/*!
 * @file aaa_api.c
 *
 * @brief Alert and Alarm task API
 *
 * @details API definition to interface *Alert and alarms module with others modules
 *          such as *Process or *Measure
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Id:: aaa_api.c 650 2014-1#$: Revision of last commit
 * $Rev:: 650                 $: Revision of last commit
 * $Author:: m175711          $: Author of last commit
 * $Date:: 2014-11-24 15:54:2#$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/alarms/src/aaa_api.c $: URL of this file
 * @endcond
 *
 */

/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @ingroup Aaa_API
 */


/* Includes ------------------------------------------------------------------*/
#include "millipore_types.h"
#include "msg_type.h"
#include "aaa_data.h"
#include "do_itf.h"
#include "sys_softerr.h"

/* Defines -------------------------------------------------------------------*/
#define MORE_LOG        /*!< use to get alarm  name rather than alarm Id  */

/*! Fill Alert and alarm message macro */
#define AAA_FILL_MSG(_pmsg, _cmd, _aaatype)\
{  \
   (_pmsg).cmd=(_cmd);\
   (_pmsg).alarmType=(_aaatype);\
}

/* Declarations --------------------------------------------------------------*/

#ifdef MORE_LOG
CONST PLAINCHAR * CONST szAlarmNameArray[]=
{
   "PERMEATE_TEMPERATURE_HIGH_SP",
   "FEED_CONDUCTIVITY_MAX       ",
   "FEED_CONDUCTIVITY_MIN       ",
   "FEED_CONDUCTIVITY_HIGH_SP   ",
   "DRAIN_AND_PERMEATE_FR_PRES  ",
   "DRAIN_AND_PERMEATE_FR_LOW   ",
   "FEED_PRESSURE_SENSOR_ERROR  ",
   "FEED_PRESSURE_MAX           ",
   "FEED_PRESSURE_MIN           ",

   /* Ro */
   "RO_PRESSURE_SENSOR_ERROR    ",
   "RO_PRESSURE_MAX             ",
   "RO_PRESSURE_MIN             ",
   "RO_PRESSURE_FEED_EQUAL      ",
   "RO_CONDUC_REJ_LOW_SP        ",
   "RO_RECOVERY_HIGH_SP         ",
   "RO_RECOVERY_LOW_SP          ",

   /* Permeate */
   "PERMEATE_TEMPERATURE_MAX    ",
   "PERMEATE_TEMPERATURE_MIN    ",
   "PERMEATE_CONDUCTIVITY_MAX   ",
   "PERMEATE_CONDUCTIVITY_MIN   ",
   "FCE_HIGH_SP                 ",
   "PERMEATE_FR_HIGH_SP         ",
   "PERMEATE_FR_LOW_SP          ",

   /* EDI */
   "EDI_FR_MAX                  ",
   "EDI_FR_MIN                  ",
   "EDI_TEMPERATURE_MAX         ",
   "EDI_TEMPERATURE_MIN         ",
   "EDI_RESISTIVITY_MAX         ",
   "EDI_RESISTIVITY_MIN         ",
   "EDI_RESISTIVITY_HIGH_SP     ",

   /* Distribution and Tank level */
   "TANKLEVEL_DISCONNECTED      ",
   "TOR_TANK_OVERFLOW           ",

   "TANKLEVEL_RESERVE           ",
   "TANKLEVEL_EMPTY             ",

   "DISTRIB_TEMPERATURE_MAX     ",
   "DISTRIB_TEMPERATURE_MIN     ",
   "DISTRIB_TEMPERATURE_HIGH_SP ",

   "DISTRIB_RESISTIVITY_MAX     ",
   "DISTRIB_RESISTIVITY_MIN     ",
   "DISTRIB_RESISTIVITY_HIGH_SP ",

   "DISTRIB_PRESSURE_SENSOR_ERR ",
   "DISTRIB_PRESSURE_MAX",

   /* Csm Pack status */
   "PACK_PROGARDS_UNPAIRED      ",
   "PACK_PROGARD1_OUT           ",
   "PACK_PROGARD1_INCORRECT     ",
   "PACK_PROGARD1_INVALID       ",
   "PACK_PROGARD2_OUT           ",
   "PACK_PROGARD2_INCORRECT     ",
   "PACK_PROGARD2_INVALID       ",
   "PACK_POLISHER_OUT           ",
   "PACK_POLISHER_INCORRECT     ",
   "PACK_POLISHER_INVALID       ",

   /* TOR */
   "TOR_LEAK_DETECTOR           ",
   "TOR_SDS_LEAK_DETECTOR       ",
   "TOR_FAULT_REPORT1           ",
   "TOR_FAULT_REPORT2           ",

   /* TOC */
   "TOC_ABOVE_SP                ",
   "TOC_CHECK_A10_COM           ",
   "TOC_A10_ERROR               ",

   /* UV */
   "UV_LAMP_SDS                 ",
   "UV_LAMP_ASM                 ",
   "UV_LAMP_SDS_DUPLEX          ",
   "UV_LAMP_ASM_DUPLEX          "
};
#endif // MORE_LOG

extern AlarmData_t g_AaaDataArray[ALARM_TYPE_LAST];    /*!< Alarm data array */




/* Code ----------------------------------------------------------------------*/

/*!
 *  @brief  Start Alerts and Alarms task
 *
 *  @details  Task is declared statically but not started because
 *            start order is set to 0. Once called , the task is started
 *
 */
VOID AAA_TaskStart(VOID)
{
   eOsRc_t rc= OS_TaskExecute("AAATASK");
   ASSERT( OS_GOOD ==rc ,"Unable to start  Alerts and Alarms task");
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *
 *  @brief  ISR reset Alert and Alarm  API procedure
 *
 *  @details   TBC
 *
 */
VOID AAA_ResetAlarmFromIsr(AlarmType_t alarmType)
{
   AAA_HandleAlarmFromIsr(AAA_CMD_ALARM_RESET, alarmType );
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  ISR reset pending Alert and Alarm  API procedure
 *
 *  @details   None
 *
 */
VOID AAA_ResetPendingAlarmFromIsr(AlarmType_t alarmType)
{
   AAA_HandleAlarmFromIsr(AAA_CMD_ALARM_RESET_PENDING, alarmType );
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  ISR Set Alert and Alarm  API procedure
 *
 *  @details   None
 *
 */
VOID AAA_SetAlarmFromIsr(AlarmType_t alarmType)
{
   AAA_HandleAlarmFromIsr(AAA_CMD_ALARM_SET, alarmType );
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *
 *  @brief  reset Alert and Alarm  API procedure
 *
 *  @details   TBC
 *
 */
VOID AAA_ResetAlarm(AlarmType_t alarmType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   AAA_FILL_MSG(Msg.body.AaaMsg, AAA_CMD_ALARM_RESET, alarmType );
   rc=  MSG_SendH(g_hAaaMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Alert and Alarm mailbox");
#ifndef MORE_LOG
   SYSLOG(INFO,"W:%d:Alarm RESET",(UINT8)alarmType);
#else
   SYSLOG(INFO,"W:%s:Alarm RESET",szAlarmNameArray[alarmType]);
#endif
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  reset pending Alert and Alarm  API procedure
 *
 *  @details   None
 *
 */
VOID AAA_ResetPendingAlarm(AlarmType_t alarmType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   AAA_FILL_MSG(Msg.body.AaaMsg, AAA_CMD_ALARM_RESET_PENDING, alarmType );
   rc = MSG_SendH(g_hAaaMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Alert and Alarm mailbox");
#ifndef MORE_LOG
   SYSLOG(INFO,"W:%d:Alarm RESET PENDING",(UINT8)alarmType);
#else
   SYSLOG(INFO,"W:%s:Alarm RESET PENDING",szAlarmNameArray[alarmType]);
#endif
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Set Alert and Alarm  API procedure
 *
 *  @details   None
 *
 */
VOID AAA_SetAlarm(AlarmType_t alarmType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   AAA_FILL_MSG(Msg.body.AaaMsg, AAA_CMD_ALARM_SET, alarmType);
   rc=MSG_SendH(g_hAaaMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Alert and Alarm mailbox");
#ifndef MORE_LOG
   SYSLOG(INFO,"W:%d:Alarm SET",(UINT8)alarmType);
#else
   SYSLOG(INFO,"W:%s:Alarm SET",szAlarmNameArray[alarmType]);
#endif
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @retval TRUE if Alarm is active
 *  @retval FALSE if Alarm is not active
 *
 *  @brief  Is Alarm active  API procedure
 *
 *  @details   If alarm is [PENDING] , alarm is NOT consider as Active.
 *
 */
BOOL AAA_IsAlarmActive(AlarmType_t alarmType)
{
   BOOL bRet=FALSE;

   ASSERT(ALARM_TYPE_LAST>alarmType ,"Alarm not managed %d",alarmType);

   if ( AAA_STATE_ACTIVE == g_AaaDataArray[(UINT32)alarmType].almState )
   {
      bRet= TRUE;
   }
   return bRet;
}

/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   alarm type
 *
 *  @retval State of the Alarm
 *
 *  @brief  Allow to access to alarm state
 *
 *  @details
 *
 */
AAA_State_t AAA_GetAlarmState(AlarmType_t alarmType)
{
   ASSERT(ALARM_TYPE_LAST>alarmType ,"Alarm not managed %d",alarmType);

   return(g_AaaDataArray[(UINT32)alarmType].almState );
}


/*!
 *  @ingroup AlarmAndAlert_func
 *  @{
 */


/*!
 *  @param  p_AaaMsg :           [IN]    AAA_Msg_t           <br>
 *                      Alarm type
 *
 *  @return None
 *
 *  @brief  Alert and Alarm message parser
 */
VOID AAA_MsgParser(CONST  AAA_Msg_t* p_AaaMsg)
{
   switch (p_AaaMsg->cmd)
   {
      case AAA_CMD_ALARM_SET:
         {
         AAA_HandleSetAlarmMsg(p_AaaMsg);
         }
         break;

      case AAA_CMD_ALARM_RESET:
         {
         AAA_HandleResetAlarmMsg(p_AaaMsg);
         }
         break;

      case AAA_CMD_ALARM_RESET_PENDING:
         {
         AAA_HandleResetPendingAlarmMsg(p_AaaMsg);
         }
         break;

      default:
         {
         ERROR("AAA_Parser : Msg type unknow: %4Xh ", p_AaaMsg->cmd );
         }
         break;
   }
}


/*!
*  @}
*/

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/


