/*!
* @file aaa_task.c
*
* @brief Measure Task
*
* @details Contain main Mesure stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: aaa_task.c 678 2014-12-1#$: Filename
* $Rev:: 678                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-12-16 13:27:08 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/alarms/src/aaa_task.c $: URL of this file
* @endcond
*
*/


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "CanOpenIface.h"
#include "millipore_types.h"
#include "os_adaptation.h"
#include "aaa_api.h"
#include "sys_softerr.h"
#include "aaa_data.h"
#include "messaging.h"
#include "mes_api.h"
#include "rop_api.h"
#include "dip_api.h"
#include "do_itf.h"
#include "objects.h"
#include "do_def.h"
#include "sys_api.h"
#include "sys_log.h"

/*!
 *  @defgroup AlarmAndAlert Alert and Alarm module
 *  @brief Alert and Alarm task module
 *  @details Alert and Alarm task is in charge of :
 *       @li Collecting alarms from measure task
 *       @li Maintaining a pending alarm list
 *       @li Trig Can PDO messages depending on alarms delay raise time.
 *       @li Waking up potential impacted process of an alarms raise.
 *
 *  @defgroup AlarmAndAlert_data Private data
 *  @brief Private data description
 *  @ingroup AlarmAndAlert
 *  @{
 */


/* Declarations --------------------------------------------------------------*/
OsHandle_t g_hAaaMboxHld;                   /*!< Alert and Alarm task mailbox */
static OsHandle_t s_hAaaSemaList[ 3u ];   /*!< Alert and Alarm semaphores handle array */


/*! contains pending delay for each alarms, and Wake up tasks in case of alarm raised
 */
static CONST AAA_pendingInf_t s_PendingInfo[ALARM_TYPE_LAST] =
{
   /* FEED */
   {  .dwDelay = 60u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_PERMEATE_TEMPERATURE_HIGH_SP,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_FEED_CONDUCTIVITY_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_FEED_CONDUCTIVITY_MIN,
   {  .dwDelay = 60u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_FEED_CONDUCTIVITY_HIGH_SP,

   {  .dwDelay = 30u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_PRES,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_LOW,

   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_FEED_PRESSURE_SENSOR_ERROR,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_FEED_PRESSURE_MAX,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_FEED_PRESSURE_MIN,

      /* RO */
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_RO_PRESSURE_SENSOR_ERROR,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_RO_PRESSURE_MAX,
   {  .dwDelay = 30u,.wakeupProc = AAA_WAKEUP_PRODUCTION  },  // ALARM_TYPE_RO_PRESSURE_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_RO_PRESSURE_FEED_EQUAL,
   {  .dwDelay = 300u,.wakeupProc = AAA_WAKEUP_NONE       },  // ALARM_TYPE_RO_CONDUCTIVITY_REJECTTION_LOW_SP,
   {  .dwDelay = 200u,.wakeupProc = AAA_WAKEUP_PRODUCTION },  // ALARM_TYPE_RO_RECOVERY_HIGH_SP,
   {  .dwDelay = 200u,.wakeupProc = AAA_WAKEUP_NONE       },  // ALARM_TYPE_RO_RECOVERY_LOW_SP,

      /* Permeate */
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_PERMEATE_TEMPERATURE_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_PERMEATE_TEMPERATURE_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_PERMEATE_CONDUCTIVITY_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_PERMEATE_CONDUCTIVITY_MIN,
   {  .dwDelay = 300u,.wakeupProc = AAA_WAKEUP_PRODUCTION }, // ALARM_TYPE_FCE_HIGH_SP,
   {  .dwDelay = 200u,.wakeupProc = AAA_WAKEUP_NONE       },  // ALARM_TYPE_PERMEATE_FLOWRATE_HIGH_SP,
   {  .dwDelay = 200u,.wakeupProc = AAA_WAKEUP_NONE       },  // ALARM_TYPE_PERMEATE_FLOWRATE_LOW_SP,

   /* EDI-Elix */
   {  .dwDelay = 60u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_FLOWRATE_MAX,
   {  .dwDelay = 60u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_FLOWRATE_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_TEMPERATURE_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_TEMPERATURE_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_RESISTIVITY_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_RESISTIVITY_MIN,
   {  .dwDelay = 60u,.wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_EDI_RESISTIVITY_HIGH_SP,

   /* Distribution and Tank level */
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_PROD_AND_DIST  },  // ALARM_TYPE_TANKLEVEL_DISCONNECTED,
   {  .dwDelay = 3u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION   },  // ALARM_TYPE_TOR_TANK_OVERFLOW,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_TANKLEVEL_RESERVE,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION   },  // ALARM_TYPE_TANKLEVEL_EMPTY,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_TEMPERATURE_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_TEMPERATURE_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_TEMPERATURE_HIGH_SP,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_RESISTIVITY_MAX,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_RESISTIVITY_MIN,
   {  .dwDelay = 10u,.wakeupProc = AAA_WAKEUP_NONE           },  // ALARM_TYPE_DISTRIB_RESISTIVITY_HIGH_SP,

   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION   },  // ALARM_TYPE_DISTRIB_PRESSURE_SENSOR_ERROR,
   {  .dwDelay = 1u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION   },  // ALARM_TYPE_DISTRIB_PRESSURE_MAX,

   /* Consumables */
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARDS_UNPAIRED,

   {  .dwDelay = 3u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD1_OUT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD1_INCORRECT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD1_INVALID,
   {  .dwDelay = 3u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD2_OUT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD2_INCORRECT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_PRODUCTION },    // ALARM_TYPE_PACK_PROGARD2_INVALID,
   {  .dwDelay = 3u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION },  // ALARM_TYPE_PACK_POLISHER_OUT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION },  // ALARM_TYPE_PACK_POLISHER_INCORRECT,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_DISTRIBUTION },  // ALARM_TYPE_PACK_POLISHER_INVALID,

   /* TOR */
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOR_LEAK_DETECTOR,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOR_SDS_LEAK_DETECTOR,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOR_FAULT_REPORT1,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOR_FAULT_REPORT2,

   /* TOC */
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOC_ABOVE_SP,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOC_CHECK_A10_COM,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_TOC_A10_ERROR,

   /* UV Lamp */
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_SDS_UV1_LAMP,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_SDS_UV2_LAMP,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        },  // ALARM_TYPE_ASM_UV1_LAMP,
   {  .dwDelay = 0u, .wakeupProc = AAA_WAKEUP_NONE        }   // ALARM_TYPE_ASM_UV2_LAMP,
};

AlarmData_t g_AaaDataArray[ALARM_TYPE_LAST];    /*!< Alarm data array */

extern CONST PLAINCHAR * CONST szAlarmNameArray[];

/* Defines -------------------------------------------------------------------*/

#define AAA_MSG_SEM_IDX ((INT32)0)
#define AAA_TIM_SEM_IDX ((INT32)1)


#define DELAY100MS     (100u)

#define MORE_LOG

/*!
 *  @}
 */

/* Code ----------------------------------------------------------------------*/
/*!
 *  @defgroup AlarmAndAlert_func Private functions
 *  @brief Private functions
 *  @ingroup AlarmAndAlert
 *  @{
 */

/*!
 *  @return None
 *
 *  @brief  Initialized Measure data
 *
 *  @details  Ensure all data are initialized before task start
 *
 */
static VOID _InitData(VOID)
{
   eOsRc_t rc;
   static const PLAINCHAR* sMBoxName = "AAA_MBX";

   static const PLAINCHAR* sSemaNameArray [3u] =
   {
      "AAA_MSG_SEM",
      "AAA_TIM_SEM",
      NULL
   };
   rc=OS_SemaLookUpL(sSemaNameArray, s_hAaaSemaList );
   /*! resolve seampahore ID to name  */
   ASSERT( OS_GOOD == rc ,"Wrong OS_SemaLookUpL");
   /*! resolve Mailbox  ID to name  */
   rc=OS_MboxLookUp( sMBoxName, &g_hAaaMboxHld );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxLookUp call");

   /*! Link MailBox to Semaphore    */
   rc=OS_MboxDefSema(sMBoxName, sSemaNameArray[AAA_MSG_SEM_IDX] );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxDefSema call");

   /*! Fill g_AaaDataArray */
   for (UINT32 i = 0u; ((UINT32) ALARM_TYPE_LAST) > i ; i++)
   {
      // LDRA_NOANALYSIS 105 d dead code found
      g_AaaDataArray[i].dwCntDown = 0u;
      g_AaaDataArray[i].pendingInf = s_PendingInfo[i];
      g_AaaDataArray[i].almState = AAA_STATE_DISABLE;
      // LDRA_ANALYSIS
   }
}


/*!
 *  @return None
 *
 *  @brief  Wake up /p Production and/or /p DIstribution task of <br>
 *                   of an alarm state change
 *
 *  @details  None
 *
 */
static VOID sAlmWarnProcess(AAA_WakeupProc_t wakeupProc, AlarmType_t alarmType, BOOL isAlarmSet)
{
   switch (wakeupProc)
   {
      case AAA_WAKEUP_PRODUCTION:
         {
         ROP_AlarmWakeup(alarmType, isAlarmSet);
         }
         break;

      case AAA_WAKEUP_DISTRIBUTION:
         {
         DIP_AlarmWakeup(alarmType, isAlarmSet);
         }
         break;

      case AAA_WAKEUP_PROD_AND_DIST:
         {
         ROP_AlarmWakeup(alarmType, isAlarmSet);
         DIP_AlarmWakeup(alarmType, isAlarmSet);
         }
         break;

      case AAA_WAKEUP_NONE:
      default:
         {
         /* There is no process to warn so we do nothing */
         }
         break;
   }
}


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Alert and Alarm task
 *
 *  @details   TBC
 *
 * @todo Improvement : following a pending list should be faster and more efficient.
 *       Check #AAA_HandleSetAlarmMsg proc.
 */
static VOID sCheckPendingAlarm(VOID)
{
   AlarmData_t* p_almData;
   AlarmType_t   alarmIdx = ALARM_TYPE_FIRST;

   /*! Fill g_AaaDataArray */
   for (UINT32 i = 0u; i < (UINT32)ALARM_TYPE_LAST; i++)
   {
      p_almData = &g_AaaDataArray[i];

      if (AAA_STATE_PENDING == p_almData->almState)
      {
         if (0u < p_almData->dwCntDown)
         {
            p_almData->dwCntDown--;
         }
         else /* if (0u == p_almData->dwCntDown) */
         {
            p_almData->almState = AAA_STATE_ACTIVE;
            /*! Update DO */
            if (ALARM_TYPE_32 > alarmIdx)
            {
               DO_OR(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms1, ((UINT32)1u << i));
            }
            else
            {
               DO_OR(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms2, ((UINT32)1u << (i - ((UINT32)ALARM_TYPE_32))));
            }
            /*! Trig PDO send */
            canSendPdo(TPDO_ALARM_AND_ALERT);
#ifndef MORE_LOG
         SYSLOG(INFO,"W:%d:Pending Alarm SET",(UINT8)i);
#else
         SYSLOG(INFO,"W:%s:Pending Alarm SET",szAlarmNameArray[i]);
#endif
            /*! Wakeup process tasks */
            sAlmWarnProcess(p_almData->pendingInf.wakeupProc, alarmIdx, TRUE);
         }
      }
      else if (AAA_STATE_PENDING_DISABLE == p_almData->almState)
      {
         /* No delay is managed to clear alarm
          * This code portion is used to clear alarm from Isr (without OS message).
          */
         p_almData->almState = AAA_STATE_DISABLE;
         /*! Update DO */
         if (ALARM_TYPE_32 > alarmIdx)
         {
            DO_AND(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms1, ~((UINT32)1u << i));
         }
         else
         {
            DO_AND(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms2, ~((UINT32)1u << (i - ((UINT32)ALARM_TYPE_32))));
         }
         /*! Trig PDO send */
         canSendPdo(TPDO_ALARM_AND_ALERT);
#ifndef MORE_LOG
         SYSLOG(INFO,"W:%d:Pending Alarm RESET",(UINT8)i);
#else
         SYSLOG(INFO,"W:%s:Pending Alarm RESET",szAlarmNameArray[i]);
#endif
         /*! Wakeup process tasks */
         sAlmWarnProcess(p_almData->pendingInf.wakeupProc, alarmIdx, FALSE);
      }
      else
      {
         /* Dear MISRA */
      }
      /* LDRA_INSPECTED 123 S wrong alert on enum inc */
      /* LDRA_INSPECTED 411 S wrong alert on enum inc */
      alarmIdx++;
   }

}


/*!
 * @param p_AaaMsg [IN] CONST  AAA_Msg_t * <br>
 *             as  pointer on Alert and Alarm message
 *
 *  @return None
 *
 *  @brief  Handle set Alert and Alarm message.\n
 *     raise Alert and Alarm on alarm specified.
 */
VOID AAA_HandleSetAlarmMsg(CONST  AAA_Msg_t* p_AaaMsg)
{
   AlarmData_t* p_almData;

   ASSERT(p_AaaMsg->alarmType < ALARM_TYPE_LAST,"Alarm not managed");

   p_almData = &g_AaaDataArray[(UINT32)(p_AaaMsg->alarmType)];

   if (AAA_STATE_DISABLE == p_almData->almState )
   {
      /*! check pending delay present */
      if (0u == p_almData->pendingInf.dwDelay)
      {
         /*! Here is no pending delay */
         /*! set new state alarm to active */
         /* LDRA_INSPECTED 105 D false dead code */
         p_almData->almState = AAA_STATE_ACTIVE;
         /*! Update DO */
         if (ALARM_TYPE_32 > p_AaaMsg->alarmType)
         {
            /* LDRA_INSPECTED 403 S wrong need detetc */
            DO_OR(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms1, ((UINT32)1u << (UINT8)(p_AaaMsg->alarmType)));
         }
         else
         {
            /* LDRA_INSPECTED 403 S wrong ned detetc */
            DO_OR(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms2, ((UINT32)1u << (UINT8)(p_AaaMsg->alarmType - ALARM_TYPE_32)));
         }
         /*! Trig PDO send */
         canSendPdo(TPDO_ALARM_AND_ALERT);
         /*! Warn Production / Distribution process */
         sAlmWarnProcess(p_almData->pendingInf.wakeupProc, p_AaaMsg->alarmType, TRUE);

      }
      else
      {
         /*! there is a pending delay */
         /*! arm a timer \n
            Depending on timer used for AAA module , here 100 ms -> x10 to get timer in sec */
         /* LDRA_INSPECTED 105 D wrong dead code */
         p_almData->dwCntDown = p_almData->pendingInf.dwDelay*10u;
         /*! set Pending state  */
         /* LDRA_INSPECTED 105 D wrong dead code */
         p_almData->almState = AAA_STATE_PENDING;
      }
      /*! Send PDO */
   }
   else //if (AAA_STATE_ACTIVE == p_almData->almState )
        //if (AAA_STATE_PENDING == p_almData->almState )
   {
      /*! Nothing is done , if already active or pending  */
   }
}


/*!
 * @param p_AaaMsg [IN] CONST  AAA_Msg_t * <br>
 *             as  pointer on Alert and Alarm message
 *
 *  @return None
 *
 *  @brief  Handle reset Alert and Alarm message
 *     clear Alert and Alarm on alarm specified.
 *
 *  @details
 *
 */
VOID AAA_HandleResetAlarmMsg(CONST  AAA_Msg_t* p_AaaMsg)
{
   AlarmData_t* p_almData;

   ASSERT( ALARM_TYPE_LAST > p_AaaMsg->alarmType ,"Alarm not managed");

   p_almData = &g_AaaDataArray[(UINT32)(p_AaaMsg->alarmType)];

   if (   AAA_STATE_ACTIVE  == p_almData->almState
       || AAA_STATE_PENDING == p_almData->almState )

   {
      /*! set new state to disable */
      p_almData->almState = AAA_STATE_DISABLE;
      /*! Clear pending delay */
      p_almData->dwCntDown = 0u;
      /*! Update DO */
      if (ALARM_TYPE_32 > p_AaaMsg->alarmType)
      {
         DO_AND(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms1, ~((UINT32)1u << p_AaaMsg->alarmType));
      }
      else
      {
         DO_AND(g_DoAlertsAndAlarms.ListOfAlertsAndAlarms2, ~((UINT32)1u << (p_AaaMsg->alarmType - ALARM_TYPE_32)));
      }
      /*! Trig PDO send */
      canSendPdo(TPDO_ALARM_AND_ALERT);
      /*! Warn Production / Distribution process , alarm clear */
      sAlmWarnProcess(p_almData->pendingInf.wakeupProc, p_AaaMsg->alarmType, FALSE);
   }
   else if (AAA_STATE_DISABLE == p_almData->almState )
   {
      /*! Noting is done , if already disable  */
   }
   else
   {
      /* Nothing to do */
   }
}


/*!
 * @param p_AaaMsg [IN] CONST  AAA_Msg_t * <br>
 *             as  pointer on Alert and Alarm message
 *
 *  @return None
 *
 *  @brief  Handle reset pending Alert and Alarm message
 *     clear Alert and Alarm on alarm specified.
 *
 *  @details
 *
 */
VOID AAA_HandleResetPendingAlarmMsg(CONST  AAA_Msg_t* p_AaaMsg)
{
   AlarmData_t* p_almData;

   ASSERT(p_AaaMsg->alarmType < ALARM_TYPE_LAST,"Alarm not managed");

   p_almData = &g_AaaDataArray[(UINT32)(p_AaaMsg->alarmType)];

   if (AAA_STATE_PENDING == p_almData->almState )
   {
      /* LDRA_INSPECTED 105 D wrong dead code */
      p_almData->almState = AAA_STATE_DISABLE;
      /* LDRA_INSPECTED 105 D wrong dead code */
      p_almData->dwCntDown = 0u;
   }
   else if (AAA_STATE_ACTIVE  == p_almData->almState)
   {
       /* Dear LDRA */
   }
   else
   {
      /* Dear LDRA */
   }
}


/*!
 * @param cmd [IN]  AAA_MsgCmd_t  <br>
 *             as  Alert and Alarm command
 *
 * @param alarmType [IN]  AlarmType_t  <br>
 *             Alarm type
 *
 *  @return None
 *
 *  @brief  Handle set, reset or , reset pending Alert and Alarm command from ISR
 *             dwCntDown is set to 0 so that sCheckPendingAlarm take into account alarm command immediatly.
 *
 *  @details
 *
 */
VOID AAA_HandleAlarmFromIsr(AAA_MsgCmd_t cmd, AlarmType_t alarmType)
{
   AlarmData_t* p_almData;

   ASSERT(alarmType < ALARM_TYPE_LAST,"Alarm not managed");

   p_almData = &g_AaaDataArray[(UINT32)(alarmType)];

   switch (cmd)
   {
      case AAA_CMD_ALARM_SET:
         {
            /* If not active, alarm will be activated asap on next \a sCheckPendingAlarm call */
            if(AAA_STATE_DISABLE == p_almData->almState)
            {
               p_almData->dwCntDown=0;
               p_almData->almState=AAA_STATE_PENDING;
            }
         }
         break;

      case AAA_CMD_ALARM_RESET:
      case AAA_CMD_ALARM_RESET_PENDING:
         {
            /* If active, alarm will be disabled asap on next \a sCheckPendingAlarm call */
            if(AAA_STATE_ACTIVE == p_almData->almState)
            {
               p_almData->dwCntDown=0;
               p_almData->almState=AAA_STATE_PENDING_DISABLE;
            }
         }
         break;

      default:
         {
         ERROR("Cmd type unknow: %4Xh ", cmd );
         }
         break;
   }

}


/*!
 *  @return None
 *
 *  @brief  Alert and Alarm task
 *
 *  @details   TBC
 *
 */
/* LDRA_INSPECTED 61 D Wrong static declaration */
ATTRIBUTE_TASK VOID alertAndAlarmTask(VOID)
{
   SYSLOG(INFO, "AlertAndAlarm task ......: START");

   _InitData();

   g_bAppTaskInit++;

   /* LDRA_INSPECTED 28 D false infinite loop */
   while ( FALSE == g_doLaunchApp )
   {
      OS_TaskDelay(DELAY100MS);
   }

   while (g_isAlertAndAlarmTaskActive)
   {
      INT32       sdwSemaListOffset;
      AAA_Msg_t   aaaMsg;

      sdwSemaListOffset = OS_SemaWaitLHI( (OsHandle_t*)s_hAaaSemaList ) ;

      switch ( sdwSemaListOffset )
      {
         case AAA_MSG_SEM_IDX:
            {
            /*! Just get the message (does not wait for it) */
            MSG_GetWaitH( g_hAaaMboxHld, (psMSG_t)&aaaMsg, (UINT8)0U /* Get */ );
            /*! Analyses message content */
            AAA_MsgParser(&aaaMsg);
            }
            break;

         case AAA_TIM_SEM_IDX:
            {
            /*! manage pending alarms list */
            sCheckPendingAlarm();
            }
            break;

         default:
            {
            ERROR("Wrong Sema receive in AlertAndAlarm task %d",sdwSemaListOffset);
            }
            break;
      }
   }

   SYSLOG(INFO, "AlertAndAlarm task ......: END");

   OS_TaskTerminate(OS_SELFTASK);
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

