/*!
 * @file act_api.c
 *
 * @brief Actuator API
 *
 * @details API definition to interface *Actuator module with others modules
 *          such as *Process or *Measure
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Id:: act_api.c 650 2014-1#$: Revision of last commit
 * $Rev:: 650                 $: Revision of last commit
 * $Author:: m175711          $: Author of last commit
 * $Date:: 2014-11-24 15:54:2#$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_api.c $: URL of this file
 * @endcond
 *
 */

/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @ingroup Act_API
 */


/* Includes ------------------------------------------------------------------*/
#include "dil_pump.h"
#include "dil_valve.h"
#include "dil_uv.h"
#include "dil_tor_out.h"
#include "dil_tor_out.h"
#include "millipore_types.h"
#include "sys_softerr.h"
#include "mes_api.h"
#include "act_api.h"
#include "act_data.h"
#include "do_def.h"
#include "do_itf.h"
#include "canopeniface.h"

/* Defines -------------------------------------------------------------------*/
#define MORE_LOG        /*!< use to get Actuators name rather than alarm Id  */

#define ACT_PUMP_LOW_CURRENT  (200u)    /*!< Low current threshold to consider pump is OFF */

#define ACT_ASM_UV_DEFAULT       (1)      /*!< Asm UV default  value */
#define ACT_ASM_UV_NO_DEFAULT    (0)      /*!< Asm UV no default value */

#define ACT_RO_PUMP_CURRENT_MIN                 (2500.0f)       /*!< Min RO Pump current */
#define ACT_DISTRIBUTION_PUMP_CURRENT_MIN       (2500.0f)       /*!< Min Distribution Pump current */

#define ACT_UV_ASM_DIAG_DELAY                (10u)  /*!< 10 seconds delays to start UV diagnose after power on. */

/* Declarations --------------------------------------------------------------*/
ActData_t g_ActDataArray[ACT_TYPE_LAST];        /*!< Actuator data array */
static UINT8 s_bActbDistribPumpDutyCycle=0;            /*!< Distribution pump  duty cycle */
static FLOAT s_ActDataReportCurrent[2]={0.0f,0.0f };   /*! 4-20 mA interfaces output current */

#ifndef D_BSP_STM3240GEVAL
/*! List of Actuators to be diagnosed when Production request */
static CONST ActType_t s_ProductionActuatorToDiagList[]=
{
/*   ACT_TYPE_EXT_SV_VALVE = ACT_TYPE_FIRST, */  /* Not possible because valve always */
   ACT_TYPE_INLET_SV_VALVE,                  /*!< SV=Valve SV FEED */
   ACT_TYPE_PERMEATE_SV_VALVE_1,             /*!< SV=Permeate Valve 1 */
   ACT_TYPE_PERMEATE_SV_VALVE_2,             /*!< SV=Permeate SV Valve 2 */
   ACT_TYPE_RO_PUMP,                         /*!< PUMP=RO Pump */
   ACT_TYPE_VACUUM_PUMP,                     /*!< TOR_OUT=TOR_OUT_VACCUM PUMP*/
   ACT_TYPE_LAST
};

/*! List of Actuators to be diagnosed when Ditribution is OFF */
static CONST ActType_t s_DitributionOffActuatorToDiagList[]=
{
   ACT_TYPE_DUMPING_SV_VALVE,
   ACT_TYPE_RECIRCULATION_SV_VALVE,
/*   ACT_TYPE_SDS_PUMP_1,                 */   /* diag not necessay */
/*   ACT_TYPE_SDS_PUMP_2,                 */   /* diag not necessay*/
/*   ACT_TYPE_UV_SDS,                     */   /*diag not necessay*/
/*   ACT_TYPE_UV_SDS_DUPLEX,              */   /* diag not necessay */
/*  ACT_TYPE_TANK_FILLING,                */   /* Not availalable for diag because connected to Gpio */
/*   ACT_TYPE_UF_TRIGGER,                 */   /* Not availalable for diag because connected to Gpio */
   ACT_TYPE_LAST
};

/*! List of Actuators to be diagnosed when Ditribution is ON */
static CONST ActType_t s_DitributionOnActuatorToDiagList[]=
{
   ACT_TYPE_DISTRIBUTION_PUMP,
   ACT_TYPE_LAST
};
#endif

#ifdef MORE_LOG
/*! Actuators name for log */
static CONST PLAINCHAR * CONST s_szActuatorNameArray[]=
{
   "EXT   SV VALVE",
   "INLET SV VALVE",
   "PERM1 SV VALVE",
   "PERM2 SV VALVE",
   "DUMP  SV VALVE",
   "REC   SV VALVE",
   "ROP   PUMP    ",
   "DIS   PUMP    ",
   "SDS   PUMP 1  ",
   "SDS   PUMP 2  ",
   "VACUM PUMP    ",
   "ALARM REPORT 1",
   "ALARM REPORT 2",
   "TANK  FILLING ",
   "UF    TRIGGER ",
   "POD   CMD     ",
   "UV    SDS     ",
   "UV    SDS DUPX",
   "UV    ASM     ",
   "UV    ASM DUPX",


   "EDI           ",
   "UV    PRODUCT ",
   "UV    DISTRIB "
};
#endif

/* Code ----------------------------------------------------------------------*/
/*!
 *  @param pActuatorList [IN] ActType_t * <br>
 *                   actuator type list.<br>
 *                   Shall be ended with ACT_TYPE_LAST.
 *
 *  @return Diagnose result on actuators list
 *
 *  @brief  Handle diagnose on actuators list.
 */
UINT32 sDiagnoseActuatotorList(ActType_t *pActuatorList)
{
   UINT32 dwDiagnotic=0;
   UINT8 actDiag;
   /*LDRA_INSPECTED 69 D variable used before assignment. */
   ActType_t actType;
   FLOAT fDistribPumpCurrent;

   /* Loop on actuator list */
   while(ACT_TYPE_LAST!=*pActuatorList)
   {
      actDiag=0u;
      actType=*pActuatorList;
      /* According actuator type , diagnose differs */
      switch(actType)
      {
         case ACT_TYPE_EXT_SV_VALVE:
         case ACT_TYPE_INLET_SV_VALVE:
         case ACT_TYPE_PERMEATE_SV_VALVE_1:
         case ACT_TYPE_PERMEATE_SV_VALVE_2:
         case ACT_TYPE_DUMPING_SV_VALVE:
         case ACT_TYPE_RECIRCULATION_SV_VALVE:
            {
            /*! Diag for Solenoid valves */
            actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_SOL_VAL_GET_STATUS);
            }
            break;

         case ACT_TYPE_DISTRIBUTION_PUMP:
            {
            /* Ensure Distribution pump is on to be tested.
             * Current is measured, if current velow distribution piump theshoold, then Pump is declared HS
             */
            fDistribPumpCurrent=MES_GetMeasure(MES_DISTRIBUTION_PUMP_CURRENT);
            if(ACT_DISTRIBUTION_PUMP_CURRENT_MIN>fDistribPumpCurrent)
            {
               WARNING("Distribution pump current low (%d mA)",(UINT32)fDistribPumpCurrent);
               actDiag=1u;
            }
            }
            break;

         case ACT_TYPE_RO_PUMP:
            {
            /*! Diag for RO Pump */
            actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_PUMP_GET_STATUS);
            }
            break;

         case ACT_TYPE_SDS_PUMP_1:
         case ACT_TYPE_SDS_PUMP_2:
         case ACT_TYPE_VACUUM_PUMP:
         case ACT_TYPE_ALARM_REPORT_1:
         case ACT_TYPE_ALARM_REPORT_2:
         case ACT_TYPE_POD_CMD:
            {
            /* TOR */
            actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_TOR_OUT_GET_STATUS);
            }
            break;

         case ACT_TYPE_UV_SDS:
         case ACT_TYPE_UV_SDS_DUPLEX:
            {
            /* UV */
            actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_UV_GET_STATUS);
            }
            break;

         case ACT_TYPE_UV_ASM:
         case ACT_TYPE_UV_ASM_DUPLEX:
            {
            /*! ASM UV Lamps are not diag here because it required to switch it on then which is not good
             * On the contrary , default is reported once UV lamp is switch on by CAN SDO.\n
             * Let refer to Can trig  siwtch on command. (in usr_301.c : sdoWrInd ).
             */
            actDiag=0u;
            }
            break;

         case ACT_TYPE_EDI:
         case ACT_TYPE_UV_PRODUCTION:
         case ACT_TYPE_UV_DISTRIBUTION:
         /* No diag too for these one : Edi, UV production, UV distribution , Tank filling , UF trigger */
         case ACT_TYPE_TANK_FILLING:
         case ACT_TYPE_UF_TRIGGER:
            {
            /* Tank filling and UF backwash can't be diag as ther are directly pluggeg throught Gpios (without drivers) */
            actDiag=0u;
            }
            break;

         default:
            {
            ASSERT(FALSE,"Wrong actuator get %d",actType);
            }
            break;
      }

      if(0u!=actDiag )
      {
#ifndef MORE_LOG
         SYSLOG(INFO,"A:%d:Failure=%d",(UINT8)actType,actDiag);
#else
         SYSLOG(INFO,"A:%s:Failure=%d",s_szActuatorNameArray[actType],actDiag);
#endif
         dwDiagnotic|= ((UINT32)1<<(UINT8)actType);
      }
      /* move to next actuator */
      /*LDRA_INSPECTED  567 S Pointer arithmetic is not on array */
      pActuatorList++;
   }

   return dwDiagnotic;
}


/*!
 *  @brief  Start Actuators
 *
 *  @details  initialised Actuator module  data according given system
 *
 */
VOID ACT_Start(VOID)
{
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type
 *
 *  @param actAction [IN] ActAction_t <br>
 *                   actuator action
 *
 *  @return None
 *
 *  @brief  set Actuator power API procedure
 */
VOID ACT_SetPower(ActType_t actType,ActAction_t actAction)
{
#ifndef D_BSP_STM3240GEVAL
   switch(actType)
   {
      case ACT_TYPE_EXT_SV_VALVE:
      case ACT_TYPE_INLET_SV_VALVE:
      case ACT_TYPE_PERMEATE_SV_VALVE_1:
      case ACT_TYPE_PERMEATE_SV_VALVE_2:
      case ACT_TYPE_DUMPING_SV_VALVE:
      case ACT_TYPE_RECIRCULATION_SV_VALVE:
         {
         /* Solenoid valve */
         ACT_ValveSetPower(actType,actAction);
         }
         break;

      case ACT_TYPE_RO_PUMP:
      case ACT_TYPE_DISTRIBUTION_PUMP:
         {
         /* Pump */
         ACT_PumpSetPower(actType,actAction);
         }
         break;

      case ACT_TYPE_SDS_PUMP_1:
      case ACT_TYPE_SDS_PUMP_2:
      case ACT_TYPE_VACUUM_PUMP:
      case ACT_TYPE_ALARM_REPORT_1:
      case ACT_TYPE_ALARM_REPORT_2:
      case ACT_TYPE_TANK_FILLING:
      case ACT_TYPE_UF_TRIGGER:
      case ACT_TYPE_POD_CMD:
         {
         /* TOR */
         ACT_TorSetPower(actType,actAction);
         }
         break;

      case ACT_TYPE_UV_SDS:
      case ACT_TYPE_UV_SDS_DUPLEX:
      case ACT_TYPE_UV_ASM:
      case ACT_TYPE_UV_ASM_DUPLEX:
         {
         /* UV Lamp */
         ACT_UvSetPower(actType,actAction);
         /*! Test ASM Uv Lamps only if switch ON, Otherwise no status can be produced */
         if( ACT_SWITCH_ON==actAction )
         {

            if( (ACT_TYPE_UV_ASM==actType)
            ||  (ACT_TYPE_UV_ASM_DUPLEX==actType ))
            {
               /* Diagnose UV ASM lamp */
               ACT_UvLaunchDelayedDiagnose(actType,ACT_UV_ASM_DIAG_DELAY);
            }
            else
            {
               /* Other UV lamp are not diagnosed */
            }
         }
         else if( ACT_SWITCH_OFF==actAction )
         {
            if( (ACT_TYPE_UV_ASM==actType)
            ||  (ACT_TYPE_UV_ASM_DUPLEX==actType ))
            {
               /* Cancel Diagnose UV ASM lamp */
               ACT_UvCancelDiagnose(actType);
            }
            else
            {
               /* Other UV lamp are not diagnosed */
            }
         }
         }
         break;

      case ACT_TYPE_EDI:
         {
         /* CAN */
         DO_WRITE(g_DoEdiModule,actAction);
         canSendPdo(TPDO_EDI_AND_UV_ACTUATORS);
         }
         break;

      case ACT_TYPE_UV_PRODUCTION:
         {
         /* CAN */
         DO_WRITE(g_DoEdiUv,actAction);
         canSendPdo(TPDO_EDI_AND_UV_ACTUATORS);
         }
         break;

      case ACT_TYPE_UV_DISTRIBUTION:
         {
         /* CAN */
         DO_WRITE(g_DoDistributionUv,actAction);
         canSendPdo(TPDO_EDI_AND_UV_ACTUATORS);
         }
         break;

      default:
         {
         /* Unmanaged actuators: Nothing to do */
         }
         break;
   }
#endif
#ifndef MORE_LOG
   SYSLOG(INFO,"A:%d:Set=%d",(UINT8)actType,(UINT8)actAction);
#else
   SYSLOG(INFO,"A:%s:Set=%d",s_szActuatorNameArray[(UINT8)actType],(UINT8)actAction);
#endif
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type
 *
 *  @return ActState_t
 *  @retval None
 *
 *  @brief  return actuator state API procedure
 *
 */
ActState_t ACT_GetState(ActType_t actType)
{
   ActState_t actState=ACT_SWITCH_OFF;
#ifndef D_BSP_STM3240GEVAL
   switch(actType)
   {
      case ACT_TYPE_EXT_SV_VALVE:
      case ACT_TYPE_INLET_SV_VALVE:
      case ACT_TYPE_PERMEATE_SV_VALVE_1:
      case ACT_TYPE_PERMEATE_SV_VALVE_2:
      case ACT_TYPE_DUMPING_SV_VALVE:
      case ACT_TYPE_RECIRCULATION_SV_VALVE:
         {
         /* Solenoid valve */
         actState=ACT_ValveGetState(actType);
         }
         break;

      case ACT_TYPE_RO_PUMP:
      case ACT_TYPE_DISTRIBUTION_PUMP:
         {
         /* Pump */
         actState=ACT_PumpGetState(actType);
         }
         break;

      case ACT_TYPE_SDS_PUMP_1:
      case ACT_TYPE_SDS_PUMP_2:
      case ACT_TYPE_VACUUM_PUMP:
      case ACT_TYPE_ALARM_REPORT_1:
      case ACT_TYPE_ALARM_REPORT_2:
      case ACT_TYPE_POD_CMD:
         {
         /* TOR */
         actState=ACT_TorGetState(actType);
         }
         break;

      case ACT_TYPE_UV_SDS:
      case ACT_TYPE_UV_SDS_DUPLEX:
         {
         /* UV Lamp */
         actState=ACT_UvGetState(actType);
         }
         break;

      /* Following actuators are not managed because there command is controlled with Can command */
      case ACT_TYPE_EDI:
      case ACT_TYPE_UV_PRODUCTION:
      case ACT_TYPE_UV_DISTRIBUTION:
      /* Following actuators are not available for getstate procedure */
      case ACT_TYPE_UF_TRIGGER:
      case ACT_TYPE_TANK_FILLING:
      case ACT_TYPE_UV_ASM:
      case ACT_TYPE_UV_ASM_DUPLEX:
         {
         ERROR("ACT_GetState not available for act=%d",actType);
         }
         break;

      default:
         {
         ERROR("Wrong actuator get %d",actType);
         }
         break;
   }
#else
   NOT_USED_PARAM(actType);
#endif

   return actState;
}


/*!
 *  @param bDutyCycle [IN] UINT8 <br>
 *                   Duty cycle (0-100)
 *
 *  @return None
 *
 *  @brief  Set motor valve duty cycle API procedure
 *
 */
VOID ACT_SetDistributionPumpDutyCycle(UINT8 bDutyCycle)
{
#ifndef D_BSP_STM3240GEVAL
   ASSERT((100u>=bDutyCycle) ,"Wrong duty cycle %d",bDutyCycle);
   ACT_PumpSetDutyCycle(bDutyCycle);
#endif
   s_bActbDistribPumpDutyCycle=bDutyCycle;
}


/*!
 *  @return UINT8
 *
 *  @brief  return distribution pump duty cycle API procedure
 *
 */
UINT8 ACT_GetDistributionPumpDutyCycle(VOID)
{
   return s_bActbDistribPumpDutyCycle;
}


/*!
 *  @param actDataReport [IN] ActDataReport_t <br>
 *                   data report actuator
 *
 *  @param fCurrent [IN] FLOAT <br>
 *                   data report current to set  (4-20)
 *
 *  @return None
 *
 *  @brief  set Data report current API procedure
 *
 */
VOID ACT_SetDataReport(ActDataReport_t actDataReport, FLOAT fCurrent)
{
   ASSERT(((ACT_DATA_REPORT_1==actDataReport) || (ACT_DATA_REPORT_2==actDataReport)),"Wrong data report ID=%d",actDataReport);
   ASSERT(((20.0f>=fCurrent) || (4.0f<=fCurrent)),"Wrong data report current %d",(UINT8)fCurrent);
#ifndef D_BSP_STM3240GEVAL
   ACT_SetDataReportCurrent(actDataReport, fCurrent);
#endif
   s_ActDataReportCurrent[actDataReport]=fCurrent;
}


/*!
 *  @param actDataReport [IN] ActDataReport_t <br>
 *                   data report actuator
 *
 *  @return UINT8
 *
 *  @brief  return Data report current API procedure
 *
 */
FLOAT ACT_GetDataReport(ActDataReport_t actDataReport)
{
   ASSERT(((ACT_DATA_REPORT_1==actDataReport) || (ACT_DATA_REPORT_2==actDataReport)),"Wrong data report ID=%d",actDataReport);
   return s_ActDataReportCurrent[actDataReport];
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type
 *
 *  @brief Trig UV diagnostic API procedure.
 *           And update CAN Uv diag field according diag result.
 *
 */
VOID ACT_GetUvDiagnosticReport(ActType_t actType)
{
   ASSERT((ACT_TYPE_UV_ASM==actType || ACT_TYPE_UV_ASM_DUPLEX==actType),"Wrong acttype for UV diag req %d",actType);

   /* We expect UV to be switched on before */
   if(ACT_SWITCH_ON==ACT_UvGetState(actType))
   {
      ACT_UvLaunchDelayedDiagnose(actType,1L);
   }
   else
   {
      WARNING("UV diag expect to be used once power ON.");
   }
}


/*!
 *  @param actDiagType [IN] ActDiagType_t <br>
 *                   Diag type
 *
 *
 *  @brief Trig diagnostic API procedure.
 *
 * @details May be called at init stage, at Production idle stage, Distriubtion idle stage or specific stages depending on
 *          diagnostic request. Ie to get a diagnotic on Distribution pump, it is necessary to test disritution pump when this one is ON.\n
 *          If ACT_DIAG_INIT is selected, all actuators are switch  off before Diag.\n
 *          If ACT_DIAG_PRODUCTION,ACT_DIAG_DISTRIBUTION, or ACT_DIAG_DISTRIBUTION  is selected, only a selection of actuators are diagnosed.\n
 */
VOID ACT_GetDiagnosticReport(ActDiagType_t actDiagType)
{
   UINT32 dwDiagnotic=0;
#ifndef D_BSP_STM3240GEVAL
   ActType_t actType;
   UINT8 actDiag;

   switch(actDiagType)
   {
      case ACT_DIAG_INIT:
            {
            /* Last one is ASM dupleix because following are controle with CAN and no diagnose is available for them */
            for(actType=ACT_TYPE_FIRST;ACT_TYPE_UV_ASM_DUPLEX>=actType;actType++)
            {
               ACT_SetPower(actType,ACT_SWITCH_OFF);
               switch(actType)
               {
                  case ACT_TYPE_EXT_SV_VALVE:
                  case ACT_TYPE_INLET_SV_VALVE:
                  case ACT_TYPE_PERMEATE_SV_VALVE_1:
                  case ACT_TYPE_PERMEATE_SV_VALVE_2:
                  case ACT_TYPE_DUMPING_SV_VALVE:
                  case ACT_TYPE_RECIRCULATION_SV_VALVE:
                     {
                     /*! Diag for Solenoid valves */
                     actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_SOL_VAL_GET_STATUS);
                     }
                     break;

                  case ACT_TYPE_DISTRIBUTION_PUMP:
                     {
                     /*! Diag for Distibution Pump */
                     /* Distribution pump can be tested as it required to switch on Pump which not possible during init */
                     actDiag=0u;
                     }
                     break;

                  case ACT_TYPE_RO_PUMP:
                     {
                     /*! Diag for RO Pump */
                     actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_PUMP_GET_STATUS);
                     }
                     break;

                  case ACT_TYPE_SDS_PUMP_1:
                  case ACT_TYPE_SDS_PUMP_2:
                  case ACT_TYPE_VACUUM_PUMP:
                  case ACT_TYPE_ALARM_REPORT_1:
                  case ACT_TYPE_ALARM_REPORT_2:
                  case ACT_TYPE_POD_CMD:
                     {
                     /* TOR */
                     actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_TOR_OUT_GET_STATUS);
                     }
                     break;

                  case ACT_TYPE_UV_SDS:
                  case ACT_TYPE_UV_SDS_DUPLEX:
                     {
                     /* UV */
                     actDiag=ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_UV_GET_STATUS);
                     }
                     break;

                  case ACT_TYPE_UV_ASM:
                  case ACT_TYPE_UV_ASM_DUPLEX:
                     {
                     /*! ASM UV Lamps are not diag here because it required to switch it on then which is not good
                      * On the contrary , default is reported once UV lamp is switch on by CAN SDO.\n
                      * Let refer to Can trig  siwtch on command. (in usr_301.c : sdoWrInd ).
                      */
                     actDiag=0u;
                     }
                     break;

                  case ACT_TYPE_EDI:
                  case ACT_TYPE_UV_PRODUCTION:
                  case ACT_TYPE_UV_DISTRIBUTION:
                  /* No diag too for these one : Edi, UV production, UV distribution , Tank filling , UF trigger */
                  case ACT_TYPE_TANK_FILLING:
                  case ACT_TYPE_UF_TRIGGER:
                     {
                     /* Tank filling and UF backwash can't be diag as ther are directly pluggeg throught Gpios (without drivers) */
                     actDiag=0u;
                     }
                     break;

                  default:
                     {
                     ASSERT(FALSE,"Wrong actuator get %d",actType);
                     }
                     break;
               }

               if(0u!=actDiag )
               {
#ifndef MORE_LOG
                  SYSLOG(INFO,"A:%d:Failure=%d",(UINT8)actType,actDiag);
#else
                  SYSLOG(INFO,"A:%s:Failure=%d",s_szActuatorNameArray[(UINT8)actType],actDiag);
#endif
                  dwDiagnotic|= ((UINT32)1<<actType);
               }
            }
            /* Update DO actuator diagnotic value */
            DO_WRITE(g_DoHardwareDiagnostic.ActuatorsDiagResult,dwDiagnotic);
            }
            break;

      case ACT_DIAG_PRODUCTION:
            {
            dwDiagnotic=sDiagnoseActuatotorList((ActType_t*)s_ProductionActuatorToDiagList);
            /* Update DO actuator diagnotic value */
            DO_OR(g_DoHardwareDiagnostic.ActuatorsDiagResult,dwDiagnotic);
            }
            break;
      case ACT_DIAG_DISTRIBUTION_OFF:
            {
            dwDiagnotic=sDiagnoseActuatotorList((ActType_t*)s_DitributionOffActuatorToDiagList);
            /* Update DO actuator diagnotic value */
            DO_OR(g_DoHardwareDiagnostic.ActuatorsDiagResult,dwDiagnotic);
            }
            break;
      case ACT_DIAG_DISTRIBUTION_ON:
            {
            dwDiagnotic=sDiagnoseActuatotorList((ActType_t*)s_DitributionOnActuatorToDiagList);
            /* Update DO actuator diagnotic value */
            DO_OR(g_DoHardwareDiagnostic.ActuatorsDiagResult,dwDiagnotic);
            }
            break;
      default:
            {
            WARNING("actDiagType %d not managed",actDiagType);
            }
            break;
   }
#else
   NOT_USED_PARAM(actDiagType);
#endif
   DO_READ(dwDiagnotic,g_DoHardwareDiagnostic.ActuatorsDiagResult);
   /* related PDO is sens by measure task */
   SYSLOG(INFO,"A:Diag=%08Xh",dwDiagnotic);
}


/*!
 *  @param actType [IN] ActType_t <br>
 *                   actuator type
 *
 *  @return Pump current
 *  @retval None
 *
 *  @brief  return Pump current API procedure
 *         and set pumps  diagnotic .\n
 *         Available for for RO pump and Distribution pump.
 *
 *  @details called by Measure task every second.
 *          -# For RO pump , diagnose is done by retrieving pump status and checking current presence if pump is ON.\n
 *          -# For Distribution pump , diagnose is done by checking current presence if pump is ON.\n
 *           Min current level to trig default is set to 200 mA (ACT_PUMP_LOW_CURRENT)
 */
UINT16 ACT_GetPumpCurrent(ActType_t actType)
{
   UINT16 wPumpCurrent;
   ASSERT(ACT_TYPE_DISTRIBUTION_PUMP==actType || ACT_TYPE_RO_PUMP==actType ,"Wrong actuator get cuurent req (%d)",actType);
   wPumpCurrent=ACT_PumpGetCurrent(actType);

   /* ACT_GetDiagnostic/GetStatus only available for RO Pump */
   if(ACT_TYPE_RO_PUMP == actType)
   {
      if(0u != ACT_GetDiagnostic(actType,(UINT32)IOCTL_CMD_PUMP_GET_STATUS))
      {
         /* Update DO actuator diagnotic value */
         DO_OR(g_DoHardwareDiagnostic.ActuatorsDiagResult,(UINT32)1u<<(UINT8)actType);
      }
      else
      {
         DO_AND(g_DoHardwareDiagnostic.ActuatorsDiagResult,~((UINT32)1u<<(UINT8)actType));
      }
   }
   else
   {
      /* For distribution pump, check connection when pump is ON */
      if(ACT_SWITCH_ON == ACT_PumpGetState(actType))
      {
         if(ACT_PUMP_LOW_CURRENT > wPumpCurrent )
         {
            /* Update DO actuator diagnotic value */
            DO_OR(g_DoHardwareDiagnostic.ActuatorsDiagResult,(UINT32)1u<<(UINT8)actType);
         }
         else
         {
            DO_AND(g_DoHardwareDiagnostic.ActuatorsDiagResult,~((UINT32)1u<<(UINT8)actType));
         }
      }
   }

   return (wPumpCurrent);
}

/*!
*  @}
*/

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

