/*!
* @file  act_proc.c
*
* @brief Actuators procedures
*
* @details NA
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: act_proc.c 521 2014-08-0#$: FileName
* $Rev:: 521                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-08-06 15:47:32 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_proc.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "dil_4_20_out.h"

#include "millipore_types.h"
#include "act_api.h"
#include "act_data.h"
#include "dil_api.h"
#include "dil_iface.h"
#include "sys_softerr.h"
#include "os_adaptation.h"

/*!
 *  @defgroup Actuators Actuators
 *  @brief Actuators module
 *  @details Acuators module is in charge of manage actuators.\n
 *           Actuators list contains :
 *           @li External Solenoid Valve
 *           @li Feed Solenoid Valve
 *           @li Permeate Solenoid Valve 1 (Production)
 *           @li Permeate Solenoid Valve 2 (Rinsing)
 *           @li Dumping Solenoid Valve
 *           @li Recirculation Loop Solenoid Valve
 *           @li RO Pump
 *           @li Distribution Pump
 *           @li SDS Pump 1
 *           @li SDS Pump 2 (Duplex)
 *           @li SDS UV 1
 *           @li SDS UV 2 (Duplex)
 *           @li ASM UV 1
 *           @li ASM UV 2 (Duplex)
 *           @li Vacuum Pump.
 *  Actuators module provides an api to activate/deactivate actuator.\n
 *  It also includes a api to retrieve actuator state.\n
 *  It includes a diagnositc procedure which allow produce a diagnotic report.\n
 *  At least, it includes a api to modify Pump actuators duty cycle.
 *
 *  @defgroup Actuators_data Private data
 *  @brief Private data
 *  @ingroup Actuators
 *  @{
 */
/* Defines -------------------------------------------------------------------*/

/* Declarations --------------------------------------------------------------*/
static DIL_HANDLE  _Act_4_20_mA_DilHdlArray[2]={0u,0u}; /*!< 4-20 ma out dil handle array store */

/*! Store DIL actuators name */
CONST DIL_CHAR * CONST g_strDilNameArray[]=
{
   DIL_NAME_SV_EXTERNAL_INLET,   // ACT_TYPE_EXT_SV_VALVE
   DIL_NAME_SV_FEED,   // ACT_TYPE_INLET_SV_VALVE
   DIL_NAME_SV_PERMEATE_1,   // ACT_TYPE_PERMEATE_SV_VALVE_1
   DIL_NAME_SV_PERMEATE_2,   // ACT_TYPE_PERMEATE_SV_VALVE_2
   DIL_NAME_SV_DUMPING,   // ACT_TYPE_DUMPING_SV_VALVE
   DIL_NAME_SV_LOOP_RECIRCULATION,   // ACT_TYPE_RECIRCULATION_SV_VALVE
   DIL_NAME_PUMP_RO,   // ACT_TYPE_RO_PUMP
   DIL_NAME_PUMP_DISTRIBUTION,   // ACT_TYPE_DISTRIBUTION_PUMP
   DIL_NAME_TOR_OUTPUT_SDS_PUMP,   // ACT_TYPE_SDS_PUMP_1
   DIL_NAME_TOR_OUTPUT_SDS_PUMP_DUPLEX,   // ACT_TYPE_SDS_PUMP_2
   DIL_NAME_TOR_OUTPUT_VACUUM_PUMP,   // ACT_TYPE_VACUUM_PUMP
   DIL_NAME_TOR_OUTPUT_ALARM_REPORT_1,   // ACT_TYPE_ALARM_REPORT_1
   DIL_NAME_TOR_OUTPUT_ALARM_REPORT_2,   // ACT_TYPE_ALARM_REPORT_2
   DIL_NAME_TOR_OUTPUT_TANKREFILL,   // ACT_TYPE_TANK_FILLING
   DIL_NAME_TOR_OUTPUT_UF_BACKWASH_TRIGGER,   // ACT_TYPE_UF_TRIGGER
   DIL_NAME_TOR_OUTPUT_POD,   // ACT_TYPE_POD_CMD
   DIL_NAME_UV_SDS_LAMP,   // ACT_TYPE_UV_SDS
   DIL_NAME_UV_SDS_LAMP_DUPLEX,   // ACT_TYPE_UV_SDS_DUPLEX
   DIL_NAME_UV_ASM_LAMP,   // ACT_TYPE_UV_ASM
   DIL_NAME_UV_ASM_LAMP_DUPLEX,   // ACT_TYPE_UV_ASM_DUPLEX (20)
   NULL,   // ACT_TYPE_EDI
   NULL,   // ACT_TYPE_UV1
   NULL,   // ACT_TYPE_UV2
};


/*
Bit 1  = External Solenoid Valve
Bit 2  = Feed Solenoid Valve
Bit 3  = Permeate Solenoid Valve 1 (Production)
Bit 4  = Permeate Solenoid Valve 2 (Rinsing)
Bit 5  = Dumping Solenoid Valve
Bit 6  = Recirculation Loop Solenoid Valve
Bit 7  = RO Pump
Bit 8  = Distribution Pump
Bit 9  = SDS Pump 1
Bit 10 = SDS Pump 2 (Duplex)
Bit 11 = SDS UV 1
Bit 12 = SDS UV 2 (Duplex)
Bit 13 = ASM UV 1
Bit 14 = ASM UV 2 (Duplex)
Bit 15 = Vacuum Pump

*/

extern CONST DIL_CHAR * g_strDilNameArray[];

/*!
 *   @}
 */

/* Code ----------------------------------------------------------------------*/
/*!
 *  @defgroup Actuators_func Private functions
 *  @brief Private functions
 *  @ingroup Actuators
 */

VOID ACT_Reset(ActType_t actType)
{
   DIL_RC ret;
   if(0u==g_ActDataArray[actType].dilHdl)
   {
      ret=dil_closedevice(g_strDilNameArray[actType],NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL actuator reset :  %s",g_strDilNameArray[actType]);
   }
   g_ActDataArray[actType].dilHdl=0u;
}

VOID ACT_Init(ActType_t actType)
{
   DIL_RC ret;
   if(0u==g_ActDataArray[actType].dilHdl)
   {
      ret=dil_opendevice(g_strDilNameArray[actType],&g_ActDataArray[actType].dilHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL actuator init :  %s",g_strDilNameArray[actType]);
   }
}

/*!
 * @fn ACT_CheckDilInit
 *
 * @param  actType  [IN]   ActType_t  <br>
 *   actuator type
 *
 * @param  pActData  [IN]   ActData_t*  <br>
 *
 *
 * @return DIL_HANDLE
 * @retval dih handle
 *
 * @brief Returns Hanlde on Actuator driver
 *
 * @details   Handle actuator driver initialisation if not initialized
 *
 */
DIL_HANDLE ACT_CheckDilInit(ActType_t actType,ActData_t *pActData)
{
   DIL_RC ret;
   if(0u==pActData->dilHdl)
   {
      ret=dil_opendevice(g_strDilNameArray[actType],&pActData->dilHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL actuator init :  %s",g_strDilNameArray[actType]);
      OS_TaskDelay(10);
   }
   return pActData->dilHdl;
}


/*!
 * @fn ACT_GetDiagnostic
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return UINT8
 * @retval 0:OK, 1:Disconnected, 2:Short cirtcuit
 *
 * @brief  Return  actuator diagnostic
 *
 * @details  NA
 *
 */
UINT8 ACT_GetDiagnostic(ActType_t actType,UINT32 dilDiagCmd)
{
   UINT8 bDiag=0;
   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret=DIL_FAILURE;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, dilDiagCmd,(DIL_LPVOID)&bDiag, sizeof(bDiag), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL get status  call %d",actType);
   }
   return bDiag;
}


/*!
 * @fn ACT_SetDataReportCurrent
 *
 * @param  actDataReport  [IN]   ActDataReport_t  <br>
 *   Data report actuator type
 *
 * @param  fCurrent  [IN]   FLOAT  <br>
 *          Current
 *
 * @return None
 * @retval NA
 *
 * @brief Set data report current
 *
 * @details   NA
 *
 */
VOID ACT_SetDataReportCurrent(ActDataReport_t actDataReport,FLOAT fCurrent)
{
   DIL_RC ret;
   /* Hanlde Init if necessary */
   if(0uL==_Act_4_20_mA_DilHdlArray[actDataReport])
   {
      if(ACT_DATA_REPORT_1==actDataReport)
      {
         ret=dil_opendevice(DIL_NAME_4_20MA_OUT_DATA_REPORT_1,&_Act_4_20_mA_DilHdlArray[actDataReport], (DIL_PVOID)NULL);
      }
      else
      {
         ret=dil_opendevice(DIL_NAME_4_20MA_OUT_DATA_REPORT_2,&_Act_4_20_mA_DilHdlArray[actDataReport], (DIL_PVOID)NULL);
      }

      ASSERT(DIL_GOOD == ret,"Error in DIL 420out actuator init :  %d",actDataReport);
   }
   /* call set value DIL */
   ret=dil_ioctldevice(_Act_4_20_mA_DilHdlArray[actDataReport],(UINT32)IOCTL_CMD_4_20mA_OUT_SET_VALUE, 
                            (DIL_LPVOID)&fCurrent, (UINT32)sizeof(fCurrent), (DIL_LPUINT32)NULL); 
   ASSERT(DIL_GOOD == ret,"Error in DIL 4-20 ma Out set current  call");
}


/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

