/*!
* @file act_pump.c
*
* @brief Pump interface procedures
*
* @details Actuators pump procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: act_pump.c 377 2014-04-0#$: FileName
* $Rev:: 377                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-04-01 14:34:00 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/actuators/src/act_pump.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include "dil_pump.h"
#include "millipore_types.h"
#include "do_itf.h"
#include "do_def.h"
#include "objects.h"
#include "act_api.h"
#include "act_data.h"
#include "dil_api.h"
#include "dil_iface.h"
#include "canopeniface.h"
#include "sys_softerr.h"

/* Defines -------------------------------------------------------------------*/

/* Declarations --------------------------------------------------------------*/

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/
/*!
 *  @ingroup Actuators_func
 *  @{
 */

/*!
 * @fn ACT_PumpSetPower
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @param  actAction  [IN]   ActAction_t  <br>
 *         Action order on actuator
 *
 * @return None
 *
 * @brief  Set power on actuator
 *
 * @details   NA
 *
 */
VOID ACT_PumpSetPower(ActType_t actType,ActAction_t actAction)
{
   BOOL isPoweredDoField=FALSE;

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      UINT8 bCmd=0;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);
      if(ACT_SWITCH_ON == actAction )
      {
         bCmd= 1u ;
         isPoweredDoField=TRUE;
      }
      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_PUMP_CONTROL , 
                            (DIL_LPVOID)bCmd, (UINT32)sizeof(bCmd), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL PUMP Control call");
   }
   /* write power status in OD */
   switch(actType)
   {
      case ACT_TYPE_RO_PUMP:
         DO_WRITE(g_DoRoPump.IsPowered,isPoweredDoField);
         break;
      case ACT_TYPE_DISTRIBUTION_PUMP:
         DO_WRITE(g_DoDistributionPump.IsPowered,isPoweredDoField);
         break;
      default:
         ASSERT(FALSE,"Wrong actuator type %d",actType);
         break;
   }
   /* Update OD actuator status */
   if(TRUE==isPoweredDoField)
   {
      DO_OR(g_DoActuatorsStatus.ActuatorsStatus , ((UINT16)1<<actType));
   }
   else
   {
      DO_AND(g_DoActuatorsStatus.ActuatorsStatus , (~((UINT16)1<<actType)));
   }
   /* Send PDO */
   canSendPdo(TPDO_ACTUATOR_STATUS);
}


/*!
 * @fn ACT_PumpGetState
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return ActState_t
 * @retval
 *
 * @brief  Return  actuator state
 *
 * @details   If Actuator 's state differs from the one save , it assert
 *
 */
ActState_t ACT_PumpGetState(ActType_t actType)
{
   ActState_t actState;
   UINT8 bState=0;
   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_PUMP_GET_STATE ,
                            (DIL_LPVOID)&bState, (UINT32)sizeof(bState), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL PUMP get state  call");
   }
   actState=(1u==bState ? ACT_SWITCH_ON : ACT_SWITCH_OFF);
   return actState;
}


/*!
 * @fn ACT_PumpSetDutyCycle
 *
 * @param  bDutyCycle  [IN]   UINT8  <br>
 *                Duty cycle (0..100)
 *
 * @return None
 * @retval
 *
 * @brief  Set Distribution Pump duty cycle voltage
 *
 * @details NA
 *
 */
VOID ACT_PumpSetDutyCycle(UINT8 bDutyCycle)
{
   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(ACT_TYPE_DISTRIBUTION_PUMP))
   {
      DIL_RC   ret;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(ACT_TYPE_DISTRIBUTION_PUMP,&g_ActDataArray[ACT_TYPE_DISTRIBUTION_PUMP]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_PUMP_SET_DUTY_CYCLE,
                            (DIL_LPVOID)bDutyCycle, (UINT32)sizeof(bDutyCycle), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL PUMP set duty cycle  call");
   }
}


/*!
 * @fn ACT_PumpGetCurrent
 *
 * @param  actType  [IN]   ActType_t  <br>
 *                Actuator  type
 *
 * @return Pump current (mA)
 * @retval
 *
 * @brief  Return  Pump actuator current used (mA)
 *
 * @details   NA
 *
 */
UINT16 ACT_PumpGetCurrent(ActType_t actType)
{
   UINT16 wPumpCurrent=0u;
   

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == (BOOL)IS_DIL_WRITE_BYPASSED(actType))
   {
      DIL_RC   ret;
      /*retrieve dilHdl */
      DIL_HANDLE dilHdl=ACT_CheckDilInit(actType,&g_ActDataArray[actType]);

      /* call DIL */
      ret = dil_ioctldevice(dilHdl, (UINT32)IOCTL_CMD_PUMP_GET_CURRENT ,
                            (DIL_LPVOID)&wPumpCurrent, (UINT32)sizeof(wPumpCurrent), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL PUMP get current  call");
   }

   return wPumpCurrent;
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

