/*!
 * @file dip_api.c
 *
 * @brief Distribution task API
 *
 * @details API definition to interface *Distribution module with others modules
 *          such as *Process or *Measure
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Id:: dip_api.c 690 2014-12-2#$: Revision of last commit
 * $Rev:: 690                    $: Revision of last commit
 * $Author:: m175711             $: Author of last commit
 * $Date:: 2014-12-24 10:31:40 +#$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_api.c $: URL of this file
 * @endcond
 *
 */

/* Includes ------------------------------------------------------------------*/
#include "millipore_types.h"
#include "os_adaptation.h"
#include "msg_type.h"
#include "sys_softerr.h"
#include "dip_data.h"
#include "dip_private.h"


/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @defgroup Dip_API  API
 *
 *  @ingroup Distribution
 *
 *  @brief Distibution module API description
 *
 *  @details Messages list :
 *       @li DIP_Start
 *       @li DIP_SendComand
 *       @li DIP_SetSanityFlags
 *       @li DIP_GetSanityFlags
 *       @li DIP_CsnStatusChange
 *       @li DIP_AlarmWakeup
 *  @{
 */



/* Declarations --------------------------------------------------------------*/
#ifndef NDEBUG
UINT32   g_dipWakeUpCnt = 0u; /*! allow testing wake messag tx rx for Unit test*/
#endif

/* Code ----------------------------------------------------------------------*/


/*!
 *  @brief  Start Distribution task
 *
 *  @details  Task is declared statically but not started because
 *            start order is set to 0. Once called , the task is started
 *
 */
VOID DIP_TaskStart(VOID)
{
   eOsRc_t rc= OS_TaskExecute("DIPTASK");
   ASSERT( OS_GOOD == rc,"Unable to start Distribution task");
}


/*!
 *  @param order [IN] DipOrder_e <br>
 *                   Order type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  send comand (order) to /p Distribution task
 *
 *  @details This message is send order to Distribution parameters by /p CANopen task
 *             upon SDO comand receiption (from IHM).
 *
 */
VOID DIP_SendComand(DipOrder_e order)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_SEND_ORDER;
   Msg.body.Dip.extraField.order = order;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Production task mailbox");
}

/*!
 *  @param autoDumpFlag [IN] BOOL <br>
 *
 *  @return None
 *  @retval None
 *
 *  @brief  set Auto-Dump Flag
 *
 *  @details Change value for the Auto-Dump Flag
 *
 */
VOID DIP_SetAutoDumpFlag(BOOL autoDumpFlag)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_SET_AUTO_DUMP_FLAG;
   Msg.body.Dip.extraField.AutoDumpFlag = autoDumpFlag;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}

/*!
 *  @return BOOL
 *
 *  @brief  Get isProductionDisable flag
 *
 *  @details   Allow \p Production task to know if production shall be disabled
 *
 */
BOOL DIP_isProductionDisable(VOID)
{
   return DistributionProcessParameters.ProcessDescription.isProductionDisable;
}


/*!
 *  @return BOOL Auto-Dump flag value
 *
 *  @brief  Get Auto Dump flag  API procedure
 *
 *  @details   Allow \p Production task to retrieve auto-dump flag which is handle by
 *                \p Distribution task.
 *
 */
BOOL DIP_GetAutoDumpFlag(VOID)
{
   return DistributionProcessParameters.ProcessDescription.AutoDumpFlag;
}


/*!
 *  @param csmStatus [IN] CSM_Status_t <br>
 *                   new CSM status
 *
 *  @brief  CSM status change API
 *
 *  @details   Allow \p Consumable task to inform \p Distribution task from a consumable change.
 *
 */
VOID DIP_CsmStatusChange(CSM_Status_t csmStatus)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_CSM_STATUS_CHANGE;
   Msg.body.Dip.extraField.csmStatus = csmStatus;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}


/*!
 *  @param isPodPres [IN] BOOL <br>
 *                  TRUE if POd is present
 *
 *  @brief  MES pad status  change API
 *
 *  @details   Allow \p Mesure task to inform \p Distribution task from Pod status change
 *
 */
VOID DIP_MesPodStatusChange(BOOL isPodPres)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_MES_POD_STATUS_CHANGE;
   Msg.body.Dip.extraField.isPodPres = isPodPres;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   Alarm type
 *
 *  @param isAlarmSet [IN] BOOL <br>
 *                   TRUE is Alarm set
 *
 *  @brief  New alarm raised API
 *
 *  @details   Allow \p Alert and Alarm  task to inform \p Distribution task from a new alarm.
 *
 */
VOID DIP_AlarmWakeup(AlarmType_t alarmType, BOOL isAlarmSet)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_ALARM_WAKEUP;
   Msg.body.Dip.extraField.alarmWakeup.alarmType = alarmType;
   Msg.body.Dip.extraField.alarmWakeup.isAlarmSet = isAlarmSet;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}

/*!
 *  @brief  A setpoint has been modified
 *
 *  @details   Allow \p CanOpen task to infomr \p Distribution task that setpoint has been modified
 *
 */
VOID DIP_UpdateSetpoints (VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_UPDATE_SETPOINT;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}

/*!
 *  @brief  First start flag has been modified
 *
 *  @details   Allow \p CanOpen task to infomr \p Distribution task that setpoint has been modified
 *
 */
VOID DIP_UpdateFirstStart (VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Dip.cmd = DIP_CMD_UPDATE_FIRST_START_FLAG;
   rc=MSG_SendH(hDipMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Distribution mailbox");
}

/*!
 *  @brief  is the process in Test Mode
 *
 *  @details   Allow \p CanOpen task to know \p if Distribution task is in test mode
 *
 */
BOOL DIP_isProcessInTestMode (VOID)
{
   BOOL isTestMode = FALSE;
   
   if (DIP_MODE_TEST == DistributionProcessParameters.ProcessDescription.ProcessMode)
   {
      isTestMode = TRUE;
   }
   return(isTestMode);
}
/*!
 *  @}
 */


/*!
 *  @ingroup Distribution
 *  @{
 */


/*!
 *  @param  p_DipMsg :           [IN]    DIP_Msg_t           <br>
 *                      Distribution message pointer
 *
 *  @return None
 *
 *  @brief  Distribution message parser
 */
VOID DIP_MsgParser(CONST DIP_Msg_t* p_DipMsg)
{
   switch (p_DipMsg->cmd)
   {
      case DIP_CMD_SEND_ORDER:
      {
         dip_distributionStateMachine(p_DipMsg->extraField.order);
         break;
      }
      
      case DIP_CMD_SET_AUTO_DUMP_FLAG:
      {
         DistributionProcessParameters.ProcessDescription.AutoDumpFlag = p_DipMsg->extraField.AutoDumpFlag;
         dip_distributionStateMachine(DIP_LAST_ORDER);
         break;
      }
      
      case DIP_CMD_CSM_STATUS_CHANGE:
      case DIP_CMD_MES_POD_STATUS_CHANGE:
      {
         dip_distributionStateMachine(DIP_LAST_ORDER);
         break;
      }

      case DIP_CMD_ALARM_WAKEUP:
      {
#ifndef NDEBUG
         g_dipWakeUpCnt++;
#endif
         SYSLOG(INFO, "D: R Wakeup type:%d",p_DipMsg->extraField.alarmWakeup.alarmType);
         dip_distributionStateMachine(DIP_LAST_ORDER);
         break;
      }

      case DIP_CMD_UPDATE_SETPOINT:
      {
         dip_readCanOpenSetpoints();
         break;
      }

      case DIP_CMD_UPDATE_FIRST_START_FLAG:
      {
         dip_readFirstStartFlag ();
         break;
      }

      default:
      {
         ERROR("DIP_Parser : Msg type unknow: %4Xh ", p_DipMsg->cmd );
         break;
      }
   }
}


/*!
*  @}
*/

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

