/*!
* @file dip_sequence.c
*
* @brief DIstribution Task
*
* @details Contain all Distribution stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: dip_sequence.c 683 2014-#$: Filename
* $Rev:: 683                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-12-22 11:36:49 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_sequence.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/

#define DIP_ACTUATORS_TEST_FREQUENCY (300000U) /* 5 minutes in msec */
#define DIP_EMERGENCY_FLOW_RATE (1.8F) /* 1.8 L/min */
#define DIP_TANK_EMPTYING_FLOWRATE (4.0F) /* 4 L/min */
#define DIP_INIT_DELAY (19000U) /* 19 seconds in msec */

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "dip_api.h"
#include "dip_rec.h"
#include "sys_softerr.h"
#include "dip_data.h"
#include "mes_api.h"
#include "sys_log.h"
#include "dip_private.h"
#include "system_parameters.h"
#include "act_api.h"
#include "mvm_api.h"
#include "toc_api.h"
#include "do_itf.h"
#include "do_def.h"

/* Declarations --------------------------------------------------------------*/

BOOL DistributionSkipSequence;
UINT8 dip_numberOfSanCycle;

static BOOL dip_measureEnabled = FALSE;
static BOOL dip_tankFillingSpeedMeasureEnabled = FALSE;
static BOOL dip_firstRecircHasBeenDone = FALSE;
static TimeCounter_t dip_exitAlarmStopCounter;

static VOID dip_manageSequenceInitialization(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceStandby(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceListening(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceRunning(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceDumping(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceAlarmStop(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequencePolisherFlush(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceOpticapFlush(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceSan1(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceSan2(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceSan3(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceSan4(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceStartPackTankFilling(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceTankEmptying(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceTest(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceTestRecirc(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageSequenceTocCleaning(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_setNewState(DIP_ProcessParam_t * p_ProcessParam, DipState_e NewState);
static BOOL dip_isStartDistributionConditions (VOID);
static BOOL dip_isStopDistributionConditions (VOID);
static VOID dip_stopMesureAndCounters (VOID);
static VOID dip_startRecirculationMeasureRecordAndAlarm (VOID);

static VOID dip_manageStopTankLevelSpeed(DipTransition_e new_transition, DipTransition_e old_transition);
static VOID dip_manageStartTankLevelSpeedAndDiagnose(DIP_ProcessParam_t * p_ProcessParam);
static VOID dip_manageProdDisablingFlag(DIP_ProcessParam_t * p_ProcessParam);

static VOID dip_manageTocRequest (UINT32 stateTimer);
static BOOL dip_isTocMeterRunning (VOID);
static FLOAT dip_getVolumeFromMm (FLOAT TankLevelMm);
static UINT32 dip_computeRemainingTimeToEmptyTankFromMm (FLOAT TankLevelMm);
static UINT32 dip_computeRemainingTimeToEmptyTankFromVol (FLOAT TankVolume);
static UINT32 dip_computeRemainingTimeToFillTank (FLOAT wantedVolume, FLOAT productFlowRate, FLOAT TankLevelMm);
static FLOAT dip_getProductFlowrate (VOID);
static VOID dip_updateRemainingTime (DIP_ProcessParam_t * p_ProcessParam, UINT32 RemainingTime);

/* Code ----------------------------------------------------------------------*/

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the different type of sequence to call
 *
 *  @details:
 *
 */
 VOID dip_manageSequence (DIP_ProcessParam_t * p_ProcessParam)
 {
   BOOL ReEnterSequence = TRUE;

   while (TRUE == ReEnterSequence)
   {
      switch(p_ProcessParam->ProcessDescription.ProcessSequence)
      {
         case DIP_SEQUENCE_INITIALIZATION:         {dip_manageSequenceInitialization(p_ProcessParam);        break;}
         case DIP_SEQUENCE_STANDBY:                {dip_manageSequenceStandby(p_ProcessParam);               break;}
         case DIP_SEQUENCE_LISTENING:              {dip_manageSequenceListening(p_ProcessParam);             break;}
         case DIP_SEQUENCE_RUNNING:                {dip_manageSequenceRunning(p_ProcessParam);               break;}
         case DIP_SEQUENCE_DUMPING:                {dip_manageSequenceDumping(p_ProcessParam);               break;}
         case DIP_SEQUENCE_ALARM_STOP:             {dip_manageSequenceAlarmStop(p_ProcessParam);             break;}
         case DIP_SEQUENCE_POLISHER_FLUSH:         {dip_manageSequencePolisherFlush(p_ProcessParam);         break;}
         case DIP_SEQUENCE_OPTICAP_FLUSH:          {dip_manageSequenceOpticapFlush(p_ProcessParam);          break;}
         case DIP_SEQUENCE_STARTPACK_TANK_FILLING: {dip_manageSequenceStartPackTankFilling(p_ProcessParam);  break;}
         case DIP_SEQUENCE_TANK_EMPTYING:          {dip_manageSequenceTankEmptying(p_ProcessParam);          break;}
         case DIP_SEQUENCE_TEST:                   {dip_manageSequenceTest(p_ProcessParam);                  break;}
         case DIP_SEQUENCE_TEST_RECIRC:            {dip_manageSequenceTestRecirc(p_ProcessParam);            break;}
         case DIP_SEQUENCE_SAN1:                   {dip_manageSequenceSan1(p_ProcessParam);                  break;}
         case DIP_SEQUENCE_SAN2:                   {dip_manageSequenceSan2(p_ProcessParam);                  break;}
         case DIP_SEQUENCE_SAN3:                   {dip_manageSequenceSan3(p_ProcessParam);                  break;}
         case DIP_SEQUENCE_SAN4:                   {dip_manageSequenceSan4(p_ProcessParam);                  break;}
         case DIP_SEQUENCE_TOC_CLEANING:           {dip_manageSequenceTocCleaning(p_ProcessParam);           break;}
         default:
         {
            ERROR("Unknown Distribution Process Sequence: %d", p_ProcessParam->ProcessDescription.ProcessSequence);
            break;
         }
      }

      dip_manageStartTankLevelSpeedAndDiagnose(p_ProcessParam);
      dip_manageProdDisablingFlag(p_ProcessParam);
      ReEnterSequence = p_ProcessParam->ProcessDescription.isEnteringSequence;
   }
 }

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewState [OUT] DipState_e <br>
 *                    New Process State
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Set a new state
 *
 *  @details: If the new state is different from the old one, the parameters are reset
 *            and if the associated transition is different, the process re-enter a new transition
 *
 */
static VOID dip_setNewState(DIP_ProcessParam_t * p_ProcessParam, DipState_e NewState)
{
   DipTransition_e dip_transition;
   DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter();

   if (NewState != p_ProcessParam->ProcessDescription.ProcessState)
   {
      p_ProcessParam->ProcessDescription.ProcessState = NewState;
      dip_transition = dip_getAssociatedTransition(NewState);
      if (dip_transition != p_ProcessParam->TransitionDescription.TransitionType)
      {
         dip_manageStopTankLevelSpeed(dip_transition, p_ProcessParam->TransitionDescription.TransitionType);
         p_ProcessParam->TransitionDescription.TransitionType = dip_transition;
         p_ProcessParam->TransitionDescription.TransitionStatus = DIP_TRANSITION_START;
         dip_manageTransition(&(p_ProcessParam->TransitionDescription));
      }
      else
      {
         /* Do Nothing */
      }
   }
   else
   {
      /* Do Nothing */
   }
}

 /*!
 *  @brief  Check if the TankLevelSpeed computation needed to be started
 */
static VOID dip_manageStopTankLevelSpeed(DipTransition_e new_transition, DipTransition_e old_transition)
{
   if ((DIP_TRANSITION_TO_WAIT == new_transition) ||
       (DIP_TRANSITION_TO_WAIT_TEST == new_transition) ||
       (DIP_TRANSITION_TO_DEPRESSURIZATION == new_transition) ||
       (DIP_TRANSITION_TO_START_KIT_DUMP == new_transition) ||
       (DIP_TRANSITION_TO_START_KIT_FILL_TANK == new_transition))
   {
      /* Dist Pump will be of */
      if ((DIP_TRANSITION_TO_RECIRCULATION == old_transition) ||
          (DIP_TRANSITION_TO_SAN_RECIRCULATION == old_transition) ||
          (DIP_TRANSITION_TO_DUMP == old_transition))
      {
         /* Dist Pump was on, so stop record */
         MES_RecordStop(MES_TANK_FILLING_SPEED);
         dip_tankFillingSpeedMeasureEnabled = FALSE;
      }
   }
   else
   {
      /* Dist Pump will be on */
      if ((DIP_TRANSITION_TO_WAIT == old_transition) ||
          (DIP_TRANSITION_TO_WAIT_TEST == old_transition) ||
          (DIP_TRANSITION_TO_DEPRESSURIZATION == old_transition) ||
          (DIP_TRANSITION_TO_START_KIT_DUMP == old_transition) ||
          (DIP_TRANSITION_TO_START_KIT_FILL_TANK == old_transition))
      {
         /* Dist Pump was off, so stop record */
         MES_RecordStop(MES_TANK_FILLING_SPEED);
         dip_tankFillingSpeedMeasureEnabled = FALSE;
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @brief  Check if the TankLevelSpeed computation needed to be started
 */
static VOID dip_manageStartTankLevelSpeedAndDiagnose(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);
   if((20000U <= stateTimer) &&
      (FALSE == dip_tankFillingSpeedMeasureEnabled))
   {
      MES_RecordStart(MES_TANK_FILLING_SPEED, FALSE);
      dip_tankFillingSpeedMeasureEnabled = TRUE;
   }

   if (((UINT32)DIP_ACTUATORS_TEST_FREQUENCY <= stateTimer) &&
       (0U == (stateTimer % (UINT32)DIP_ACTUATORS_TEST_FREQUENCY)))
   {
      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case DIP_STATE_WAIT:
         case DIP_STATE_DEPRESSURIZATION:
         {
            ACT_GetDiagnosticReport(ACT_DIAG_DISTRIBUTION_OFF);
            break;
         }
         case DIP_STATE_RECIRCULATION:
         case DIP_STATE_RECIRCULATION_MEASURE:
         case DIP_STATE_DISTRIBUTION_MEASURE:
         case DIP_STATE_DUMP:
         {
            ACT_GetDiagnosticReport(ACT_DIAG_DISTRIBUTION_ON);
            break;
         }
         case DIP_STATE_START_KIT_DUMP:
         case DIP_STATE_START_KIT_FILL_TANK:
         case DIP_STATE_WAIT_TEST:
         default:
         {
            /* Do Nothing */
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @brief  Check the coherence for Flag isProdDisabled
 */
static VOID dip_manageProdDisablingFlag(DIP_ProcessParam_t * p_ProcessParam)
{
   if((DIP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN1_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_CAN_NOT_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_SAN2_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (DIP_MODE_CAN_NOT_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      p_ProcessParam->ProcessDescription.isProductionDisable = TRUE;
   }
   else
   {
      p_ProcessParam->ProcessDescription.isProductionDisable = FALSE;
   }
}

 /*!
 *  @brief  Check if the start Distribution Conditions are TRUE
 */
static BOOL dip_isStartDistributionConditions (VOID)
{
   BOOL StartDistribution;
   FLOAT DistributionPressure = MES_GetMeasure(MES_DISTRIBUTION_PRESSURE)*1000.0; // In mBar
   UINT32 stateTimer = TMC_GetTimeCounter(DistributionProcessParameters.ProcessDescription.StateTimeCounter);

   if ((1000U * (UINT32)DIST_START_DIST_DELAY) <= stateTimer)
   {
      if(DistributionPressure <= DistributionProcessSetpoints.StartDistributionPressure)
      {
         StartDistribution = TRUE;
      }
      else
      {
         StartDistribution = FALSE;
      }
   }
   else
   {
      StartDistribution = FALSE;
   }

   return(StartDistribution);
}

 /*!
 *  @brief  Check if the stop Distribution Conditions are TRUE
 */
static BOOL dip_isStopDistributionConditions (VOID)
{
   BOOL StopDistribution = FALSE;
   FLOAT DistributionPressure = MES_GetMeasure(MES_DISTRIBUTION_PRESSURE)*1000.0F; // In mBar
   if(DistributionPressure >= DistributionProcessSetpoints.StopDistributionPressure)
   {
      StopDistribution = TRUE;
   }
   return(StopDistribution);
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceInitialization(DIP_ProcessParam_t * p_ProcessParam)
{
   CSM_Status_t PolisherStatus = CSM_STATUS_MAX;
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if((UINT32)DIP_INIT_DELAY <= stateTimer) // After DIP_INIT_DELAY
      {
         PolisherStatus = CSM_GetStatus(CSM_POLISHER);

         if(CSM_STATUS_INIT != PolisherStatus)
         {
            if(TRUE == p_ProcessParam->pProcessRecord->isFirstStartUp)
            {
               if((DIP_MODE_STARTPACK_TANK_FILLING == p_ProcessParam->pProcessRecord->InterruptedProcessMode) ||
                  (DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP == p_ProcessParam->pProcessRecord->InterruptedProcessMode))
               {
                  dip_setNewMode(p_ProcessParam, p_ProcessParam->pProcessRecord->InterruptedProcessMode, FALSE);
               }
               else
               {
                  dip_setNewMode(p_ProcessParam, DIP_MODE_CAN_NOT_RESTART_STARTPACK_TANK_FILLING, TRUE);
               }
            }
            else
            {
               // Recover the interrupted sequence if it is coherent
               if ((DIP_MODE_INITIALIZATION != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (DIP_MODE_TEST != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (DIP_MODE_TEST_ALARM_STOP != p_ProcessParam->pProcessRecord->InterruptedProcessMode) &&
                   (DIP_MODE_TEST_RECIRC != p_ProcessParam->pProcessRecord->InterruptedProcessMode))
               {
                  dip_setNewMode(p_ProcessParam, p_ProcessParam->pProcessRecord->InterruptedProcessMode, FALSE);
               }
               else
               {
                  // else force to Standby
                  dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
               }
            }
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceStandby(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DEPRESSURIZATION);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      dip_stopMesureAndCounters();
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      TOC_Stop();
      if(TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable)
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      if (DIP_STATE_DEPRESSURIZATION == p_ProcessParam->ProcessDescription.ProcessState)
      {
         stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

         if((1000U * (UINT32)DIST_DEPRESS_DURATION) <= stateTimer) // After DIST_DEPRESS_DURATION of DEPRESSURIZATION
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceListening(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   UINT32 recirculationDelay;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      TOC_Stop();
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case DIP_STATE_WAIT:
         {
            if (FALSE == dip_firstRecircHasBeenDone)
            {
               // Force the recirculation, if the first has not been done
               recirculationDelay = 0U;
            }
            else if ((TRUE == SYS_GetDeviceOption(SYSTEM_OPT_TOC)) &&
                     (TRUE == DistributionProcessSetpoints.TocMeasureRequested))
            {
               // Force the recirculation, if a TOC measure is requested
               recirculationDelay = 0U;
            }
            else if((3600U * (UINT32)DIST_AUTO_RECIRC_FREQUENCY) > ((UINT32)DistributionProcessSetpoints.AutoRecirculationDuration * 60U)) // DIST_AUTO_RECIRC_FREQUENCY is in hour, AutoRecirculationDuration in minutes
            {
               recirculationDelay = (3600U * (UINT32)DIST_AUTO_RECIRC_FREQUENCY) - ((UINT32)DistributionProcessSetpoints.AutoRecirculationDuration * 60U); // DIST_AUTO_RECIRC_FREQUENCY is in hour, AutoRecirculationDuration in minutes
            }
            else
            {
               recirculationDelay = 0U;
            }

            if (TRUE == dip_isStartDistributionConditions())
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_RUNNING, TRUE);
            }
            else if ((TRUE == DistributionProcessSetpoints.AutoDumpOptionActivation) &&
                     (TRUE == p_ProcessParam->ProcessDescription.AutoDumpFlag))
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_DUMPING, TRUE);
            }
            else if((recirculationDelay * 1000U) <= stateTimer) // recirculationDelay in secondes
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION_MEASURE);
               DistributionSkipSequence = FALSE;
            }
            else
            {
               /* Do Nothing: Continue to wait */
            }
               break;
         }

         case DIP_STATE_RECIRCULATION_MEASURE:
         {
            if(((1000U * (UINT32)START_WATER_QUALITY_MEASURE_DELAY) <= stateTimer) && // Timer ok for mesure
               (FALSE == dip_measureEnabled))
            {
               dip_startRecirculationMeasureRecordAndAlarm();
            }
            dip_manageTocRequest(stateTimer);

            if (TRUE == dip_isStartDistributionConditions())
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_RUNNING, TRUE);
            }
            else if ((TRUE == DistributionProcessSetpoints.AutoDumpOptionActivation) &&
                (TRUE == p_ProcessParam->ProcessDescription.AutoDumpFlag))
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_DUMPING, TRUE);
            }
            else if(((((UINT32)DistributionProcessSetpoints.AutoRecirculationDuration * 60000U) <= stateTimer) || // AutoRecirculationDuration is in min
                    (TRUE == DistributionSkipSequence)) &&
                    (FALSE == dip_isTocMeterRunning()))
            {
               dip_stopMesureAndCounters();
               dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
               dip_firstRecircHasBeenDone = TRUE;
               DistributionSkipSequence = FALSE;
            }
            else
            {
            /* Do Nothing: Continue the recirculation */
            }

            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Listening", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceRunning(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DISTRIBUTION_MEASURE);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case DIP_STATE_DISTRIBUTION_MEASURE:
         {
            if(((1000U * (UINT32)START_WATER_QUALITY_MEASURE_DELAY) <= stateTimer) && // Timer ok for mesure
               (FALSE == dip_measureEnabled))
            {
               dip_startRecirculationMeasureRecordAndAlarm();
            }

            if(TRUE == dip_isStopDistributionConditions())
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION_MEASURE);
               DistributionSkipSequence = FALSE;
            }
            break;
         }

         case DIP_STATE_RECIRCULATION_MEASURE:
         {
            if(((1000U * (UINT32)START_WATER_QUALITY_MEASURE_DELAY) <= stateTimer) && // Timer ok for mesure
               (FALSE == dip_measureEnabled))
            {
               dip_startRecirculationMeasureRecordAndAlarm();
            }
            dip_manageTocRequest(stateTimer);
            if (TRUE == dip_isStartDistributionConditions())
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DISTRIBUTION_MEASURE);
            }
            else if(((((UINT32)DistributionProcessSetpoints.PostDistributionRecirculationDuration * 60000U) <= stateTimer) || // PostDistributionRecirculationDuration is in min
                    (TRUE == DistributionSkipSequence)) &&
                    (FALSE == dip_isTocMeterRunning()))
            {
               dip_stopMesureAndCounters();
               dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
               dip_firstRecircHasBeenDone = TRUE;
               DistributionSkipSequence = FALSE;
            }
            else
            {
               /* Do nothing */
            }
            break;
         }

         default:
         {
            ERROR("Inconsistent state (%d) for the sequence Running", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceDumping(DIP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if(FALSE == p_ProcessParam->ProcessDescription.AutoDumpFlag) // The Production process manage the Auto-Dump Flag and set it to FALSE when it starts a tank Filling
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
            p_ProcessParam->ProcessDescription.AutoDumpFlag = FALSE;
         }
         else
         {
            /* Do nothing */
         }
      }
      else
      {
         ERROR("Inconsistent state (%d) for the sequence Dumping", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceAlarmStop(DIP_ProcessParam_t * p_ProcessParam)
{
   BOOL isAlarmPresent = TRUE;
   DipMode_e DipModeNominal;
   DipMode_e DipModeAlarmStop;
   BOOL isModeFound = FALSE;
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_stopMesureAndCounters();
      dip_setNewState(p_ProcessParam, DIP_STATE_DEPRESSURIZATION);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      TOC_Stop();
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      if (DIP_STATE_DEPRESSURIZATION == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((1000U * (UINT32)DIST_DEPRESS_DURATION) <= stateTimer) // After DIST_DEPRESS_DURATION of DEPRESSURIZATION
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
         }
         else
         {
            /* Do nothing */
         }
      }
      else
      {
         /* Do nothing */
      }

      if (DIP_STATE_WAIT == p_ProcessParam->ProcessDescription.ProcessState)
      {
         isAlarmPresent = dip_isAlarmStopPresent(TRUE, TRUE);
         if(FALSE == isAlarmPresent) // There is no more Alarm Stop
         {
            if (1600U < TMC_GetTimeCounter(dip_exitAlarmStopCounter))
            {
               isModeFound = dip_getAssociatedMode (p_ProcessParam->ProcessDescription.ProcessMode, &DipModeNominal, &DipModeAlarmStop);
               if (TRUE == isModeFound)
               {
                  if(DIP_MODE_TEST != DipModeNominal)
                  {
                     dip_setNewMode(p_ProcessParam, DipModeNominal, TRUE);
                  }
                  else
                  {
                     dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
                  }
               }
               else
               {
                  ERROR("Unknown Associated Mode to Distribution: %d", (INT)p_ProcessParam->ProcessDescription.ProcessMode);
               }
            }
         }
         else if (DIP_MODE_TANK_EMPTYING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            // Check if there is an alarm stop other than tank empty
            isAlarmPresent = dip_isAlarmStopPresent(TRUE, FALSE);
            if (FALSE == isAlarmPresent)
            {
               // There is no alarm stop except tank empty so we return to tank emptying
               dip_setNewMode(p_ProcessParam, DIP_MODE_TANK_EMPTYING, TRUE);
            }
         }
         else if (DIP_MODE_STARTPACK_TANK_FILLING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            // Check if there is an alarm stop other than tank empty
            isAlarmPresent = dip_isAlarmStopPresent(TRUE, FALSE);
            if (FALSE == isAlarmPresent)
            {
               // There is no alarm stop except tank empty so we return to startpack tank filling
               dip_setNewMode(p_ProcessParam, DIP_MODE_STARTPACK_TANK_FILLING, TRUE);
            }
         }
         else
         {
            dip_exitAlarmStopCounter = TMC_StartTimeCounter();
         }
      }
      else
      {
         /* Do nothing */
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequencePolisherFlush(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)DIST_POLISHER_FLUSH_DURATION;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((1000U * (UINT32)DIST_POLISHER_FLUSH_DURATION) <= stateTimer) // After DIST_POLISHER_FLUSH_DURATION
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
            p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
         }
         else
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_POLISHER_FLUSH_DURATION - (stateTimer / 1000U);
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence Polisher Flush", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceOpticapFlush(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)DIST_OPTICAP_FLUSH_DURATION;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((1000U * (UINT32)DIST_OPTICAP_FLUSH_DURATION) <= stateTimer) // After DIST_OPTICAP_FLUSH_DURATION
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
            p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
         }
         else
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_OPTICAP_FLUSH_DURATION - (stateTimer / 1000U);
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence Opticap Flush", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceStartPackTankFilling(DIP_ProcessParam_t * p_ProcessParam)
{
   CSM_Status_t PolisherStatus = CSM_GetStatus(CSM_POLISHER);
   UINT32 stateTimer;
   UINT32 remainingTime;
   FLOAT TankLevelMm;

   TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);
   remainingTime = dip_computeRemainingTimeToFillTank ((FLOAT)DIST_FIRST_INSTALL_VOLUME, (FLOAT)DIP_EMERGENCY_FLOW_RATE, TankLevelMm);

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      if(CSM_STATUS_NEW == PolisherStatus)
      {
         CSM_WriteInstallDate(CSM_POLISHER,TRUE);
         dip_setNewState(p_ProcessParam, DIP_STATE_START_KIT_DUMP);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration = (remainingTime + (UINT32)DIST_START_KIT_DUMP_DURATION);
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }
      else
      {
         dip_setNewState(p_ProcessParam, DIP_STATE_START_KIT_FILL_TANK);
         p_ProcessParam->ProcessDescription.SequenceTotalDuration = remainingTime;
         p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      }

      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if((5000u <= stateTimer) &&
         (TRUE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         p_ProcessParam->ProcessDescription.isCheckNewPackDisable = FALSE;
      }

      switch(p_ProcessParam->ProcessDescription.ProcessState)
      {
         case DIP_STATE_START_KIT_DUMP:
         {
            if((1000U * (UINT32)DIST_START_KIT_DUMP_DURATION) <= stateTimer) // After DIST_START_KIT_DUMP_DURATION
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_START_KIT_FILL_TANK);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = ((remainingTime + (UINT32)DIST_START_KIT_DUMP_DURATION) - (stateTimer / 1000U));
            }
            break;
         }

         case DIP_STATE_START_KIT_FILL_TANK:
         {
            /* Nothing to do, transition to Standby is done in StateMachine */
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = remainingTime;
            break;
         }

         default:
         {
            ASSERT(FALSE,"Inconsistent state (%d) for the sequence Start Pack Tank Filling", p_ProcessParam->ProcessDescription.ProcessState);
            break;
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceSan1(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION);
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)DIST_SAN_RECIRC_DURATION;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if (DIP_STATE_RECIRCULATION == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((1000U * (UINT32)DIST_SAN_RECIRC_DURATION) <= stateTimer) // After DIST_SAN_RECIRC_DURATION
         {
            dip_setNewMode(p_ProcessParam, DIP_MODE_SAN2, TRUE);
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
         }
         else
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_SAN_RECIRC_DURATION - (stateTimer / 1000U);
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence San1", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceSan2(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT TankLevelMm;
   UINT32 remainingTime;
   FLOAT volumeMax = dip_getVolumeFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX);
   FLOAT productFlowrate = dip_getProductFlowrate();
   UINT32 San2Duration;
   UINT32 San3Duration = ((dip_computeRemainingTimeToFillTank (volumeMax, productFlowrate, 0.0f) + dip_computeRemainingTimeToEmptyTankFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX)) + (UINT32)DIST_EMPTYING_DURATION);
   UINT32 San4Duration = DIST_SAN_CYCLE_MAX *
                           ((((dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, 0.0f) +
                               dip_computeRemainingTimeToEmptyTankFromVol((FLOAT)DIST_SAN_VOLUME)) +
                               (UINT32)DIST_EMPTYING_DURATION)));

   TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);
   remainingTime = dip_computeRemainingTimeToEmptyTankFromMm (TankLevelMm) + (UINT32)DIST_EMPTYING_DURATION;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
      p_ProcessParam->ProcessDescription.StateNumber = 0U;
      San2Duration = dip_computeRemainingTimeToEmptyTankFromVol ((FLOAT)DIST_SAN_VOLUME);
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = ((San2Duration + San3Duration) + San4Duration);
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if (0U == p_ProcessParam->ProcessDescription.StateNumber)
         {
            /* First Dump to empty to Alarm Stop */
            if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW >= TankLevelMm)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 1U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (((UINT32)DIST_EMPTYING_DURATION + San3Duration) + San4Duration);
            }
            else
            {
               remainingTime = ((remainingTime + San3Duration) + San4Duration);
               dip_updateRemainingTime(p_ProcessParam, remainingTime);
            }
         }
         else
         {
            if((1000U * (UINT32)DIST_EMPTYING_DURATION) <= stateTimer) // After DIST_EMPTYING_DURATION
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN3, TRUE);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (San3Duration + San4Duration);
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (((UINT32)DIST_EMPTYING_DURATION - (stateTimer / 1000U) + San3Duration) + San4Duration);
            }
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence San2", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceSan3(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);
   UINT32 remainingTime;
   FLOAT volumeMax = dip_getVolumeFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX);
   FLOAT productFlowrate = dip_getProductFlowrate();
   UINT32 remainingTimeFirstEmpty = dip_computeRemainingTimeToEmptyTankFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX);
   UINT32 San2Duration;
   UINT32 San3Duration;
   UINT32 San4Duration = DIST_SAN_CYCLE_MAX *
                           ((((dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, 0.0f) +
                               dip_computeRemainingTimeToEmptyTankFromVol((FLOAT)DIST_SAN_VOLUME)) +
                               (UINT32)DIST_EMPTYING_DURATION)));

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.StateNumber = 0U;
      San2Duration = dip_computeRemainingTimeToEmptyTankFromVol ((FLOAT)DIST_SAN_VOLUME);
      San3Duration = ((dip_computeRemainingTimeToFillTank (volumeMax, productFlowrate, 0.0f) + dip_computeRemainingTimeToEmptyTankFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX)) + (UINT32)DIST_EMPTYING_DURATION);

      p_ProcessParam->ProcessDescription.SequenceTotalDuration = ((San2Duration + San3Duration) + San4Duration);
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = (San3Duration + San4Duration);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if (DIP_STATE_WAIT == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH < TankLevelMm)
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION);
         }

         /* Remaining Time computation */
         remainingTime = dip_computeRemainingTimeToFillTank (volumeMax, productFlowrate, TankLevelMm);
         remainingTime = (((remainingTime + remainingTimeFirstEmpty) + (UINT32)DIST_EMPTYING_DURATION) + San4Duration);
         dip_updateRemainingTime(p_ProcessParam, remainingTime);
      }
      else if (DIP_STATE_RECIRCULATION == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW > TankLevelMm)
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
            p_ProcessParam->ProcessDescription.StateNumber = 0U;
         }
         else
         {
            if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX < TankLevelMm)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
            }
         }

         /* Remaining Time computation */
         remainingTime = dip_computeRemainingTimeToFillTank (volumeMax, productFlowrate, TankLevelMm);
         remainingTime = remainingTime + remainingTimeFirstEmpty + (UINT32)DIST_EMPTYING_DURATION + San4Duration;
         dip_updateRemainingTime(p_ProcessParam, remainingTime);
      }
      else if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if (0U == p_ProcessParam->ProcessDescription.StateNumber)
         {
            /* Remaining Time computation */
            remainingTime = dip_computeRemainingTimeToEmptyTankFromMm (TankLevelMm);
            remainingTime = remainingTime + (UINT32)DIST_EMPTYING_DURATION + San4Duration;
            dip_updateRemainingTime(p_ProcessParam, remainingTime);

            /* First Dump to empty to Alarm Stop */
            if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW >= TankLevelMm)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 1U;
            }
         }
         else
         {
            if((1000U * (UINT32)DIST_EMPTYING_DURATION) <= stateTimer) // After DIST_EMPTYING_DURATION
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_SAN4, TRUE);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = San4Duration;
               dip_numberOfSanCycle = 0;
            }
            else
            {
               /* Remaining Time computation */
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_EMPTYING_DURATION - (stateTimer / 1000U) + San4Duration;
               dip_updateRemainingTime(p_ProcessParam, remainingTime);
            }
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceSan4(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);
   FLOAT TankLevelVol = MES_GetMeasure(MES_TANK_LEVEL_VOL);
   UINT32 remainingTime;
   FLOAT volumeMax = dip_getVolumeFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX);
   FLOAT productFlowrate = dip_getProductFlowrate();
   UINT32 remainingTimeFill = dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, 0.0f);
   UINT32 remainingTimeEmpty = dip_computeRemainingTimeToEmptyTankFromVol((FLOAT)DIST_SAN_VOLUME);
   UINT32 remainingTimeCycle = remainingTimeFill + remainingTimeEmpty + (UINT32)DIST_EMPTYING_DURATION;
   UINT32 San2Duration;
   UINT32 San3Duration;
   UINT32 San4Duration;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
      p_ProcessParam->ProcessDescription.StateNumber = 0U;
      San2Duration = dip_computeRemainingTimeToEmptyTankFromVol ((FLOAT)DIST_SAN_VOLUME);
      San3Duration = ((dip_computeRemainingTimeToFillTank (volumeMax, productFlowrate, 0.0f) + dip_computeRemainingTimeToEmptyTankFromMm((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX)) + (UINT32)DIST_EMPTYING_DURATION);
      San4Duration = DIST_SAN_CYCLE_MAX *
                        ((((dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, 0.0f) +
                            dip_computeRemainingTimeToEmptyTankFromVol((FLOAT)DIST_SAN_VOLUME)) +
                            (UINT32)DIST_EMPTYING_DURATION)));
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = ((San2Duration + San3Duration) + San4Duration);
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = remainingTimeCycle * (DIST_SAN_CYCLE_MAX - dip_numberOfSanCycle);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if (DIP_STATE_WAIT == p_ProcessParam->ProcessDescription.ProcessState)
      {
         /* Remaining Time computation */
         remainingTime = dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, TankLevelMm);
         remainingTime = remainingTime + remainingTimeEmpty + (UINT32)DIST_EMPTYING_DURATION + (remainingTimeCycle * (DIST_SAN_CYCLE_MAX - dip_numberOfSanCycle - 1U));
         dip_updateRemainingTime(p_ProcessParam, remainingTime);

         if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH < TankLevelMm)
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION);
         }
      }
      else if (DIP_STATE_RECIRCULATION == p_ProcessParam->ProcessDescription.ProcessState)
      {
         /* Remaining Time computation */
         remainingTime = dip_computeRemainingTimeToFillTank ((FLOAT)DIST_SAN_VOLUME, productFlowrate, TankLevelMm);
         remainingTime = remainingTime + remainingTimeEmpty + (UINT32)DIST_EMPTYING_DURATION + (remainingTimeCycle * (DIST_SAN_CYCLE_MAX - dip_numberOfSanCycle - 1U));
         dip_updateRemainingTime(p_ProcessParam, remainingTime);

         if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW > TankLevelMm)
         {
            dip_setNewState(p_ProcessParam, DIP_STATE_WAIT);
         }
         else
         {
            if((FLOAT)DIST_SAN_VOLUME < TankLevelVol)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
            }
         }
      }
      else if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if (0U == p_ProcessParam->ProcessDescription.StateNumber)
         {
            /* Remaining Time computation */
            remainingTime = dip_computeRemainingTimeToEmptyTankFromMm (TankLevelMm);
            remainingTime = remainingTime + (UINT32)DIST_EMPTYING_DURATION + (remainingTimeCycle * (DIST_SAN_CYCLE_MAX - dip_numberOfSanCycle - 1U));
            dip_updateRemainingTime(p_ProcessParam, remainingTime);

            /* First Dump to empty to Alarm Stop */
            if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW >= TankLevelMm)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 1U;
            }
         }
         else
         {
            if((1000U * (UINT32)DIST_EMPTYING_DURATION) <= stateTimer) // After DIST_EMPTYING_DURATION
            {
               dip_numberOfSanCycle++;
               if ((UINT8)DIST_SAN_CYCLE_MAX > dip_numberOfSanCycle)
               {
                  dip_setNewMode(p_ProcessParam, DIP_MODE_TEST, FALSE);
                  dip_setNewMode(p_ProcessParam, DIP_MODE_SAN4, TRUE);
                  p_ProcessParam->ProcessDescription.StateNumber = 0U;
               }
               else
               {
                  dip_setNewMode(p_ProcessParam, DIP_MODE_SAN_STOP, TRUE);
                  p_ProcessParam->ProcessDescription.StateNumber = 0U;
                  p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
                  p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
                  dip_numberOfSanCycle = 0;
               }
            }
            else
            {
               remainingTime = (UINT32)DIST_EMPTYING_DURATION - (stateTimer / 1000U) + (remainingTimeCycle * (DIST_SAN_CYCLE_MAX - dip_numberOfSanCycle - 1U));
               dip_updateRemainingTime(p_ProcessParam, remainingTime);
            }
         }
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceTankEmptying(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   FLOAT TankLevelMm;
   UINT32 remainingTime;

   TankLevelMm = MES_GetMeasure(MES_TANK_LEVEL_MM);
   remainingTime = dip_computeRemainingTimeToEmptyTankFromMm (TankLevelMm);

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
      p_ProcessParam->ProcessDescription.StateNumber = 1U;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = remainingTime + (UINT32)DIST_EMPTYING_DURATION;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter);

      if (DIP_STATE_DUMP == p_ProcessParam->ProcessDescription.ProcessState)
      {
         if (1U == p_ProcessParam->ProcessDescription.StateNumber)
         {
            /* First Dump to empty to Alarm Stop */
            if((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW >= TankLevelMm)
            {
               dip_setNewState(p_ProcessParam, DIP_STATE_DUMP);
               p_ProcessParam->ProcessDescription.StateNumber = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_EMPTYING_DURATION;
            }
            else
            {
               remainingTime = remainingTime + (UINT32)DIST_EMPTYING_DURATION;
               dip_updateRemainingTime(p_ProcessParam, remainingTime);
            }
         }
         else
         {
            if((1000U * (UINT32)DIST_EMPTYING_DURATION) <= stateTimer) // After DIST_EMPTYING_DURATION
            {
               dip_setNewMode(p_ProcessParam, DIP_MODE_STANDBY_OLD_PACK, TRUE);
               p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
            }
            else
            {
               p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_EMPTYING_DURATION - (stateTimer / 1000U);
            }
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence Tank Emptying", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceTest(DIP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_WAIT_TEST);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceTestRecirc(DIP_ProcessParam_t * p_ProcessParam)
{
   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION_MEASURE);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
   }
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the sequence
 *
 */
static VOID dip_manageSequenceTocCleaning(DIP_ProcessParam_t * p_ProcessParam)
{
   UINT32 stateTimer;
   TOC_Status_t tocStatus;

   if (TRUE == p_ProcessParam->ProcessDescription.isEnteringSequence)
   {
      dip_setNewState(p_ProcessParam, DIP_STATE_RECIRCULATION_MEASURE);
      p_ProcessParam->ProcessDescription.isEnteringSequence = FALSE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = (UINT32)DIST_TOC_CLEANING_DURATION;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = p_ProcessParam->ProcessDescription.SequenceTotalDuration;
   }

   if(DIP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
   {
      stateTimer = TMC_GetTimeCounter(p_ProcessParam->ProcessDescription.StateTimeCounter); // (ms)

      if (DIP_STATE_RECIRCULATION_MEASURE == p_ProcessParam->ProcessDescription.ProcessState)
      {
         tocStatus = TOC_GetStatus();

         if (TOC_STATUS_IDLE == tocStatus)
         {
            TOC_StartCleaning();
         }

         if((1000U * (UINT32)DIST_TOC_CLEANING_DURATION) <= stateTimer) // After DIST_TOC_CLEANING_DURATION
         {
            TOC_Stop();
            dip_setNewMode(p_ProcessParam, DIP_MODE_LISTENING, TRUE);
            p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
         }
         else
         {
            p_ProcessParam->ProcessDescription.SequenceRemainingTime = (UINT32)DIST_TOC_CLEANING_DURATION - (stateTimer / 1000U);
         }
      }
      else
      {
         ASSERT(FALSE,"Inconsistent state (%d) for the sequence TOC Cleaning", p_ProcessParam->ProcessDescription.ProcessState);
      }
   }
}

/*!
 *
 *  @return None
 *
 *  @brief  Manage the stop of measure and counter
 *
 */
static VOID dip_stopMesureAndCounters (VOID)
{
   dip_measureEnabled = FALSE;
   MES_RecordStop(MES_DISTRIBUTION_RESISTIVITY);
   MES_RecordStop(MES_DISTRIBUTION_TEMPERATURE);
   MES_AlarmChkStop(MES_CHK_ALM_PRODUCT_TEMP_SP);
   MES_AlarmChkStop(MES_CHK_ALM_DIST_RES_MIN);
   MES_AlarmChkStop(MES_CHK_ALM_DIST_RES_MAX);
   MES_AlarmChkStop(MES_CHK_ALM_DIST_RES_SP);
   TOC_Stop();
}

/*!
 *
 *  @return None
 *
 *  @brief  Manage the start of measure record and alarm
 *
 */
static VOID dip_startRecirculationMeasureRecordAndAlarm (VOID)
{
   // Record and Check Distribution Resistivity and Temperature
   MES_RecordStart(MES_DISTRIBUTION_RESISTIVITY, FALSE);
   MES_RecordStart(MES_DISTRIBUTION_TEMPERATURE, FALSE);
   MES_AlarmChkStart(MES_CHK_ALM_PRODUCT_TEMP_SP);
   MES_AlarmChkStart(MES_CHK_ALM_DIST_RES_MIN);
   MES_AlarmChkStart(MES_CHK_ALM_DIST_RES_MAX);
   MES_AlarmChkStart(MES_CHK_ALM_DIST_RES_SP);
   dip_measureEnabled = TRUE;
}

/*!
 *  @param stateTimer [IN] UINT32 <br>
 *                    nbr of milliseconds in the state
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the TOC meter request to start a measure
 *
 */
static VOID dip_manageTocRequest (UINT32 stateTimer)
{
   if(TRUE == SYS_GetDeviceOption(SYSTEM_OPT_TOC))
   {
      if(TRUE == DistributionProcessSetpoints.TocMeasureRequested)
      {
         TOC_Status_t tocStatus = TOC_GetStatus();
         if((TOC_STATUS_IDLE == tocStatus) &&
            ((1000U * (UINT32)DIST_RECIRCULATION_MIN_FOR_TOC) < stateTimer))
         {
            TOC_StartMeasure();
         }
         else if (TOC_STATUS_MEASURING == tocStatus)
         {
            dip_informCanOpenTocMeasureStarted();
         }
         else
         {
            /* Do Nothing */
         }
      }
   }
}

/*!
 *  @return BOOL
 *  @retval is the TOC Meter is performing a measure
 *
 *  @brief  Allow the process to know if the TOC Meter is running or not
 *
 */
static BOOL dip_isTocMeterRunning (VOID)
{
   BOOL TocMeterInProgress = FALSE;

   if(TRUE == SYS_GetDeviceOption(SYSTEM_OPT_TOC))
   {
      if(TOC_STATUS_IDLE != TOC_GetStatus())
      {
         TocMeterInProgress = TRUE;
      }
   }
   return (TocMeterInProgress);
}

/*!
 *  @param TankLevelMm [OUT] FLOAT <br>
 *         Tank level in mm
 *
 *  @return FLOAT
 *  @retval is the number of liter in the tank
 *
 *  @brief  Allow the process to know if the volume in the tank
 *
 */
static FLOAT dip_getVolumeFromMm (FLOAT TankLevelMm)
{
   FLOAT fTankLevelVol;
   UINT8 bTankType;

   DO_READ(bTankType,g_DoTankLevelMeasure.TankType);

   /* Compute real volume remaining in tank */
   if(MES_TANK_TYPE_SMALL == bTankType)
   {
      fTankLevelVol = (FLOAT)MM_TO_LITER_COEFF_FOR_SMALL_TANK * TankLevelMm;
   }
   else if(MES_TANK_TYPE_BIG == bTankType)
   {
      fTankLevelVol = (FLOAT)MM_TO_LITER_COEFF_FOR_BIG_TANK * TankLevelMm;
   }
   else if(MES_TANK_TYPE_CUSTOM == bTankType)
   {
      fTankLevelVol= MES_GetMeasure(MES_TANK_LEVEL_VOL);
   }
   else
   {
      fTankLevelVol = 0.0f;
   }

   return (fTankLevelVol);
}

/*!
 *  @param TankLevelMm [IN] FLOAT  <br>
 *        Tank level in mm
 *
 *  @return UINT32
 *  @retval is the number of seconds estimated to empty the tank
 *
 *  @brief  Allow the process to know the remaining time to empty the tank
 *
 */
static UINT32 dip_computeRemainingTimeToEmptyTankFromMm (FLOAT TankLevelMm)
{
   FLOAT fTankLevelVol;
   UINT32 remainingTime;

   if ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW < TankLevelMm)
   {
      TankLevelMm = TankLevelMm - (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW;
      fTankLevelVol = dip_getVolumeFromMm(TankLevelMm);
   }
   else
   {
      fTankLevelVol = 0.0f;
   }

   /* Compute the remaining time */
   fTankLevelVol = ((fTankLevelVol * 60.0f) / (FLOAT)DIP_TANK_EMPTYING_FLOWRATE);
   remainingTime =(UINT32)fTankLevelVol;
   return (remainingTime);
}

/*!
 *  @param TankVolume [IN] FLOAT <br>
 *        Tank volume
 *
 *  @return UINT32
 *  @retval is the number of seconds estimated to empty the tank
 *
 *  @brief  Allow the process to know the remaining time to empty the tank
 *
 */
static UINT32 dip_computeRemainingTimeToEmptyTankFromVol (FLOAT TankVolume)
{
   UINT32 remainingTime = 0;

   if (0.0f < TankVolume)
   {
      /* Compute the remaining time */
      TankVolume = ((TankVolume * 60.0f) / (FLOAT)DIP_TANK_EMPTYING_FLOWRATE);
      remainingTime =(UINT32)TankVolume;
   }

   return (remainingTime);
}

/*!
 *  @param wantedVolume [OUT] FLOAT <br>
 *        volume wanted in the tank
 *
 *  @param productFlowRate [OUT] FLOAT <br>
 *        tank filling flowrate.
 *
 *  @param TankLevelMm [OUT] FLOAT <br>
 *        Tank level.
 *
 *  @return UINT32
 *  @retval is the number of seconds estimated to fill the tank to the wanted volume
 *
 *  @brief  Allow the process to know the needed time in second to fill the tank
 *
 */
static UINT32 dip_computeRemainingTimeToFillTank (FLOAT wantedVolume, FLOAT productFlowRate, FLOAT TankLevelMm)
{
   FLOAT fTankLevelVol;
   UINT32 remainingTime;

   if ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH <= TankLevelMm)
   {
      fTankLevelVol = MES_GetMeasure(MES_TANK_LEVEL_VOL);
      if (fTankLevelVol < wantedVolume)
      {
         /* Compute the remaining time */
         fTankLevelVol = (((wantedVolume - fTankLevelVol) * 60.0f) / productFlowRate);
         remainingTime =(UINT32)fTankLevelVol;
      }
      else
      {
         remainingTime = 0U;
      }
   }
   else
   {
      /* compute the remaining mm to reach MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH */
      TankLevelMm = (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH - TankLevelMm;
      fTankLevelVol = dip_getVolumeFromMm(TankLevelMm);
      fTankLevelVol = fTankLevelVol + wantedVolume;

      /* Compute the remaining time */
      fTankLevelVol = ((fTankLevelVol * 60.0f) / (FLOAT)productFlowRate);
      remainingTime =(UINT32)fTankLevelVol;
   }
   return (remainingTime);
}

/*!
 *  @return FLOAT
 *  @retval is the product flowrate
 *
 *  @brief  Allow the process to know the product flowrate in L/min
 *
 */
static FLOAT dip_getProductFlowrate (VOID)
{
   FLOAT productFlowrate;
   UINT8 nbrCartridges = SYS_GetNbCartridges();

   if (1U == nbrCartridges)
   {
      productFlowrate = (40.0f / 60.0f);
   }
   else if (2U == nbrCartridges)
   {
      productFlowrate = (80.0f / 60.0f);
   }
   else if (3U == nbrCartridges)
   {
      productFlowrate = (120.0f / 60.0f);
   }
   else
   {
      productFlowrate = (150.0f / 60.0f);
   }
   return(productFlowrate);
}

/*!
 *  @param p_ProcessParam [IN] DIP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *  @param RemainingTime [IN] UINT32 <br>
 *         new remainig time
 *
 *  @brief  Update the remaining time of a sequence
 *
 */
static VOID dip_updateRemainingTime (DIP_ProcessParam_t * p_ProcessParam, UINT32 RemainingTime)
{
   if (RemainingTime  < p_ProcessParam->ProcessDescription.SequenceRemainingTime)
   {
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = RemainingTime;
   }
}

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

