/*!
* @file dip_transition.c
*
* @brief Distribution Task
*
* @details Manage the stransition step from one state to another
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: dip_transition.c 624 201#$: Filename
* $Rev:: 624                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-11-05 13:44:14 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/distribution/src/dip_transition.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "dip_api.h"
#include "sys_softerr.h"
#include "dip_data.h"
#include "act_api.h"
#include "tmc_api.h"
#include "sys_log.h"
#include "dip_private.h"
#include "system_parameters.h"



/* Declarations --------------------------------------------------------------*/

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/

static VOID dip_manageTransitionToWait(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToRecirculation(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToSanRecirculation(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToDepressurization(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToDump(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToStartKitDump(DIP_TransitionDescription_t * p_TransitionDescription);
static VOID dip_manageTransitionToStartKitTankFilling(DIP_TransitionDescription_t * p_TransitionDescription);

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage the different type of transition
 *
 *  @details  Check which type of transition is start and call the associated transition
 *
 */
VOID dip_manageTransition (DIP_TransitionDescription_t * p_TransitionDescription)
{
   if (DIP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus)
   {
      switch(p_TransitionDescription->TransitionType)
      {
         case DIP_TRANSITION_TO_WAIT:
         case DIP_TRANSITION_TO_WAIT_TEST:
         {
            dip_manageTransitionToWait(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_RECIRCULATION:
         {
            dip_manageTransitionToRecirculation(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_SAN_RECIRCULATION:
         {
            dip_manageTransitionToSanRecirculation(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_DEPRESSURIZATION:
         {
            dip_manageTransitionToDepressurization(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_DUMP:
         {
            dip_manageTransitionToDump(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_START_KIT_DUMP:
         {
            dip_manageTransitionToStartKitDump(p_TransitionDescription);
            break;
         }

         case DIP_TRANSITION_TO_START_KIT_FILL_TANK:
         {
            dip_manageTransitionToStartKitTankFilling(p_TransitionDescription);
            break;
         }

         default:
         {
            ERROR("Wrong TransitionType %d",p_TransitionDescription->TransitionType);
            break;
         }
      }
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToWait(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Close the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_OFF);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Stop the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_OFF);
      // Close the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_OFF);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToRecirculation(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch ON the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_ON);
      // Open the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_ON);
      // Close the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_OFF);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Start the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_ON);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToSanRecirculation(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch ON the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Open the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_ON);
      // Close the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_OFF);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Start the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_ON);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToDepressurization(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Close the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_OFF);
      // Open the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_ON);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Stop the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_OFF);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToDump(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Open the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_ON);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Start the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_ON);
      // Close the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_OFF);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToStartKitDump(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Open the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_ON);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Stop the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_OFF);
      // Close the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_OFF);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/*!
 *  @param p_TransitionDescription [IN] DIP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID dip_manageTransitionToStartKitTankFilling(DIP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(DIP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_DISTRIBUTION, ACT_SWITCH_OFF);
      // Open the recirculation valve
      ACT_SetPower(ACT_TYPE_RECIRCULATION_SV_VALVE, ACT_SWITCH_ON);
      // Set duty cycle to 100
      ACT_SetDistributionPumpDutyCycle(100U);
      // Stop the distribution pump
      ACT_SetPower(ACT_TYPE_DISTRIBUTION_PUMP, ACT_SWITCH_OFF);
      // Close the dumping valve
      ACT_SetPower(ACT_TYPE_DUMPING_SV_VALVE, ACT_SWITCH_OFF);

      p_TransitionDescription->TransitionStatus = DIP_TRANSITION_DONE;
      DistributionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
   }
}

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

