/*!
* @file measure.c
*
* @brief Measure Task
*
* @details Contain main Mesure stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: measure.c 624 2014-11#$:  Filename
* $Rev:: 624                  $: Revision of last commit
* $Author:: m175711           $: Author of last commit
* $Date:: 2014-11-05 13:44:14#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/measure.c $: URL of this file
* @endcond
*
*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include "millipore_types.h"
#include "sys_api.h"
#include "sys_softerr.h"
#include "sys_log.h"

#include "os_adaptation.h"
#include <intrinsics.h>
#include "measure.h"
#include "mes_data.h"
#include "mes_api.h"
#include "mvm_api.h"
#include "mes_can.h"
#include "do_itf.h"
#include "do_def.h"
#include "msg_type.h"
#include "messaging.h"
#include "aaa_api.h"
#include "system_parameters.h"
#include "tmc_api.h"

/*! @defgroup Measure Measure module
 *
 *  @brief Measure task module
 *
 *  @details Measure task is in charge of :
 *    @li collecting raw measures from DIL access and maintain readable measures .
 *    @li providing an interface to get measure at anytime.
 *    @li providing measure record capability.  Measure record trigs CAN message sent from Process board to IHM board containing measure.
 *    @li providing alternative measure record capability to trigs a CAN message sent from Process board to IHM board containing averaged measure.
 *    @li Providing Volume measure capability for Permeate and Drain flows
 *    @li Trigs some alarms if measure are inconsistent.
 *  \n
 *  Before any action, measure task waits configuration. Once configuration received, it updates measure stored in an array and send optionally CAN messages.\n
 *  It treats Process task measure request. In addition it subtracts alarm and alerts to AlarmAndAlert task.
 *
 *  @defgroup Measure_data Private data
 *  @brief Private data description
 *  @ingroup Measure
 *  @{
 */

/* Declarations ------------------------------------------------------------- */
#define N_MAX_RECORD (500)
typedef struct MES_MeasuresRecord_s
{
   UINT32 TimeStamp;
   FLOAT RecircValvePosition;
   FLOAT DrainValvePosition;
   UINT16 ActuatorsStatus;
   FLOAT FeedPressure;
   FLOAT RoPressure;
   FLOAT DistPressure;
   FLOAT DrainFlowrate;
   FLOAT PermeateFlowrate;
   FLOAT EDIRejectFlowrate;
   FLOAT TankLevelMM;
   FLOAT TankLevelSpeed;
   FLOAT FeedCond;
   FLOAT PermeateCond;
   FLOAT EdiRes;
   FLOAT DistRes;
   FLOAT PermeatTemp;
   FLOAT EdiTemp;
   FLOAT DistTemp;
} MES_MeasuresRecord_s;

typedef struct MES_MeasuresRecordStruct_s
{
   BOOL isTabFull;
   UINT32 nIndexNextRecord;
   MES_MeasuresRecord_s MeasuresRecordTab[N_MAX_RECORD];
} MES_MeasuresRecordStruct_s;

MES_MeasuresRecordStruct_s g_mesRecordStruct;

/*! Bitmap to allow DIL RAW measure bypass. */
UINT32 g_dwIsDilBypassed[2] =
{
#ifdef D_BSP_STM3240GEVAL
   (UINT32) -1,    (UINT32) -1 // Full DIL bypassed
#else
//   0xFFFFFFF0u ,0xFFFFFFFFu // DIL temp         : bits 03-00
//   0xFFFFFC0Fu ,0xFFFFFFFFu // DIL conductivity   : bits 09-04
//   0xFFF803FFu ,0xFFFFFFFFu // DIL flowrate       : bits 18-10
//   0xFF07FFFFu ,0xFFFFFFFFu // DIL pressure       : bits 23-19
//   0xFEFFFFFFu ,0xFFFFFFFFu // DIL Tanklevel      : bits 24
//   0x01FFFFFFu ,0xFFFFFFFEu // DIL Tor            : bits 32-25
   0x00000000u ,0x00000000u // Full DIL access
#endif
};

UINT16 g_wLastRoRejectionRec=(UINT16)-1; /*!< Last RO rejection record save. INitialized at NON AVAILABLE (-1) */

OsHandle_t  hOsMeasureMutexHdl;           /*!< Measure tasks Mutex */

BOOL       g_bRoFeedCondMeasEn;           /*! TRUE if  RO Feed Conducitivity Measure Enable */

MeasData_t g_MeasDataArray[MEASURE_TYPE_LAST];     /*!< Measure data array. */

OsHandle_t hMeasureMboxHld;               /*!< Measure task mailbox */

BOOL g_doMesUpdateTorStatus=FALSE;        /*!< TRUE if TOR Gpio status shall be read */

FLOAT g_fMeasTankLvlPercent=0.0f;         /*!< store Tank level percentage */

FLOAT g_fTankLevelVolume = (FLOAT)OUT_OF_RANGE_MAX_MEASURE; /*!< store Tank level volume in L */

FLOAT g_fFeedPressureMinAlarmThreshold;

BOOL  g_bLowTdsEn;                        /*! TRUE if Low TDS is enable -> Reset Alarm FEED COD < MIN and RO REJECTION < SP */

/*! This varialble is set as volatile because value can change during Isr treatment.
 *  TRUE if TOR state is changed, and thus need to read .
 *  Set is done #MES_WarnTORChangedCallBack which is trig in ISR\n
 *  Once set,  measure task proceed state read and clear it.*/
volatile BOOL g_isTorsStatusModified=FALSE;

static OsHandle_t hMeasSemaList[ 3u ];   /*!< Measure semaphores handle array */

static VOID (* CONST* pUpdateMeasure1000msProc)(BOOL isSecondBoundary);
static VOID (* CONST* pUpdateMeasure500msProc)(BOOL isSecondBoundary);
static VOID (* CONST* pUpdateMeasure100msProc)(BOOL isSecondBoundary);

/*! Volume counter data array */
MeasVolumeCounterData_t g_MeasVolumeCounterData[MES_VOL_CNT_MAX];

UINT32 g_dwPeriodicSensorDiagResult=0; /*!< Periodic sensors diagnostic */

/*! @brief Check Alarm array
 *
 * @details According this array content, Alarm are checked or not
 *
 */
MeasCheckAlarmData_t g_MeasCheckAlarmArray[MES_CHK_ALM_LAST]=
{
   { .isEnable=FALSE,   .almType=ALARM_TYPE_PERMEATE_TEMPERATURE_HIGH_SP }, // MES_CHK_ALM_FEED_TEMP_SP,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_DISTRIB_TEMPERATURE_HIGH_SP  }, // MES_CHK_ALM_PRODUCT_TEMP_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_FEED_CONDUCTIVITY_MIN }, // MES_CHK_ALM_FEED_COND_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_FEED_CONDUCTIVITY_MAX }, // MES_CHK_ALM_FEED_COND_MAX,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_FEED_CONDUCTIVITY_HIGH_SP }, // MES_CHK_ALM_FEED_COND_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_PERMEATE_CONDUCTIVITY_MIN }, // MES_CHK_ALM_PERMEATE_COND_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_PERMEATE_CONDUCTIVITY_MAX }, // MES_CHK_ALM_PERMEATE_COND_MAX,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_RO_CONDUCTIVITY_REJECTTION_LOW_SP }, // MES_CHK_ALM_RO_REJECTION_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_FCE_HIGH_SP }, // MES_CHK_ALM_FCE_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_EDI_RESISTIVITY_MIN }, // MES_CHK_ALM_ELIX_RES_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_EDI_RESISTIVITY_MAX }, // MES_CHK_ALM_ELIX_RES__MAX,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_EDI_RESISTIVITY_HIGH_SP }, // MES_CHK_ALM_ELIX_RES_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_DISTRIBUTION_RESISTIVITY_MIN }, // MES_CHK_ALM_DIST_RES_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_DISTRIBUTION_RESISTIVITY_MAX }, // MES_CHK_ALM_DIST_RES_MAX,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_DISTRIBUTION_RESISTIVITY_HIGH_SP }, // MES_CHK_ALM_DIST_RES_SP,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_PRES }, // MES_CHK_ALM_PERMEATE_FR_STANDBY,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_LOW }, // MES_CHK_ALM_PERMEATE_FR_FLUSH,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_PERMEATE_FLOWRATE_LOW_SP }, // MES_CHK_ALM_PERMEATE_FR_SP_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_PERMEATE_FLOWRATE_HIGH_SP }, // MES_CHK_ALM_PERMEATE_FR_SP_MAX,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_EDI_FLOWRATE_MIN }, // MES_CHK_ALM_PERMEATE_EDI_FR_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_EDI_FLOWRATE_MAX }, // MES_CHK_ALM_PERMEATE_EDI_FR_MAX,  // Permeate flow rate

   { .isEnable=FALSE,   .almType=ALARM_TYPE_RO_RECOVERY_LOW_SP }, // MES_CHK_ALM_RO_RECOVERY_SP_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_RO_RECOVERY_HIGH_SP }, // MES_CHK_ALM_RO_RECOVERY_SP_MAX,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_FEED_PRESSURE_MIN }, // MES_CHK_ALM_FEED_PRESSURE_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_FEED_PRESSURE_MAX }, // MES_CHK_ALM_FEED_PRESSURE_MAX,

   { .isEnable=FALSE,   .almType=ALARM_TYPE_RO_PRESSURE_MIN }, // MES_CHK_ALM_RO_PRESSURE_MIN,
   { .isEnable=FALSE,   .almType=ALARM_TYPE_RO_PRESSURE_FEED_EQUAL }  // MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE,
};


/*! Measure type to sensors type map array
 *
 * @note MeasData_t is indexed with sensors type
 *       Process see only Meastype
 */
CONST UINT32 g_MeastypToSensorMap [] =
{
   PERMEATE_TEMPERATURE,
   EDI_TEMPERATURE,
   DISTRIBUTION_TEMPERATURE,
   FEED_CONDUCTIVITY_1,
   FEED_CONDUCTIVITY_2,
   PERMEATE_CONDUCTIVITY,
   RO_REJECTION,
   FCE,
   EDI_RESISTIVITY,
   DISTRIBUTION_RESISTIVITY,
   DRAIN_FLOWRATE,
   PERMEATE_FLOWRATE,
   RO_PERMEABILITY,
   RO_RECOVERY,
   RO_PUMP_FLOWRATE,
   PROGARD_DELTA_P_FLOWRATE,
   EDI_PRODUCTION_FLOWRATE,
   EDI_RECOVERY_FLOWRATE,
   FEED_PRESSURE,
   TAP_FEED_PRESSURE,
   PROGARD_DELTA_PRESSURE,
   RO_PRESSURE,
   DISTRIBUTION_PRESSURE,
   TANK_LEVEL,
   TANK_LEVEL,
   TANK_LEVEL,
   TANK_FILLING_SPEED,
   LEAK_DETECTOR,
   TOR_SDS_LEAK_DETECTOR,
   TOR_TANK_OVERFLOW,
   TOR_FAULT_REPORT1,
   TOR_FAULT_REPORT2,
   TOR_BACKWASH_CARBON,
   TOR_POD_STATUS,
   TOR_TANK_REFILL,
   RO_PUMP_CURRENT,
   DISTRIBUTION_PUMP_CURRENT
};


/*! SYSTEM_RIOS */
VOID (* CONST RiosMesureOperationProcArray[MEAS_MAX_NBR_PERIOD][MEAS_MAX_UPDATE_PER_PERIOD])(BOOL isSecondBoundary) =
{
   /*! 100ms period */
   {
      MES_UpdateFeedPressure,
      MES_UpdateROPressure,
      MES_UpdateTankLevel,
      MES_UpdateTORLeakDetector,
      MES_UpdateTORTankRefill,
      MES_END_MES_OPERATION
   },
   /*! 500ms period */
   {
      /* Take care to keep DrainFlowrate update procedure before Permeate update procedure ,
       some alarms depend on call order */
      MES_UpdateDrainFlowrate,
      MES_UpdatePermeateFlowrate,
      MES_UpdateAnalogLeakDetector,
      MES_END_MES_OPERATION
   },
   /*! 1000ms period */
   {
      MES_HardwareMonitor,
      MES_UpdateFeedTemperature,
      MES_UpdateFeedConductivity,
      MES_UpdatePermeateTemperature,
      MES_UpdatePermeateConductivity,
      MES_UpdateRoPumpCurrent,
      MES_UpdateDistributionPumpCurrent,
      MES_END_MES_OPERATION
   }
};


/*! SYSTEM_ELIX */
VOID (* CONST ElixMesureOperationProcArray[MEAS_MAX_NBR_PERIOD][MEAS_MAX_UPDATE_PER_PERIOD])(BOOL isSecondBoundary) =
{
   /*! 100ms period */
   {
      MES_UpdateFeedPressure,
      MES_UpdateROPressure,
      MES_UpdateDistributionPressure,
      MES_UpdateTankLevel,
      MES_UpdateTORLeakDetector,
      MES_UpdateTORTankRefill,
      MES_END_MES_OPERATION
   },
   /*! 500ms period */
   {
      MES_UpdateDrainFlowrate,
      MES_UpdatePermeateFlowrate,
      MES_UpdateEDIRejectFlowrate,
      MES_UpdateAnalogLeakDetector,
      MES_END_MES_OPERATION
   },
   /*! 1000ms period */
   {
      MES_HardwareMonitor,
      MES_UpdateFeedTemperature,
      MES_UpdateFeedConductivity,
      MES_UpdatePermeateTemperature,
      MES_UpdatePermeateConductivity,
      MES_UpdateEDITemperature,
      MES_UpdateEDIResistivity,
      MES_UpdateRoPumpCurrent,
      MES_UpdateDistributionPumpCurrent,
      MES_END_MES_OPERATION
   }
};


/*! SYSTEM_WPSH */
/*! SYSTEM_AFS */
VOID (* CONST AfsMesureOperationProcArray[MEAS_MAX_NBR_PERIOD][MEAS_MAX_UPDATE_PER_PERIOD])(BOOL isSecondBoundary) =
{
   /*! 100ms period */
   {
      MES_UpdateFeedPressure,
      MES_UpdateROPressure,
      MES_UpdateDistributionPressure,
      MES_UpdateTankLevel,
      MES_UpdateTORLeakDetector,
      MES_UpdateTORTankOverflow,
      MES_UpdateTORBackwashCarbon,
      MES_END_MES_OPERATION
   },
   /*! 500ms period */
   {
      /* Take care to keep DrainFlowrate update procedure before Permeate update procedure ,
       some alarms depend on call order */
      MES_UpdateDrainFlowrate,
      MES_UpdatePermeateFlowrate,
      MES_UpdateEDIRejectFlowrate,
      MES_UpdateAnalogLeakDetector,
      MES_END_MES_OPERATION
   },
   /*! 1000ms period */
   {
      MES_HardwareMonitor,
      MES_UpdateFeedTemperature,
      MES_UpdateFeedConductivity,
      MES_UpdatePermeateTemperature,
      MES_UpdatePermeateConductivity,
      MES_UpdateEDITemperature,
      MES_UpdateEDIResistivity,
      MES_UpdateDistributionTemperature,
      MES_UpdateDistributionRestivity,
      MES_UpdateRoPumpCurrent,
      MES_UpdateDistributionPumpCurrent,
      MES_END_MES_OPERATION
   }
};


/* Defines -------------------------------------------------------------------*/

#define MES_MSG_SEM_IDX ((INT32)0)
#define MES_TIM_SEM_IDX ((INT32)1)


/*! macro to set Digitial entries according provided value */
#define UPDATE_STATUS_ACCORDING_VALUE(_value,_status,_digentrienbr)\
{\
   if((_value) & ((UINT8)1<<(_digentrienbr)))\
   {\
      (_status)|=(UINT8)((UINT8)1<<(_digentrienbr));\
   }\
   else\
   {\
      (_status)&=~(UINT8)((UINT8)1<<(_digentrienbr));\
   }\
}

/*! OS delay defintion , here in ms */
#define  DELAY100MS     ((UINT32)100)

/*! Measure operation index defintions */
enum
{
   MEAS_OPERATION_100MS_IDX = 0,
   MEAS_OPERATION_500MS_IDX,
   MEAS_OPERATION_1000MS_IDX
};

/*! Debug mode bit index for sensors (used for bypass purpose)*/
enum
{
   FEED_CONDUCTIVITY_DBG = 0,
   PERMEATE_CONDUCTIVITY_DBG,
   EDI_RESISTIVITY_DBG,
   DISTRIBUTION_RESISTIVITY_DBG,
   FEED_TEMPERATURE_DBG,
   PERMEATE_TEMPERATURE_DBG,
   EDI_TEMPERATURE_DBG,
   DISTRIBUTION_TEMPERATURE_DBG,
   FEED_PRESSURE_DBG,
   RO_PRESSURE_DBG,
   DISTRIBUTION_PRESSURE_DBG,
   TANK_LEVEL_DBG,
   DRAIN_FLOW_RATE_DBG,
   PERMEATE_FLOW_RATE_DBG,
   EDI_REJECT_FLOW_RATE_DBG,
   TOC_DBG
};

/*! Debug mode bit index for digital entries (used for bypass purpose)
 *  CAUTION : in same order than sensor LEAK_DETECTOR to TANK_REFILL
 * Refer to Sensor_t
 */
enum
{
   LEAK_DETECTOR_DIGENT_DBG = 0,   // LEAK_DETECTOR,
   SDS_LEAK_DETECTOR_DIGENT_DBG,    // TOR_SDS_LEAK_DETECTOR
   TANK_OVERFLOW_DIGENT_DBG,        // TOR_TANK_OVERFLOW
   FAULT_REPORT1_DIGENT_DBG,        // TOR_FAULT_REPORT1
   FAULT_REPORT2_DIGENT_DBG,        // TOR_FAULT_REPORT2
   BACKWASH_DIGENT_DBG,             // TOR_BACKWASH_CARBON
   POD_DIGENT_DBG,                  // TOR_POD_STATUS
   TANK_REFILL_DIGENT_DBG           // TOR_TANK_REFILL
};


/*!
 *  @}
 */
 extern VOID MES_MsgParser(CONST  MES_Msg_t* pMesMsg);
/* Code ----------------------------------------------------------------------*/


/*! @defgroup Measure_func Private functions
 *  @brief Private functions
 *  @ingroup Measure
 *  @{
 */

/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Initialized Measure data
 *
 *  @details  Ensure all measure data are initialized before Measure task start
 *
 */
static VOID InitData(VOID)
{
   eOsRc_t rc;
   static const PLAINCHAR* MBoxName = "MES_MBX";

   static const PLAINCHAR* SemaNameArray [3u] =
   {
      "MES_MSG_SEM",
      "MES_TIM_SEM",
      NULL
   };

   /*! resolve seampahore ID to name  */
   rc=OS_SemaLookUpL(SemaNameArray, hMeasSemaList );
   ASSERT( OS_GOOD == rc ,"Wrong OS_SemaLookUpL call");
   /*! resolve Mailbox  ID to name  */
   rc=OS_MboxLookUp( MBoxName, &hMeasureMboxHld );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxLookUp call");

   /*! Link MailBox to Semaphore    */
   rc=OS_MboxDefSema(MBoxName, SemaNameArray[MES_MSG_SEM_IDX] );
   ASSERT( OS_GOOD == rc ,"Wrong OS_MboxDefSema call");
}


/*!
 *  @brief  Update DO sensors diagnositic field from \b g_DoHardwareDiagnostic \n
 *             \b g_DoHardwareDiagnostic is update periodically at 1 sec period.
 */
static VOID UpdateSensorsDoDiag(VOID)
{
   UINT32 dwDoSensorDiag = 0u;

#ifndef NDEBUG
   static UINT32 dwOldSensorDiag = 0u;
#endif

   /* Read from DO sensor diag */
   DO_READ(dwDoSensorDiag,g_DoHardwareDiagnostic.SensorsDiagResult);
   /* Clear relevant bits */
   dwDoSensorDiag &=~(  ( 1u<<(UINT32)FEED_TEMPERATURE) |
                        ( 1u<<(UINT32)PERMEATE_TEMPERATURE) |
                        ( 1u<<(UINT32)EDI_TEMPERATURE) |
                        ( 1u<<(UINT32)DISTRIBUTION_TEMPERATURE) |

                        ( 1u<<(UINT32)FEED_PRESSURE) |
                        ( 1u<<(UINT32)RO_PRESSURE) |
                        ( 1u<<(UINT32)DISTRIBUTION_PRESSURE) |
                        ( 1u<<(UINT32)TANK_LEVEL) );

   /* Update them with new value */
   dwDoSensorDiag|=g_dwPeriodicSensorDiagResult;

#ifndef NDEBUG
   if(dwOldSensorDiag!= dwDoSensorDiag)
   {
      SYSLOG(INFO,"M:SensorDiag : %08Xh",dwDoSensorDiag);
      dwOldSensorDiag = dwDoSensorDiag;
   }
#endif

   /* Write to DO new sensor diag */
   DO_WRITE(g_DoHardwareDiagnostic.SensorsDiagResult, dwDoSensorDiag);
}




/*! @fn MES_CheckAlarmToReset
 *
 * @brief Check Alarms manually reset to update status in Alert and alarm module
 */
VOID MES_CheckAlarmToReset(UINT8 bResetCommand)
{
   switch (bResetCommand)
   {
      /* Feed water  */
      case AAA_RESET_FEED_FR_INF_FLUSH:   {AAA_ResetAlarm(ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_LOW);break;}/* No water */
      case AAA_RESET_FEED_PRES_SUP_MAX:
      {
         if (ALARM_FEED_PRESSURE_MAX > g_MeasDataArray[FEED_PRESSURE].fMeasure)
         {
            AAA_ResetAlarm(ALARM_TYPE_FEED_PRESSURE_MAX);
         }
         break;
      }/* Feed over pressure */
      case AAA_RESET_FEED_PRES_INF_MIN:   {AAA_ResetAlarm(ALARM_TYPE_FEED_PRESSURE_MIN);              break;}/* Feed under pressure */
      /* RO */
      case AAA_RESET_RO_PRES_SUP_MAX:
      {
         if (ALARM_RO_PRESSURE_MAX > g_MeasDataArray[RO_PRESSURE].fMeasure)
         {
            AAA_ResetAlarm(ALARM_TYPE_RO_PRESSURE_MAX);
         }
         break;
      }
      case AAA_RESET_RO_PRES_INF_MIN:     {AAA_ResetAlarm(ALARM_TYPE_RO_PRESSURE_MIN);                break;}
      case AAA_RESET_RO_AND_FEED_PRES_INEQUAL:{AAA_ResetAlarm(ALARM_TYPE_RO_PRESSURE_FEED_EQUAL);     break;}
      /* EDI */
      case AAA_RESET_PERMEATE_FR_SUP_MAX: {AAA_ResetAlarm(ALARM_TYPE_EDI_FLOWRATE_MAX);               break;}/* Permeate too high for EDI */
      case AAA_RESET_PERMEATE_FR_INF_MIN: {AAA_ResetAlarm(ALARM_TYPE_EDI_FLOWRATE_MIN);               break;}/* Permeate too low for EDI */
      /* Permeate */
      case AAA_RESET_RECOVERY_SUP_MAX:    {AAA_ResetAlarm(ALARM_TYPE_RO_RECOVERY_HIGH_SP);            break;}/* RO Recovery high sp */
      case AAA_RESET_FCE_SUP_SP:          {AAA_ResetAlarm(ALARM_TYPE_FCE_HIGH_SP);                    break;}/* FCE */
      /* Tank and distribution */
      case AAA_RESET_DIST_PRES_SUP_MAX:
      {
         if (ALARM_DISTRIBUTION_PRESSURE_MAX > g_MeasDataArray[DISTRIBUTION_PRESSURE].fMeasure)
         {
            AAA_ResetAlarm(ALARM_TYPE_DISTRIB_PRESSURE_MAX);
         }
         break;
      }/* Distribution loop */
      case AAA_RESET_SDS_LEAK_DETECTION:  {MES_CheckTORResetAlarm(TOR_SDS_LEAK_DETECTOR);              break;}/* Water SDS leak detector */
      case AAA_RESET_TANK_OVERFLOW:       {MES_CheckTORResetAlarm(TOR_TANK_OVERFLOW);                  break;}/* Tank overflow */
      case AAA_RESET_LEAK_DETECTION:
      {
#ifndef INCLUDE_ANALOG_LEAK_DETECTOR
         /* TOR Water leak detector */
         MES_CheckTORResetAlarm(LEAK_DETECTOR);
#else
         /* Analog Water leak detector */
         MES_AllowAnalogLeakDetectorResetAlarm();
#endif
         break;
      }
      default:
      {
         break;
      }
   }
}


/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Check debug mode change and update \b g_dwIsDilBypassed
 *
 *  @details First part is relative to sensors,
 *           When DIL bypassed, the Can RAW field measure is expected to be updated by IHM
 *           or extern CAN applicative
 *           Second part is relative to Digital entries.
 *           When bypass, digital entries are updated in CANOpen field DigitalEntries
 *
 */
VOID MES_CheckDebugModeChange(VOID)
{
   UINT32 localDebugMode;
   UINT8  localDigitalEntriesMask, localDigitalEntriesStatus, localDigitalEntriesValue;

   DO_READ(localDebugMode, g_DoDebugMode);

   RESET_DIL_READ_BYPASSED();

   if (localDebugMode & ((UINT32)1 << (UINT32)FEED_CONDUCTIVITY_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)FEED_CONDUCTIVITY_1);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)PERMEATE_CONDUCTIVITY_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)PERMEATE_CONDUCTIVITY);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)EDI_RESISTIVITY_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)EDI_RESISTIVITY);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)DISTRIBUTION_RESISTIVITY_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_RESISTIVITY);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)FEED_TEMPERATURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)FEED_TEMPERATURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)PERMEATE_TEMPERATURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)PERMEATE_TEMPERATURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)EDI_TEMPERATURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)EDI_TEMPERATURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)DISTRIBUTION_TEMPERATURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_TEMPERATURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)FEED_PRESSURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)FEED_PRESSURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)RO_PRESSURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)RO_PRESSURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)DISTRIBUTION_PRESSURE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_PRESSURE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)TANK_LEVEL_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TANK_LEVEL);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)DRAIN_FLOW_RATE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)DRAIN_FLOWRATE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)PERMEATE_FLOW_RATE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)PERMEATE_FLOWRATE);
   }
   if (localDebugMode & ((UINT32)1 << (UINT32)EDI_REJECT_FLOW_RATE_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)EDI_REJECT_FLOWRATE);
   }
   /* we don't care the followed one s because there deducted from other sensors
   RO_REJECTION,
   RO_PERMEABILITY,
   RO_RECOVERY,
   RO_PUMP_FLOWRATE,
   PROGARD_DELTA_P_FLOWRATE,

   EDI_PRODUCTION_FLOWRATE,
   EDI_RECOVERY,

   TAP_FEED_PRESSURE,
   PROGARD_DELTA_PRESSURE,
   */
   /*! Digital entries update */
   DO_READ(localDigitalEntriesMask, g_DoDigitalEntriesStatus[DISABLE_DIGITAL_ENTRIES_MASK]);
   DO_READ(localDigitalEntriesStatus, g_DoDigitalEntriesStatus[DIGITAL_ENTRIES_STATUS]);
   DO_READ(localDigitalEntriesValue, g_DoDigitalEntriesStatus[DIGITAL_ENTRIES_VALUE]);
   /* if dig entries masked */
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)LEAK_DETECTOR_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)LEAK_DETECTOR);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, LEAK_DETECTOR_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)SDS_LEAK_DETECTOR_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_SDS_LEAK_DETECTOR);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, SDS_LEAK_DETECTOR_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)TANK_OVERFLOW_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_TANK_OVERFLOW);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, TANK_OVERFLOW_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)FAULT_REPORT1_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_FAULT_REPORT1);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, FAULT_REPORT1_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)FAULT_REPORT2_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_FAULT_REPORT2);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, FAULT_REPORT2_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)BACKWASH_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_BACKWASH_CARBON);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, BACKWASH_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)POD_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_POD_STATUS);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, POD_DIGENT_DBG);
   }
   if (localDigitalEntriesMask & ((UINT8)1 << (UINT8)TANK_REFILL_DIGENT_DBG))
   {
      SET_DIL_READ_BYPASSED((UINT32)TOR_TANK_REFILL);
      /* update status with given value */
      UPDATE_STATUS_ACCORDING_VALUE(localDigitalEntriesValue, localDigitalEntriesStatus, TANK_REFILL_DIGENT_DBG);
   }
   DO_WRITE(g_DoDigitalEntriesStatus[DIGITAL_ENTRIES_STATUS], localDigitalEntriesStatus);

   /* To forced read status by measure loop, thus Process will be warned  */
   __disable_interrupt();
   g_isTorsStatusModified=TRUE;
   __enable_interrupt();

   SYSLOG(INFO,"M:DIL ByPass=%0Xh",g_dwIsDilBypassed);
   SYSLOG(INFO,"M:DIG Input M-V-S=%0Xh:%0Xh:%0Xh",localDigitalEntriesMask,localDigitalEntriesValue, localDigitalEntriesStatus);
}


/*!
 *  @brief  Configure Measure according system device
 *
 *  @return None
 */
VOID MES_ConfigSystemDevice(VOID)
{
   SYS_DeviceType_t systemDeviceType = SYS_GetDeviceType();

   if (SYSTEM_DEVICE_AFS == systemDeviceType)
   {
      pUpdateMeasure100msProc = AfsMesureOperationProcArray[MEAS_OPERATION_100MS_IDX];
      pUpdateMeasure500msProc = AfsMesureOperationProcArray[MEAS_OPERATION_500MS_IDX];
      pUpdateMeasure1000msProc = AfsMesureOperationProcArray[MEAS_OPERATION_1000MS_IDX];
   }
   else if (SYSTEM_DEVICE_ELIX == systemDeviceType)
   {
      pUpdateMeasure100msProc = ElixMesureOperationProcArray[MEAS_OPERATION_100MS_IDX];
      pUpdateMeasure500msProc = ElixMesureOperationProcArray[MEAS_OPERATION_500MS_IDX];
      pUpdateMeasure1000msProc = ElixMesureOperationProcArray[MEAS_OPERATION_1000MS_IDX];
   }
   else if (SYSTEM_DEVICE_WPSH == systemDeviceType)
   {
      pUpdateMeasure100msProc = AfsMesureOperationProcArray[MEAS_OPERATION_100MS_IDX];
      pUpdateMeasure500msProc = AfsMesureOperationProcArray[MEAS_OPERATION_500MS_IDX];
      pUpdateMeasure1000msProc = AfsMesureOperationProcArray[MEAS_OPERATION_1000MS_IDX];
   }
   else // Default  (SYSTEM_DEVICE_RIOS==systemDeviceType)
   {
      pUpdateMeasure100msProc = RiosMesureOperationProcArray[MEAS_OPERATION_100MS_IDX];
      pUpdateMeasure500msProc = RiosMesureOperationProcArray[MEAS_OPERATION_500MS_IDX];
      pUpdateMeasure1000msProc = RiosMesureOperationProcArray[MEAS_OPERATION_1000MS_IDX];
   }

   switch(SYS_GetNbCartridges())
   {
      case SYSTEM_ONE_CARTRIDGE:
      {
         if (SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_1_CARTRIDGES_50HZ;
         }
         else
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_1_CARTRIDGES_60HZ;
         }
         break;
      }
      case SYSTEM_TWO_CARTRIDGES:
      case SYSTEM_THREE_CARTRIDGES:
      {
         if (SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_2_3_CARTRIDGES_50HZ;
         }
         else
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_2_3_CARTRIDGES_60HZ;
         }
         break;
      }
      case SYSTEM_FOUR_CARTRIDGES:
      {
         if (SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_4_CARTRIDGES_50HZ;
         }
         else
         {
            g_fFeedPressureMinAlarmThreshold = ALARM_FEED_PRESSURE_MIN_4_CARTRIDGES_60HZ;
         }
         break;
      }

      default:
         ERROR("Unknown Number of cartridges");
         break;
   }
}


/*!
 *  @brief  Dump the recorded measure values in SYSLOG
 *
 *  @return None
 */
VOID MES_DumpRecords(VOID)
{
   UINT32 counter;
   UINT32 firstRecord;
   UINT32 numberOfRecord;
   UINT32 l_index;

   if (TRUE == g_mesRecordStruct.isTabFull)
   {
      firstRecord = g_mesRecordStruct.nIndexNextRecord;
      numberOfRecord = N_MAX_RECORD;
   }
   else
   {
      firstRecord = 0;
      numberOfRecord = g_mesRecordStruct.nIndexNextRecord;
   }

   SYSLOG(TEST, "Time InV RoP RecircValve DrainValv RinsV PermV DistP RecircV DumpV FeedP RoP DistP DrainF PermF EdiRejF TankMm TankSpeed FeedC PermC EdiR DistR PermT EdiT DistT");
   for(counter = 0; counter < numberOfRecord; counter++)
   {
      l_index = counter + firstRecord;
      if(N_MAX_RECORD <= l_index)
      {
         l_index = l_index - N_MAX_RECORD;
      }

      SYSLOG(TEST, "%d %d %d %.02f %.02f %d %d %d %d %d %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f %.02f",
             g_mesRecordStruct.MeasuresRecordTab[l_index].TimeStamp,
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x02) >0),
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x40) >0),
             g_mesRecordStruct.MeasuresRecordTab[l_index].RecircValvePosition,
             g_mesRecordStruct.MeasuresRecordTab[l_index].DrainValvePosition,
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x08) >0),
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x04) >0),
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x80) >0),
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x20) >0),
             ((g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus & 0x10) >0),
             g_mesRecordStruct.MeasuresRecordTab[l_index].FeedPressure,
             g_mesRecordStruct.MeasuresRecordTab[l_index].RoPressure,
             g_mesRecordStruct.MeasuresRecordTab[l_index].DistPressure,
             g_mesRecordStruct.MeasuresRecordTab[l_index].DrainFlowrate,
             g_mesRecordStruct.MeasuresRecordTab[l_index].PermeateFlowrate,
             g_mesRecordStruct.MeasuresRecordTab[l_index].EDIRejectFlowrate,
             g_mesRecordStruct.MeasuresRecordTab[l_index].TankLevelMM,
             g_mesRecordStruct.MeasuresRecordTab[l_index].TankLevelSpeed,
             g_mesRecordStruct.MeasuresRecordTab[l_index].FeedCond,
             g_mesRecordStruct.MeasuresRecordTab[l_index].PermeateCond,
             g_mesRecordStruct.MeasuresRecordTab[l_index].EdiRes,
             g_mesRecordStruct.MeasuresRecordTab[l_index].DistRes,
             g_mesRecordStruct.MeasuresRecordTab[l_index].PermeatTemp,
             g_mesRecordStruct.MeasuresRecordTab[l_index].EdiTemp,
             g_mesRecordStruct.MeasuresRecordTab[l_index].DistTemp);
   }

}


/*!
 *  @brief  Call every seconds to record the measure values
 *
 *  @return None
 */
static VOID mes_Trace(VOID)
{
   UINT32 l_index = g_mesRecordStruct.nIndexNextRecord;
   g_mesRecordStruct.MeasuresRecordTab[l_index].TimeStamp = TMC_StartTimeCounter();
   g_mesRecordStruct.MeasuresRecordTab[l_index].RecircValvePosition = MVM_GetPosition(MVM_VALVE_RECIRCULATION);
   g_mesRecordStruct.MeasuresRecordTab[l_index].DrainValvePosition = MVM_GetPosition(MVM_VALVE_DRAIN);
   g_mesRecordStruct.MeasuresRecordTab[l_index].ActuatorsStatus = g_DoActuatorsStatus.ActuatorsStatus;
   g_mesRecordStruct.MeasuresRecordTab[l_index].FeedPressure = g_MeasDataArray[FEED_PRESSURE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].RoPressure = g_MeasDataArray[RO_PRESSURE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].DistPressure = g_MeasDataArray[DISTRIBUTION_PRESSURE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].DrainFlowrate = g_MeasDataArray[DRAIN_FLOWRATE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].PermeateFlowrate = g_MeasDataArray[PERMEATE_FLOWRATE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].EDIRejectFlowrate = g_MeasDataArray[EDI_REJECT_FLOWRATE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].TankLevelMM = g_MeasDataArray[TANK_LEVEL].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].TankLevelSpeed = g_MeasDataArray[TANK_FILLING_SPEED].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].FeedCond = g_MeasDataArray[FEED_CONDUCTIVITY_1].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].PermeateCond = g_MeasDataArray[PERMEATE_CONDUCTIVITY].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].EdiRes = g_MeasDataArray[EDI_RESISTIVITY].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].DistRes = g_MeasDataArray[DISTRIBUTION_RESISTIVITY].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].PermeatTemp = g_MeasDataArray[PERMEATE_TEMPERATURE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].EdiTemp = g_MeasDataArray[EDI_TEMPERATURE].fMeasure;
   g_mesRecordStruct.MeasuresRecordTab[l_index].DistTemp = g_MeasDataArray[DISTRIBUTION_TEMPERATURE].fMeasure;
   g_mesRecordStruct.nIndexNextRecord++;
   if (N_MAX_RECORD <= g_mesRecordStruct.nIndexNextRecord)
   {
      g_mesRecordStruct.nIndexNextRecord = 0;
      g_mesRecordStruct.isTabFull = TRUE;
   }
}


/*!
 *  @brief  Measure task
 *
 *  @return None
 *  @retval None
 *
 *  @details   Measure task retrieve physical measures with DIL helps
 *       at differents periods (100 ms , 500ms , 1sec) according
 *       measure requirements.<br>
 *       It converts physical measures into readable measures.<br>
 *       It deals with AlarmAndAlert task to trigs alerts.<br>
 *       It receive specific commands from Process task (refer to API).<br>
 *       Task in addition, count few flow meters.<br>
 */
ATTRIBUTE_TASK void measureTask(VOID)
{
   UINT8 bMeasurePeriod = 0U;

   SYSLOG(INFO, "Measure task ............: START");

   InitData();

   MES_InitPressureDrivers();
   MES_InitTORDrivers();
   MES_InitFlowrateDrivers();
   MES_InitTemperatureDrivers();
   MES_InitConductivityDrivers();
   MES_InitTankLevelDrivers();
   MES_InitHardwareMonitoringDriver();
   MVM_InitMvmDrivers();
   MES_InitCurrentDrivers();

   g_bAppTaskInit++;

   while ( FALSE == g_doLaunchApp )
   {
      OS_TaskDelay(DELAY100MS);
   }

   while (g_isMeasureTaskActive)
   {
      BOOL        l_isSecondBoudary;
      INT32       sdwSemaListOffset;
      MES_Msg_t   mesMsg;

      sdwSemaListOffset = OS_SemaWaitLHI( (OsHandle_t*)hMeasSemaList ) ;

      switch ( sdwSemaListOffset )
      {
         case MES_MSG_SEM_IDX:
            /* Just get the message (does not wait for it) */
            MSG_GetWaitH( hMeasureMboxHld, (psMSG_t)&mesMsg, (UINT8)0U /* Get */ );
            /* Analyses message content */
            MES_MsgParser(&mesMsg);
            break;

         case MES_TIM_SEM_IDX:
            {
               UINT32      dwMeasOperationIdx = 0U;

               bMeasurePeriod++;

               if (0U == (bMeasurePeriod % 10U))
               {
                  l_isSecondBoudary = TRUE;
               }
               else
               {
                  l_isSecondBoudary = FALSE;
               }

               /* CAUTIONS ** MOTOR VALVE MANAGEMENT ** :
                * === Always call BEFORE measure update ==
                * === Always call MVM_UpdateMotorValveCommand BEFORE MVM_Update500ms
                * === Always call MVM_Update500ms BEFORE MVM_Update100ms
                */
               /* Update New position to reach at Motor Valve Step Motor Drivers */
               MVM_UpdateMotorValveCommand();

               /* Update Matlab model computation every 500 ms with Permeate and Drain flowrate  */
               if(   (UINT8) 0 == (bMeasurePeriod % (UINT8) 5 ))
               {
                  MVM_Update500ms(  g_MeasDataArray[PERMEATE_FLOWRATE].fMeasure,
                                    g_MeasDataArray[DRAIN_FLOWRATE].fMeasure);
               }
               /* Update Matlab model computation every 100 ms with RO pressure */
               MVM_Update100ms(g_MeasDataArray[RO_PRESSURE].fMeasure);

               /* reset TOR update status which is set under Ext line ISR in #MES_WarnTORChangedCallBack */
               __disable_interrupt();
               if(TRUE== g_isTorsStatusModified)
               {
                  g_doMesUpdateTorStatus=TRUE;
                  g_isTorsStatusModified=FALSE;
               }
               else
               {
                  g_doMesUpdateTorStatus=FALSE;
               }
               __enable_interrupt();


               /* MEASURE UPDATE
                */
               /* # every 100 ms  */
               /*******************/
               dwMeasOperationIdx = 0U;
               while (MES_END_MES_OPERATION != pUpdateMeasure100msProc[dwMeasOperationIdx])
               {
                  pUpdateMeasure100msProc[dwMeasOperationIdx](l_isSecondBoudary);
                  dwMeasOperationIdx++;
               }

               /* # every 500 ms  */
               /*******************/
               dwMeasOperationIdx = 0U;
               if ((UINT8) 0 == (bMeasurePeriod % (UINT8) 5 ))
               {
                  while (MES_END_MES_OPERATION != pUpdateMeasure500msProc[dwMeasOperationIdx])
                  {
                     pUpdateMeasure500msProc[dwMeasOperationIdx](l_isSecondBoudary);
                     dwMeasOperationIdx++;
                  }
               }

               /* # every 1000 ms  */
               /*********************/
               dwMeasOperationIdx = 0U;
               if ((UINT8)0 ==(bMeasurePeriod % (UINT8)10 )  )
               {
                  while (MES_END_MES_OPERATION != pUpdateMeasure1000msProc[dwMeasOperationIdx])
                  {
                     pUpdateMeasure1000msProc[dwMeasOperationIdx](l_isSecondBoudary);
                     dwMeasOperationIdx++;
                  }

                  /* Update sensors Diag DO field */
                  UpdateSensorsDoDiag();

                  /*! send PDO records */
                  MES_CanSendRecords();

                  /*! Toggle led measure period */
                  SYS_Led_Toggle(SYSTEM_LED_1_SEC_MEAS);

                  bMeasurePeriod = 0U;

                  mes_Trace();
               }
            }
            break;

         default:
            ASSERT( FALSE,"Wrong message ID receive in Measure task");
            break;
      }

   }

   SYSLOG(INFO, "Measure task ............: END");

   OS_TaskTerminate(OS_SELFTASK);
}

// For debug purpose, use to deactivate Alarm trig
#if 0
#define AAA_SetAlarm(_x)
#define AAA_ResetAlarm(_x)
#endif


/*!
 *
 *  @param alarmType [IN] AlarmType_t <br>
 *        Alarm type to raise
 *
 *  @brief  Set alarm only if disable
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY independant mode alarm
 */
VOID MES_SetAlarm(AlarmType_t alarmType)
{
   /* If alarm not set , set it */
   if (AAA_STATE_DISABLE == AAA_GetAlarmState(alarmType))
   {
      AAA_SetAlarm(alarmType);
   }
}


/*!
 *
 *  @param alarmType [IN] AlarmType_t <br>
 *        Alarm type to raise
 *
 *  @brief Reset alarm if  active or pending.
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY independant mode alarm
 */
VOID MES_ClearAlarm(AlarmType_t alarmType)
{
   if (AAA_STATE_DISABLE != AAA_GetAlarmState(alarmType))
   {
      AAA_ResetAlarm(alarmType);
   }
}

/*!
 *
 *  @param alarmType [IN] AlarmType_t <br>
 *        Alarm type to raise
 *
 *  @brief  Clear pending alarm saves in bitmap located in measure context and Reset Alarm
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY independant mode alarm
 */
VOID MES_ClearPendingAlarm(AlarmType_t alarmType)
{
   if (AAA_STATE_PENDING == AAA_GetAlarmState(alarmType))
   {
      AAA_ResetPendingAlarm(alarmType);
   }
}

/*!
 *  @param chkAlarmType [IN]MeasureChkAlarm_t <br>
 *        Index of mode dependnant alamr to raised
 *
 *  @brief  Set Alarm if disable
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY \p Process dependant alarms (mode dependant)
 */
VOID MES_SetCheckAlarm(MeasureChkAlarm_t chkAlarmType)
{
   /* If alarm not set , set it */
   if(AAA_STATE_DISABLE== AAA_GetAlarmState(g_MeasCheckAlarmArray[chkAlarmType].almType))
   {
      AAA_SetAlarm(g_MeasCheckAlarmArray[chkAlarmType].almType);
   }
}


/*!
 *  @param chkAlarmType [IN]MeasureChkAlarm_t <br>
 *        Index of mode dependnant alamr to raised
 *
 *  @brief  Reset Alarm if pending or active
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY \p Process dependant alarms (mode dependant)
 */
VOID MES_ClearCheckAlarm(MeasureChkAlarm_t chkAlarmType)
{
   /* If alarm set or pending , clear it */
   if(AAA_STATE_DISABLE != AAA_GetAlarmState(g_MeasCheckAlarmArray[chkAlarmType].almType))
   {
      AAA_ResetAlarm(g_MeasCheckAlarmArray[chkAlarmType].almType);
   }
}


/*!
 *  @param chkAlarmType [IN]MeasureChkAlarm_t <br>
 *        Index of mode dependnant alamr to raised
 *
 *  @brief  Reset Alarm if pending.
 *
 *  @return None
 *  @retval None
 *
 *  @details  Process ONLY \p Process dependant alarms (mode dependant)
 *            Called upon \a MES_AlarmChkStop procedure call
 */
VOID MES_ClearCheckPendingAlarm(MeasureChkAlarm_t chkAlarmType)
{
   /* If alarm set , reset pending  */
   if(AAA_STATE_PENDING == AAA_GetAlarmState(g_MeasCheckAlarmArray[chkAlarmType].almType))
   {
      AAA_ResetPendingAlarm(g_MeasCheckAlarmArray[chkAlarmType].almType);
   }
}


/*!
 *
 *  @param sensor [IN] Sensor_t <br>
 *          sensor type status to report diag
 *
 *  @param isError [IN] BOOL  <br>
 *            sensor error status
 *
 * @return None
 *
 * @brief Periodic update  sensor error status. Called on all sensors measure except flowrates.
 */
VOID MES_WarnPeriodicSensorErrorDiag(Sensor_t sensor,BOOL isError)
{
   /* write diagnostic in OD */
   if(FALSE==isError)
   {
      g_dwPeriodicSensorDiagResult &= ~ ((UINT32)1<<sensor);
   }
   else
   {
      g_dwPeriodicSensorDiagResult |= ((UINT32)1<<sensor);
   }
}


/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/
