/*!
 * @file mes_api.c
 *
 * @brief Measure module API
 *
 * @details API definition to interface *Measure module with others modules
 *          such as *Process or *AlertAndAlarms
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Rev:: 626                   $: Revision of last commit
 * $Author:: m175711            $: Author of last commit
 * $Date:: 2014-11-06 11:44:31 #$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_api.c $: URL of this file
 * @endcond
 *
 */


/* Includes ------------------------------------------------------------------*/
#include "millipore_types.h"    // application types
#include "msg_type.h"
#include "mes_data.h"
#include "measure.h"
#include "do_itf.h"
#include "sys_softerr.h"



/*!
 *  @ingroup Measure_API
 *  @{
 */
/* Defines -------------------------------------------------------------------*/
#define  MEAS_TASK_SIZE (256)
#define  MEAS_TASK_PRIO (8)

/* Declarations --------------------------------------------------------------*/

/*! Fill Measure message macro */
#define MES_FILL_MSG(_pmsg, _cmd, _meastype, _optparam)\
{  \
   (_pmsg).cmd=(_cmd);\
   (_pmsg).uParam.measType=(_meastype);\
   (_pmsg).dwOptParam=(UINT32)(_optparam);\
}


/* Code ----------------------------------------------------------------------*/

/*!
 * @param measType [IN] CONST MeasureType_t
 *
 * @return MeasVolumeCounterData_t* pointer to measure counter data
 *
 * @brief  retyurn volume counter data pointer coresponding to flowrate  counter measurement type
 *
 * @details  TBC
 *
 */
static MeasVolumeCounterData_t* GetMeasVolumeCounterDataPtr(MeasureType_t measType)
{
   MeasVolumeCounterData_t *pMeasVolCnt=NULL;

   switch(measType)
   {
      case MES_DRAIN_FLOWRATE:
         pMeasVolCnt=&g_MeasVolumeCounterData[MES_VOL_DRAIN];
         break;
      case MES_PERMEATE_FLOWRATE:
         pMeasVolCnt=&g_MeasVolumeCounterData[MES_VOL_PERMEATE];
         break;
      case MES_EDI_PRODUCTION_FLOWRATE:
         pMeasVolCnt=&g_MeasVolumeCounterData[MES_VOL_PRODUCE];
         break;
      default:
         /* Not available */
         ASSERT(FALSE,"Wrong Meastype for vol counter");
         break;
   }
   return pMeasVolCnt;
}


/*!
 *  @brief  Start Measure task
 *
 *  @details  Task is declared statically but not started because
 *            start order is set to 0. Once called , the task is started
 *
 */
VOID MES_TaskStart(VOID)
{
   MES_ConfigSystemDevice();
   eOsRc_t rc= OS_TaskExecute("MEASTASK");
   ASSERT( OS_GOOD == rc,"Unable to start Production task");
}

#if 0 // Not used anymore
/*!
 *  @param  measType [IN]    MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return None
 *
 *  @brief Allow start measure acquistion
 *
 *  @note Not used anymore
 */
VOID MES_AcquisitionStart(MeasureType_t measType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_ACQ_START, measType, 0U );
   ASSERT( MSG_GOOD == MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL ),
           "Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Mes ON  %d",measType);
}


/*!
 *  @param  measType  [IN]    MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return None
 *
 *  @brief Allow stop measure acquistion
 *
 *  @note Not used anymore
 */
VOID MES_AcquisitionStop(MeasureType_t measType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_ACQ_STOP, measType, 0u );
   ASSERT( MSG_GOOD == MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL ),
           "Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Mes OFF %d",measType);
}
#endif //0


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return Measure              FLOAT
 *
 *  @brief The gaol of this procedure is to return last record send saved when record was enable
 *
 *  @details TBC
 *
 */
UINT16 MES_GetLastRecord(MeasureType_t measType)
{
   UINT16 wLastRecord=(UINT16)-1;
   if(MES_RO_REJECTION==measType)
   {
       wLastRecord=g_wLastRoRejectionRec;
   }
   return wLastRecord;
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @param  fRoBackPressure [IN]  FLOAT                 <br>
 *       RO back pressure usefull to compute Ro Premeability
 *
 *  @return None
 *
 *  @brief Setter for RO back pressure variable
 *
 *  @details TBC
 *
 */
VOID MES_SetRoBackPressure(MeasureType_t measType,FLOAT fRoBackPressure)
{
   if(MES_RO_PERMEABILITY==measType)
   {
      g_fBackPressure=fRoBackPressure;
   }
}

/*!
 *  @param  isFlushLimited [IN]  BOOL                 <br>
 *       Value for global
 *
 *  @return None
 *
 *  @brief Let the measure task know if the flush is limited or not
 *
 *  @details TBC
 *
 */
VOID MES_SetFlushLimited(BOOL isFlushLimited)
{
   g_isFlushLimited = isFlushLimited;
}


/*!
 *  @param  measType    [ IN ]   MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @param  shouldComputeAverage :  [ IN ]    BOOL                 <br>
 *       FALSE if simple record to be done
 *       TRUE if average to be compute and record trig one average done
 *
 *  @return None
 *
 *  @brief Called by Process task in order to
 *                        - ask for Record
 *                        - ask for Record with average
 *
 *  @details When record is requested, Record is send at second period.\n
 *             Record is averaged on measure done over the last second.\n
 *             On the contrary, when average computation is ordered,\n
 *             Average accumulation only is started, and no record is produced
 *             until #MES_RecordStop is called.
 *
 */
VOID MES_RecordStart(MeasureType_t measType, BOOL shouldComputeAverage )
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_RECORD_START, measType, (UINT32)shouldComputeAverage );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"->M:Rec ON  %d",measType);
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return None
 *
 *  @brief Alow stop recording or trig an average Record.
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_RecordStop(MeasureType_t measType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_RECORD_STOP, measType, (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Rec OFF %d",measType);
}


/*!
 *  @param  measChkAlarm [IN]  MeasureChkAlarm_t                 <br>
 *       Process depednant alarm ID to check
 *
 *  @return None
 *  @retval x
 *
 *  @brief Allow \p Process to start alarm check.
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_AlarmChkStart(MeasureChkAlarm_t measChkAlarm)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Mes.cmd=MES_CMD_ALARM_CHK_START;
   Msg.body.Mes.uParam.measChkAlm=measChkAlarm;
   Msg.body.Mes.dwOptParam=(UINT32)0u;

   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Chk ON  %d",measChkAlarm);
}


/*!
 *  @param  measChkAlarm [IN]  MeasureChkAlarm_t                 <br>
 *       Process depednant alarm ID to check
 *
 *  @return None
 *  @retval x
 *
 *  @brief Allow \p Process to Stop alarm check
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_AlarmChkStop(MeasureChkAlarm_t measChkAlarm)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Mes.cmd=MES_CMD_ALARM_CHK_STOP;
   Msg.body.Mes.uParam.measChkAlm=measChkAlarm;
   Msg.body.Mes.dwOptParam=(UINT32)0u;

   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Chk OFF  %d",measChkAlarm);
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return None
 *  @retval x
 *
 *  @brief Allow \p Process to start flow counters
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_FlowCounterStart(MeasureType_t measType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_FLOW_CNT_START, measType, (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"->M:FlowCnt ON  %d",measType);
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return None
 *
 *  @brief Allow \p Process to  flow counters API
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_FlowCounterStop(MeasureType_t measType)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_FLOW_CNT_STOP, measType, (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->FlowCnt OFF %d",measType);
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return Volume count [OUT]  UINT32
 *  @retval -1 if not available
 *
 *  @brief Allow \p Process to get volume counter.
 *
 */
DOUBLE MES_VolumeCounterGet(MeasureType_t measType)
{
   MeasVolumeCounterData_t* p_MeasVolCntData;

   p_MeasVolCntData=GetMeasVolumeCounterDataPtr(measType);

   return p_MeasVolCntData->dCnt;
}


/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @return Measure              FLOAT
 *
 *  @brief Allow \p Process to get measure value coresponding to measure type given.
 *
 *  @details
 *       if measure is not available (because it has not been done in the time),\n
 *       it returns <OUT_OF_RANGE_MAX_MEASURE> or <OUT_OF_RANGE_MIN_MEASURE>.
 */
FLOAT MES_GetMeasure(MeasureType_t measType)
{
   FLOAT measure = MES_FLOAT_NOT_AVAILABLE;
   UINT32 SensorId = g_MeastypToSensorMap[measType];
   measure=g_MeasDataArray[SensorId].fMeasure;
   /*! Special treatment for percentage measure request,
    * Normally, it is recommended that MES moduel to not perform any units conversion
    */
   if(MES_TANK_LEVEL_PC==measType)
   {
      measure = g_fMeasTankLvlPercent;
   }
   else if(MES_TANK_LEVEL_VOL==measType)
   {
      measure = g_fTankLevelVolume;
   }
   else
   {
      /* Do Nothing */
   }

   return measure;
}



/*!
 *  @param  measType [IN]  MeasureType_t                 <br>
 *       Measure type ID
 *
 *  @param  fSetPoint [IN]  FLOAT                 <br>
 *       New setpoint
 *
 *  @return None
 *
 *  @brief Allow \p Process to change alarm setpoint.
 *
 */
VOID MES_SetNewSetPointAlarmThres(MeasureType_t measType,FLOAT fSetPoint)
{
   UINT32 SensorId = g_MeastypToSensorMap[measType];
   g_MeasDataArray[SensorId].fSetPointAlarmThres=fSetPoint;
   SYSLOG( INFO,"M:->Set new SP %d = %6d",measType,(INT32)fSetPoint);
}


/*!
 *  @brief Allow advertise Measure task of a modification in DIL bypass.
 *
 *  @details
 *      Called from CanOpen task on writing in \b DebugMode or \n
 *         \n Digital Entries Can object dictionnary field to forced a value.
 */
VOID MES_SetDebugStatusModification(VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_DEBUG_MODE_CHG, MES_RO_TEMPERATURE , (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->Debug mode change");
}


/*!
 *  @brief Retrieve diagnotic report
 *
 *  @details
 *         Called from CanOpen task on writing in \b Command OD field.
 */
VOID MES_GetDiagnosticReport(VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   MES_FILL_MSG(Msg.body.Mes, MES_CMD_DIAGNOSTIC_REQ, MES_RO_TEMPERATURE , (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->GetDiagnose");

}


/*!
 *  @brief Update measure alarms setpoints
 *
 *  @details
 *         Called from CanOpen task on writing in \b g_DoAlertAndAlarmsSetpoints OD field.
 */
VOID MES_UpdateSetpoints(VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   MES_FILL_MSG(Msg.body.Mes, MES_CMD_SP_UPDATE, MES_RO_TEMPERATURE , (UINT32)0u );
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"M:->SP Update");
}


/*!
 *  @param  bResetAlarmCmd [IN]  UINT8                 <br>
 *       Reset alarm command.
 *
 *  @return None
 *  @retval x
 *
 *  @brief Allow \p CanOpen task to reset alarm.
 *         To reset alarm, source alarm is recheck to ensure alarm desappear.
 *         If alarm disapaerd , Alarm is reseted.
 *
 *  @details Refer to #MES_MsgParser.
 *
 */
VOID MES_ResetAlarmReq(UINT8 bResetAlarmCmd)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Mes.cmd=MES_CMD_RESET_ALARM_REQ;
   Msg.body.Mes.uParam.bResetAlarmCmd=bResetAlarmCmd;
   Msg.body.Mes.dwOptParam=0u;   /*not used  */

   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );

   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
   SYSLOG( INFO,"->M:Reset alrm  %d",bResetAlarmCmd);
}


/*!
 *  @brief Dump of measure records.
 */
VOID MES_DumpRecord(VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Mes.cmd=MES_CMD_DUMP_RECORDS;
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
}

/*!
 *  @brief Enable/Disable the tank level percent saturation.
 */
VOID MES_TankLevelSaturation(BOOL isSaturationNeeded)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Mes.cmd=MES_CMD_TAN_LEVEL_SAT;
   Msg.body.Mes.dwOptParam= (UINT32)isSaturationNeeded;
   rc=MSG_SendH(hMeasureMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc ,"Unable to post a message to Measure mailbox");
}

/*!
 *  @brief Get the minimum feed pressure needed when RO Pump is ON to avoid cavitation
 */
FLOAT MES_GetFeedPressureMin(VOID)
{
   return(g_fFeedPressureMinAlarmThreshold);
}

/*!
 *  @}
 */


/*!
 *  @ingroup Measure_func
 *  @{
 */

/*!
 *  @param  p_MeasMsg :           [IN]    MES_Msg_t           <br>
 *                      Measure message
 *
 *  @return None
 *
 *  @brief Measure task message parser
 */
void MES_MsgParser(CONST  MES_Msg_t* p_MeasMsg)
{
   MeasVolumeCounterData_t *p_MeasVolCntData;
   UINT32 SensorId = g_MeastypToSensorMap[p_MeasMsg->uParam.measType];

   switch (p_MeasMsg->cmd)
   {
      case MES_CMD_RECORD_START:
      {
         BOOL shouldComputeAverage = (BOOL) p_MeasMsg->dwOptParam;

         if(FEED_CONDUCTIVITY_2 == SensorId)
         {
            SensorId&=~((UINT32)ALT_MEASURE);
            g_bRoFeedCondMeasEn=TRUE;
         }
         ASSERT(TRUE == g_MeasDataArray[SensorId].isProcDepMeasEn,"Measure not active");

         if (FALSE == shouldComputeAverage)
         {
            g_MeasDataArray[SensorId].isProcDepRecordEn = TRUE;
            g_MeasDataArray[SensorId].isProcDepAverageEn = FALSE;
         }
         else
         {
            g_MeasDataArray[SensorId].isProcDepRecordEn = FALSE;
            g_MeasDataArray[SensorId].isProcDepAverageEn = TRUE;
         }
         break;
      }

      case MES_CMD_RECORD_STOP:
      {
         if(FEED_CONDUCTIVITY_2 == SensorId)
         {
            SensorId&=~((UINT32)ALT_MEASURE);
            g_bRoFeedCondMeasEn=FALSE;
         }

         if (TRUE == g_MeasDataArray[SensorId].isProcDepRecordEn)
         {
            /*! simple record production end */
            g_MeasDataArray[SensorId].isProcDepRecordEn = FALSE;
         }
         else if(TRUE == g_MeasDataArray[SensorId].isProcDepAverageEn )
         {
            /*! Average computation finalization */
            MeasData_t* CONST pMeasData = &g_MeasDataArray[SensorId];

            /*! Call Stop record procedure */
            if(NULL != pMeasData->pStopRecordFunc)
            {
               pMeasData->pStopRecordFunc(pMeasData);
            }

            /*!  Average computing is stopped */
            g_MeasDataArray[SensorId].isProcDepAverageEn = FALSE;
         }
         else
         {
            /* Nothing to do */
         }
         break;
      }

      case MES_CMD_ALARM_CHK_START:
      {
         /* Enable mode dependant alarm */
         g_MeasCheckAlarmArray[p_MeasMsg->uParam.measChkAlm].isEnable=TRUE;
         break;
      }

      case MES_CMD_ALARM_CHK_STOP:
      {
         /* If alarm set , reset pending  */
         MES_ClearCheckPendingAlarm(p_MeasMsg->uParam.measChkAlm);
         /* Disable mode depedant alarm */
         g_MeasCheckAlarmArray[p_MeasMsg->uParam.measChkAlm].isEnable=FALSE;
         break;
      }

      case MES_CMD_FLOW_CNT_START:
      {
         p_MeasVolCntData=GetMeasVolumeCounterDataPtr((MeasureType_t)p_MeasMsg->uParam.measType);
         p_MeasVolCntData->isFlowCntRunning=TRUE;
         break;
      }

      case MES_CMD_FLOW_CNT_STOP:
      {
         p_MeasVolCntData=GetMeasVolumeCounterDataPtr((MeasureType_t)p_MeasMsg->uParam.measType);
         p_MeasVolCntData->isFlowCntRunning=FALSE;
         break;
      }

      case MES_CMD_VOL_CNT_GET:
      {
         /* Treat directly */
         break;
      }

      case MES_CMD_GET_MEASURE:
      {
         /* Treat directly */
         break;
      }

      case MES_CMD_GET_LAST_REC:
      {
         /* Treat directly */
         break;
      }

      case MES_CMD_SET_RO_BACK_PRESSURE:
      {
         /* Treat directly */
         break;
      }

      case MES_CMD_DEBUG_MODE_CHG:
      {
         MES_CheckDebugModeChange();
         break;
      }

      case MES_CMD_DIAGNOSTIC_REQ:
      {
         MES_DiagnoseFlowrates();
         /* Acknowledge command reception by clear command */
         DO_WRITE(g_DoHardwareDiagnostic.Command,0u);
         break;
      }

      case MES_CMD_SP_UPDATE:
      {
         DO_READ(g_MeasDataArray[FEED_CONDUCTIVITY_1].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.FeedConductivitySp);
         DO_READ(g_MeasDataArray[PERMEATE_TEMPERATURE].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.FeedTemperatureSp);
         DO_READ(g_MeasDataArray[FCE].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.FceSp);
         DO_READ(g_MeasDataArray[RO_REJECTION].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.RoRejectionSp);
         DO_READ(g_MeasDataArray[EDI_RESISTIVITY].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.ElixResistivitySp);
         DO_READ(g_MeasDataArray[TANK_LEVEL].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.TankReserveSp);
         DO_READ(g_MeasDataArray[DISTRIBUTION_RESISTIVITY].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.DistributionResistivitySp);
         DO_READ(g_MeasDataArray[DISTRIBUTION_TEMPERATURE].fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.ProductTemperatureSp);
         DO_READ(g_bLowTdsEn, g_DoAlertAndAlarmsSetpoints.LowTdsSp);
         break;
      }

      case MES_CMD_RESET_ALARM_REQ:
      {
         /*! Message send by CAN (Ihm) to reset one alarm.
          *  Before reseting the alarm, we ensure alarm source is cleaed for Leak detector or overflow .
          */
         MES_CheckAlarmToReset(p_MeasMsg->uParam.bResetAlarmCmd);
         break;
      }

      case MES_CMD_DUMP_RECORDS:
      {
         MES_DumpRecords();
         break;
      }
      
      case MES_CMD_TAN_LEVEL_SAT:
      {
         g_isTankLevelSaturated = (BOOL) p_MeasMsg->dwOptParam;
         break;
      }

      default:
      {
         ERROR("MES_Parser : Msg type unknow: %4Xh ", p_MeasMsg->cmd );
         break;
      }
   }
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

