/*!
* @file mes_conductivity.c
* @author alain Keraudy
* @date $Date: 2014-09-29 13:59:51 +0200 (lun. 29 sept. 2014) $
*
* @brief Conductivity/Resistivity acquisition
*
* @details Contain Conductivity/Resistivity acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Rev:: 585                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-09-29 13:59:51 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_conductivity.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>


#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_res.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "system_parameters.h"
#include "measure.h"


/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup Conductivity_Measure Conductivity/Resistivity measure
 *  @ingroup Measure
 *  @brief Conductivity/Resistivity measure acquistion procedures
 *  @{
 */

/* Definitions ---------------------------------------------------------------*/

#define DEFAULT_TEMP_CALIBRATION_GAIN  ((FLOAT)1.0F)        /*!< Default Temperature gain */
#define REFERENCE_VOLTAGE              ((FLOAT)3.3F)        /*!< Reference voltage = 3.3V */
#define RESISTIVITY_REC_MAX            ((FLOAT)65.5F)       /*!< Reference voltage = 3.3V */
#define FCE_REC_MAX                    ((FLOAT)655.0F)      /*!< Reference voltage = 3.3V */

#define L_RES_UP_COND_25               ((DOUBLE)0.0549)
#define L_RES_NA_CL_MOBILITY_25        ((DOUBLE)126.4552)
#define L_CONDUCTIVITY_MIN             ((DOUBLE)0.0000000001)

#define FEED_COND_FILTER_COEF          (0.2f)   /*!< Filtering coef for volume    */

/* Declarations --------------------------------------------------------------*/
static DIL_HANDLE dilFeedConductivityHdl = NULL;           /*!<Feed conductivity DIL handle */
static DIL_HANDLE dilPermeateConductivityHdl = NULL;       /*!<Permeate conductivity DIL handle */
static DIL_HANDLE dilEdiResistivityHdl = NULL;             /*!<Edi Resistivity DIL handle */
static DIL_HANDLE dilDistributionResistivityHdl = NULL;    /*!<Distribution Resistivity DIL handle */

static FLOAT fOldFeedResistance = FLT_MAX;

FLOAT g_fRoFeedConductivityNC;  /*!< RO feed conductivity NC, saved for RO Rejection computation */
FLOAT g_fLastRoRejectionRec=MES_FLOAT_NOT_AVAILABLE;  /*!< Contain Last RO Rejection computed */

/* Codes ---------------------------------------------------------------------*/

/*!
 *  @param  dRawVal :         IN    DOUBLE                           <br>
 *       Conductivity raw value to compensate in uS/cm
 *
 *  @param  dTemp :           IN    DOUBLE                           <br>
 *       Measurement temperature in \C [0.0 - 50.0]
 *
 *  @return DOUBLE : Temperature-compensated conductivity value.
 *
 *  @brief
 *       Compute temperature-compensated conductivity value.
 *
 *  @details
 *   This function computes temperature-compensated conductivity value in double in uS/cm.<br>
 *  @f$ TCCond (\mbox{uS/cm}) = \frac {NaCl_25 \cdot (dRawVal - UPCond}{NaCl} + UPCond_25 @f$
 *
 */
static DOUBLE ComputeCondTCValueDouble(DOUBLE dRawVal, DOUBLE dTemp)
{
   DOUBLE dTCVal;
   DOUBLE dUltraPureCond, dNaClMobility;
   DOUBLE dTempSq = dTemp * dTemp;

   // Limit temp
   if ( dTemp < 0.0f)
   {
      dTemp = 0.0f;
   }
   else if ( dTemp > 50.0f )
   {
      dTemp = 50.0f;
   }
   else
   {
      /* Do Nothing */
   }

   // Ultrapure water conductivity
   dUltraPureCond = ((DOUBLE)120126E-7)
                  + ((DOUBLE)80198E-8 * dTemp)
                  + ((DOUBLE)292E-7 * dTempSq)
                  + ((DOUBLE)3E-7 * dTempSq * dTemp);

   //NaCl mobility
   dNaClMobility = ((DOUBLE)669252E-4)
                 + ((DOUBLE)21337E-4 * dTemp)
                 + ((DOUBLE)99E-4 * dTempSq);

   /****************************************************************************
   *          NaCl_25 * (RawCond - UPCond)
   * TCCond = ---------------------------- + UPCond_25
   *                    NaCl
   ****************************************************************************/
   dTCVal = ((((dRawVal - dUltraPureCond) * L_RES_NA_CL_MOBILITY_25) / dNaClMobility) + L_RES_UP_COND_25);
   
   if (L_CONDUCTIVITY_MIN > dTCVal)
   {
      /* Conductivity can not be negative */
      dTCVal = L_CONDUCTIVITY_MIN;
   }

   // return TC conductivity in uS/cm
   return (dTCVal);
}


/*!
 * @param fResistance [IN] FLOAT <br>
 *                   Resistance measured (from DIL)
 *
 * @param fCellCoef [IN] FLOAT <br>
 *                   Cell coefficient (from DO)
 *
 *
 * @return Not conpensated conductivity.as uS
 *
 * @brief returns Conductivity from given voltage raw measure<br>
 *              and calibration data.<br>
 * @details <br>
 * @f$ NcConduc (uS/cm )= \frac {1E6 \cdot fCellCoef}{RawRes} ~\\ ~\\
 *     RawRes = 100k\Omega  @f$
 */
static FLOAT ComputeNotCompensateConductivity(FLOAT fResistance, FLOAT fCellCoef)
{
   FLOAT fNcConductivity;        /* Not compensated conductivity */

   if(0.0f <= fResistance)
   {
      /* 1E6 is used to convert to uS */
      fNcConductivity = ((1E6f * fCellCoef) / fResistance);
   }
   else
   {
      fNcConductivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   return fNcConductivity;
}


/*!
 *  @param fPermeateConductivityNC [IN] FLOAT <br>
 *                  Not compensated  permeate conductivity measure.
 *
 *  @param fRoFeedConductivityNC [IN] FLOAT <br>
 *                  Not compensated feed conductivity.
 *
 *  @return None.
 *
 *  @brief Update RO rejection every 1000 ms.
 *  @f$ RO rejection (\%)= ( 1-  \frac{  PermeateCond}{ ROFeedCond } ) \cdot 100  @f$
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# Compute RO rejection
 *     -# write RO Rejection in OD
 *     -# Check SP mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write  RO Rejection  in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
static VOID UpdateRoRejection(FLOAT fPermeateConductivityNC, FLOAT fRoFeedConductivityNC)
{
   FLOAT fRoRejectionPerCent;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[RO_REJECTION];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Compensated RO Rejection computation */
   /* =====================================*/

   if(((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fPermeateConductivityNC) &&
      ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateConductivityNC) &&
      ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fRoFeedConductivityNC) &&
      ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fRoFeedConductivityNC) &&
      ( 0.0f != fRoFeedConductivityNC))
   {
      fRoRejectionPerCent=((1.0f - (fPermeateConductivityNC / fRoFeedConductivityNC)) * 100.0f);
      if(100.0f < fRoRejectionPerCent)
      {
         fRoRejectionPerCent = 100.0f;
      }
      else if (0.0f > fRoRejectionPerCent)
      {
         fRoRejectionPerCent = 0.0f;
      }
      else
      {
         /* Do Nothing */
      }
   }
   else
   {
      fRoRejectionPerCent = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }

   /* Save measure */
   pMeasData->fMeasure=fRoRejectionPerCent;

   /* Check Alarms */
   if((TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_RO_REJECTION_SP].isEnable)  &&
      (FALSE == g_bLowTdsEn))
   {
      /* Set Point conductivity check  */
      if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
          (pMeasData->fMeasure < pMeasData->fSetPointAlarmThres))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_RO_REJECTION_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_RO_REJECTION_SP);
      }
   }
   else if (TRUE == g_bLowTdsEn)
   {
      MES_ClearCheckAlarm(MES_CHK_ALM_RO_REJECTION_SP);
   }
   else
   {
      /* Do Nothing */
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fRoRejectionPerCent)
      {
         /* Write RO Rejection in OD REC History field */
         DO_WRITE(g_DoWaterQuality.RoRejection, ((UNSIGNED16)((fRoRejectionPerCent*100.0f)+0.5f)));
      }
      else
      {
         /* Write RO Rejection in OD REC History field */
         DO_WRITE(g_DoWaterQuality.RoRejection, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }
      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_RO_REJECTION);
      /* Saved RO rejection Record */
      g_wLastRoRejectionRec=g_DoWaterQuality.RoRejection;
   }
}


/*!
 *  @param fPermeateConductivity [IN] FLOAT <br>
 *                  Permeate conductivity measure.
 *
 *  @return void
 *
 *  @brief Update FCE (Feed Conductivity Equivalent)  every 1000 ms <br>
 *
 *  @f$ Fce = PermeateCond + ( 2.77 \cdot CO^2 )  @f$.
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# Compute FCE
 *     -# write FCE in OD
 *     -# Check SP mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Fce in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
static VOID UpdateFce(FLOAT fPermeateConductivity)
{
   FLOAT fCo2Level;
   FLOAT fFce;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[FCE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Compensated FCE computation */
   /* ============================*/
   DO_READ(fCo2Level, g_DoAlertAndAlarmsSetpoints.TapFeedCO2);
   if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fPermeateConductivity) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateConductivity))
   {
      fFce = (fPermeateConductivity + (2.79F * fCo2Level));
   }
   else
   {
      fFce = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   /* Save measure */
   pMeasData->fMeasure=fFce;
   DO_WRITE(g_DoFce, fFce);

   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FCE_SP].isEnable)
   {
      /* Greater Set Point check  */
      if(((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
         (pMeasData->fMeasure > pMeasData->fSetPointAlarmThres))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FCE_SP);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_FCE_SP);
         /* Alarm is cleared by an user acknowledgement */
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (FCE_REC_MAX > fFce)
      {
         /* Write FCE in OD REC History field */
         DO_WRITE(g_DoWaterQuality.FceRecord, ((UNSIGNED16)((fFce*100.0f)+0.5f)));
      }
      else
      {
         /* Write RO Rejection in OD REC History field */
         DO_WRITE(g_DoWaterQuality.FceRecord, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }
      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_FCE);
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 1000ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Permeate conductivity every 1000 ms.
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL conductivity read (RawResistance, Range, Voltage)
 *     -# Write RawResistance, Range, Voltage values in OD
 *     -# Compute Not Compensated Permeate conductivity
 *     -# write Instant NT PermeateConductivity in OD
 *     -# Compute Compensated Permeate conductivity
 *     -# write Instant TC PermeateConductivity in OD
 *     -# Check mode independant Alarms (Min, Max, Sp)
 *     -# Check mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Instant NT permeate conductivity in OD
 *          ->write Instant TC permeate conductivity in OD
 *     -# Call RO rejection measure
 *     -# Call Fce measure
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdatePermeateConductivity(BOOL isSecondBoundary)
{
   FLOAT conductivityRes = 0.0f;   /* temp variable.*/
   FLOAT fNcConductivity = 0.0f;
   FLOAT fCellCoef;
   FLOAT fCalibGain;
   FLOAT fTemperature;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[PERMEATE_CONDUCTIVITY];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      ASSERT(FALSE==g_MeasDataArray[RO_REJECTION].isProcDepMeasEn,"Can't measure RO rejection if Permeate Conductivity measure is disabled");
      return;
   }

   /* If DIL not bypassed, DIL feed conductivity read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_CONDUCTIVITY))
   {
      DIL_RC   ret;
      FLOAT fValue = 0.0F;
      UINT8 bRange = 0U;

      ret = dil_ioctldevice(dilPermeateConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RESISTANCE ,
                            (DIL_LPVOID)&conductivityRes, sizeof(conductivityRes), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Permeate conductivity  call");
      /* Save Raw data */
      pMeasData->rawMeas.fResistance = conductivityRes;
      /* write resistance value in OD */
      DO_WRITE(g_DoPermeateWaterQuality.RawResistance , pMeasData->rawMeas.fResistance);

      ret = dil_ioctldevice(dilPermeateConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RANGE ,
                            (DIL_LPVOID)&bRange, sizeof(bRange), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Permeate conductivity  call");
      /* write range value in OD */
      DO_WRITE(g_DoPermeateWaterQuality.Range , bRange);

      ret = dil_ioctldevice(dilPermeateConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_VOLTAGE ,
                            (DIL_LPVOID)&fValue, sizeof(fValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Permeate conductivity  call");
      /* write voltage value in OD */
      DO_WRITE(g_DoPermeateWaterQuality.Voltage ,fValue);
   }

   /* Read Raw resistance data */
   DO_READ(pMeasData->rawMeas.fResistance ,g_DoPermeateWaterQuality.RawResistance);

   if(FLT_MAX == pMeasData->rawMeas.fResistance )
   {
      fNcConductivity = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.InstantConductivityNC,fNcConductivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);
   }
   else if(FLT_MIN == pMeasData->rawMeas.fResistance )
   {
      fNcConductivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.InstantConductivityNC,fNcConductivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);
   }
   else
   {
      /* We assume pMeasData->rawMeas.fResistance cant' be NULL */

      /* Not compensated conductivity computation for CALIB */
      /* ================================================== */
      /* Read Calibration cell coefficient */
      DO_READ(fCellCoef,g_DoPermeateWaterQuality.CalibAdjConductivityCell);
      /* Read  Calibration circuit gain  */
      DO_READ(fCalibGain,g_DoPermeateWaterQuality.CalibAdjCircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjResistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);

      /* Not compensated conductivity computation */
      /* ======================================== */
      /* Read  Cell coefficient */
      DO_READ(fCellCoef,g_DoPermeateWaterQuality.ConductivityCell);
      /* Read  Circuit gain  */
      DO_READ(fCalibGain,g_DoPermeateWaterQuality.CircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      /* Write compensated Resistance */
      DO_WRITE(g_DoPermeateWaterQuality.Resistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoPermeateWaterQuality.InstantConductivityNC,fNcConductivity);

      /* Compensated conductivity computation */
      /* =====================================*/
      /* Get temperature  */
      fTemperature=g_MeasDataArray[PERMEATE_TEMPERATURE].fMeasure;
      if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
      {
         pMeasData->fMeasure=(FLOAT)ComputeCondTCValueDouble((DOUBLE)fNcConductivity, (DOUBLE)fTemperature);
      }
      else
      {
         pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
   }

   /* write Temperature compensated conductivity value in OD */
   DO_WRITE(g_DoPermeateWaterQuality.InstantConductivityTC , pMeasData->fMeasure);


   /* Check Alarms */
   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_COND_MAX].isEnable)
   {
      /* Check Max conductivity */
      if (ALARM_PERMEATE_COND_MAX < fNcConductivity )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_COND_MAX);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PERMEATE_COND_MAX);
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_COND_MIN].isEnable)
   {
      /* Check Min conductivity */
      if (ALARM_PERMEATE_COND_MIN > fNcConductivity )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_COND_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PERMEATE_COND_MIN);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fNcConductivity) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
      {
         /* Write Not compensated conductivity x100  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.PermeateConductivityNTC,((UNSIGNED16)((fNcConductivity*100.0f)+0.5f)));
      }
      else
      {
         /* Write Not compensated conductivity x1000  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.PermeateConductivityNTC, (UNSIGNED16)INVALID_RECORD_MEASURE );
      }

      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
      {
         /* Write Temperature compensated conductivity x in OD REC History field */
         FLOAT fTempValue = ((pMeasData->fMeasure * 100.0f)+0.5f);
         DO_WRITE(g_DoWaterQuality.PermeateConductivityTC, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Temperature compensated conductivity x 1000 in OD REC History field */
         DO_WRITE(g_DoWaterQuality.PermeateConductivityTC, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }
      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_PERMEATE_CONDUCTIVITY);
   }

   /* Update RO Rejection */
   UpdateRoRejection(fNcConductivity, g_fRoFeedConductivityNC);
   UpdateFce(pMeasData->fMeasure);
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 1000ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Feed conductivity every 1000 ms
 *
 *  @details
 *     -# if DIL not bypassed, DIL feed conductivity read
 *     -# Write Resistance value in OD
 *     -# Compute Feed conductivity
 *     -# write Instant NT Feed Conductivity in OD
 *     -# write Instant TC Feed Conductivity in OD
 *     -# Check mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->if Tap feed conducitvity measure enable
 *          ---->write Tap feed NT conductivity in OD
 *          ----->write Tap feed TC conductivity in OD
 *          if Ro feed conducitvity measure enable
 *          ----->write Ro feed  NC conductivity in OD
 *          ----->write Ro feed  TC conductivity in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateFeedConductivity(BOOL isSecondBoundary)
{
   FLOAT conductivityRes = 0;   /* temp variable.*/
   FLOAT fNcConductivity=0;
   FLOAT fCellCoef;
   FLOAT fCalibGain;
   UINT16 fRoTemperature;
   FLOAT fTemperature;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[FEED_CONDUCTIVITY_1];

   NOT_USED_PARAM(isSecondBoundary);

   /* If DIL not bypassed, DIL feed conductivity read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_CONDUCTIVITY_1))
   {
      DIL_RC   ret;
      FLOAT fValue = 0.0F;
      UINT8 bRange = 0U;

      ret = dil_ioctldevice(dilFeedConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RESISTANCE ,
                            (DIL_LPVOID)&conductivityRes, sizeof(conductivityRes), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed conductivity  call");
      /* Filtering raw resistance */
      if ((FLT_MAX == fOldFeedResistance) ||
          (FLT_MIN == fOldFeedResistance) ||
          (FLT_MAX == conductivityRes)    ||
          (FLT_MIN == conductivityRes))
      {
         fOldFeedResistance = conductivityRes;
      }
      else
      {
         conductivityRes = (((FLOAT)FEED_COND_FILTER_COEF * conductivityRes) + ((1.0f - (FLOAT)FEED_COND_FILTER_COEF) * fOldFeedResistance));
         fOldFeedResistance = conductivityRes;
      }
      
      /* Save Raw data */
      pMeasData->rawMeas.fResistance = conductivityRes;
      /* write resistance value in OD */
      DO_WRITE(g_DoFeedWaterQuality.RawResistance , pMeasData->rawMeas.fResistance);

      ret = dil_ioctldevice(dilFeedConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RANGE ,
                            (DIL_LPVOID)&bRange, sizeof(bRange), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed conductivity  call");
      /* write range value in OD */
      DO_WRITE(g_DoFeedWaterQuality.Range , bRange);

      ret = dil_ioctldevice(dilFeedConductivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_VOLTAGE ,
                            (DIL_LPVOID)&fValue, sizeof(fValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed conductivity  call");
      /* write voltage value in OD */
      DO_WRITE(g_DoFeedWaterQuality.Voltage ,fValue);
   }

   /* Read Raw resistance data */
   DO_READ(pMeasData->rawMeas.fResistance ,g_DoFeedWaterQuality.RawResistance);

   if(FLT_MAX == pMeasData->rawMeas.fResistance )
   {
      fNcConductivity = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.InstantConductivityNC,fNcConductivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);
   }
   else if(FLT_MIN == pMeasData->rawMeas.fResistance )
   {
      fNcConductivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.InstantConductivityNC,fNcConductivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);
   }
   else
   {
      /* We assume pMeasData->rawMeas.fResistance can't be NULL */

      /* Not compensated conductivity computation for CALIB */
      /* ================================================== */
      /* Read Calibration cell coefficient */
      DO_READ(fCellCoef,g_DoFeedWaterQuality.CalibAdjConductivityCell);
      /* Read  Calibration circuit gain  */
      DO_READ(fCalibGain,g_DoFeedWaterQuality.CalibAdjCircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjResistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.CalibAdjInstantConductivityNC,fNcConductivity);

      /* Not compensated conductivity computation */
      /* ======================================== */
      /* Read  Cell coefficient */
      DO_READ(fCellCoef,g_DoFeedWaterQuality.ConductivityCell);
      /* Read  Circuit gain  */
      DO_READ(fCalibGain,g_DoFeedWaterQuality.CircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      /* Write compensated Resistance */
      DO_WRITE(g_DoFeedWaterQuality.Resistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoFeedWaterQuality.InstantConductivityNC,fNcConductivity);

      /* Compensated conductivity computation */
      /* =====================================*/
      /*! @note To compute TC Feed conductivity, we use last Permeate temperature recorded.
       *  First reason is there is no feed temperature sensors.
       *  Second reason is permeate temperature sensors is not always active. This depend on Mode system stand.
       *  (caution : Permeate temperature is record in Ro Temperature record as x 1000.
       */
      DO_READ(fRoTemperature, g_DoWaterQuality.RoTemperature);
      if ((fRoTemperature != INVALID_RECORD_MEASURE) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
      {
         fTemperature=((FLOAT)fRoTemperature / 1000.0F);
         pMeasData->fMeasure=(FLOAT)ComputeCondTCValueDouble((DOUBLE)fNcConductivity, (DOUBLE)fTemperature);
      }
      else
      {
         pMeasData->fMeasure= (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
   }

   /* write Temperature compensated conductivity value in OD */
   DO_WRITE(g_DoFeedWaterQuality.InstantConductivityTC , pMeasData->fMeasure);

   /* save conductivity NC for RO Rejection computation in permeate conductivity call */
   g_fRoFeedConductivityNC = fNcConductivity;

   /* Check Alarms */
   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FEED_COND_MAX].isEnable)
   {
      /* Check Max conductivity */
      if (ALARM_FEED_COND_MAX < fNcConductivity )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FEED_COND_MAX);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_FEED_COND_MAX);
      }
   }

   if((TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FEED_COND_MIN].isEnable) &&
      (FALSE == g_bLowTdsEn))
   {
      /* Check Min conductivity */
      if (ALARM_FEED_COND_MIN > fNcConductivity)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FEED_COND_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_FEED_COND_MIN);
      }
   }
   else if (TRUE == g_bLowTdsEn)
   {
      MES_ClearCheckAlarm(MES_CHK_ALM_FEED_COND_MIN);
   }
   else
   {
      /* Do Nothing */
   }

   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FEED_COND_SP].isEnable)
   {
      /* Greater Set Point conductivity check  */
      if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
          ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          (pMeasData->fMeasure > pMeasData->fSetPointAlarmThres))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FEED_COND_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_FEED_COND_SP);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* TAP Feed Conductivity */
      if(FALSE==g_bRoFeedCondMeasEn)
      {
         /* Short flush operation */
         if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fNcConductivity) &&
             ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.TapFeedConductivityNTC, (UNSIGNED16)(fNcConductivity+0.5f));
         }
         else
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.TapFeedConductivityNTC, (UNSIGNED16)INVALID_RECORD_MEASURE );
         }

         if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
             ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.TapFeedConductivityTC, (UNSIGNED16)(pMeasData->fMeasure+0.5f));
         }
         else
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.TapFeedConductivityTC, (UNSIGNED16)INVALID_RECORD_MEASURE );
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_TAP_FEED_CONDUCTIVITY);
      }
      /* RO Feed Conductivity */
      else
      {
         /* Tank filling operation */
         if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fNcConductivity) &&
             ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.RoFeedConductivityNTC, (UNSIGNED16)(fNcConductivity+0.5f));
         }
         else
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.RoFeedConductivityNTC, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
             ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.RoFeedConductivityTC, (UNSIGNED16)(pMeasData->fMeasure+0.5f));
         }
         else
         {
            /* Write Not compensated conductivity  in OD REC History field  */
            DO_WRITE(g_DoWaterQuality.RoFeedConductivityTC, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_RO_FEED_CONDUCTIVITY);
      }
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 1000ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update EDI resistivity every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL EDI resistivity read
 *     -# Write Resistance value in OD
 *     -# Compute EDI NT resistivity
 *     -# write Instant NT EDI resistivity in OD
 *     -# Compute EDI TC resistivity
 *     -# write Instant TC EDI resistivity in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Elix Instant NC  resitivity in OD
 *          ->write Elix Instant TC  resitivity in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateEDIResistivity(BOOL isSecondBoundary)
{
   FLOAT conductivityRes = 0;   /* temp variable.*/
   FLOAT fNcConductivity=0;
   FLOAT fNcResistivity=0;
   FLOAT fCellCoef;
   FLOAT fCalibGain;
   FLOAT fTemperature;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[EDI_RESISTIVITY];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If DIL not bypassed, DIL Edi resistivity read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_RESISTIVITY))
   {
      DIL_RC   ret;
      FLOAT fValue = 0.0F;
      UINT8 bRange = 0U;

      ret = dil_ioctldevice(dilEdiResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RESISTANCE ,
                            (DIL_LPVOID)&conductivityRes, sizeof(conductivityRes), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Edi conductivity  call");
      /* Save Raw data */
      pMeasData->rawMeas.fResistance = conductivityRes;
      /* write resistance value in OD */
      DO_WRITE(g_DoEdiWaterQuality.RawResistance, pMeasData->rawMeas.fResistance);

      ret = dil_ioctldevice(dilEdiResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RANGE ,
                            (DIL_LPVOID)&bRange, sizeof(bRange), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Edi conductivity  call");
      /* write range value in OD */
      DO_WRITE(g_DoEdiWaterQuality.Range , bRange);

      ret = dil_ioctldevice(dilEdiResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_VOLTAGE ,
                            (DIL_LPVOID)&fValue, sizeof(fValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Edi conductivity  call");
      /* write voltage value in OD */
      DO_WRITE(g_DoEdiWaterQuality.Voltage ,fValue);
   }

   /* Read Raw resistance data */
   DO_READ(pMeasData->rawMeas.fResistance ,g_DoEdiWaterQuality.RawResistance);

   if(FLT_MAX == pMeasData->rawMeas.fResistance )
   {
      fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.InstantResistivityNC,fNcResistivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);
   }
   else if(FLT_MIN == pMeasData->rawMeas.fResistance )
   {
      fNcResistivity = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.InstantResistivityNC,fNcResistivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);
   }
   else
   {
      /* We assume pMeasData->rawMeas.fResistance cant' be NULL */

      /* Not compensated resistivity computation for CALIB */
      /* ================================================= */
      /* Read Calibration cell coefficient */
      DO_READ(fCellCoef,g_DoEdiWaterQuality.CalibAdjConductivityCell);
      /* Read  Calibration circuit gain  */
      DO_READ(fCalibGain,g_DoEdiWaterQuality.CalibAdjCircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity)
      {
         fNcResistivity = (1.0f / fNcConductivity);
      }
      else
      {
         fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjResistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);

      /* Not compensated resistivity computation */
      /* ======================================== */
      /* Read  Cell coefficient */
      DO_READ(fCellCoef,g_DoEdiWaterQuality.ConductivityCell);
      /* Read  Circuit gain  */
      DO_READ(fCalibGain,g_DoEdiWaterQuality.CircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity)
      {
         fNcResistivity = (1.0f / fNcConductivity);
      }
      else
      {
         fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
      /* Write compensated Resistance */
      DO_WRITE(g_DoEdiWaterQuality.Resistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoEdiWaterQuality.InstantResistivityNC,fNcResistivity);

      /* Compensated resistivity computation */
      /* =====================================*/
      /* Get temperature  */
      fTemperature=g_MeasDataArray[EDI_TEMPERATURE].fMeasure;
      if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
      {
         pMeasData->fMeasure=(FLOAT)(1.0 / ComputeCondTCValueDouble(fNcConductivity,fTemperature));
      }
      else
      {
         pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
   }

   /* write Temperature compensated resistivity value in OD */
   DO_WRITE(g_DoEdiWaterQuality.InstantResistivityTC , pMeasData->fMeasure);


   /* Check Alarms */
   /* Check Process depedant Alarm */
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_ELIX_RES_MAX].isEnable)
   {
      /* Check Max conductivity */
      if (ALARM_ELIX_RESISTIVITY_MAX < fNcResistivity)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_ELIX_RES_MAX);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_ELIX_RES_MAX);
      }
   }
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_ELIX_RES_MIN].isEnable)
   {
      /* Check Min conductivity */
      if (ALARM_ELIX_RESISTIVITY_MIN > fNcResistivity )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_ELIX_RES_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_ELIX_RES_MIN);
      }
   }

   /* Greater Set Point conductivity check  */
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_ELIX_RES_SP].isEnable)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
          (pMeasData->fMeasure < pMeasData->fSetPointAlarmThres))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_ELIX_RES_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_ELIX_RES_SP);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fNcResistivity) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcResistivity))
      {
         /* Write Not compensated conductivity x1000  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.ElixResistivityNTC,((UNSIGNED16)((fNcResistivity*1000.0F)+0.5f)));
      }
      else
      {
         /* Write Not compensated conductivity x1000  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.ElixResistivityNTC, (UNSIGNED16)INVALID_RECORD_MEASURE );
      }

      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          ( RESISTIVITY_REC_MAX > pMeasData->fMeasure))
      {
         /* Write Temperature compensated conductivity x 1000 in OD REC History field */
         FLOAT fTempValue = ((pMeasData->fMeasure * 1000.0f)+0.5f);
         DO_WRITE(g_DoWaterQuality.ElixResistivityTC, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Temperature compensated conductivity x 1000 in OD REC History field */
         DO_WRITE(g_DoWaterQuality.ElixResistivityTC, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }

      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_ELIX_RESISTIVITY);
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 1000ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Distribution resistivity every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL resistivity read
 *     -# Write Resistance value in OD
 *     -# Compute Distribution NT resistivity
 *     -# write Instant NT EDI resistivity in OD
 *     -# Compute Distribution TC resistivity
 *     -# write Instant TC Distribution resistivity in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Distribution Instant NC  resitivity in OD
 *          ->write Distribution Instant TC  resitivity in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateDistributionRestivity(BOOL isSecondBoundary)
{
   FLOAT conductivityRes = 0.0f;   /* temp variable.*/
   FLOAT fNcConductivity = 0.0f;
   FLOAT fNcResistivity = 0.0f;
   FLOAT fCellCoef;
   FLOAT fCalibGain;
   FLOAT fTemperature;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[DISTRIBUTION_RESISTIVITY];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If DIL not bypassed, DIL Edi resistivity read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_RESISTIVITY))
   {
      DIL_RC   ret;
      FLOAT fValue = 0.0F;
      UINT8 bRange = 0U;

      ret = dil_ioctldevice(dilDistributionResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RESISTANCE ,
                            (DIL_LPVOID)&conductivityRes, sizeof(conductivityRes), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Distribution conductivity  call");
      /* Save Raw data */
      pMeasData->rawMeas.fResistance = conductivityRes;
      /* write resistance value in OD */
      DO_WRITE(g_DoDistributionWaterQuality.RawResistance , pMeasData->rawMeas.fResistance);

      ret = dil_ioctldevice(dilDistributionResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_RANGE ,
                            (DIL_LPVOID)&bRange, sizeof(bRange), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Distribution conductivity  call");
      /* write range value in OD */
      DO_WRITE(g_DoDistributionWaterQuality.Range , bRange);

      ret = dil_ioctldevice(dilDistributionResistivityHdl, (DIL_UINT32)IOCTL_CMD_COND_GET_VOLTAGE ,
                            (DIL_LPVOID)&fValue, sizeof(fValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Distribution conductivity  call");
      /* write voltage value in OD */
      DO_WRITE(g_DoDistributionWaterQuality.Voltage ,fValue);
   }

   /* Read Raw resistance data */
   DO_READ(pMeasData->rawMeas.fResistance ,g_DoDistributionWaterQuality.RawResistance);

   if(FLT_MAX == pMeasData->rawMeas.fResistance )
   {
      fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.InstantResistivityNC,fNcResistivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);
   }
   else if(FLT_MIN == pMeasData->rawMeas.fResistance )
   {
      fNcResistivity = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.Resistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.InstantResistivityNC,fNcResistivity);
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjResistance,pMeasData->rawMeas.fResistance);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);
   }
   else
   {
      /* We assume pMeasData->rawMeas.fResistance cant' be NULL */


      /* Not compensated resistivity computation for CALIB */
      /* ================================================= */
      /* Read Calibration cell coefficient */
      DO_READ(fCellCoef,g_DoDistributionWaterQuality.CalibAdjConductivityCell);
      /* Read  Calibration circuit gain  */
      DO_READ(fCalibGain,g_DoDistributionWaterQuality.CalibAdjCircuitGain);
      /* Compute Temperature NOT-Compensated conductivity */
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity)
      {
         fNcResistivity = (1.0f / fNcConductivity);
      }
      else
      {
         fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjResistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.CalibAdjInstantResistivityNC,fNcResistivity);

      /* Not compensated resistivity computation */
      /* ======================================== */
      /* Read  Cell coefficient */
      DO_READ(fCellCoef,g_DoDistributionWaterQuality.ConductivityCell);
      /* Read  Calibration gain  */
      DO_READ(fCalibGain,g_DoDistributionWaterQuality.CircuitGain);
      conductivityRes = pMeasData->rawMeas.fResistance * fCalibGain;
      fNcConductivity = ComputeNotCompensateConductivity(conductivityRes,fCellCoef);
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity)
      {
         fNcResistivity = (1.0f / fNcConductivity);
      }
      else
      {
         fNcResistivity = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
      /* Write compensated Resistance */
      DO_WRITE(g_DoDistributionWaterQuality.Resistance,conductivityRes);
      /* Write Not compensated resitivity in OD */
      DO_WRITE(g_DoDistributionWaterQuality.InstantResistivityNC,fNcResistivity);

      /* Compensated resistivity computation */
      /* =====================================*/
      /* Get temperature  */
      fTemperature = g_MeasDataArray[DISTRIBUTION_TEMPERATURE].fMeasure;

      if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fTemperature) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcConductivity))
      {
         pMeasData->fMeasure=(FLOAT)(1.0f / ComputeCondTCValueDouble(fNcConductivity,fTemperature));
      }
      else
      {
         pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      }
   }

   /* write Temperature compensated resistivity value in OD */
   DO_WRITE(g_DoDistributionWaterQuality.InstantResistivityTC , pMeasData->fMeasure);


   /* Check Alarms */
   /* Check Process depedant Alarm */
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_DIST_RES_MAX].isEnable)
   {
      /* Check Max conductivity */
      if (ALARM_DISTRIBUTION_RESISTIVITY_MAX < fNcResistivity)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_DIST_RES_MAX);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_DIST_RES_MAX);
      }
   }
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_DIST_RES_MIN].isEnable)
   {
      /* Check Min conductivity */
      if (ALARM_DISTRIBUTION_RESISTIVITY_MIN > fNcResistivity )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_DIST_RES_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_DIST_RES_MIN);
      }
   }
   if (TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_DIST_RES_SP].isEnable)
   {
      /* Greater Set Point conductivity check  */
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
          (pMeasData->fMeasure < pMeasData->fSetPointAlarmThres))

      {
         MES_SetCheckAlarm(MES_CHK_ALM_DIST_RES_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_DIST_RES_SP);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fNcResistivity) &&
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fNcResistivity))
      {
         /* Write Not compensated conductivity x1000  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.DistributionNTC,((UNSIGNED16)((fNcResistivity*1000.0f)+0.5f)));
      }
      else
      {
         /* Write Not compensated conductivity x1000  in OD REC History field  */
         DO_WRITE(g_DoWaterQuality.DistributionNTC, INVALID_RECORD_MEASURE );
      }

      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure) &&
          ( RESISTIVITY_REC_MAX > pMeasData->fMeasure))
      {
         FLOAT fTempValue = ((pMeasData->fMeasure * 1000.0f)+0.5f);
         /* Write Temperature compensated conductivity x 1000 in OD REC History field */
         DO_WRITE(g_DoWaterQuality.DistributionTC, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Temperature compensated conductivity x 1000 in OD REC History field */
         DO_WRITE(g_DoWaterQuality.DistributionTC, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }

      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_DISTRIBUTION_RESISTIVITY);
   }
}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details
 */
VOID MES_InitConductivityDrivers(VOID)
{
   MeasData_t* CONST pMeasFeedConductivityData = &g_MeasDataArray[FEED_CONDUCTIVITY_1];
   MeasData_t* CONST pMeasPermeateConductivityData = &g_MeasDataArray[PERMEATE_CONDUCTIVITY];
   MeasData_t* CONST pMeasEdiResistivityData = &g_MeasDataArray[EDI_RESISTIVITY];
   MeasData_t* CONST pMeasDistributionResistivityData = &g_MeasDataArray[DISTRIBUTION_RESISTIVITY];
   MeasData_t* CONST pMeasRoRejectionData = &g_MeasDataArray[RO_REJECTION];
   MeasData_t* CONST pMeasFceData = &g_MeasDataArray[FCE];

   g_bRoFeedCondMeasEn = FALSE;
   DIL_RC   ret;

   /* DIL feed conductivity init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_CONDUCTIVITY_1))
   {
      DIL_COND_MEASURE_CONFIG measConf;
      UINT16 wValue;
      BOOL bValue;

      ret = dil_opendevice(DIL_NAME_RESISTIVITY_FEED, &dilFeedConductivityHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed Conductivity init");

      // write config
      measConf.bHalfNbrOfSample   = (DIL_UINT8)(MEASURE_FEED_COND_SENSOR_NBR_OF_SAMPLE / 2U) ;
      measConf.bDutyCycle         = (DIL_UINT8)MEASURE_FEED_COND_SENSOR_DUTY_CYCLE;
      measConf.bInhibitTime       = (DIL_UINT8)MEASURE_FEED_COND_SENSOR_INHIBIT_TIME / 5U;
      measConf.fMinValue          = (DIL_FLOAT)MEASURE_FEED_COND_SENSOR_MIN_VOLTAGE;
      measConf.fMaxValue          = (DIL_FLOAT)MEASURE_FEED_COND_SENSOR_MAX_VOLTAGE;
      measConf.fSwitchToHighRes   = (DIL_FLOAT)MEASURE_FEED_COND_SENSOR_SWITCH_TO_HIGH_VOLTAGE;
      measConf.fSwitchToLowRes    = (DIL_FLOAT)MEASURE_FEED_COND_SENSOR_SWITCH_TO_LOW_VOLTAGE;
      ret = dil_ioctldevice(dilFeedConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_CONFIG,
                            (DIL_PVOID)&measConf,
                            sizeof(measConf),
                            NULL );

      if(SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
      {
         wValue = MEASURE_FEED_COND_SENSOR_50HZ_FREQ;
      }
      else
      {
         wValue = MEASURE_FEED_COND_SENSOR_60HZ_FREQ;
      }
      ret = dil_ioctldevice(dilFeedConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_SET_FREQUENCY,
                            (DIL_PVOID)wValue,
                            sizeof(DIL_UINT16),
                            NULL );

      bValue = 1;
      ret = dil_ioctldevice(dilFeedConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_ENABLE,
                            (DIL_PVOID)bValue,
                            sizeof(DIL_BOOL),
                            NULL );
   }
   /* Read TAP Feed conductivity SP in OD */
   DO_READ(pMeasFeedConductivityData->fSetPointAlarmThres, g_DoAlertAndAlarmsSetpoints.FeedConductivitySp);
   DO_READ(g_bLowTdsEn, g_DoAlertAndAlarmsSetpoints.LowTdsSp);
   pMeasFeedConductivityData->dwNbSplMeasure = 0u;
   pMeasFeedConductivityData->fMeasAccumulator = 0.0f;
   pMeasFeedConductivityData->pStopRecordFunc = NULL;
   pMeasFeedConductivityData->isProcDepMeasEn = TRUE;
   pMeasFeedConductivityData->isProcDepAverageEn = FALSE;
   pMeasFeedConductivityData->isProcDepRecordEn = FALSE;

   pMeasFeedConductivityData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* DIL Permeate resistivity init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_CONDUCTIVITY))
   {
      DIL_COND_MEASURE_CONFIG measConf;
      UINT16 wValue;
      UINT8 bValue;

      ret = dil_opendevice(DIL_NAME_RESISTIVITY_PERMEATE, &dilPermeateConductivityHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed Conductivity init");

      // write config
      measConf.bHalfNbrOfSample   = (DIL_UINT8)(MEASURE_PERMEATE_COND_SENSOR_NBR_OF_SAMPLE / 2U);
      measConf.bDutyCycle         = (DIL_UINT8)MEASURE_PERMEATE_COND_SENSOR_DUTY_CYCLE;
      measConf.bInhibitTime       = (DIL_UINT8)MEASURE_PERMEATE_COND_SENSOR_INHIBIT_TIME / 5U;
      measConf.fMinValue          = (DIL_FLOAT)MEASURE_PERMEATE_COND_SENSOR_MIN_VOLTAGE;
      measConf.fMaxValue          = (DIL_FLOAT)MEASURE_PERMEATE_COND_SENSOR_MAX_VOLTAGE;
      measConf.fSwitchToHighRes   = (DIL_FLOAT)MEASURE_PERMEATE_COND_SENSOR_SWITCH_TO_HIGH_VOLTAGE;
      measConf.fSwitchToLowRes    = (DIL_FLOAT)MEASURE_PERMEATE_COND_SENSOR_SWITCH_TO_LOW_VOLTAGE;
      ret = dil_ioctldevice(dilPermeateConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_CONFIG,
                            (DIL_PVOID)&measConf,
                            sizeof(measConf),
                            NULL );

      if(SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
      {
         wValue = MEASURE_PERMEATE_COND_SENSOR_50HZ_FREQ;
      }
      else
      {
         wValue = MEASURE_PERMEATE_COND_SENSOR_60HZ_FREQ;
      }
      ret = dil_ioctldevice(dilPermeateConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_SET_FREQUENCY,
                            (DIL_PVOID)wValue,
                            sizeof(DIL_UINT16),
                            NULL );

      bValue = 1U;
      ret = dil_ioctldevice( dilPermeateConductivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_ENABLE,
                            (DIL_PVOID)bValue,
                            sizeof(DIL_BOOL),
                            NULL );
   }
   pMeasPermeateConductivityData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pMeasPermeateConductivityData->dwNbSplMeasure = 0u;
   pMeasPermeateConductivityData->fMeasAccumulator = 0.0f;
   pMeasPermeateConductivityData->pStopRecordFunc = NULL;
   pMeasPermeateConductivityData->isProcDepMeasEn = TRUE;
   pMeasPermeateConductivityData->isProcDepAverageEn = FALSE;
   pMeasPermeateConductivityData->isProcDepRecordEn = FALSE;

   pMeasPermeateConductivityData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* DIL Edi resistivity init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_RESISTIVITY))
   {
      DIL_COND_MEASURE_CONFIG measConf;
      UINT16 wValue;
      UINT8 bValue;

      ret = dil_opendevice(DIL_NAME_RESISTIVITY_EDI, &dilEdiResistivityHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Edi resisttivity init");
      // write config
      measConf.bHalfNbrOfSample   = (DIL_UINT8)(MEASURE_EDI_RES_SENSOR_NBR_OF_SAMPLE / 2U);
      measConf.bDutyCycle         = (DIL_UINT8)MEASURE_EDI_RES_SENSOR_DUTY_CYCLE;
      measConf.bInhibitTime       = (DIL_UINT8)MEASURE_EDI_RES_SENSOR_INHIBIT_TIME / 5U;
      measConf.fMinValue          = (DIL_FLOAT)MEASURE_EDI_RES_SENSOR_MIN_VOLTAGE;
      measConf.fMaxValue          = (DIL_FLOAT)MEASURE_EDI_RES_SENSOR_MAX_VOLTAGE;
      measConf.fSwitchToHighRes   = (DIL_FLOAT)MEASURE_EDI_RES_SENSOR_SWITCH_TO_HIGH_VOLTAGE;
      measConf.fSwitchToLowRes    = (DIL_FLOAT)MEASURE_EDI_RES_SENSOR_SWITCH_TO_LOW_VOLTAGE;
      ret = dil_ioctldevice(dilEdiResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_CONFIG,
                            (DIL_PVOID)&measConf,
                            sizeof(measConf),
                            NULL );
      /* selection upon frequency config */
      if(SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
      {
         wValue = MEASURE_EDI_RES_SENSOR_50HZ_FREQ;
      }
      else
      {
         wValue = MEASURE_EDI_RES_SENSOR_60HZ_FREQ;
      }
      ret = dil_ioctldevice(dilEdiResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_SET_FREQUENCY,
                            (DIL_PVOID)wValue,
                            sizeof(DIL_UINT16),
                            NULL );

      bValue = 1U;
      ret = dil_ioctldevice(dilEdiResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_ENABLE,
                            (DIL_PVOID)bValue,
                            sizeof(DIL_BOOL),
                            NULL );
   }
   /* Read Elix conduc SP in OD */
   DO_READ(pMeasEdiResistivityData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.ElixResistivitySp);
   pMeasEdiResistivityData->dwNbSplMeasure = 0u;
   pMeasEdiResistivityData->fMeasAccumulator = 0.0f;
   pMeasEdiResistivityData->pStopRecordFunc = NULL;
   pMeasEdiResistivityData->isProcDepMeasEn = TRUE;
   pMeasEdiResistivityData->isProcDepAverageEn = FALSE;
   pMeasEdiResistivityData->isProcDepRecordEn = FALSE;

   pMeasEdiResistivityData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* DIL Distribution resisttivity init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_RESISTIVITY))
   {
      DIL_COND_MEASURE_CONFIG measConf;
      UINT16 wValue;
      UINT8 bValue;

      ret = dil_opendevice(DIL_NAME_RESISTIVITY_DISTRIBUTION, &dilDistributionResistivityHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Distribution resisttivity init");

      // write config
      measConf.bHalfNbrOfSample   = (DIL_UINT8)(MEASURE_DIST_RES_SENSOR_NBR_OF_SAMPLE / 2U);
      measConf.bDutyCycle         = (DIL_UINT8)MEASURE_DIST_RES_SENSOR_DUTY_CYCLE ;
      measConf.bInhibitTime       = (DIL_UINT8)MEASURE_DIST_RES_SENSOR_INHIBIT_TIME / 5U;
      measConf.fMinValue          = (DIL_FLOAT)MEASURE_DIST_RES_SENSOR_MIN_VOLTAGE;
      measConf.fMaxValue          = (DIL_FLOAT)MEASURE_DIST_RES_SENSOR_MAX_VOLTAGE;
      measConf.fSwitchToHighRes   = (DIL_FLOAT)MEASURE_DIST_RES_SENSOR_SWITCH_TO_HIGH_VOLTAGE;
      measConf.fSwitchToLowRes    = (DIL_FLOAT)MEASURE_DIST_RES_SENSOR_SWITCH_TO_LOW_VOLTAGE;
      ret = dil_ioctldevice(dilDistributionResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_CONFIG,
                            (DIL_PVOID)&measConf,
                            sizeof(measConf),
                            NULL );

      /* selection upon frequency config */
      if(SYSTEM_50_HERTZ == SYS_GetPowerSupplyFreq())
      {
         wValue = MEASURE_DIST_RES_SENSOR_50HZ_FREQ;
      }
      else
      {
         wValue = MEASURE_DIST_RES_SENSOR_60HZ_FREQ;
      }
      ret = dil_ioctldevice(dilDistributionResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_SET_FREQUENCY,
                            (DIL_PVOID)wValue,
                            sizeof(DIL_UINT16),
                            NULL );

      bValue = 1U;
      ret = dil_ioctldevice(dilDistributionResistivityHdl,
                            (DIL_UINT32)IOCTL_CMD_COND_ENABLE,
                            (DIL_PVOID)bValue,
                            sizeof(DIL_BOOL),
                            NULL );

   }
   /* Read Distribution conduc SP in OD */
   DO_READ(pMeasDistributionResistivityData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.DistributionResistivitySp);
   pMeasDistributionResistivityData->dwNbSplMeasure = 0u;
   pMeasDistributionResistivityData->fMeasAccumulator = 0.0f;
   pMeasDistributionResistivityData->pStopRecordFunc = NULL;
   pMeasDistributionResistivityData->isProcDepMeasEn = TRUE;
   pMeasDistributionResistivityData->isProcDepAverageEn = FALSE;
   pMeasDistributionResistivityData->isProcDepRecordEn = FALSE;

   pMeasDistributionResistivityData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* RO Rejection  : No Dil use */
   /* Read RO Rejection conductivity SP in OD */
   DO_READ(pMeasRoRejectionData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.RoRejectionSp);
   pMeasRoRejectionData->dwNbSplMeasure = 0u;
   pMeasRoRejectionData->fMeasAccumulator = 0.0f;
   pMeasRoRejectionData->pStopRecordFunc = NULL;
   pMeasRoRejectionData->isProcDepMeasEn = TRUE;
   pMeasRoRejectionData->isProcDepAverageEn = FALSE;
   pMeasRoRejectionData->isProcDepRecordEn = FALSE;

   pMeasRoRejectionData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* FCE : No Dil use */
   /* Read FCE conductivity SP in OD */
   DO_READ(pMeasFceData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.FceSp);
   pMeasFceData->dwNbSplMeasure = 0u;
   pMeasFceData->fMeasAccumulator = 0.0f;
   pMeasFceData->pStopRecordFunc = NULL;
   pMeasFceData->isProcDepMeasEn = TRUE;
   pMeasFceData->isProcDepAverageEn = FALSE;
   pMeasFceData->isProcDepRecordEn = FALSE;

   pMeasFceData->bAverageTpdo = TPDO_NOT_MAPPED;
}


/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

