/*!
* @file mes_current.c
*
* @brief Currents acquisition
*
* @details Contain Pump currents acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_current.c 638 2014#$: Filename
* $Rev:: 638                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-11-19 10:26:12 #$: Date of last commit
* $URL: svn:$: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "act_api.h"
#include "aaa_api.h"
#include "measure.h"
#include "mes_can.h"
#include "system_parameters.h"



/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup Current_Measure Pump currents measure
 *  @ingroup Measure
 *  @brief  Pump currents measure acquistion procedures
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#define RO_PUMP_SENSOR_CURRENT_MIN                 (300U)       /*!< Min RO Pump current */
#define DISTRIBUTION_PUMP_SENSOR_CURRENT_MIN       (100U)       /*!< Min Distribution Pump current */

/* Declarations --------------------------------------------------------------*/

/* Codes ---------------------------------------------------------------------*/


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update RO pump current measure every 100ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, Ro pump current read
 *     -# write Ro pump current in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according _Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant RO Pump current in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateRoPumpCurrent(BOOL isSecondBoundary)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[RO_PUMP_CURRENT];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL Pump current read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)RO_PUMP_CURRENT))
   {
      UINT16 wCurrent;

      /* call DIL/ACT */
      wCurrent = ACT_GetPumpCurrent(ACT_TYPE_RO_PUMP);

      if (RO_PUMP_SENSOR_CURRENT_MIN > wCurrent)
      {
         wCurrent = 0U;
      }

      pMeasData->rawMeas.fCurrent = (FLOAT) wCurrent;

      /* write current value in OD */
      DO_WRITE(g_DoRoPump.Current , wCurrent);
   }

   pMeasData->fMeasure=pMeasData->rawMeas.fCurrent;
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Distribution current measure every 100ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL current read
 *     -# Write voltage value in OD
 *     -# Compute Instant Distribution pump current
 *     -# write Instant Distribution pump current in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according /p Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant Distribution pump current in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateDistributionPumpCurrent(BOOL isSecondBoundary)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[DISTRIBUTION_PUMP_CURRENT];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL Pump current read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_PUMP_CURRENT))
   {
      UINT16 wCurrent;

      /* call DIL/ACT */
      wCurrent= ACT_GetPumpCurrent(ACT_TYPE_DISTRIBUTION_PUMP);

      if (DISTRIBUTION_PUMP_SENSOR_CURRENT_MIN > wCurrent)
      {
         wCurrent = 0U;
      }

      pMeasData->rawMeas.fCurrent =  (FLOAT) wCurrent;

      /* write current value in OD */
      DO_WRITE(g_DoDistributionPump.Current , wCurrent);
   }

   pMeasData->fMeasure=pMeasData->rawMeas.fCurrent;
}


/*!
 *  @return None
 *
 *  @brief handle Stop record with average computation
 *
 *  @details Called upon MES_RecordStop trig when start record has been ordered with average computation
 *
 */
static VOID MES_StopRecordAverageCurrentFunc(MeasData_t* CONST pMeasData )
{
   /* Average computation finalization */
   if ((TRUE == pMeasData->isProcDepAverageEn) &&
       (0 < pMeasData->dwNbSplMeasure))
   {
      /* Send Record containing average */
      pMeasData->fMeasure = pMeasData->fMeasAccumulator / pMeasData->dwNbSplMeasure;
   }

   /* Reset average data */
   pMeasData->fMeasAccumulator = 0.0f;
   pMeasData->dwNbSplMeasure = 0u;
}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details
 */
VOID MES_InitCurrentDrivers(VOID)
{
   MeasData_t* CONST pRoPumpCurrentMeasData = &g_MeasDataArray[RO_PUMP_CURRENT];
   MeasData_t* CONST pDistributionPumpCurrentMeasData = &g_MeasDataArray[DISTRIBUTION_PUMP_CURRENT];

   /* No DIL pump current init as it is self managed in ACT module */

   pRoPumpCurrentMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pRoPumpCurrentMeasData->dwNbSplMeasure = 0u;
   pRoPumpCurrentMeasData->fMeasAccumulator = 0.0f;
   pRoPumpCurrentMeasData->pStopRecordFunc = MES_StopRecordAverageCurrentFunc;
   pRoPumpCurrentMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.RoPumpInletPressure;
   pRoPumpCurrentMeasData->isProcDepMeasEn = TRUE;
   pRoPumpCurrentMeasData->isProcDepAverageEn = FALSE;
   pRoPumpCurrentMeasData->isProcDepRecordEn = FALSE;

   pRoPumpCurrentMeasData->bAverageTpdo = TPDO_RO_PUMP_INLET_PRESSURE;

   /* No DIL pump current init as it is self managed in ACT module */
   pDistributionPumpCurrentMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pDistributionPumpCurrentMeasData->dwNbSplMeasure = 0u;
   pDistributionPumpCurrentMeasData->fMeasAccumulator = 0.0f;
   pDistributionPumpCurrentMeasData->pStopRecordFunc = MES_StopRecordAverageCurrentFunc;
   pDistributionPumpCurrentMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.RoPressure;
   pDistributionPumpCurrentMeasData->isProcDepMeasEn = TRUE;
   pDistributionPumpCurrentMeasData->isProcDepAverageEn = FALSE;
   pDistributionPumpCurrentMeasData->isProcDepRecordEn = FALSE;

   pDistributionPumpCurrentMeasData->bAverageTpdo = TPDO_RO_PRESSURE;


}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

