/*!
* @file mes_flowrate.c
*
* @brief Flow rate  acquisition
*
* @details Contain Flow rate s acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_flowrate.c 626 201#$: Filename
* $Rev:: 626                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-11-06 11:44:31 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_flowrate.c $: URL of this file
* @endcond
*
*/




/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>

#include "boardcfg.h"
#include "sys_softerr.h"
#include "sys_api.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_flow.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "system_parameters.h"
#include "measure.h"
#include "mvm_api.h"
#include "act_api.h"


/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup Flowrates_Measure Flowrate measure
 *  @ingroup Measure
 *  @brief Flowrate measure acquistion procedures
 *  @{
 */

/* Defines -------------------------------------------------------------------*/
#define FLOWRATE_MES_FREQ      (2.0)    /*!< FLOW RATE measure frequency (Hz) */
#define T25KELVIN              (25.0F + 273.15F)

/* Declarations --------------------------------------------------------------*/
static BOOL IsFlowrateOperational(DIL_HANDLE dilHdl);

static DIL_HANDLE dilDrainFlowRateHdl = NULL; /*!< Drain flow rate DIL handle */
static DIL_HANDLE dilPermeateFlowRateHdl = NULL; /*!< Permeate flow rate DIL handle */
static DIL_HANDLE dilEdiRejectFlowRateHdl = NULL; /*!< Reject flow rate DIL handle */

FLOAT g_fBackPressure;  /*!< 0 in RINSE mode , EdiDeltaP=(RoPressure_RINSING - RoPressure_Tankfilling ) in TANK Filing
                                    To be provide by PROCESS */

BOOL g_isFlushLimited;

/* Codes ---------------------------------------------------------------------*/

/*!
 * @param  fFlowrate  [IN]   FLOAT  <br>
 *                    Measured frequency (DIL)
 *
 * @param  p_VolCounterData  [IN]   MeasVolumeCounterData_t  <br>
 *                    Volume data context pointer
 *
 * @return None
 *
 * @brief  Compute volume for a given flowrate provided in mL/hour
 *             \b Volume (mL/hour) = \b fFlowrate (L/h) x FLOWRATE_MES_FREQ x 1000 / \b 3600
 *              Unit :  mL/hour.
 *
 * @details   Flowate is accumulated over\a VOLUME_ACCUMULATE_PERIOD period (60sec)
 *            Once accumulation period  ends, Volume is compute.
 *            It is not proposed to check overflow count check as a DOUBLE can store
 *            2^52 L. This reprensents  more 1 E6 Years use for a 400 L/H flowrate.
 *           (Using a FLOAT could only store 20 hours).
 *     \f[
 *   Volume = \frac{ Flowrate \cdot 1000}{3600 \cdot Freq_FR }
 *  \f]
  */
static VOID ComputeVolume(FLOAT fFlowrate, MeasVolumeCounterData_t *p_VolCounterData)
{
   if((TRUE == p_VolCounterData->isFlowCntRunning) &&
      ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fFlowrate))
   {
      p_VolCounterData->dCnt += (((DOUBLE)fFlowrate * 1000.0) / (3600.0 * FLOWRATE_MES_FREQ));
   }
}


/*!
 * @param  fFreq  [IN]   FLOAT  <br>
 *                    Measured frequency (DIL)
 *
 * @param  kFactor  [IN]   FLOAT  <br>
 *                    K Factor
 *
 * @return flowrate [OUT] FLOAT
 *
 * @brief  Compute flow rate according K Factor
 *             \b Flow rate = \b fFreq x 3600 / \b kFactor
 *              Unit :  L/h
 *
 * @details   <br>
 *       \f$ Flowrate = \frac{Freq \times 3600}{KFactor} \f$
 *
 */
static FLOAT ComputeFlowRate(FLOAT fFreq, UINT16 kFactor)
{
   FLOAT fFlowRate;
   if(0.0f == fFreq)
   {
      fFlowRate = 0.0F;
   }
   else if (0u == kFactor)
   {
      fFlowRate = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   else
   {
      fFlowRate = ((fFreq * 3600.0F) / (FLOAT)kFactor);
   }
   return (fFlowRate);
}


/*!
 * @param  fPermeateFlowRate [IN] FLOAT  <br>
 *          Permeate flowrate (L/h)
 *
 * @param  fRoTemperature [IN] FLOAT  <br>
 *         Ro teperature (C)
 *
 * @param  fRoPresssure [IN] FLOAT  <br>
 *         Ro pressure (bar)
 *
 * @return RoPermeability [OUT] FLOAT
 *
 * @brief  Compute RO Permeability
 *
 * @details    It is expected \p Process  provide  #g_fBackPressure  with  #MES_SetRoBackPressure API procedure <br>
 *             before use because Measure has no information on process running <br>
 *              <br>
 * \f$
 * RoPermeability = ( P_{Ro} - P_{RoBack} - P_{Osmotic} ) \cdot \frac { e^{ const \cdot \left( \frac{1}{T_{nominal}(K)} - \frac{1}{T_{Ro}(K)} \right)  } } { Fr_{Permeate} }\\
 * \\
 * P_{Ro} $ : RO Pressure.$\\
 *  \\
 * P_{Osmotic} $ : Osmotic pressure.$\\
 * = \frac { ConductivityTC_{RoFeed} }{121 \cdot R \cdot T_{Ro}(K) \cdot i \cdot 10^{-5} } \\
 * \\
 * P_{RoBack} $ : RO Back pressure.$\\
 * = 0 (bar) $ if rinsing mode.$\\
 * = DeltaP_{Ed} - P_{Ro} (bars) $ if tank filling mode.$\\
 * \\
 * const : \\
 * = 2640 $ if temp is over  25 $^\circ Celsius\\
 * = 3020 $ if temp is below 25 $^\circ Celsius\\
 * \\
 * R=8.314\\
 * i=2\\
 * \\
 * T_{Ro} ( ^\circ Kelvin ) $ : RO Temperature.$\\
 * \\
 * Fr_{Permeate} $ : Permeate flowrate.$\\
 * \f$
 */
static FLOAT ComputeRoPermeability(FLOAT fPermeateFlowRate, FLOAT fRoTemperature, FLOAT fRoPresssure)
{
   FLOAT fRoPermeability;
   FLOAT fOsmoticPressure;
   FLOAT fRoFeedConductivityTC = g_MeasDataArray[FEED_CONDUCTIVITY_1].fMeasure;
   FLOAT fSuperCoef;
   FLOAT fKelvinRoTemperature;

   if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fRoFeedConductivityTC) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fRoFeedConductivityTC) &&
       ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != fRoTemperature) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fRoTemperature) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fRoPresssure) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateFlowRate))
   {
      fKelvinRoTemperature = (273.15F + fRoTemperature);

      /* Compute Osmotic pressure */
      fOsmoticPressure = (((((fRoFeedConductivityTC / 121.0F) * 8.314F) * fKelvinRoTemperature) * 2.0F) * 1.0e-5F) ;

      /* Compute RO Permeability */
      fRoPermeability = ((fRoPresssure - g_fBackPressure) - fOsmoticPressure);
      if(25.0F < fRoTemperature)
      {
         fSuperCoef = 2640.0F;
      }
      else
      {
         fSuperCoef = 3020.0F;
      }
      fRoPermeability *= expf(fSuperCoef * ((1.0F / T25KELVIN) - (1.0F / fKelvinRoTemperature)));
      fRoPermeability /= fPermeateFlowRate;
      fRoPermeability = (1.0f / fRoPermeability);
   }
   else
   {
      fRoPermeability = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }

   return (fRoPermeability);
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @param fPermeateFlowRate [IN] FLOAT <br>
 *                   Permeate flow rate used to determine Ro permeability
 *
 *  @return void
 *
 *  @brief Update  Ro permeability every 500ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write RO permeability flowrate in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 */
static VOID UpdateRoPermeability(BOOL isSecondBoundary,FLOAT fPermeateFlowRate)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[RO_PERMEABILITY];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Raw measure pressure valid */

   pMeasData->fMeasure = ComputeRoPermeability(fPermeateFlowRate,
                                               g_MeasDataArray[PERMEATE_TEMPERATURE].fMeasure,
                                               g_MeasDataArray[RO_PRESSURE].fMeasure
                                               );
   /*No Alarms to check */

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         /* Accumuate Instant Flow rate  */
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0U < pMeasData->dwNbSplMeasure)
         {
            pMeasData->fMeasure = (pMeasData->fMeasAccumulator / (FLOAT)pMeasData->dwNbSplMeasure);
            fMeasure = (((pMeasData->fMeasure * 1000.0f) / 60.0f) + 0.5f);

            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoMembranePermeability, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoMembranePermeability, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_RO_PERMEABILITY_FLOWRATE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if ((TRUE == pMeasData->isProcDepAverageEn) &&
            ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
   {
      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do Nothing */
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @param fPermeateFlowRate [IN] FLOAT <br>
 *                   Permeate flow rate used to determine
 *
 *  @return void
 *
 *  @brief Update RO recovery flow rate every 500ms
 *
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# compute RO recovery = Permeate flow rate / ( Permeate flow rate + Drain flow rate )
 *     -# check \p process dependnat alarms
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write RO recovery flowrate in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated. <br>
 *    <br>
 *    \f$ RO recovery = \frac { 100 \cdot Flowrate_{Permeate} }{Flowrate_{Permeate} + Flowrate_{Drain}} \f$
 *
 */
static VOID UpdateRoRecovery(BOOL isSecondBoundary,FLOAT fPermeateFlowRate)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[RO_RECOVERY];
   FLOAT fDrainFlowRate = g_MeasDataArray[DRAIN_FLOWRATE].fMeasure;

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* compute Ro Recovery */
   if(((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateFlowRate) &&
      ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fDrainFlowRate))
   {
      if (0u == fPermeateFlowRate)
      {
         pMeasData->fMeasure = 0.0f;
      }
      else if (0.0f < (fPermeateFlowRate + fDrainFlowRate))
      {
         /* compute RO recovery = Permeate flow rate / ( Permeate flow rate + Drain flow rate ) */
         pMeasData->fMeasure = ((100.0f * fPermeateFlowRate) / (fPermeateFlowRate + fDrainFlowRate)) ;
      }
      else
      {
         pMeasData->fMeasure = 0.0f;
      }
   }
   else
   {
      /* Invalid value */
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }

   /* Process dependant alarms check */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_RO_RECOVERY_SP_MAX].isEnable)
   {
      /* Check RO recovery > Set point high  */
      if (pMeasData->fMeasure
        > ((FLOAT)MVM_GetRegulationRoRecoverySetpoint() + (FLOAT)ALARM_RO_RECOVERY_REGULATION_HYSTERESIS_SETPOINT_UP))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_RO_RECOVERY_SP_MAX);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_RO_RECOVERY_SP_MAX);
         /* Alarm is cleared by an user acknowledgement */
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_RO_RECOVERY_SP_MIN].isEnable)
   {
      /* Check RO recovery < Set point low  */
      if (pMeasData->fMeasure
        < ((FLOAT)MVM_GetRegulationRoRecoverySetpoint() - (FLOAT)ALARM_RO_RECOVERY_REGULATION_HYSTERESIS_SETPOINT_DOWN))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_RO_RECOVERY_SP_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_RO_RECOVERY_SP_MIN);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         /* Accumuate Instant Flow rate  */
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            fMeasure = (((pMeasData->fMeasAccumulator * 100.0f) / (FLOAT)pMeasData->dwNbSplMeasure) + 0.5f); // Rec * 100

            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoRecovery, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoRecovery, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_RO_RECOVERY);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if ((TRUE == pMeasData->isProcDepAverageEn) &&
            ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
   {
      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do nothing */
   }
}


/*!
 *  @param fPermeateFlowRate [IN] FLOAT <br>
 *                   Permeate flow rate used to determine
 *
 *  @return void
 *
 *  @brief Update Progard flowrate every 500ms
 *             \f$ Progard flowrate = Permeate Flowrate + Drain Flowrate \f$
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# compute Progard delta P flow rate  = Permeate flow rate + Drain flow rate
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant Progard delata P flowrate in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 */
static VOID UpdateProgardDeltaPFlowRate(FLOAT fPermeateFlowRate)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[PROGARD_DELTA_P_FLOWRATE];
   FLOAT fDrainFlowRate = g_MeasDataArray[DRAIN_FLOWRATE].fMeasure;

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Check Process dependant Average */
   if ((TRUE == pMeasData->isProcDepAverageEn) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateFlowRate) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fDrainFlowRate))
   {
      /* compute Progard delta P flow rate  = Permeate flow rate + Drain flow rate */
      pMeasData->fMeasure = fPermeateFlowRate + fDrainFlowRate;

      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @param fEdiRejectFlowRate [IN] FLOAT <br>
 *                   Edi flow rate used to determine EDI recovery
 *
 *  @return void
 *
 *  @brief Update Drain flow rate every 500ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant Edi recovery  flowrate in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     -# Compute Volume <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.

 */
static VOID UpdateEdiProductFlowRate(BOOL isSecondBoundary,FLOAT fEdiRejectFlowRate)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[EDI_PRODUCTION_FLOWRATE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Raw measure pressure valid */
   if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fEdiRejectFlowRate) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != g_MeasDataArray[PERMEATE_FLOWRATE].fMeasure))
   {
      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_PERMEATE_SV_VALVE_1))
      {
         pMeasData->fMeasure = g_MeasDataArray[PERMEATE_FLOWRATE].fMeasure - fEdiRejectFlowRate;
      }
      else
      {
         pMeasData->fMeasure = 0.0f;
      }
   }
   else
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   DO_WRITE(g_DoEdiProductFlowRate.InstantFlowRate, pMeasData->fMeasure);

   /*No Alarms to check */

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         /* Accumuate Instant Flow rate  */
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            /* Conversion to ml/min */
            fMeasure = ((((pMeasData->fMeasAccumulator / (FLOAT)pMeasData->dwNbSplMeasure) * 1000.0f) / 60.0f) + 0.5f);

            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.EdiProductFlowrate, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.EdiProductFlowrate, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_EDI_PRODUCTION_FLOWRATE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if ((TRUE == pMeasData->isProcDepAverageEn) &&
            ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
   {
      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do nothing */
   }

   /* Compute Volume */
   ComputeVolume(pMeasData->fMeasure, &g_MeasVolumeCounterData[MES_VOL_PRODUCE]);
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @param fEdiProductFlowRate [IN] FLOAT <br>
 *                   Edi Product flow rate used to determine EDI recovery.
 *
 *  @return void
 *
 *  @brief Update EDI Recovery flow rate percentage every 500ms.
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant Edi recovery  flowrate in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 * @note  use Permeate flow rate measure
 *
 * @details      <br>
 *   \f$ EDI_Recovery (\%) = \frac{ FR_EdiProduction }{FR_Permeate} \cdot 100 \f$
 *
 */
static VOID UpdateEdiRecovery(BOOL isSecondBoundary, FLOAT fEdiProductFlowRate)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[EDI_RECOVERY_FLOWRATE];
   FLOAT fPermeateFlowRate = g_MeasDataArray[PERMEATE_FLOWRATE].fMeasure;

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* Raw measure pressure valid */
   if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fPermeateFlowRate) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fEdiProductFlowRate))
   {
      if (0.0f < fPermeateFlowRate)
      {
         pMeasData->fMeasure = ((fEdiProductFlowRate * 100.0f) / fPermeateFlowRate);
      }
      else
      {
         pMeasData->fMeasure = 0.0f;
      }
      if(100.0f < pMeasData->fMeasure)
      {
         pMeasData->fMeasure=100.0f;
      }
      else if(0.0f > pMeasData->fMeasure)
      {
         pMeasData->fMeasure=0.0f;
      }
      else
      {
         /* Measure is in range */
      }
   }
   else
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   DO_WRITE(g_DoEdiProductFlowRate.InstantRecovery, pMeasData->fMeasure);

   /*No Alarms to check */

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Flow rate  */
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            fMeasure = (((pMeasData->fMeasAccumulator * 100.0f) / (FLOAT)pMeasData->dwNbSplMeasure) + 0.5f);  /* Record in *100 */

            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.EdiRecovery, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.EdiRecovery, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_EDI_RECOVERY_FLOWRATE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if ((TRUE == pMeasData->isProcDepAverageEn) &&
            ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
   {
      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do nothing */
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Drain flow rate every 500ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL flow rate read
 *     -# Write Freq value in OD
 *     -# Compute Instant Drain flowrate
 *     -# write Instant Feed Pressure in OD
 *     -# Compute Volume <br>
 */
VOID MES_UpdateDrainFlowrate(BOOL isSecondBoundary)
{
   FLOAT rawFreq = 0.0f;      /*!< temp variable.*/
   UINT16 wKFactor=0u;      /*!< K factor */
   FLOAT fCalibFlowrate;   /*!< Calibrated computed flowrate */
   MeasData_t* CONST pMeasData = &g_MeasDataArray[DRAIN_FLOWRATE];
   NOT_USED_PARAM(isSecondBoundary);
   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DRAIN_FLOWRATE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilDrainFlowRateHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_GET_VALUE ,
                            (DIL_LPVOID)&rawFreq, sizeof(rawFreq), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate  call");
      /* Save Raw data */
      pMeasData->rawMeas.fFreq = rawFreq;

      /* write freq value in OD */
      DO_WRITE(g_DoDrainFlowRate.FrequenceValue , pMeasData->rawMeas.fFreq);
   }

   /* Standart measure computation */
   /* ============================ */
   /* Read RAW Flow rate in OD*/
   DO_READ(pMeasData->rawMeas.fFreq , g_DoDrainFlowRate.FrequenceValue);
   /* Read RAW  KFactor4 in OD*/
   DO_READ(wKFactor,g_DoDrainFlowRate.CalibKfactor4);
   /* Compute Instant Flow rate */
   pMeasData->fMeasure = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoDrainFlowRate.InstantFlowRate, pMeasData->fMeasure);
   /* Compute Volume */
   ComputeVolume(pMeasData->fMeasure, &g_MeasVolumeCounterData[MES_VOL_DRAIN]);
   /* Calibration measure computation */
   /* =============================== */
   /* Read RAW  KFactor4 in OD*/
   DO_READ(wKFactor,g_DoDrainFlowRate.CalibAdjCalibKfactor);
   /* Compute Instant Flow rate */
   fCalibFlowrate = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoDrainFlowRate.CalibAdjInstantFlowRate, fCalibFlowrate);

   /* Compute average */
   if ((TRUE == pMeasData->isProcDepAverageEn) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure))
   {
      /* Accumuate Instant Instant Flow rate  */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update EDI Reject flow rate every 500ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL flow rate read
 *     -# Write freq value in OD
 *     -# Compute Instant Drain flowrate
 *     -# write Instant Feed Pressure in OD
 *     -# Call EDI production specific treatement
 *     -# Call EDI Recovery specific treatement
 */
VOID MES_UpdateEDIRejectFlowrate(BOOL isSecondBoundary)
{
   FLOAT rawFreq = 0.0f;      /*!< temp variable.*/
   UINT16 wKFactor=0u;      /*!< K factor */
   FLOAT fCalibFlowrate;   /*!< Calibrated computed flowrate */
   MeasData_t* CONST pMeasData = &g_MeasDataArray[EDI_REJECT_FLOWRATE];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_REJECT_FLOWRATE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilEdiRejectFlowRateHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_GET_VALUE ,
                            (DIL_LPVOID)&rawFreq, sizeof(rawFreq), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate  call");
      /* Save Raw data */
      pMeasData->rawMeas.fFreq = rawFreq;

      /* write voltage value in OD */
      DO_WRITE(g_DoEdiRejectFlowRate.FrequenceValue , pMeasData->rawMeas.fFreq);
   }
   /* Standart measure computation */
   /* ============================ */
   /* Read RAW Flow rate in OD*/
   DO_READ(pMeasData->rawMeas.fFreq , g_DoEdiRejectFlowRate.FrequenceValue);
   /* Read KFactor5 in OD*/
   DO_READ(wKFactor,g_DoEdiRejectFlowRate.CalibKfactor1);
   /* Compute Instant Flow rate */
   pMeasData->fMeasure = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoEdiRejectFlowRate.InstantFlowRate, pMeasData->fMeasure);

   /* Calibration measure computation */
   /* =============================== */
   /* Read RAW  KFactor4 in OD*/
   DO_READ(wKFactor,g_DoEdiRejectFlowRate.CalibAdjCalibKfactor);
   /* Compute Instant Flow rate */
   fCalibFlowrate = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoEdiRejectFlowRate.CalibAdjInstantFlowRate, fCalibFlowrate);

   /* In the end, */
   /* -    Call EDI production specific treatement */
   UpdateEdiProductFlowRate(isSecondBoundary, pMeasData->fMeasure);
   /* -    Call EDI recovery specific treatement */
   UpdateEdiRecovery(isSecondBoundary, g_MeasDataArray[EDI_PRODUCTION_FLOWRATE].fMeasure);
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 500ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Permeate flow rate every 500ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Flow rate read
 *     -# Write voltage value in OD
 *     -# Compute Instant Flow rate
 *     -# write Instant Flow rate in OD
 *     -# Check mode Dependant Alarms (according _Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant RO Pump Inlet Pressure in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     -# Compute Volume <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 * @warning it is expected to call MES_UpdatePermeateFlowrate after MES_UpdateDrainFlowrate
 *        to avoid using wrong value when updating Ro pump flow rate
 */
VOID MES_UpdatePermeateFlowrate(BOOL isSecondBoundary)
{
   FLOAT rawFreq = 0.0f;      /*!< temp variable.*/
   UINT16 wKFactor=0u;      /*!< K factor */
   FLOAT fCalibFlowrate;   /*!< Calibrated computed flowrate */
   FLOAT fFeedFlowrate;    /*!< Feed Flowrate = Permeate + Drain Flowrates */
   FLOAT fFeedFlowrateMin;
   FLOAT AlertPermeateFlowRateMaxThreshold;
   FLOAT AlertPermeateFlowRateMinThreshold;
   FLOAT AlertEdiFlowRateMaxThreshold;
   FLOAT AlertEdiFlowRateMinThreshold;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[PERMEATE_FLOWRATE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      /* Check to ensure UpdateTapFeedPressure or progard delat_P is not requested  */
      ASSERT((FALSE == g_MeasDataArray[RO_PERMEABILITY].isProcDepMeasEn
            && FALSE == g_MeasDataArray[RO_RECOVERY].isProcDepMeasEn
            && FALSE == g_MeasDataArray[RO_PUMP_FLOWRATE].isProcDepMeasEn
            && FALSE == g_MeasDataArray[PROGARD_DELTA_P_FLOWRATE].isProcDepMeasEn ),
            "Can't measure some flow rates if permeate flow rate measure disable");
      return;
   }

   /* If DIL not bypassed, DIL flow rate read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_FLOWRATE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilPermeateFlowRateHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_GET_VALUE ,
                            (DIL_LPVOID)&rawFreq, sizeof(rawFreq), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate  call");
      /* Save Raw data */
      pMeasData->rawMeas.fFreq = rawFreq;

      /* write voltage value in OD */
      DO_WRITE(g_DoPermeateFlowRate.FrequenceValue , pMeasData->rawMeas.fFreq);
   }

   /* Standart measure computation */
   /* ============================ */
   /* Read RAW Feed Pressure in OD*/
   DO_READ(pMeasData->rawMeas.fFreq , g_DoPermeateFlowRate.FrequenceValue);
   /* Read RAW Feed Pressure in OD*/
   switch(g_bSysSelectedKFactor)
   {
      case 5u:ERROR("Wrong cartridge selection in Permeate flow meter K Factor(5)");break;
      default:
      case 1u:DO_READ(wKFactor,g_DoPermeateFlowRate.CalibKfactor1);break;
      case 2u:DO_READ(wKFactor,g_DoPermeateFlowRate.CalibKfactor2);break;
      case 3u:DO_READ(wKFactor,g_DoPermeateFlowRate.CalibKfactor3);break;
      case 4u:DO_READ(wKFactor,g_DoPermeateFlowRate.CalibKfactor4);break;

   }

   /* Compute Instant Feed pressure */
   pMeasData->fMeasure = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoPermeateFlowRate.InstantFlowRate, pMeasData->fMeasure);

   /* Calibration measure computation */
   /* =============================== */
   /* Read RAW  KFactor4 in OD*/
   DO_READ(wKFactor,g_DoPermeateFlowRate.CalibAdjCalibKfactor);
   /* Compute Instant Flow rate */
   fCalibFlowrate = ComputeFlowRate((FLOAT)pMeasData->rawMeas.fFreq, wKFactor);
   /* Write Instant Flow rate in OD */
   DO_WRITE(g_DoPermeateFlowRate.CalibAdjInstantFlowRate, fCalibFlowrate);


   /* Compute Feed Flowrate for alarm check */
   if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure) &&
       ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != g_MeasDataArray[DRAIN_FLOWRATE].fMeasure))
   {
      fFeedFlowrate = pMeasData->fMeasure + g_MeasDataArray[DRAIN_FLOWRATE].fMeasure;
   }
   else
   {
      fFeedFlowrate = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }

   /* Check Alarms */
   /* No Check indepedant Alarms */
   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_FR_STANDBY].isEnable)
   {
      /* Check Permeate and Drain flow rate > 0 alarm */
      if (0.0f < fFeedFlowrate)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_FR_STANDBY);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PERMEATE_FR_STANDBY);
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_FR_FLUSH].isEnable)
   {
      if (TRUE == g_isFlushLimited)
      {
         fFeedFlowrateMin = (FLOAT)ALARM_FEED_FLOWRATE_LIMITED_FLUSH_MIN;
      }
      else
      {
         fFeedFlowrateMin = (FLOAT)ALARM_FEED_FLOWRATE_FLUSH_MIN;
      }

      /* Check Permeate and Drain flow rate < Feed flow rate flush MIN */
      if ((fFeedFlowrateMin > fFeedFlowrate ) ||
          ((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fFeedFlowrate))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_FR_FLUSH);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_PERMEATE_FR_FLUSH);
         /* Alarm is cleared by an user acknowledgement */
      }
   }

   AlertPermeateFlowRateMaxThreshold = ((MVM_GetRegulationPermeateFlowrateSetpoint() * (100.0F + (FLOAT)ALARM_PERMEATE_FLOW_RATE_REGULATION_HYSTERESIS_SETPOINT_UP)) / 100.0F);
   AlertPermeateFlowRateMinThreshold = ((MVM_GetRegulationPermeateFlowrateSetpoint() * (100.0F - (FLOAT)ALARM_PERMEATE_FLOW_RATE_REGULATION_HYSTERESIS_SETPOINT_DOWN)) / 100.0F);
   AlertEdiFlowRateMaxThreshold = ((MVM_GetRegulationPermeateFlowrateSetpoint()* (FLOAT)ALARM_EDI_MAX_FLOWRATE) / 100.0F);
   AlertEdiFlowRateMinThreshold = ((MVM_GetRegulationPermeateFlowrateSetpoint()* (FLOAT)ALARM_EDI_MIN_FLOWRATE) / 100.0F);

   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_FR_SP_MAX].isEnable)
   {
      /* Check Permeate > Set point high  */
      if ((pMeasData->fMeasure > AlertPermeateFlowRateMaxThreshold) &&
          (pMeasData->fMeasure <= AlertEdiFlowRateMaxThreshold))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_FR_SP_MAX);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PERMEATE_FR_SP_MAX);
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_FR_SP_MIN].isEnable)
   {
      /* Check Permeate < Set point low  */
      if ((pMeasData->fMeasure < AlertPermeateFlowRateMinThreshold) &&
          (pMeasData->fMeasure >= AlertEdiFlowRateMinThreshold))
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_FR_SP_MIN);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PERMEATE_FR_SP_MIN);
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_EDI_FR_MAX].isEnable)
   {
      /* Check Permeate  < EDI MAX flow rate */
      if (pMeasData->fMeasure > AlertEdiFlowRateMaxThreshold)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_EDI_FR_MAX);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_PERMEATE_EDI_FR_MAX);
         /* Alarm is cleared by an user acknowledgement */
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PERMEATE_EDI_FR_MIN].isEnable)
   {
      /* Check Permeate  > EDI MIN flow rate */
      if (pMeasData->fMeasure < AlertEdiFlowRateMinThreshold)
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PERMEATE_EDI_FR_MIN);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_PERMEATE_EDI_FR_MIN);
         /* Alarm is cleared by an user acknowledgement */
      }
   }

   /* Check Process dependant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         /* Accumuate Instant Flow rate  */
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            pMeasData->fMeasure = (pMeasData->fMeasAccumulator / (FLOAT)pMeasData->dwNbSplMeasure);
            /* Conversion to ml/min */
            fMeasure = (((pMeasData->fMeasure * 1000.0f) / 60.0f) + 0.5f);

            /* Write averaged Instant Instant Flow rate   in OD REC History field */
            DO_WRITE(g_DoWaterProcess.PermeateFlowrate, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write averaged Instant Instant Flow rate   in OD REC History field */
            DO_WRITE(g_DoWaterProcess.PermeateFlowrate, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_PERMEATE_FLOWRATE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure)
      {
         /* Accumuate Instant Instant Flow rate  */
         pMeasData->fMeasAccumulator += pMeasData->fMeasure;
         pMeasData->dwNbSplMeasure++;
      }
   }
   else
   {
      /* Do Nothing */
   }

   /* Compute Volume */
   ComputeVolume(pMeasData->fMeasure, &g_MeasVolumeCounterData[MES_VOL_PERMEATE]);

   /* In the end, */
   /* -    Call RO permeability specific treatement */
   UpdateRoPermeability(isSecondBoundary, pMeasData->fMeasure);
   /* -    Call RO recovery  specific treatement */
   UpdateRoRecovery(isSecondBoundary, pMeasData->fMeasure);
   /* -    Call Progard deltaP specific treatement */
   UpdateProgardDeltaPFlowRate(pMeasData->fMeasure);
}


/*!
 *  @brief Test flowrate sensor
 *
 *  @details This command requires to switches OFF and ON flowrate sensors to retreive status.\n
 *           Then it switches on flowrate sensors to retreive status.\n
 *           Whatever status is , when  ends, flow rate sensors are ON.
 */
static BOOL IsFlowrateOperational(DIL_HANDLE dilHdl)
{
   DIL_RC   ret;
   UINT8 bStatus = 0u;
   DIL_UINT8 isCmdEn;
   BOOL ReturnValue = FALSE;

   /* call DIL to disable FR */
   isCmdEn = 0u;
   ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL ,
                         (DIL_LPVOID)isCmdEn, sizeof(isCmdEn), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate control call");
   OS_TaskDelay(50u);
   /* Test FR  */
   ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_GET_STATUS ,
                         (DIL_LPVOID)&bStatus, sizeof(bStatus), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate test call");

   if(1u == bStatus)
   {
      /* if disabe and connected -> Error */
      /* Switches ON  Flow rates even if failed to keep   */
      isCmdEn = 1u;
      ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL ,
                            (DIL_LPVOID)isCmdEn, sizeof(isCmdEn), (DIL_LPUINT32)NULL);
      ReturnValue = FALSE;
   }
   else
   {
      /* call DIL to enable FR */
      isCmdEn = 1u;
      ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL ,
                            (DIL_LPVOID)isCmdEn, sizeof(isCmdEn), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate control call");

      /* Test FR  */
      ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_FLOWRATE_GET_STATUS ,
                            (DIL_LPVOID)&bStatus, sizeof(bStatus), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  flow rate test call");

      if(1u == bStatus)
      {
         /* if enabe and short circuit -> Error */
         ReturnValue = FALSE;
      }
      else
      {
         /* if enabe and short circuit -> Error */
         ReturnValue = TRUE;
      }
   }
   return (ReturnValue);
}


/*!
 *  @brief Diagnose flowrate sensors
 *
 *  @details NA
 */
VOID MES_DiagnoseFlowrates(VOID)
{
   UINT32 dwSensDiag;
   // DO_WRITE(g_DoHardwareDiagnostic.SensorsDiagResult,0u);
   /* If DIL not bypassed, DIL flow rate read */
   if(   (FALSE == IS_DIL_READ_BYPASSED((UINT32)DRAIN_FLOWRATE)
      && (FALSE == IsFlowrateOperational(dilDrainFlowRateHdl)) ) )
   {
      /* write voltage value in OD */
      DO_OR(g_DoHardwareDiagnostic.SensorsDiagResult ,((UINT32)1<<(UINT32)DRAIN_FLOWRATE) );
   }
   else
   {
      DO_AND(g_DoHardwareDiagnostic.SensorsDiagResult,~(((UINT32)1<<(UINT32)DRAIN_FLOWRATE)));
   }

   /* If DIL not bypassed, DIL flow rate read */
   if(   (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_FLOWRATE))
      && (FALSE == IsFlowrateOperational(dilPermeateFlowRateHdl)) )
   {
      /* write voltage value in OD */
      DO_OR(g_DoHardwareDiagnostic.SensorsDiagResult ,((UINT32)1<<(UINT32)PERMEATE_FLOWRATE) );
   }
   else
   {
      DO_AND(g_DoHardwareDiagnostic.SensorsDiagResult,~(((UINT32)1<<(UINT32)PERMEATE_FLOWRATE)));
   }

   /* If DIL not bypassed, DIL flow rate read */
   if(   (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_REJECT_FLOWRATE))
      && (FALSE == IsFlowrateOperational(dilEdiRejectFlowRateHdl)) )
   {
      /* write voltage value in OD */
      DO_OR(g_DoHardwareDiagnostic.SensorsDiagResult ,((UINT32)1<<(UINT32)EDI_REJECT_FLOWRATE) );
   }
   else
   {
      DO_AND(g_DoHardwareDiagnostic.SensorsDiagResult,~(((UINT32)1<<(UINT32)EDI_REJECT_FLOWRATE)));
   }

   DO_READ(dwSensDiag,g_DoHardwareDiagnostic.SensorsDiagResult);
   SYSLOG(INFO,"M:Diag=%08Xh",dwSensDiag);
}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details NA
 */
VOID MES_InitFlowrateDrivers(VOID)
{
   MeasData_t* CONST pDrainFlowRateMeasData = &g_MeasDataArray[DRAIN_FLOWRATE];
   MeasData_t* CONST pPermeateRateMeasData = &g_MeasDataArray[PERMEATE_FLOWRATE];
   MeasData_t* CONST pEdiRejectRateMeasData = &g_MeasDataArray[EDI_REJECT_FLOWRATE];

   MeasData_t* CONST pRoPermeabilityFlowRateMeasData = &g_MeasDataArray[RO_PERMEABILITY];
   MeasData_t* CONST pRoRecoveryFlowRateMeasData = &g_MeasDataArray[RO_RECOVERY];
   MeasData_t* CONST pRoPumpFlowRateMeasData = &g_MeasDataArray[RO_PUMP_FLOWRATE];
   MeasData_t* CONST pProgardDeltaPFlowRateMeasData = &g_MeasDataArray[PROGARD_DELTA_P_FLOWRATE];
   MeasData_t* CONST pEdiProductionFlowRateMeasData = &g_MeasDataArray[EDI_PRODUCTION_FLOWRATE];
   MeasData_t* CONST pEdiRecoveryFlowRateMeasData = &g_MeasDataArray[EDI_RECOVERY_FLOWRATE];

   DIL_RC   ret;
   UINT8    bCmd=1;
   /*! Drain flow rate   init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DRAIN_FLOWRATE))
   {
      ret = dil_opendevice(DIL_NAME_FLOWRATE_DRAIN, &dilDrainFlowRateHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL Drain flow rate init");
      ret=dil_ioctldevice(  dilDrainFlowRateHdl,
                            (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL,
                            (DIL_PVOID)bCmd,
                            sizeof(bCmd),
                            NULL );
      ASSERT(DIL_GOOD == ret,"Error in DIL Drain flow rate control");
   }
   pDrainFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pDrainFlowRateMeasData->dwNbSplMeasure = 0u;
   pDrainFlowRateMeasData->fMeasAccumulator = 0.0f;
   pDrainFlowRateMeasData->pStopRecordFunc = NULL;
   pDrainFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pDrainFlowRateMeasData->isProcDepMeasEn = TRUE;
   pDrainFlowRateMeasData->isProcDepAverageEn = FALSE;
   pDrainFlowRateMeasData->isProcDepRecordEn = FALSE;

   pDrainFlowRateMeasData->bAverageTpdo = TPDO_NOT_MAPPED;

   /*! permeate flow rate  init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_FLOWRATE))
   {
      ret = dil_opendevice(DIL_NAME_FLOWRATE_PERMEATE, &dilPermeateFlowRateHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL Permeate flow rate init");
      ret=dil_ioctldevice(  dilPermeateFlowRateHdl,
                            (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL,
                            (DIL_PVOID)bCmd,
                            sizeof(bCmd),
                            NULL );
      ASSERT(DIL_GOOD == ret,"Error in DIL Permeate flow rate control");
   }
   /* Read set point in DO */
   pPermeateRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pPermeateRateMeasData->dwNbSplMeasure = 0u;
   pPermeateRateMeasData->fMeasAccumulator = 0.0f;
   pPermeateRateMeasData->pStopRecordFunc = NULL;
   pPermeateRateMeasData->pDoRecord = (VOID*)NULL;
   pPermeateRateMeasData->isProcDepMeasEn = TRUE;
   pPermeateRateMeasData->isProcDepAverageEn = FALSE;
   pPermeateRateMeasData->isProcDepRecordEn = FALSE;

   pPermeateRateMeasData->bAverageTpdo = TPDO_PERMEATE_FLOWRATE;

   /*! Edi reject flow rate   init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_REJECT_FLOWRATE))
   {
      ret = dil_opendevice(DIL_NAME_FLOWRATE_DISTRIBUTION, &dilEdiRejectFlowRateHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL Distribution flow rate init");
      ret=dil_ioctldevice(  dilEdiRejectFlowRateHdl,
                            (DIL_UINT32)IOCTL_CMD_FLOWRATE_CONTROL,
                            (DIL_PVOID)bCmd,
                            sizeof(bCmd),
                            NULL );
      ASSERT(DIL_GOOD == ret,"Error in DIL Distribution flow rate control");
   }
   pEdiRejectRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;  /* not used */
   pEdiRejectRateMeasData->dwNbSplMeasure = 0u;
   pEdiRejectRateMeasData->fMeasAccumulator = 0.0f;
   pEdiRejectRateMeasData->pStopRecordFunc = NULL;
   pEdiRejectRateMeasData->pDoRecord = (VOID*)NULL;
   pEdiRejectRateMeasData->isProcDepMeasEn = TRUE;
   pEdiRejectRateMeasData->isProcDepAverageEn = FALSE;
   pEdiRejectRateMeasData->isProcDepRecordEn = FALSE;

   pEdiRejectRateMeasData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* RO_PERMEABILITY flow rate */
   pRoPermeabilityFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;  /* not used */
   pRoPermeabilityFlowRateMeasData->dwNbSplMeasure = 0u;
   pRoPermeabilityFlowRateMeasData->fMeasAccumulator = 0.0f;
   pRoPermeabilityFlowRateMeasData->pStopRecordFunc = NULL;
   pRoPermeabilityFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pRoPermeabilityFlowRateMeasData->isProcDepMeasEn = TRUE;
   pRoPermeabilityFlowRateMeasData->isProcDepAverageEn = FALSE;
   pRoPermeabilityFlowRateMeasData->isProcDepRecordEn = FALSE;

   pRoPermeabilityFlowRateMeasData->bAverageTpdo = TPDO_RO_PERMEABILITY_FLOWRATE;

   /* RO_Recovery flow rate */
   /* Get Set poinbt from Production Process data */
   pRoRecoveryFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pRoRecoveryFlowRateMeasData->dwNbSplMeasure = 0u;
   pRoRecoveryFlowRateMeasData->fMeasAccumulator = 0.0f;
   pRoRecoveryFlowRateMeasData->pStopRecordFunc = NULL;
   pRoRecoveryFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pRoRecoveryFlowRateMeasData->isProcDepMeasEn = TRUE;
   pRoRecoveryFlowRateMeasData->isProcDepAverageEn = FALSE;
   pRoRecoveryFlowRateMeasData->isProcDepRecordEn = FALSE;

   pRoRecoveryFlowRateMeasData->bAverageTpdo = TPDO_RO_RECOVERY;

   /* RO Pump flow rate */
   pRoPumpFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pRoPumpFlowRateMeasData->dwNbSplMeasure = 0u;
   pRoPumpFlowRateMeasData->fMeasAccumulator = 0.0f;
   pRoPumpFlowRateMeasData->pStopRecordFunc = NULL;
   pRoPumpFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pRoPumpFlowRateMeasData->isProcDepMeasEn = TRUE;
   pRoPumpFlowRateMeasData->isProcDepAverageEn = FALSE;
   pRoPumpFlowRateMeasData->isProcDepRecordEn = FALSE;

   pRoPumpFlowRateMeasData->bAverageTpdo = TPDO_NOT_MAPPED;

   /* Progard delata p flow rate */
   pProgardDeltaPFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pProgardDeltaPFlowRateMeasData->dwNbSplMeasure = 0u;
   pProgardDeltaPFlowRateMeasData->fMeasAccumulator = 0.0f;
   pProgardDeltaPFlowRateMeasData->pStopRecordFunc = NULL;
   pProgardDeltaPFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pProgardDeltaPFlowRateMeasData->isProcDepMeasEn = TRUE;
   pProgardDeltaPFlowRateMeasData->isProcDepAverageEn = FALSE;
   pProgardDeltaPFlowRateMeasData->isProcDepRecordEn = FALSE;

   pProgardDeltaPFlowRateMeasData->bAverageTpdo = TPDO_NOT_MAPPED;


   /* Edi production  flow rate */
   pEdiProductionFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pEdiProductionFlowRateMeasData->dwNbSplMeasure = 0u;
   pEdiProductionFlowRateMeasData->fMeasAccumulator = 0.0f;
   pEdiProductionFlowRateMeasData->pStopRecordFunc = NULL;
   pEdiProductionFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pEdiProductionFlowRateMeasData->isProcDepMeasEn = TRUE;
   pEdiProductionFlowRateMeasData->isProcDepAverageEn = FALSE;
   pEdiProductionFlowRateMeasData->isProcDepRecordEn = FALSE;

   pEdiProductionFlowRateMeasData->bAverageTpdo = TPDO_EDI_PRODUCTION_FLOWRATE;


   /* Edi recovery  flow rate */
   pEdiRecoveryFlowRateMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pEdiRecoveryFlowRateMeasData->dwNbSplMeasure = 0u;
   pEdiRecoveryFlowRateMeasData->fMeasAccumulator = 0.0f;
   pEdiRecoveryFlowRateMeasData->pStopRecordFunc = NULL;
   pEdiRecoveryFlowRateMeasData->pDoRecord = (VOID*)NULL;
   pEdiRecoveryFlowRateMeasData->isProcDepMeasEn = TRUE;
   pEdiRecoveryFlowRateMeasData->isProcDepAverageEn = FALSE;
   pEdiRecoveryFlowRateMeasData->isProcDepRecordEn = FALSE;

   pEdiRecoveryFlowRateMeasData->bAverageTpdo = TPDO_EDI_RECOVERY_FLOWRATE;
}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

