/*!
* @file mes_pressure.c
*
* @brief Pressures acquisition
*
* @details Contain pressures acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_pressure.c 537 201#$: Filename
* $Rev:: 537                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-08-20 09:54:18 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_pressure.c $: URL of this file
* @endcond
*
*/

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>

#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_pres.h"
#include "aaa_api.h"
#include "measure.h"
#include "mes_can.h"
#include "system_parameters.h"



/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup Pressures_Measure Pressures measure
 *  @ingroup Measure
 *  @brief Pressures measure acquistion procedures
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#define FEED_PRESSURE_SPREAD     ( MEASURE_FEED_PRESSSURE_SENSOR_MAX - MEASURE_FEED_PRESSSURE_SENSOR_MIN)
#define RO_PRESSURE_SPREAD       ( MEASURE_RO_PRESSSURE_SENSOR_MAX - MEASURE_RO_PRESSSURE_SENSOR_MIN)
#define DISTRIB_PRESSURE_SPREAD  ( MEASURE_DIST_PRESSSURE_SENSOR_MAX - MEASURE_DIST_PRESSSURE_SENSOR_MIN)
#define PRESSURE_SENSOR_VOLTAGE_MAX       (4.5F)   /*!< Max pressure sensor voltage */
#define PRESSURE_SENSOR_VOLTAGE_MIN       (0.5F)   /*!< Min pressure sensor voltage */
#define PRESSURE_SENSOR_VOLTAGE_TOLERANCE (0.3F)   /*!< Voltage fluctuation tolerance before alarm raised  */
#define FEED_PRESSURE_VOLTAGE_COEF        (0.25f)  /*!< Filtered coef for feed pressure  */

/* Declarations --------------------------------------------------------------*/

static DIL_HANDLE dilFeedPressureHdl = NULL; /*!< Feed pressure DIL handle */
static DIL_HANDLE dilRoPressureHdl = NULL; /*!< RO pressure DIL handle */
static DIL_HANDLE dilDistributionPressureHdl = NULL; /*!< Distribution pressure DIL handle */

static FLOAT fOldFeedPressureVoltageValue = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;

/* Codes ---------------------------------------------------------------------*/

/*!
 * @brief returns pressure from given voltage raw measure<br>
 *              and Min / Max boundaries.<br>
 *
 * \f$ Pressure = Pmin + \frac{ (RawVoltage - fOffset) \cdot fGain \cdot (Pmax - Pmin) }{4}  \f$
 *
 * @param fRawVoltage [IN] FLOAT <br>
 *                   voltage measured (from DIL)
 *
 * @param fGain [IN] FLOAT <br>
 *                   voltage gain (Default Value = 1.0)
 *
 * @param fOffset [IN] FLOAT <br>
 *                   voltage offset (Default Value = 0.5V)
 *
 * @param fPmin [IN] FLOAT <br>
 *                   Min pressure
 *
 * @param fPspread [IN] FLOAT <br>
 *                   Pressure spread (Pmax-Pmin)
 *
 * @return Pressure.
 *
 */
static FLOAT ComputePressure( FLOAT fRawVoltage, FLOAT fGain, FLOAT fOffset, FLOAT fPmin, FLOAT fPspread)
{
   FLOAT fVoltageCorrected;
   FLOAT Pressure = fPmin;

   fVoltageCorrected = (fRawVoltage - fOffset);

   Pressure +=  (((fVoltageCorrected * fGain) * fPspread)  / (PRESSURE_SENSOR_VOLTAGE_MAX - PRESSURE_SENSOR_VOLTAGE_MIN));

   return Pressure ;
}


/*!
 *  @param feedPressure  IN    FLOAT               <br>
 *              Feed pressure
 *
 *  @return None
 *
 *  @brief Update Tap Feed pressure every 100ms
 *              \f$ Tap Feed = \overline{Feed presure} \f$
 *
 *  @details
 *     -# Compute Tap Feed Pressure average
 */
static VOID UpdateTapFeedPressure( FLOAT feedPressure)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TAP_FEED_PRESSURE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /*No Alarms to check */

   /* Check Process depedant Record */
   if( (TRUE == pMeasData->isProcDepAverageEn)
   &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != feedPressure) )
   {
      /* Accumuate Instant Ro Pressure */
      pMeasData->fMeasAccumulator += feedPressure;
      pMeasData->dwNbSplMeasure++;
   }
}


/*!
 *  @param feedPressure [IN] FLOAT Feed pressure
 *
 *  @return None
 *
 *  @brief Update Progard Delta every 100ms
 *              \f$ Progard Feed = \overline{Feed presure} \f$
 *
 *  @details
 *     -# Compute Instant Feed Pressure average
 */
static VOID UpdateProGardDeltaP(FLOAT feedPressure)
{
   MeasData_t* CONST pMeasData = &g_MeasDataArray[PROGARD_DELTA_PRESSURE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /*No Alarms to check */

   /* Check Process depedant Record */
   if( (TRUE == pMeasData->isProcDepAverageEn)
   &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != feedPressure) )
   {
      /* Accumuate Instant Ro Pressure */
      pMeasData->fMeasAccumulator += feedPressure;
      pMeasData->dwNbSplMeasure++;
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update RO pressure every 100ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL feed pressure read
 *     -# Write voltage value in OD
 *     -# Compute Instant Feed Pressure
 *     -# write Instant Feed Pressure in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according _Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant RO Pump Inlet Pressure in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateROPressure(BOOL isSecondBoundary)
{
   FLOAT fRoPressureVoltage = 0.0f;   /*< temp variable.*/
   FLOAT fCalibGain,fCalibOffset,fAdjCalibGain,fAdjCalibOffset;

   MeasData_t* CONST pMeasData = &g_MeasDataArray[RO_PRESSURE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL feed pressure read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)RO_PRESSURE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilRoPressureHdl, (DIL_UINT32)IOCTL_CMD_PRESSURE_GET_VALUE ,
                            (DIL_LPVOID)&fRoPressureVoltage, sizeof(fRoPressureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure call");

      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = fRoPressureVoltage;

      /* write voltage value in OD */
      DO_WRITE(g_DoRoPressure.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Ro Pressure in OD*/
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoRoPressure.Voltage);

   /* Send alarm if sensor error */
   if(   ((PRESSURE_SENSOR_VOLTAGE_MIN - PRESSURE_SENSOR_VOLTAGE_TOLERANCE) > pMeasData->rawMeas.fVoltage)
      || ((PRESSURE_SENSOR_VOLTAGE_MAX + PRESSURE_SENSOR_VOLTAGE_TOLERANCE) < pMeasData->rawMeas.fVoltage) )
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      MES_SetAlarm(ALARM_TYPE_RO_PRESSURE_SENSOR_ERROR);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(RO_PRESSURE,TRUE);

      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoRoPressure.CalibAdjPressure, (FLOAT)OUT_OF_RANGE_MAX_MEASURE);
   }
   else
   {
      /* Otherwise ,*/

      /* Compute calibration pressure  */
      /* ============================= */
      FLOAT fCalibInstantPressure;
      /* Read calibration gain */
      DO_READ(fAdjCalibGain, g_DoRoPressure.CalibAdjCalibrationGain);
      /* Read calibration offset  */
      DO_READ(fAdjCalibOffset, g_DoRoPressure.CalibAdjCalibrationOffset);
      /* Compute Instant Feed pressure */
      fCalibInstantPressure = ComputePressure(pMeasData->rawMeas.fVoltage, fAdjCalibGain, fAdjCalibOffset, MEASURE_RO_PRESSSURE_SENSOR_MIN, RO_PRESSURE_SPREAD);
      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoRoPressure.CalibAdjPressure, fCalibInstantPressure);

      /* Compute Standart pressure  */
      /* ========================== */
      MES_ClearAlarm(ALARM_TYPE_RO_PRESSURE_SENSOR_ERROR);
      /* Read calibration gain */
      DO_READ(fCalibGain, g_DoRoPressure.CalibrationGain);
      /* Read calibration offset  */
      DO_READ(fCalibOffset, g_DoRoPressure.CalibrationOffset);
      /* Compute Instant Feed pressure */
      pMeasData->fMeasure = ComputePressure(pMeasData->rawMeas.fVoltage, fCalibGain, fCalibOffset, MEASURE_RO_PRESSSURE_SENSOR_MIN, RO_PRESSURE_SPREAD);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(RO_PRESSURE,FALSE);
   }

   /* Write Instant Ro Pressure in OD */
   DO_WRITE(g_DoRoPressure.Pressure, pMeasData->fMeasure);

   /* Check Alarms */

   /* Check High level alarm */
   if( (ALARM_RO_PRESSURE_MAX < pMeasData->fMeasure )
   &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
   {
      MES_SetAlarm(ALARM_TYPE_RO_PRESSURE_MAX);
   }
   else
   {
      MES_ClearPendingAlarm(ALARM_TYPE_RO_PRESSURE_MAX);
      /* Alarm is cleared by an user acknowledgement */
   }

   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_RO_PRESSURE_MIN].isEnable)
   {
      FLOAT fMinRoPressureThreshold;
      /* According system type ,
       *   if Rios , ALARM_RO_PRESSURE_MIN_WITHOUT_EDI
       *   for other  ,  ALARM_RO_PRESSURE_MIN_WITH_EDI
       */
      if(SYSTEM_DEVICE_RIOS==SYS_GetDeviceType())
      {
         fMinRoPressureThreshold=ALARM_RO_PRESSURE_MIN_WITHOUT_EDI;
      }
      else
      {
         fMinRoPressureThreshold=ALARM_RO_PRESSURE_MIN_WITH_EDI;
      }
      /* Check Min Pump pressure accodrding system type */
      if( (fMinRoPressureThreshold > pMeasData->fMeasure )
      &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_RO_PRESSURE_MIN);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_RO_PRESSURE_MIN);
         /* Alarm is cleared by an user acknowledgement */
      }
   }
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE].isEnable)
   {
      /* Check RO pressure compare to Feed pressure
       * In order to compare significantly , a theshold is defined.
       */
      if( (fabs(g_MeasDataArray[FEED_PRESSURE].fMeasure - pMeasData->fMeasure ) > ALARM_PRESSURE_NOT_EQUAL_THRESHOLD )
      &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_RO_TO_FEED_PRESSURE_COMPARE);
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Ro Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            fMeasure = (((pMeasData->fMeasAccumulator * 1000.0f) / (FLOAT)pMeasData->dwNbSplMeasure) + 0.5f); // Record in mBar
            if (0.0f > fMeasure)
            {
               fMeasure = 0.0f;
            }
            /* Write averaged Instant Feed Pressure  in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoPressure, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoPressure, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }
         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_RO_PRESSURE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Ro Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Nothing to do */
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Distribution pressure every 100ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL pressure read
 *     -# Write voltage value in OD
 *     -# Compute Instant Distribution Pressure
 *     -# write Instant Distribution Pressure in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according /p Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant RO Pump Inlet Pressure in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateDistributionPressure(BOOL isSecondBoundary)
{
   FLOAT fDistributionPressureVoltage = 0.0f;   /*< temp variable.*/
   FLOAT fCalibGain,fCalibOffset,fAdjCalibGain,fAdjCalibOffset;

   MeasData_t* CONST pMeasData = &g_MeasDataArray[DISTRIBUTION_PRESSURE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL distribution pressure read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_PRESSURE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilDistributionPressureHdl, (DIL_UINT32)IOCTL_CMD_PRESSURE_GET_VALUE ,
                            (DIL_LPVOID)&fDistributionPressureVoltage, sizeof(fDistributionPressureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure call");
      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = fDistributionPressureVoltage;

      /* write voltage value in OD */
      DO_WRITE(g_DoDistributionPressure.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Distribution Pressure in OD*/
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoDistributionPressure.Voltage);

   /* Send alarm if sensor error */
   if ( ((PRESSURE_SENSOR_VOLTAGE_MIN - PRESSURE_SENSOR_VOLTAGE_TOLERANCE) > pMeasData->rawMeas.fVoltage)
    ||  ((PRESSURE_SENSOR_VOLTAGE_MAX + PRESSURE_SENSOR_VOLTAGE_TOLERANCE) < pMeasData->rawMeas.fVoltage)  )
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      MES_SetAlarm(ALARM_TYPE_DISTRIB_PRESSURE_SENSOR_ERROR);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(DISTRIBUTION_PRESSURE,TRUE);
      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoDistributionPressure.CalibAdjPressure, (FLOAT)OUT_OF_RANGE_MAX_MEASURE);
   }
   else
   {
      /* Otherwise ,*/

      /* Compute calibration pressure  */
      /* ============================= */
      FLOAT fCalibInstantPressure;
      /* Read calibration gain */
      DO_READ(fAdjCalibGain, g_DoDistributionPressure.CalibAdjCalibrationGain);
      /* Read calibration offset  */
      DO_READ(fAdjCalibOffset, g_DoDistributionPressure.CalibAdjCalibrationOffset);
      /* Compute Instant Feed pressure */
      fCalibInstantPressure = ComputePressure(pMeasData->rawMeas.fVoltage, fAdjCalibGain, fAdjCalibOffset, MEASURE_DIST_PRESSSURE_SENSOR_MIN, DISTRIB_PRESSURE_SPREAD);
      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoDistributionPressure.CalibAdjPressure, fCalibInstantPressure);

      /* Compute Standart pressure  */
      /* ========================== */
      MES_ClearAlarm(ALARM_TYPE_DISTRIB_PRESSURE_SENSOR_ERROR);
      /* Read calibration gain */
      DO_READ(fCalibGain, g_DoDistributionPressure.CalibrationGain);
      /* Read calibration offset  */
      DO_READ(fCalibOffset, g_DoDistributionPressure.CalibrationOffset);
      /* Raw measure pressure valid */
      pMeasData->fMeasure = ComputePressure(pMeasData->rawMeas.fVoltage, fCalibGain, fCalibOffset, MEASURE_DIST_PRESSSURE_SENSOR_MIN, DISTRIB_PRESSURE_SPREAD);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(DISTRIBUTION_PRESSURE,FALSE);
   }

   /* Write Instant Distribution Pressure in OD */
   DO_WRITE(g_DoDistributionPressure.Pressure, pMeasData->fMeasure);

   /* Check Alarms */
   /* Check High level alarm */
   if( (ALARM_DISTRIBUTION_PRESSURE_MAX < pMeasData->fMeasure )
   &&  ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ))
   {
      MES_SetAlarm(ALARM_TYPE_DISTRIB_PRESSURE_MAX);
   }
   else
   {
      MES_ClearPendingAlarm(ALARM_TYPE_DISTRIB_PRESSURE_MAX);
      /* Alarm is cleared by an user acknowledgement */
   }

   /* No Check Process depedant Alarm */

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Distribution Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         //INT16 swMeasure;

         ASSERT(0u != pMeasData->dwNbSplMeasure,"Average spl null");

         //pMeasData->fMeasure = pMeasData->fMeasAccumulator / pMeasData->dwNbSplMeasure;
         //swMeasure = (INT16)pMeasData->fMeasure;

         /* Write averaged Instant Distribution Pressure  in OD REC History field */
         /* No write in Water hisrory because we prefer MMI to ask for SDO.. */
         // DO_WRITE(g_DoWaterProcess.DistributionPumpInletPressure, swMeasure);
         /* Set PDO record to be send */
         //MES_CanRecordUpdate(pMeasData->bTpdo);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Distribution Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do Nothing */
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called every 100ms and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Feed pressure every 100ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL feed pressure read
 *     -# Write voltage value in OD
 *     -# Compute Instant Feed Pressure
 *     -# write Instant Feed Pressure in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant Alarms (according \p Process order)
 *     -# if Record enable, <br>
 *          ->accumulate. <br>
 *          ->if second boundary,  <br>
 *                -->compute average <br>
 *                -->write Instant RO Pump Inlet Pressure in OD
 *     -# if average enable, <br>
 *          ->accumulate. <br>
 *     -#  Call TAP Feed Pressure specific treatement
 *     -#  Call ProGard Delta P  specific treatement
  *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateFeedPressure(BOOL isSecondBoundary)
{
   FLOAT feedPressureVoltage = 0.0f;   /*< temp variable.*/
   FLOAT fCalibGain,fCalibOffset,fAdjCalibGain,fAdjCalibOffset;

   MeasData_t* CONST pMeasData = &g_MeasDataArray[FEED_PRESSURE];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      /* Check to ensure UpdateTapFeedPressure or progard delat_P is not requested  */
      ASSERT(((FALSE == g_MeasDataArray[TAP_FEED_PRESSURE].isProcDepMeasEn) &&
	          (FALSE == g_MeasDataArray[PROGARD_DELTA_PRESSURE].isProcDepMeasEn)),
            "Can't measure Tap feed pressure or ProgardDeltaP if Feed pressure measure disable");
      return;
   }

   /* If DIL not bypassed, DIL feed pressure read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_PRESSURE))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilFeedPressureHdl, (DIL_UINT32)IOCTL_CMD_PRESSURE_GET_VALUE ,
                            (DIL_LPVOID)&feedPressureVoltage, sizeof(feedPressureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure call");

      /* Filtering Voltage measure */
      if ((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fOldFeedPressureVoltageValue)
      {
         /* First Call, we need to initialize old value */
         fOldFeedPressureVoltageValue = feedPressureVoltage;
      }
      else
      {
         feedPressureVoltage = ((FEED_PRESSURE_VOLTAGE_COEF * feedPressureVoltage) + ((1.0f - FEED_PRESSURE_VOLTAGE_COEF) * fOldFeedPressureVoltageValue));
         fOldFeedPressureVoltageValue = feedPressureVoltage;
      }

      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = feedPressureVoltage;

      /* write voltage value in OD */
      DO_WRITE(g_DoFeedPressure.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Feed Pressure in OD*/
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoFeedPressure.Voltage);

   /* Send alarm if sensor error */
   if (  ((PRESSURE_SENSOR_VOLTAGE_MIN-PRESSURE_SENSOR_VOLTAGE_TOLERANCE) > pMeasData->rawMeas.fVoltage )
      || ((PRESSURE_SENSOR_VOLTAGE_MAX+PRESSURE_SENSOR_VOLTAGE_TOLERANCE) < pMeasData->rawMeas.fVoltage ) )
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      MES_SetAlarm(ALARM_TYPE_FEED_PRESSURE_SENSOR_ERROR);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(FEED_PRESSURE,TRUE);
      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoFeedPressure.CalibAdjPressure, (FLOAT)OUT_OF_RANGE_MAX_MEASURE);
   }
   else
   {
      /* Compute calibration pressure  */
      /* ============================= */
      FLOAT fCalibInstantPressure;
      /* Read calibration gain */
      DO_READ(fAdjCalibGain, g_DoFeedPressure.CalibAdjCalibrationGain);
      /* Read calibration offset  */
      DO_READ(fAdjCalibOffset, g_DoFeedPressure.CalibAdjCalibrationOffset);
      /* Compute Instant Feed pressure */
      fCalibInstantPressure = ComputePressure(pMeasData->rawMeas.fVoltage, fAdjCalibGain, fAdjCalibOffset, MEASURE_FEED_PRESSSURE_SENSOR_MIN, FEED_PRESSURE_SPREAD);
      /* Write Instant Ro Pressure in OD */
      DO_WRITE(g_DoFeedPressure.CalibAdjPressure, fCalibInstantPressure);

      /* Compute Standart pressure  */
      /* ========================== */
      /* Otherwise ,*/
      MES_ClearAlarm(ALARM_TYPE_FEED_PRESSURE_SENSOR_ERROR);
      /* Read calibration gain */
      DO_READ(fCalibGain, g_DoFeedPressure.CalibrationGain);
      /* Read calibration offset  */
      DO_READ(fCalibOffset, g_DoFeedPressure.CalibrationOffset);
      /* Compute Instant Feed pressure */
      pMeasData->fMeasure = ComputePressure(pMeasData->rawMeas.fVoltage, fCalibGain, fCalibOffset, MEASURE_FEED_PRESSSURE_SENSOR_MIN, FEED_PRESSURE_SPREAD);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(FEED_PRESSURE,FALSE);
   }

   /* Write Instant Feed Pressure in OD */
   DO_WRITE(g_DoFeedPressure.Pressure, pMeasData->fMeasure);

   /* Check Process Indepedant Alarm */
   /* Check High level alarm */
   if((ALARM_FEED_PRESSURE_MAX < pMeasData->fMeasure)
    && (pMeasData->fMeasure != (FLOAT)OUT_OF_RANGE_MAX_MEASURE ))
   {
      MES_SetAlarm(ALARM_TYPE_FEED_PRESSURE_MAX);
   }
   else
   {
      MES_ClearPendingAlarm(ALARM_TYPE_FEED_PRESSURE_MAX);
      /* Alarm is cleared by an user acknowledgement */
   }

   /* Check Process depedant Alarm */
   /* Check Low level alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FEED_PRESSURE_MIN].isEnable)
   {
      /* Check Min pressure */
      if( (g_fFeedPressureMinAlarmThreshold > pMeasData->fMeasure )
       && (pMeasData->fMeasure != (FLOAT)OUT_OF_RANGE_MAX_MEASURE )  )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FEED_PRESSURE_MIN);
      }
      else
      {
         MES_ClearCheckPendingAlarm(MES_CHK_ALM_FEED_PRESSURE_MIN);
         /* Alarm is cleared by an user acknowledgement */
      }
   }

   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Feed Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         FLOAT fMeasure;

         if (0u < pMeasData->dwNbSplMeasure)
         {
            fMeasure = (((pMeasData->fMeasAccumulator * 1000.0f) / (FLOAT)pMeasData->dwNbSplMeasure) + 0.5f); // Record in mBar
            if (0.0f > fMeasure)
            {
               fMeasure = 0.0f;
            }
            /* Write averaged Instant Feed Pressure  in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoPumpInletPressure, (UNSIGNED16)fMeasure);
         }
         else
         {
            /* Write flow rate in OD REC History field */
            DO_WRITE(g_DoWaterProcess.RoPumpInletPressure, (UNSIGNED16)INVALID_RECORD_MEASURE);
         }

         /* Set PDO record to be send */
         MES_CanRecordUpdate(TPDO_RO_PUMP_INLET_PRESSURE);

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }
   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Feed Pressure */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
   else
   {
      /* Do nothing */
   }

   /* In the end, */
   /* -    Call TAP Feed Pressure specific treatement */
   UpdateTapFeedPressure( pMeasData->fMeasure);

   /* -    Call ProGard Delta P  specific treatement */
   UpdateProGardDeltaP(pMeasData->fMeasure);
}


/*!
 *  @return None
 *
 *  @brief handle Stop record with average computation
 *
 *  @details Called upon MES_RecordStop trig when start record has been ordered with average computation
 *
 */
static VOID MES_StopRecordAveragePressureFunc(MeasData_t* CONST pMeasData )
{
   /* Average computation finalization */
   if ((TRUE == pMeasData->isProcDepAverageEn) &&
       (0u < pMeasData->dwNbSplMeasure))
   {
      /* Send Record containing average */
      pMeasData->fMeasure = (pMeasData->fMeasAccumulator / (FLOAT)pMeasData->dwNbSplMeasure);
   }

   /* Reset average data */
   pMeasData->fMeasAccumulator = 0.0f;
   pMeasData->dwNbSplMeasure = 0u;
}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details
 */
VOID MES_InitPressureDrivers(VOID)
{
   DIL_RC   ret;
   MeasData_t* CONST pFeedPressureMeasData = &g_MeasDataArray[FEED_PRESSURE];
   MeasData_t* CONST pRoPressureMeasData = &g_MeasDataArray[RO_PRESSURE];
   MeasData_t* CONST pDistributionPressureMeasData = &g_MeasDataArray[DISTRIBUTION_PRESSURE];
   MeasData_t* CONST pTapFeedPressureMeasData = &g_MeasDataArray[TAP_FEED_PRESSURE];
   MeasData_t* CONST pProgardDeltaPMeasData = &g_MeasDataArray[PROGARD_DELTA_PRESSURE];

   /* DIL feed pressure init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_PRESSURE))
   {
      ret = dil_opendevice(DIL_NAME_PRES_FEED, &dilFeedPressureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure init");
   }
   pFeedPressureMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pFeedPressureMeasData->dwNbSplMeasure = 0u;
   pFeedPressureMeasData->fMeasAccumulator = 0.0f;
   pFeedPressureMeasData->pStopRecordFunc = MES_StopRecordAveragePressureFunc;
   pFeedPressureMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.RoPumpInletPressure;
   pFeedPressureMeasData->isProcDepMeasEn = TRUE;
   pFeedPressureMeasData->isProcDepAverageEn = FALSE;
   pFeedPressureMeasData->isProcDepRecordEn = FALSE;

   pFeedPressureMeasData->bAverageTpdo = TPDO_RO_PUMP_INLET_PRESSURE;

   /* DIL Ro pressure init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)RO_PRESSURE))
   {
      ret = dil_opendevice(DIL_NAME_PRES_RO, &dilRoPressureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure init");
   }
   pRoPressureMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pRoPressureMeasData->dwNbSplMeasure = 0u;
   pRoPressureMeasData->fMeasAccumulator = 0.0f;
   pRoPressureMeasData->pStopRecordFunc = MES_StopRecordAveragePressureFunc;
   pRoPressureMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.RoPressure;
   pRoPressureMeasData->isProcDepMeasEn = TRUE;
   pRoPressureMeasData->isProcDepAverageEn = FALSE;
   pRoPressureMeasData->isProcDepRecordEn = FALSE;

   pRoPressureMeasData->bAverageTpdo = TPDO_RO_PRESSURE;

   /* DIL Distribution pressure init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_PRESSURE))
   {
      ret = dil_opendevice(DIL_NAME_PRES_DISTRIBUTION, &dilDistributionPressureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure init");
   }
   pDistributionPressureMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pDistributionPressureMeasData->dwNbSplMeasure = 0u;
   pDistributionPressureMeasData->fMeasAccumulator = 0.0f;
   pDistributionPressureMeasData->pStopRecordFunc = MES_StopRecordAveragePressureFunc;
   pDistributionPressureMeasData->pDoRecord = (VOID*)NULL;
   pDistributionPressureMeasData->isProcDepMeasEn = TRUE;
   pDistributionPressureMeasData->isProcDepAverageEn = FALSE;
   pDistributionPressureMeasData->isProcDepRecordEn = FALSE;

//   pDistributionPressureMeasData->bTpdo = TPDO_NOT_MAPPED;

   /* DIL TAP Feed pressure init */
   pTapFeedPressureMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pTapFeedPressureMeasData->dwNbSplMeasure = 0u;
   pTapFeedPressureMeasData->fMeasAccumulator = 0.0f;
   pTapFeedPressureMeasData->pStopRecordFunc = MES_StopRecordAveragePressureFunc;
   pTapFeedPressureMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.TapFeedPressure;
   pTapFeedPressureMeasData->isProcDepMeasEn = TRUE;
   pTapFeedPressureMeasData->isProcDepAverageEn = FALSE;
   pTapFeedPressureMeasData->isProcDepRecordEn = FALSE;

   pTapFeedPressureMeasData->bAverageTpdo = TPDO_TAP_FEED_PRESSURE;

   /* DIL Progard Delta pressure init */
   pProgardDeltaPMeasData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pProgardDeltaPMeasData->dwNbSplMeasure = 0u;
   pProgardDeltaPMeasData->fMeasAccumulator = 0.0f;
   pProgardDeltaPMeasData->pStopRecordFunc = MES_StopRecordAveragePressureFunc;
   pProgardDeltaPMeasData->pDoRecord = (VOID*)&g_DoWaterProcess.ProgardDeltaPressure;
   pProgardDeltaPMeasData->isProcDepMeasEn = TRUE;
   pProgardDeltaPMeasData->isProcDepAverageEn = FALSE;
   pProgardDeltaPMeasData->isProcDepRecordEn = FALSE;

   pProgardDeltaPMeasData->bAverageTpdo = TPDO_PROGARD_DELTA_PRESSURE;

}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

