/*!
* @file mes_tanklevel.c
*
* @brief Tank level acquisition
*
* @details Contains Tank level acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_tanklevel.c 683 20#$: Filename
* $Rev:: 683                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-12-22 11:36:49 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_tanklevel.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_data.h"
#include "mes_api.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_4_20_in.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "system_parameters.h"
#include "measure.h"


/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup TankLevel_Measure Tank level measure
 *  @ingroup Measure
 *  @brief Tank level measure acquistion procedures
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#define TANK_LEVEL_CUR_FLOOR     (4.0f)   /*!< lowest sensor current       */
#define TANK_LEVEL_CUR_TOP       (20.0f)  /*!< highest sensor current      */
#define TANK_LEVEL_CUR_COEF      (0.1f)   /*!< Filtered coef for current   */
#define TANK_LEVEL_MM_COEF       (0.1f)   /*!< Filtered coef for current   */
#define TANK_LEVEL_VOL_COEF      (0.1f)   /*!< Filtered coef for volume    */

#define TANK_LEVEL_LOW_CURRENT_THRESHOLD (3.46f)/*!< Low current alarm threshold (equivalent to 0.4V of level sensor) */

#define TANK_LEVEL_MM_TOP        (1000.0f)/*!< Max tank level millimeter = 1000 mm  */
#define TANK_LEVEL_MM_FLOOR      (0u)     /*!< Min tank level millimeter = 0 mm     */

#define TANK_LEVEL_PERCENT_TOP            (100.0f) /*!< Max tank level percentage 100% */
#define TANK_LEVEL_PERCENT_NULL           (0.0f)   /*!< Min tank level percentage 0%   */

#define  TFS_FILTER_COEF         (0.01f)  /*!< Filtered coef for tank filling computation */
#define  TFS_SPEED_MIN           (15.0f)  /*!< Tank Filling Minimum Speed */
#define  TFS_SPEED_MAX           (250.0f) /*!< Tank Filling Maximum Speed */
#define  TFS_MIN_COUNTER         (600u)   /*!< Nbr of measure of the same sign = 1 min */


/* Declarations --------------------------------------------------------------*/
BOOL g_isTankLevelSaturated;

static DIL_HANDLE dil4_20mAHdl = NULL; /*!< 4-20mA DIL handle */
static FLOAT fOldCurrentValue = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
static FLOAT fOldMmValue = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
static FLOAT fOldVolumeValue = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
static FLOAT fOldTankFillingSpeed = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
static UINT32 lTankFillingSpeedCounter = 0u;
static BOOL isPercentForcedToZero = FALSE;

/* Codes ---------------------------------------------------------------------*/

/*!
 *
 *  @param fCurrent [IN] FLOAT <br>
 *                   4-20 in measured curent.
 *
 *  @param pfTankLevelMm [OUT] FLOAT *  CONST<br>
 *      pointer to place tank level in mm
 *
 *  @param pfTankLevelPercentage [OUT] FLOAT * CONST CONST<br>
 *      pointer to place Percentage coresponding value
 *
 *  @param pfTankLevelVolume [OUT] FLOAT * CONST CONST<br>
 *      pointer to place Volume coresponding value
 *
 *  @return void
 *
 *  @brief Conversion function to convert 4-20 in current to Millimeters and percentage.
 */
static VOID ConvertTanklevel(CONST FLOAT fCurrent, FLOAT * CONST pfTankLevelMm, FLOAT * CONST pfTankLevelPercentage, FLOAT * CONST pfTankLevelVolume)
{
   FLOAT fValue;
   FLOAT fFilteredCurrent;
   UINT16 wTankTotalVol;
   UINT8 bTankType;
   FLOAT fCustomTankCurrentMin;
   FLOAT fCustomTankCurrentMax;
   DO_READ(bTankType,g_DoTankLevelMeasure.TankType);

   /* Filtering Current measure */
   if ((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fOldCurrentValue)
   {
      /* First Call, we need to initialize old value */
      fOldCurrentValue = fCurrent;
      fFilteredCurrent = fCurrent;
   }
   else
   {
      fFilteredCurrent = ((TANK_LEVEL_CUR_COEF * fCurrent) + ((1.0f - TANK_LEVEL_CUR_COEF) * fOldCurrentValue));
      fOldCurrentValue = fFilteredCurrent;
   }

   /* Convert current in mm*/
   fValue=(fFilteredCurrent - TANK_LEVEL_CUR_FLOOR ) ;
   fValue*=(TANK_LEVEL_MM_TOP / (TANK_LEVEL_CUR_TOP - TANK_LEVEL_CUR_FLOOR));
   if ((fValue<0.0f) || (fValue > 32767.0f))
   {
      fValue=0.0f;
   }

   /* Filtering Mm measure */
   if ((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fOldMmValue)
   {
      /* First Call, we need to initialize old value */
      fOldMmValue = fValue;
   }
   else
   {
      fValue = ((TANK_LEVEL_MM_COEF * fValue) + ((1.0f - TANK_LEVEL_MM_COEF) * fOldMmValue));
      fOldMmValue = fValue;
   }

   *pfTankLevelMm=fValue;
   
   /* Compute percent value */
   if((MES_TANK_TYPE_SMALL == bTankType) ||
      (MES_TANK_TYPE_BIG == bTankType))
   {
      /* For small and big tank, we use the mm measure */
      fValue=(fValue - (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW);
      fValue*=(TANK_LEVEL_PERCENT_TOP/ ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MAX - (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW) );
   }
   else
   {
      /* for custom tank, we use the current measure */
      DO_READ(fCustomTankCurrentMin, g_DoTankLevelMeasure.TankEmptyCalibration);
      DO_READ(fCustomTankCurrentMax, g_DoTankLevelMeasure.TankFullCalibration);
      fValue = (((fCurrent - fCustomTankCurrentMin) * TANK_LEVEL_PERCENT_TOP) / (fCustomTankCurrentMax - fCustomTankCurrentMin));
   }
   
   if (fValue>TANK_LEVEL_PERCENT_TOP)
   {
      *pfTankLevelPercentage = TANK_LEVEL_PERCENT_TOP;
   }
   else if (fValue > TANK_LEVEL_PERCENT_NULL)
   {
      *pfTankLevelPercentage = fValue;
   }
   else
   {
      *pfTankLevelPercentage = TANK_LEVEL_PERCENT_NULL;
   }

   /* Compute volume value */
   *pfTankLevelVolume = 0.0f;
   if(MES_TANK_TYPE_SMALL == bTankType)
   {
      if ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW < (*pfTankLevelMm))
      {
         *pfTankLevelVolume = (FLOAT)MM_TO_LITER_COEFF_FOR_SMALL_TANK * ((*pfTankLevelMm) - (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW);
      }
   }
   else if(MES_TANK_TYPE_BIG == bTankType)
   {
      if ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW < (*pfTankLevelMm))
      {
         *pfTankLevelVolume = (FLOAT)MM_TO_LITER_COEFF_FOR_BIG_TANK * ((*pfTankLevelMm) - (FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW);
      }
   }
   else if(MES_TANK_TYPE_CUSTOM == bTankType)
   {
      DO_READ(wTankTotalVol,g_DoTankLevelMeasure.TankTotalVolume);
      *pfTankLevelVolume= (((FLOAT)wTankTotalVol * (*pfTankLevelPercentage)) / 100.0f);
   }
   else
   {
      *pfTankLevelVolume=0.0f;
   }
}


/*!
 *
 *  @param fTankLevelVol [IN] FLOAT <br>
 *            New tank level measured (L)
 *
 *  @return void
 *
 *  @brief Update Tank filling speed every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# Compute Filtered Instant Tank Filling speed
 *
 */
static VOID UpdateTankFillingSpeed(FLOAT fTankLevelVol)
{
   FLOAT fTfs = 0.0f;

   MeasData_t* CONST pMeasData = &g_MeasDataArray[TANK_FILLING_SPEED];

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Filtering Volume measure */
      if ((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fOldVolumeValue)
      {
         /* First Call, we need to initialize old value */
         fOldVolumeValue = fTankLevelVol;
      }
      else
      {
         fTankLevelVol = ((TANK_LEVEL_VOL_COEF * fTankLevelVol) + ((1.0f - TANK_LEVEL_VOL_COEF) * fOldVolumeValue));

         /* compute Tanklevel variation = Tfs*/
         if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fTankLevelVol) &&
             ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != fOldVolumeValue))
         {
            fTfs = ((fTankLevelVol - fOldVolumeValue) * 36000.0f);

            /* Saturation */
            if (fTfs > (FLOAT)TFS_SPEED_MAX)
            {
               fTfs = (FLOAT)TFS_SPEED_MAX;
            }
            else if (fTfs < -(FLOAT)TFS_SPEED_MAX)
            {
               fTfs = -(FLOAT)TFS_SPEED_MAX;
            }
            else
            {
               /* Measure is in the range */
            }
         }

         /* Filtering Tank Filling Speed */
         if ((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fOldTankFillingSpeed)
         {
            fOldTankFillingSpeed = fTfs;
         }
         else
         {
            fTfs = ((TFS_FILTER_COEF * fTfs) + ((1.0f - TFS_FILTER_COEF) * fOldTankFillingSpeed));
         }

         pMeasData->fMeasure = fTfs;

         /* Write Tank filling speed in OD*/
         if ((FLOAT)TFS_SPEED_MIN > fabs(fTfs))
         {
            DO_WRITE(g_DoTankLevelMeasure.TankFillingFlowRate, 0.0f);
            lTankFillingSpeedCounter = 0u;
         }
         else
         {
            if (0.0f > (fOldTankFillingSpeed * fTfs))
            {
               lTankFillingSpeedCounter = 0u;
            }
            else
            {
               lTankFillingSpeedCounter++;
               if (lTankFillingSpeedCounter > TFS_MIN_COUNTER)
               {
                  lTankFillingSpeedCounter = TFS_MIN_COUNTER;
               }
            }

            if (TFS_MIN_COUNTER <= lTankFillingSpeedCounter)
            {
               DO_WRITE(g_DoTankLevelMeasure.TankFillingFlowRate, fTfs);
            }
            else
            {
               DO_WRITE(g_DoTankLevelMeasure.TankFillingFlowRate, 0.0f);
            }
         }
         fOldTankFillingSpeed = fTfs;
      }
      
      fOldVolumeValue = fTankLevelVol;
   }
   else
   {
      fOldVolumeValue = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
   }
   
}

/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used.
 *
 *  @return void
 *
 *  @brief Update Tank level every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL TankLevel read
 *     -# Write voltage value in OD
 *     -# Compute Instant TankLevel
 *     -# write Instant TankLevel in OD
 *     -# Check mode independant Alarms
 *
 */
VOID MES_UpdateTankLevel(BOOL isSecondBoundary)
{
   FLOAT fTankLevelCurrent = 0.0f;
   FLOAT fTankLevelMillimeter = 0.0f;   /*!< Tank level volume in mm */
   FLOAT fTankLevelPercentage = 0.0f;   /*< temp variable.*/

   MeasData_t* CONST pMeasData = &g_MeasDataArray[TANK_LEVEL];
   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If DIL not bypassed, DIL feed pressure read */
   if (FALSE == IS_DIL_READ_BYPASSED(TANK_LEVEL))
   {
      DIL_RC   ret;
      ret = dil_ioctldevice(dil4_20mAHdl, (DIL_UINT32)IOCTL_CMD_4_20mA_GET_VALUE ,
                            (DIL_LPVOID)&fTankLevelCurrent, sizeof(fTankLevelCurrent), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL 4-20 in call");

      /* Save Raw data */
      pMeasData->rawMeas.fCurrent = fTankLevelCurrent;

      /* write voltage value in OD */
      DO_WRITE(g_DoTankLevelMeasure.Input , pMeasData->rawMeas.fCurrent);
   }

   /* Read RAW Ro Pressure in OD */
   DO_READ(pMeasData->rawMeas.fCurrent , g_DoTankLevelMeasure.Input);

   /* Convert Tank level from current measure to Mm measure and Percentage */
   ConvertTanklevel(pMeasData->rawMeas.fCurrent, &fTankLevelMillimeter, &fTankLevelPercentage, &g_fTankLevelVolume);

   /* Update Tank filling speed */
   UpdateTankFillingSpeed(g_fTankLevelVolume);

   pMeasData->fMeasure=fTankLevelMillimeter;

   /* Check out of range  current and voltage level alarm  */
   if(TANK_LEVEL_LOW_CURRENT_THRESHOLD > pMeasData->rawMeas.fCurrent)
   {
      /* Trig tank level alarm */
      MES_SetAlarm(ALARM_TYPE_TANKLEVEL_DISCONNECTED);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(TANK_LEVEL,TRUE);
   }
   else
   {
      MES_ClearAlarm(ALARM_TYPE_TANKLEVEL_DISCONNECTED);
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(TANK_LEVEL,FALSE);
   }

   /* Alarms check */
   /* We check alarms with Millimeter tank level variable */
   if(((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_LOW >= pMeasData->fMeasure) &&
      (AAA_STATE_ACTIVE != AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_DISCONNECTED)))
   {
      /* Trig tank level alarm */
      MES_SetAlarm(ALARM_TYPE_TANKLEVEL_EMPTY);
      isPercentForcedToZero = TRUE;
   }
   else if((pMeasData->fSetPointAlarmThres <= fTankLevelPercentage) ||
           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_DISCONNECTED)))
   {
      /* Clear TANK EMPTY alarm remaining */
      MES_ClearAlarm(ALARM_TYPE_TANKLEVEL_EMPTY);
   }

   if ((FLOAT)MEASURE_TANK_LEVEL_MM_PERCENT_MIN_HIGH < pMeasData->fMeasure)
   {
      isPercentForcedToZero = FALSE;
   }

   if (TRUE == isPercentForcedToZero)
      {
         /* Force Tanklevel percentage report to 0% if tank level < 165 mm */
         fTankLevelPercentage = (FLOAT)TANK_LEVEL_PERCENT_NULL;
         /* Force Tanklevel volume to 0 if tank level < 165 mm */
         g_fTankLevelVolume = 0.0f;
      }

   /* Check Reserve SP alarm 10% - 25 % and Alarm Tank empty alarm clear */
   if((pMeasData->fSetPointAlarmThres > fTankLevelPercentage ) &&
      (AAA_STATE_ACTIVE != AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_DISCONNECTED)) &&
      (AAA_STATE_ACTIVE != AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_EMPTY)))
   {
      /* Trig Reserve tank level alarm */
      MES_SetAlarm(ALARM_TYPE_TANKLEVEL_RESERVE);
   }
   else
   {
      MES_ClearAlarm(ALARM_TYPE_TANKLEVEL_RESERVE);
   }

   g_fMeasTankLvlPercent=fTankLevelPercentage;

   /* Write Tank level mm in OD*/
   DO_WRITE(g_DoTankLevelMeasure.TankLevelValueMillimeters, (UINT16)(fTankLevelMillimeter+0.5f));

   /* Saturation for MMI of the Tank Level Percent */
   if (TRUE == g_isTankLevelSaturated)
   {
      if(99.0f < fTankLevelPercentage)
      {
         fTankLevelPercentage = 99.0f;
      }
   }
   
   /* Write Tank level percentage in OD*/
   DO_WRITE(g_DoTankLevelMeasure.TankLevelValuePercent, (UNSIGNED8)fTankLevelPercentage);

   /* Write Tank level volume in OD*/
   DO_WRITE(g_DoTankLevelMeasure.TankLevelVolume, (UNSIGNED16)(g_fTankLevelVolume+0.5f));

#ifdef LOG_MORE
   {
      static FLOAT _OldTanklevel=0.0f;
      if(fTankLevelMillimeter != _OldTanklevel)
      {
         _OldTanklevel= fTankLevelMillimeter;
         SYSLOG( INFO, "M:TnkLvl Vol=%.1f mm: %.1f %%",
            fTankLevelMillimeter,
            fTankLevelPercentage);
      }
   }
#endif

}


/*!
 *  @return None
 *
 *  @brief Initialize drivers
 *
 *  @details
 */
VOID MES_InitTankLevelDrivers(VOID)
{
   DIL_RC   ret;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TANK_LEVEL];
   MeasData_t* CONST pMeasTankFillingSpeedData = &g_MeasDataArray[TANK_FILLING_SPEED];

   /* DIL feed pressure init */
   if (FALSE == IS_DIL_READ_BYPASSED(TANK_LEVEL))
   {
      ret = dil_opendevice(DIL_NAME_4_20MA_IN_TANK_LEVEL, &dil4_20mAHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  4-20 in init");
   }
   /* Read Reserve setpoint in OD */
   DO_READ( pMeasData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.TankReserveSp);
   pMeasData->dwNbSplMeasure = 0u;
   pMeasData->fMeasAccumulator = 0.0f;
   pMeasData->pStopRecordFunc = NULL;
   pMeasData->pDoRecord = NULL;
   pMeasData->isProcDepMeasEn = TRUE;
   pMeasData->isProcDepAverageEn = FALSE;
   pMeasData->isProcDepRecordEn = FALSE;

   pMeasData->bAverageTpdo = TPDO_NOT_MAPPED;

   pMeasTankFillingSpeedData->dwNbSplMeasure = 0u;
   pMeasTankFillingSpeedData->fMeasAccumulator = 0.0f;
   pMeasTankFillingSpeedData->pStopRecordFunc = NULL;
   pMeasTankFillingSpeedData->pDoRecord = NULL;
   pMeasTankFillingSpeedData->isProcDepMeasEn = TRUE;
   pMeasTankFillingSpeedData->isProcDepAverageEn = FALSE;
   pMeasTankFillingSpeedData->isProcDepRecordEn = FALSE;

   pMeasTankFillingSpeedData->bAverageTpdo = TPDO_NOT_MAPPED;


}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

