/*!
* @file mes_temperature.c
*
* @brief Temperatures acquisition
*
* @details Contain temperatures acuisition procedures
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: mes_temperature.c 524 #$: Filename
* $Rev:: 524                   $: Revision of last commit
* $Author:: m175711            $: Author of last commit
* $Date:: 2014-08-08 09:15:11 #$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_temperature.c $: URL of this file
* @endcond
*
*/

/* Defines -------------------------------------------------------------------*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include <float.h>


#include "boardcfg.h"
#include "sys_softerr.h"
#include "os_adaptation.h"
#include "mes_api.h"
#include "mes_data.h"
#include "system_parameters.h"
#include "do_itf.h"
#include "do_def.h"
#include "dil_api.h"
#include "dil_temp.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "measure.h"



/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup Temperature_Measure Temperatures measure
 *  @ingroup Measure
 *  @brief Temperatures measure acquistion procedures
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#define DEFAULT_TEMP_CALIBRATION_GAIN  ((FLOAT)1.0F)        /*!< Default Temperature gain */
#define RESISTANCE_GAIN                ((FLOAT)100000.0F)   /*!< Resitance gain = 100 K */
#define REFERENCE_VOLTAGE              ((FLOAT)3.3F)        /*!< Reference voltage = 3.3V */
#define TEMP_SENSOR_VOLTAGE_MIN        ((FLOAT)0.5F)        /*!< Min voltage = MIN_VOLTAGEV */
#define TEMP_SENSOR_VOLTAGE_MAX        ((FLOAT)3.0F)        /*!< Max voltage = 3.0V */

/* Declarations --------------------------------------------------------------*/

static DIL_HANDLE dilFeedTemperatureHdl = NULL;         /*!< Feed Temperature DIL handle */
static DIL_HANDLE dilPermeateTemperatureHdl = NULL;     /*!< Permeate Temperature DIL handle */
static DIL_HANDLE dilEdiTemperatureHdl = NULL;          /*!< Feed Edi DIL handle */
static DIL_HANDLE dilDistributionTemperatureHdl = NULL; /*!< Distribution Temperature DIL handle */

// CTN Betatherm  Calibration table 100K6A1B r= f(TC), from 0 to 60C
static const UINT32 dwThArray[]=
{
/* + 0C   + 1C   + 2C   + 3C   + 4C   + 5C   + 6C   + 7C   + 8C     + 9C  */
   351017, 332619, 315288, 298959, 283558, 269041, 255337, 242414, 230210,   218688,   /* x  1C */
   207807, 197521, 187803, 178613, 169924, 161702, 153923, 146560, 139588,   132984,   /* x 10C */
   126729, 120799, 115179, 109850, 104796, 100000, 95449,  91128,  87026,    83129,    /* x 20C */
   79428,  75912,  72567,  69389,  66365,  63489,  60752,  58149,  55668,    53307,    /* x 30C */
   51058,  48915,  46873,  44927,  43071,  41301,  39613,  38003,  36465,    34999,    /* x 40C */
   33598,  32260,  30983,  29761,  28595,  27479,  26413,  25394,  24419,    23486,    /* x 50C */
   22593                                                                               /* x 60C */
};

/* Codes ---------------------------------------------------------------------*/


/*!
 * @param fResistanceMeas [IN] FLOAT <br>
 *                   resistance measured
 *
 * @param fCalibOffset [IN] FLOAT <br>
 *                   Calibration offset
 *
 * @return the milli-Q temperature in \C.
 *
 * @brief returns temperature from given Thermistor resistance measure.
 *        returns OUT_OF_RANGE_MIN_MEASURE or OUT_OF_RANGE_MAX_MEASURE if the temperature is not valid
 *
 * @details The temperature is retrieved from table<br>
 *          As row , is given the temperature 10th <br>
 *          As Cosl , is given the temperature  units <br>
 *          The proc retrieves the row and col range  the resistor stands in <br>
 *          Two thermisors are retrieved by (row x 10)  + cols.<br>
 *          Once detemined, the final thermistor is interpolated between twos.
 *
 */
static FLOAT ComputeTemperature( FLOAT fResistanceMeas, FLOAT fCalibOffset )
{
   UINT32 bIndex;
   FLOAT rTmp = 0.0F;
   UINT32 dwResTh=(UINT32)fResistanceMeas;

   // Resistance
   // Locate index in the table

   if (((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fResistanceMeas) ||
       (dwThArray[0] < dwResTh))
   {
      rTmp = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
   }
   else if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fResistanceMeas) ||
            (dwThArray[60] > dwResTh))
   {
      rTmp = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   else
   {
      bIndex=0u;
      while( (dwThArray[bIndex+1] > dwResTh) && 59u > bIndex )
      {
         bIndex++;
      }

      if(500000u<dwResTh) // Res > 500K =>Around -9C -> Check Thermistance
      {
         rTmp=-9.0F;
      }
      else
      {
         // decimals interpolation
         // Note: when <0, value will be approximative then false...
         rTmp = (FLOAT)bIndex + (((FLOAT)dwThArray[bIndex] - fResistanceMeas) / (FLOAT)(dwThArray[bIndex] - dwThArray[bIndex+1]));
      }
      // Add calibration offset
      rTmp += fCalibOffset;
   }

#ifdef DEBUG_FORCE_TEMP_TO_25_DEGREE
   rTmp = 25.0F;
#endif

   if (ALARM_TEMPERATURE_MIN > rTmp)
   {
      rTmp = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
   }
   else if (ALARM_TEMPERATURE_MAX < rTmp)
   {
      rTmp = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   else
   {
      /* Measure is in the range */
   }

   return rTmp;
}


/*!
 *
 *  @param fRawVoltage [IN] FLOAT <br>
 *                   voltage measured
 *
 *  @param fCalibGain [IN] FLOAT <br>
 *                   Calibration gain
 *
 * @return Thermistor resitance
 *         returns OUT_OF_RANGE_MIN_MEASURE or OUT_OF_RANGE_MAX_MEASURE if the resistance is not valid
 *
 * @brief returns Thermistor resitance from given voltage raw measure<br>
 *              and a calibration gain.
 *
 * @f$ ThermRes= \frac{ ResGain \cdot RawVoltage } { ( RefVoltage - RawVoltage ) \cdot CalibGain } @f$
 *
 */
static FLOAT ComputeThermResitivity( FLOAT fRawVoltage, FLOAT fCalibGain )
{
   FLOAT fThermRes;        /* Thermistor resistance */
   FLOAT fDenominator;

   if( TEMP_SENSOR_VOLTAGE_MIN > fRawVoltage  )
   {
      fThermRes = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
   }
   else if( TEMP_SENSOR_VOLTAGE_MAX < fRawVoltage  )
   {
      fThermRes = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
   }
   else
   {
      fDenominator = (REFERENCE_VOLTAGE - fRawVoltage);
      fThermRes = (((RESISTANCE_GAIN *  fRawVoltage)  * fCalibGain) / fDenominator);
   }
   return (fThermRes);
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used
 *
 *  @return void
 *
 *  @brief Update Feed Temperatures  every 1000 ms.
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL feed temperature read
 *     -# Write voltage value in OD
 *     -# Compute Instant Temperature
 *     -# write Instant Feed temperature in OD
 *     -# Check mode independant Alarms.
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 *  @note this procedure is kept to check resistance presence for diagnotic purpose.
 *          Temp measure is used to detect Feed sensor conductivity is unlugged
 *
 */
VOID MES_UpdateFeedTemperature(BOOL isSecondBoundary)
{
   FLOAT feedTemperatureVoltage = 0;   /* temp variable.*/
   FLOAT fThermRes; /* Thermistor resistance */

   MeasData_t* CONST pMeasData = &g_MeasDataArray[FEED_TEMPERATURE];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL feed temperature read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_TEMPERATURE))
   {
      DIL_RC   ret;
      ret = dil_ioctldevice(dilFeedTemperatureHdl, (DIL_UINT32)IOCTL_CMD_TEMP_GET_TEMP ,
                            (DIL_LPVOID)&feedTemperatureVoltage, sizeof(feedTemperatureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  feed temperature call");

      /* * Save Raw data */
      pMeasData->rawMeas.fVoltage = feedTemperatureVoltage;
      /* * write voltage value in OD */
      DO_WRITE(g_DoFeedTemperature.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Feed Temperature in OD */
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoFeedTemperature.Voltage);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,DEFAULT_TEMP_CALIBRATION_GAIN);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoFeedTemperature.Resistance,fThermRes);

   if( (FLOAT)OUT_OF_RANGE_MIN_MEASURE == fThermRes )
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(FEED_TEMPERATURE,FALSE); /* Feed Temperature is strapped for conductivity sensor detection so temperature can be Min */
   }
   else if ((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fThermRes )
   {
      pMeasData->fMeasure = (FLOAT)OUT_OF_RANGE_MIN_MEASURE;
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(FEED_TEMPERATURE,TRUE);
   }
   else
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(FEED_TEMPERATURE,FALSE);
      /* No Sensor for FEED, but temp measure is used to detect Feed sensor conductivity is unlugged */
   }
   /* No Check Process indepedant nor depednat  Alarm */
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used
 *
 *  @return void
 *
 *  @brief Update Permeate Temperatures  every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL permeate temperature read
 *     -# Write voltage value in OD
 *     -# Compute Instant Temperature
 *     -# write Instant Feed temperature in OD
 *     -# Check mode independant  Alarms
 *     -# Check mode Dependant setpoint Alarm (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Instant RO Temperature in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdatePermeateTemperature(BOOL isSecondBoundary)
{
   FLOAT permeateTemperatureVoltage = 0;   /* temp variable.*/
   MeasData_t* CONST pMeasData = &g_MeasDataArray[PERMEATE_TEMPERATURE];
   FLOAT fThermRes; /* Thermistor resistance */
   FLOAT fTemperature; /* Temperature */
   FLOAT fTempCalibGain; /* Thermistor calibration gain */
   FLOAT fTempCalibOffset; /* Thermistor calibration offset */

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL permeate temperature read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_TEMPERATURE))
   {
      DIL_RC   ret;
      ret = dil_ioctldevice(dilPermeateTemperatureHdl, (DIL_UINT32)IOCTL_CMD_TEMP_GET_TEMP ,
                            (DIL_LPVOID)&permeateTemperatureVoltage, sizeof(permeateTemperatureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  permeate temperature call");

      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = permeateTemperatureVoltage;
      /* write voltage value in OD */
      DO_WRITE(g_DoPermeateTemperature.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Permeate Temperature in OD */
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoPermeateTemperature.Voltage);

   /* Calibration measure computation */
   /* =============================== */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoPermeateTemperature.CalibAdjCalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoPermeateTemperature.CalibAdjCalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoPermeateTemperature.CalibAdjResistance,fThermRes);
   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoPermeateTemperature.CalibAdjInstantTemperature,fTemperature);

   /* Standard measure computation */
   /* ============================ */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoPermeateTemperature.CalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoPermeateTemperature.CalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoPermeateTemperature.Resistance,fThermRes);

   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* Store temperature in context */
   pMeasData->fMeasure=fTemperature;
   /* Write Instant temperature temp in OD */
   DO_WRITE(g_DoPermeateTemperature.InstantTemperature, pMeasData->fMeasure);

   /* Check Alarms */
   if((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(PERMEATE_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MIN);
   }
   else if((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(PERMEATE_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MIN);
      MES_ClearAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MAX);
   }
   else
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(PERMEATE_TEMPERATURE,FALSE);
      MES_ClearAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_PERMEATE_TEMPERATURE_MIN);
   }

   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_FEED_TEMP_SP].isEnable)
   {
      /* Set Point check */
      if( (pMeasData->fMeasure > pMeasData->fSetPointAlarmThres)
       && ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure )
       && ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_FEED_TEMP_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_FEED_TEMP_SP);
      }
   }

#if 1
   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure )
       && ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         /* Write Instant Feed Temperature in OD RO temperature field */
         FLOAT fTempValue = ((pMeasData->fMeasure * 1000.0f)+0.5f);
         DO_WRITE(g_DoWaterQuality.RoTemperature, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Instant Feed Temperature in OD RO temperature field */
         DO_WRITE(g_DoWaterQuality.RoTemperature, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }
      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_RO_TEMPERATURE);
   }
#else
   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Permeate Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         pMeasData->fMeasure = pMeasData->fMeasAccumulator / pMeasData->dwNbSplMeasure;

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }

   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Permeate Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
#endif
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used
 *
 *  @return void
 *
 *  @brief Update EDI Temperatures  every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL feed temperature read
 *     -# Write voltage value in OD
 *     -# Compute Instant Temperature
 *     -# write Instant EDI temperature in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant setpoint Alarm (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Instant Elix Temperature in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateEDITemperature(BOOL isSecondBoundary)
{
   FLOAT ediTemperatureVoltage = 0;   /* temp variable.*/
   MeasData_t* CONST pMeasData = &g_MeasDataArray[EDI_TEMPERATURE];
   FLOAT fThermRes; /* Thermistor resistance */
   FLOAT fTemperature; /* Temperature */
   FLOAT fTempCalibGain; /* Thermistor calibration gain */
   FLOAT fTempCalibOffset; /* Thermistor calibration offset */

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL edi temperature read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_TEMPERATURE))
   {
      DIL_RC   ret;
      ret = dil_ioctldevice(dilEdiTemperatureHdl, (DIL_UINT32)IOCTL_CMD_TEMP_GET_TEMP ,
                            (DIL_LPVOID)&ediTemperatureVoltage, sizeof(ediTemperatureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  edi temperature call");

      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = ediTemperatureVoltage;
      /* write voltage value in OD */
      DO_WRITE(g_DoEdiTemperature.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Edi Temperature in OD */
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoEdiTemperature.Voltage);

   /* Calibration measure computation */
   /* =============================== */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoEdiTemperature.CalibAdjCalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoEdiTemperature.CalibAdjCalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoEdiTemperature.CalibAdjResistance,fThermRes);
   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoEdiTemperature.CalibAdjInstantTemperature,fTemperature);

   /* Standard measure computation */
   /* ============================ */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoEdiTemperature.CalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoEdiTemperature.CalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoEdiTemperature.Resistance,fThermRes);

   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* Store temperature in context */
   pMeasData->fMeasure=fTemperature;
   /* Write Instant temp in OD */
   DO_WRITE(g_DoEdiTemperature.InstantTemperature, pMeasData->fMeasure);

      /* Check Alarms */
   if((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(EDI_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_EDI_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_EDI_TEMPERATURE_MIN);
   }
   else if((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(EDI_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_EDI_TEMPERATURE_MIN);
      MES_ClearAlarm(ALARM_TYPE_EDI_TEMPERATURE_MAX);
   }
   else
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(EDI_TEMPERATURE,FALSE);
      MES_ClearAlarm(ALARM_TYPE_EDI_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_EDI_TEMPERATURE_MIN);
   }

   /* No Check Process depedant Alarm */
#if 1
   /* Check Process depedant Record */
   if  (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure )
       && ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         /* Write Instant Feed Temperature in OD RO temperature field */
         FLOAT fTempValue = ((pMeasData->fMeasure * 1000.0f)+0.5f);
         DO_WRITE(g_DoWaterQuality.ElixTemperature, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Instant Feed Temperature in OD RO temperature field */
         DO_WRITE(g_DoWaterQuality.ElixTemperature, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }

      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_ELIX_TEMPERATURE);
   }
#else
   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Edi Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         pMeasData->fMeasure = pMeasData->fMeasAccumulator / pMeasData->dwNbSplMeasure;

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }

   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Edi Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
#endif
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used
 *
 *  @return void
 *
 *  @brief Update Distribution Temperatures  every 1000 ms
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL distribution temperature read
 *     -# Write voltage value in OD
 *     -# Compute Instant Temperature
 *     -# write Instant Disribution temperature in OD
 *     -# Check mode independant Alarms
 *     -# Check mode Dependant setpoint Alarm (according \p Process order)
 *     -# if Record enable, <br>
 *          ->write Instant Distribution Temperature in OD
 *     PDOsent is trigged later, once all measures are done and OD fully updated.
 *
 */
VOID MES_UpdateDistributionTemperature(BOOL isSecondBoundary)
{
   FLOAT distributionTemperatureVoltage = 0;   /* temp variable.*/
   MeasData_t* CONST pMeasData = &g_MeasDataArray[DISTRIBUTION_TEMPERATURE];
   FLOAT fThermRes; /* Thermistor resistance */
   FLOAT fTemperature; /* Temperature */
   FLOAT fTempCalibGain; /* Thermistor calibration gain */
   FLOAT fTempCalibOffset; /* Thermistor calibration offset */

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, DIL distribution temperature read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_TEMPERATURE))
   {
      DIL_RC   ret;
      ret = dil_ioctldevice(dilDistributionTemperatureHdl, (DIL_UINT32)IOCTL_CMD_TEMP_GET_TEMP ,
                            (DIL_LPVOID)&distributionTemperatureVoltage, sizeof(distributionTemperatureVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  distribution temperature call");

      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = distributionTemperatureVoltage;
      /* write voltage value in OD */
      DO_WRITE(g_DoDistributionTemperature.Voltage , pMeasData->rawMeas.fVoltage);
   }

   /* Read RAW Distribution Temperature in OD */
   DO_READ(pMeasData->rawMeas.fVoltage , g_DoDistributionTemperature.Voltage);

   /* Calibration measure computation */
   /* =============================== */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoDistributionTemperature.CalibAdjCalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoDistributionTemperature.CalibAdjCalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoDistributionTemperature.CalibAdjResistance,fThermRes);
   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoDistributionTemperature.CalibAdjInstantTemperature,fTemperature);

   /* Standard measure computation */
   /* ============================ */
   /* Read Calibration gain in OD */
   DO_READ(fTempCalibGain , g_DoDistributionTemperature.CalibrationGain);
   /* Read Calibration offset in OD */
   DO_READ(fTempCalibOffset , g_DoDistributionTemperature.CalibrationOffset);

   /* retrieve Thermistor resistivity measure */
   fThermRes=ComputeThermResitivity(pMeasData->rawMeas.fVoltage,fTempCalibGain);
   /* write Thermistor resistivity measure in DO */
   DO_WRITE(g_DoDistributionTemperature.Resistance,fThermRes);

   /* Compute Thermistor temperature */
   fTemperature=ComputeTemperature(fThermRes,fTempCalibOffset);
   /* Store temperature in context */
   pMeasData->fMeasure=fTemperature;
   /* Write Instant temp in OD */
   DO_WRITE(g_DoDistributionTemperature.InstantTemperature, pMeasData->fMeasure);

   /* Check Alarms */
   if((FLOAT)OUT_OF_RANGE_MAX_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(DISTRIBUTION_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MIN);
   }
   else if((FLOAT)OUT_OF_RANGE_MIN_MEASURE == fTemperature)
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(DISTRIBUTION_TEMPERATURE,TRUE);
      MES_SetAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MIN);
      MES_ClearAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MAX);
   }
   else
   {
      /* Update sensors diag */
      MES_WarnPeriodicSensorErrorDiag(DISTRIBUTION_TEMPERATURE,FALSE);
      MES_ClearAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MAX);
      MES_ClearAlarm(ALARM_TYPE_DISTRIB_TEMPERATURE_MIN);
   }

   /* Check Process depedant Alarm */
   if(TRUE == g_MeasCheckAlarmArray[MES_CHK_ALM_PRODUCT_TEMP_SP].isEnable)
   {
      /* Set Point check */
      if( (pMeasData->fMeasure > pMeasData->fSetPointAlarmThres)
       && ((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure )
       && ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         MES_SetCheckAlarm(MES_CHK_ALM_PRODUCT_TEMP_SP);
      }
      else
      {
         MES_ClearCheckAlarm(MES_CHK_ALM_PRODUCT_TEMP_SP);
      }
   }
#if 1
   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      if (((FLOAT)OUT_OF_RANGE_MIN_MEASURE != pMeasData->fMeasure )
       && ((FLOAT)OUT_OF_RANGE_MAX_MEASURE != pMeasData->fMeasure ) )
      {
         FLOAT fTempValue = ((pMeasData->fMeasure * 1000.0f)+0.5f);
         /* Write Instant Feed Temperature in OD RO temperature field */
         DO_WRITE(g_DoWaterQuality.DistributionTemperature, (UNSIGNED16)fTempValue);
      }
      else
      {
         /* Write Instant Feed Temperature in OD RO temperature field */
         DO_WRITE(g_DoWaterQuality.DistributionTemperature, (UNSIGNED16)INVALID_RECORD_MEASURE);
      }

      /* Set PDO record to be send */
      MES_CanRecordUpdate(TPDO_DISTRIBUTION_TEMPERATURE);
   }
#else
   /* Check Process depedant Record */
   if (TRUE == pMeasData->isProcDepRecordEn)
   {
      /* Accumuate Instant Distribution Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;

      /* on second boundary, compute average and update PDO Record */
      if (TRUE == isSecondBoundary)
      {
         pMeasData->fMeasure = pMeasData->fMeasAccumulator / pMeasData->dwNbSplMeasure;

         /* Clear average */
         pMeasData->fMeasAccumulator = 0.0f;
         pMeasData->dwNbSplMeasure = 0u;
      }

   }
   else if (TRUE == pMeasData->isProcDepAverageEn)
   {
      /* Accumuate Instant Distribution Temperature */
      pMeasData->fMeasAccumulator += pMeasData->fMeasure;
      pMeasData->dwNbSplMeasure++;
   }
#endif
}


/*!
 *  @return None
 *
 *  @brief Initialize temperature acquistion drivers
 *
 *  @details
 */
VOID MES_InitTemperatureDrivers(VOID)
{
   MeasData_t* CONST pMeasFeedTemperatureData = &g_MeasDataArray[FEED_TEMPERATURE];
   MeasData_t* CONST pMeasPermeateTemperatureData = &g_MeasDataArray[PERMEATE_TEMPERATURE];
   MeasData_t* CONST pMeasEdiTemperatureData = &g_MeasDataArray[EDI_TEMPERATURE];
   MeasData_t* CONST pMeasDistributionTemperatureData = &g_MeasDataArray[DISTRIBUTION_TEMPERATURE];

   DIL_RC   ret;

   /* DIL feed temperature init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)FEED_TEMPERATURE))
   {
      ret = dil_opendevice(DIL_NAME_TEMPERATURE_FEED, &dilFeedTemperatureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Feed Temperature init");
   }
   pMeasFeedTemperatureData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pMeasFeedTemperatureData->dwNbSplMeasure = 0u;
   pMeasFeedTemperatureData->fMeasAccumulator = 0.0f;
   pMeasFeedTemperatureData->pStopRecordFunc = NULL;
   pMeasFeedTemperatureData->isProcDepMeasEn = TRUE;
   pMeasFeedTemperatureData->isProcDepAverageEn = FALSE;
   pMeasFeedTemperatureData->isProcDepRecordEn = FALSE;

   pMeasFeedTemperatureData->bAverageTpdo = TPDO_NOT_MAPPED;


   /* DIL Permeate temperature init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)PERMEATE_TEMPERATURE))
   {
      ret = dil_opendevice(DIL_NAME_TEMPERATURE_PERMEATE, &dilPermeateTemperatureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Permeate Temperature init");
   }

   /* Read Permeate temperature SP in OD */
   DO_READ(pMeasPermeateTemperatureData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.FeedTemperatureSp);
   pMeasPermeateTemperatureData->dwNbSplMeasure = 0u;
   pMeasPermeateTemperatureData->fMeasAccumulator = 0.0f;
   pMeasPermeateTemperatureData->pStopRecordFunc = NULL;
   pMeasPermeateTemperatureData->isProcDepMeasEn = TRUE;
   pMeasPermeateTemperatureData->isProcDepAverageEn = FALSE;
   pMeasPermeateTemperatureData->isProcDepRecordEn = FALSE;

   pMeasPermeateTemperatureData->bAverageTpdo = TPDO_NOT_MAPPED;


   /* DIL Edi temperature init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)EDI_TEMPERATURE))
   {
      ret = dil_opendevice(DIL_NAME_TEMPERATURE_EDI, &dilEdiTemperatureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Edi Temperature init");
   }
   pMeasEdiTemperatureData->fSetPointAlarmThres = (FLOAT)OUT_OF_RANGE_MAX_MEASURE;  /* not used */
   pMeasEdiTemperatureData->dwNbSplMeasure = 0u;
   pMeasEdiTemperatureData->fMeasAccumulator = 0.0f;
   pMeasEdiTemperatureData->pStopRecordFunc = NULL;
   pMeasEdiTemperatureData->isProcDepMeasEn = TRUE;
   pMeasEdiTemperatureData->isProcDepAverageEn = FALSE;
   pMeasEdiTemperatureData->isProcDepRecordEn = FALSE;

   pMeasEdiTemperatureData->bAverageTpdo = TPDO_NOT_MAPPED;


   /* DIL Distribution temperature init */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)DISTRIBUTION_TEMPERATURE))
   {
      ret = dil_opendevice(DIL_NAME_TEMPERATURE_DISTRIBUTION, &dilDistributionTemperatureHdl, (DIL_PVOID)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  Distribution Temperature init");
   }
   /* Read Product temperature SP in OD */
   DO_READ(pMeasDistributionTemperatureData->fSetPointAlarmThres,g_DoAlertAndAlarmsSetpoints.ProductTemperatureSp);
   pMeasDistributionTemperatureData->dwNbSplMeasure = 0u;
   pMeasDistributionTemperatureData->fMeasAccumulator = 0.0f;
   pMeasDistributionTemperatureData->pStopRecordFunc = NULL;
   pMeasDistributionTemperatureData->isProcDepMeasEn = TRUE;
   pMeasDistributionTemperatureData->isProcDepAverageEn = FALSE;
   pMeasDistributionTemperatureData->isProcDepRecordEn = FALSE;

   pMeasDistributionTemperatureData->bAverageTpdo = TPDO_NOT_MAPPED;

}

/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

