/*!
 * @file mes_tor.c
 *
 * @brief Pressures acquisition
 *
 * @details Contain pressures acuisition procedures
 *
 * @cond Copyright
 * (c) Copyright 2013 MILLIPORE Corporation unpublished work.
 * This computer program includes Confidential, Proprietary Information
 * and is a Trade Secret of MILLIPORE corporation.
 * All use, disclosure, and/or reproduction is prohibited
 * unless authorized in writing.
 * All rights reserved.
 * @endcond
 *
 * @cond SVN
 * $Id:: mes_tor.c 523 2014-08-#$: Filename
 * $Rev:: 523                   $: Revision of last commit
 * $Author:: m175711            $: Author of last commit
 * $Date:: 2014-08-07 11:12:09 #$: Date of last commit
 * $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/measure/src/mes_tor.c $: URL of this file
 * @endcond
 *
 */

/* Defines -------------------------------------------------------------------*/



/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "boardcfg.h"
#include "sys_softerr.h"
#include "system_parameters.h"
#include "os_adaptation.h"
#include "sys_api.h"
#include "mes_api.h"
#include "mes_data.h"
#include "do_itf.h"
#include "do_def.h"
#include "rop_api.h"
#include "dip_api.h"
#include "dil_api.h"
#include "dil_tor_in.h"
#include "dil_ld_ana.h"
#include "aaa_api.h"
#include "mes_can.h"
#include "measure.h"



/* Groups Definition ---------------------------------------------------------*/
/*! @defgroup TOR_Measure TOR measure
 *  @ingroup Measure
 *  @brief TOR measure acquistion procedures
 *  @{
 */

/* Defintions -----------------------------------------------------------------*/
#define MES_USE_CALLBACK_FOR_TOR_UPDATE   /*!< Used to setup call back use for TOR in*/

/* Declarations --------------------------------------------------------------*/
static VOID MES_WarnTORChangedCallBack(VOID);

/*! This varialble is set as volatile because value can change during Isr treatment.
 *  TRUE if TOR state is changed, and thus need to read .
 */
extern volatile BOOL g_isTorsStatusModified;

extern BOOL g_doMesUpdateTorStatus; /*!< TRUE if TOR Gpio status shall be read */

static DIL_HANDLE dilTORSdsLeakDetectorHdl = NULL;   /*!< SDS leak detector DIL handle */
static DIL_HANDLE dilTORTankOverflowHdl = NULL;      /*!< Tank overflow DIL handle */
static DIL_HANDLE dilTORFaultReport1Hdl = NULL;      /*!< Fault 1 report DIL handle */
static DIL_HANDLE dilTORFaultReport2Hdl = NULL;      /*!< Fault 2 report DIL handle */
static DIL_HANDLE dilTORBackwashCarbonHdl = NULL;    /*!< Backwash carbon DIL handle */
static DIL_HANDLE dilTORPodStatusHdl = NULL;         /*!< Pod status DIL handle */
static DIL_HANDLE dilTORTankRefillHdl = NULL;        /*!< Tank refill DIL handle */
static DIL_HANDLE dilLeakDetectorHdl = NULL;         /*!< Leak detector DIL handle  (TOR or Analog) */
#ifdef INCLUDE_ANALOG_LEAK_DETECTOR
static BOOL IsAnalogLeakDetectorResetAlarmAllowed=FALSE; /*!< TRUE to allow Leak detector alarm reset  */
#endif

/*! Tor to alarm array store alarm type and Tor dil pointer relative a sensor.\n
 *   The goal is to allow test a sensor and reset coresponding alarm if needed
 */
static CONST struct
{
   AlarmType_t alarmType;
   DIL_HANDLE *pDilHdl;
} TorSensorToAlarmArray [] =
{
   { .alarmType=ALARM_TYPE_TOR_LEAK_DETECTOR,      .pDilHdl =&dilLeakDetectorHdl    }, /* LEAK_DETECTOR */
   { .alarmType=ALARM_TYPE_TOR_SDS_LEAK_DETECTOR,  .pDilHdl =&dilTORSdsLeakDetectorHdl }, /* TOR_SDS_LEAK_DETECTOR */
   { .alarmType=ALARM_TYPE_TOR_TANK_OVERFLOW,      .pDilHdl =&dilTORTankOverflowHdl    }, /* TOR_TANK_OVERFLOW */
};

#define MORE_LOG
#ifdef MORE_LOG
CONST PLAINCHAR * CONST szMesTorNameArray[]=
{
   "TOR UNKNOWN !?        ",
   "LEAK_DETECTOR         ",
   "TOR_SDS_LEAK_DETECTOR ",
   "TOR_TANK_OVERFLOW     ",
   "TOR_FAULT_REPORT1     ",
   "TOR_FAULT_REPORT2     ",
   "TOR_BACKWASH_CARBON   ",
   "TOR_POD_STATUS        ",
   "TOR_TANK_REFILL       "
};
#endif

/* Codes ---------------------------------------------------------------------*/


/*! @brief  Log Tor State
 *
 *
 *  @param  torSensor [IN] Sensor_t <br>
 *                Sensor modified ID
 *
 *  @param  isTorSet [IN] BOOL <br>
 *             Tor sensor state
 * @return None
 */
static VOID _LogTorState(Sensor_t torSensor, BOOL isTorSet)
{
   NOT_USED_PARAM(torSensor);
#ifdef MORE_LOG
   UINT32 szIdx=0;
   if(LEAK_DETECTOR <= torSensor && TOR_TANK_REFILL >= torSensor)
   {
      szIdx=(UINT32)(1u+torSensor-LEAK_DETECTOR);
   }

   if(!isTorSet)
   {
      SYSLOG(INFO,"M:TOR %s RESET",szMesTorNameArray[szIdx]);
   }
   else
   {
      SYSLOG(INFO,"M:TOR %s SET",szMesTorNameArray[szIdx]);
   }
#endif // MORE_LOG
}


/*! @brief  Update dictionnary object
 *
 *  @param  isTorSet [IN] BOOL <br>
 *             Tor sensor state
 *
 *  @param  torSensor [IN] Sensor_t <br>
 *                Sensor modified ID
 *
 * @return None
 */
static VOID UpdateOdTorStatus(BOOL isTorSet,Sensor_t torSensor)
{
   UINT8 bTorNbr;
   UINT8 bStatus, bMask;   /* bForcedValue;*/

   ASSERT((TOR_TANK_REFILL>=torSensor) && (LEAK_DETECTOR<=torSensor),"Wrong sensor");
   DO_READ(bStatus,g_DoDigitalEntriesStatus[DIGITAL_ENTRIES_STATUS]);
   DO_READ(bMask,g_DoDigitalEntriesStatus[DISABLE_DIGITAL_ENTRIES_MASK]);
   bTorNbr= (UINT8)torSensor - (UINT8)LEAK_DETECTOR;
   /* check TOR masked */
   if((UINT8)0u == (bMask & (UINT8)(1u<<bTorNbr)))
   {
      /* Not masked */
      if(TRUE == isTorSet)
      {
         /* Set TOR status */
         bStatus|=((UINT8)1<<bTorNbr);
      }
      else
      {
         /* Set TOR status */
         bStatus&=~((UINT8)1<<bTorNbr);
      }
   }
   DO_WRITE(g_DoDigitalEntriesStatus[0],bStatus);
}


/*! @brief  Return TOR status stored in dictionnay object
 *
 *  @param  torSensor [IN] Sensor_t <br>
 *                Sensor modified ID
 *
 * @return Tor statsus  [as]  BOOL
 */
static BOOL GetOdTorStatus(Sensor_t torSensor)
{
   UINT8 bTorNbr;
   UINT8 bStatus;
   ASSERT((TOR_TANK_REFILL>=torSensor) && (LEAK_DETECTOR<=torSensor),"Wrong sensor");
   bTorNbr = (UINT8)torSensor - (UINT8)LEAK_DETECTOR;
   DO_READ(bStatus,g_DoDigitalEntriesStatus[DIGITAL_ENTRIES_STATUS]);
   if((UINT8)0u != (bStatus & (UINT8)(1u<<bTorNbr)))
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
}


/*!
 *  @param  torSensor [IN] Sensor_t <br>
 *                Sensor modified ID
 *
 *  @param  dilHdl [IN] DIL_HANDLE <br>
 *                DIL tor sensor handle.
 *
 * @return none
 *
 * @brief  Provide a diagnotic on TOR
 *
 * @details run at initialization stage to provide a status on TOR.\n
 *          If TorState is high , TOR is considered broken.
 *
 */
VOID DiagnoseTor(Sensor_t torSensor,DIL_HANDLE dilHdl)
{
   BOOL isTorSet = FALSE;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[torSensor];

   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)torSensor))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0;
      ret = dil_ioctldevice(dilHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,torSensor);
   }
   /* update diagnostic */
   MES_WarnPeriodicSensorErrorDiag(torSensor,isTorSet);
}


/*!
 *  @brief Allow reseting Analog leak detector alarm.
 *
 *  @details Only available for Analog leak detector.
 *
 */
VOID MES_AllowAnalogLeakDetectorResetAlarm(VOID)
{
#ifdef INCLUDE_ANALOG_LEAK_DETECTOR
   /* Allow reset alarm  */
   if(AAA_STATE_DISABLE != AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))
   {
      IsAnalogLeakDetectorResetAlarmAllowed=TRUE;
   }
#endif
}


/*!
 *  @param sensor [IN] Sensor_t <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return Nothing
 *
 *  @brief Update TOR status before reseting alarm.
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *    Reset alarm is done with IHM command.
 *    But a check is required to see if Tor is cleared before reset alarm.\n
 *    This procedure use #TorSensorToAlarmArray to retrieve dil and alarm relative to given sensor.
 *    It is available only for LEAK_DETECTOR,TOR_SDS_LEAK_DETECTOR, and TOR_TANK_OVERFLOW sensor.\n
 *    Other TOR sensors are reseted without IHM action.
 */
VOID MES_CheckTORResetAlarm(Sensor_t sensor)
{
   BOOL isTorSet;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[sensor];

   ASSERT(LEAK_DETECTOR == sensor || TOR_SDS_LEAK_DETECTOR == sensor || TOR_TANK_OVERFLOW == sensor ,"Wrong TOR sensor test");
   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)sensor))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0;
      ret = dil_ioctldevice(*TorSensorToAlarmArray[sensor-LEAK_DETECTOR].pDilHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,sensor);
   }
   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(sensor);

   /* Check Process independant Alarm only if option available */
   if((AAA_STATE_DISABLE != AAA_GetAlarmState(TorSensorToAlarmArray[sensor-LEAK_DETECTOR].alarmType))
   && (FALSE == isTorSet) )
   {
      /* AAA_ResetAlarm allow reseting pending or active alarm. */
      AAA_ResetAlarm(TorSensorToAlarmArray[sensor-LEAK_DETECTOR].alarmType);
   }
}


/*!
 *
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   Not used.
 *
 *  @return void
 *
 *  @brief ANalag leak detector measure (every 500ms)
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, call Analaog DIL leak detetction read
 *     -# Check mode independant Alarm by comparing to a threshold define in system paramater header file.
 */
VOID MES_UpdateAnalogLeakDetector(BOOL isSecondBoundary)
{
#ifdef INCLUDE_ANALOG_LEAK_DETECTOR
   BOOL isLeakDetected=FALSE;
   FLOAT fLeakDetectorVoltage;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[LEAK_DETECTOR];

   NOT_USED_PARAM(isSecondBoundary);

   /* If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   {
      return;
   }

   /* If DIL not bypassed, call Analaog DIL leak detetction read */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)LEAK_DETECTOR))
   {
      DIL_RC   ret;
      /* call DIL */
      ret = dil_ioctldevice(dilLeakDetectorHdl, IOCTL_CMD_LEAK_DETECTOR_ANA_GET_VALUE ,
                            (DIL_LPVOID)&fLeakDetectorVoltage, sizeof(fLeakDetectorVoltage), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  pressure call");
      /* Save Raw data */
      pMeasData->rawMeas.fVoltage = fLeakDetectorVoltage;
      /*  Save Raw data */
      pMeasData->fMeasure =fLeakDetectorVoltage;

      /* Convert UINT8 to BOOL */
      isLeakDetected= (ALARM_ANALOG_LEAK_DETECTION_THRES > fLeakDetectorVoltage  ? TRUE : FALSE);
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isLeakDetected,LEAK_DETECTOR);
   }

   /* Retrieve OD TOR status */
   isLeakDetected = GetOdTorStatus(LEAK_DETECTOR);

   /* Check Process independant Alarms only if option available */
   if(TRUE==SYS_GetDeviceOption(SYSTEM_OPT_WATER_SENSOR))
   {
      /* Check set Alarm */
      if((AAA_STATE_DISABLE == AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))
      && (TRUE == isLeakDetected) )
      {
         AAA_SetAlarm(ALARM_TYPE_TOR_LEAK_DETECTOR);
         IsAnalogLeakDetectorResetAlarmAllowed=FALSE;
      }

      /* Check reset Alarm allowed / acknowledged */
      if(TRUE == IsAnalogLeakDetectorResetAlarmAllowed)
      {
         if ((AAA_STATE_DISABLE != AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))
              && (FALSE == isLeakDetected))
         {
            /* AAA_ResetAlarm allow reseting pending or active alarm. */
            AAA_ResetAlarm(ALARM_TYPE_TOR_LEAK_DETECTOR);
         }
         IsAnalogLeakDetectorResetAlarmAllowed=FALSE;
      }
   }
   else
   {
      if (AAA_STATE_DISABLE != AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))
      {
         /* If the alarm is set but the option is disabled, we must reset the alarm */
         AAA_ResetAlarm(ALARM_TYPE_TOR_LEAK_DETECTOR);
      }
   }
#else // INCLUDE_ANALOG_LEAK_DETECTOR
   NOT_USED_PARAM(isSecondBoundary);
#endif // INCLUDE_ANALOG_LEAK_DETECTOR
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Check mode independant Alarms
 *
 *   Alarm is only set here.  Reset alarm shall be done with IHM command
 */
VOID MES_UpdateTORLeakDetector(BOOL isSecondBoundary)
{
#ifndef INCLUDE_ANALOG_LEAK_DETECTOR
   BOOL isTorSet=0;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[LEAK_DETECTOR];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);

   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)LEAK_DETECTOR))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue = 0u;
      ret = dil_ioctldevice(dilLeakDetectorHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,LEAK_DETECTOR);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(LEAK_DETECTOR);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(LEAK_DETECTOR);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(LEAK_DETECTOR,isTorSet);
   }

   /* Check Process independant Alarms only if option available */
   if((TRUE == SYS_GetDeviceOption(SYSTEM_OPT_WATER_SENSOR))
   && (AAA_STATE_DISABLE == AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))
   && (TRUE == isTorSet) )
   {
      AAA_SetAlarm(ALARM_TYPE_TOR_LEAK_DETECTOR);
   }
#else // INCLUDE_ANALOG_LEAK_DETECTOR
   NOT_USED_PARAM(isSecondBoundary);
#endif // !INCLUDE_ANALOG_LEAK_DETECTOR
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Check mode independant Alarms
 *
 *   Alarm is only set here.  Reset alarm shall be done with IHM command
 */
VOID MES_UpdateTORSDSLeakDetector(BOOL isSecondBoundary)
{
   BOOL isTorSet=0;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_SDS_LEAK_DETECTOR];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);
   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_SDS_LEAK_DETECTOR))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0;
      ret = dil_ioctldevice(dilTORSdsLeakDetectorHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_SDS_LEAK_DETECTOR);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_SDS_LEAK_DETECTOR);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_SDS_LEAK_DETECTOR);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_SDS_LEAK_DETECTOR,isTorSet);
   }

   /* Check Process independant Alarms */
   if((TRUE == SYS_GetDeviceOption(SYSTEM_OPT_SDS_WATER_SENSOR))
   && (AAA_STATE_DISABLE == AAA_GetAlarmState(ALARM_TYPE_TOR_SDS_LEAK_DETECTOR))
   && (TRUE == isTorSet) )
   {
      AAA_SetAlarm(ALARM_TYPE_TOR_SDS_LEAK_DETECTOR);
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Check mode independant Alarms
 *
 *   Alarm is only set here.  Reset alarm shall be done with IHM command
 */
VOID MES_UpdateTORTankOverflow(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_TANK_OVERFLOW];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);

   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_TANK_OVERFLOW))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue = 0U;
      ret = dil_ioctldevice(dilTORTankOverflowHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_TANK_OVERFLOW);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_TANK_OVERFLOW);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_TANK_OVERFLOW);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_TANK_OVERFLOW,isTorSet);
   }


   /* Check Process independant Alarms
    * Alarm is clear/ack by user CAN msg
    */
   if( (AAA_STATE_DISABLE == AAA_GetAlarmState(ALARM_TYPE_TOR_TANK_OVERFLOW))
    && (TRUE == isTorSet) )
   {
      AAA_SetAlarm(ALARM_TYPE_TOR_TANK_OVERFLOW);
   }
   /* Bug fix ,
    * At system startup, UV switch on trig a tankoverflow detection
    * To cope this, a delay is placed to confirm alarm,
    * If during this delay [ PENDING state alarm ] ,overflow disappear, we clear the alarm
    * Otherwise alarm is raised to MMI, and shall be ACK to clear it.
    */
   if( (AAA_STATE_PENDING == AAA_GetAlarmState(ALARM_TYPE_TOR_TANK_OVERFLOW))
    && (FALSE == isTorSet) )
   {
      AAA_ResetPendingAlarm(ALARM_TYPE_TOR_TANK_OVERFLOW);
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Check mode independant Alarms
 */
VOID MES_UpdateTORFaultReport1(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   BOOL isAlarmActive;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_FAULT_REPORT1];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);

   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_FAULT_REPORT1))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0u;
      ret = dil_ioctldevice(dilTORFaultReport1Hdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_FAULT_REPORT1);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_FAULT_REPORT1);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_FAULT_REPORT1);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_FAULT_REPORT1,isTorSet);
   }

   /* Check Process independant Alarms */
   isAlarmActive= (AAA_STATE_DISABLE != AAA_GetAlarmState(ALARM_TYPE_TOR_FAULT_REPORT1) ? TRUE : FALSE);
   if( isAlarmActive != isTorSet)
   {
      if (TRUE == isTorSet)
      {
         AAA_SetAlarm(ALARM_TYPE_TOR_FAULT_REPORT1);
      }
      else
      {
         AAA_ResetAlarm(ALARM_TYPE_TOR_FAULT_REPORT1);
      }
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Check mode independant Alarms
 */
VOID MES_UpdateTORFaultReport2(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   BOOL isAlarmActive;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_FAULT_REPORT2];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);

   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_FAULT_REPORT2))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0u;
      ret = dil_ioctldevice(dilTORFaultReport2Hdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_FAULT_REPORT2);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_FAULT_REPORT2);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_FAULT_REPORT2);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_FAULT_REPORT2,isTorSet);
   }

   /* Check Process independant Alarms */
   isAlarmActive= (AAA_STATE_DISABLE != AAA_GetAlarmState(ALARM_TYPE_TOR_FAULT_REPORT2) ? TRUE : FALSE);
   if( isAlarmActive != isTorSet)
   {
      if (TRUE == isTorSet)
      {
         AAA_SetAlarm(ALARM_TYPE_TOR_FAULT_REPORT2);
      }
      else
      {
         AAA_ResetAlarm(ALARM_TYPE_TOR_FAULT_REPORT2);
      }
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Warn \p Production process of backwash state change
 */
VOID MES_UpdateTORBackwashCarbon(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_BACKWASH_CARBON];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);
   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_BACKWASH_CARBON))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0;
      ret = dil_ioctldevice(dilTORBackwashCarbonHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_BACKWASH_CARBON);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_BACKWASH_CARBON);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_BACKWASH_CARBON);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_BACKWASH_CARBON,isTorSet);

      /* No Alarms check */

      /* Warn Process of backwash state change */
      ROP_MesBackwashCarbonChange(isTorSet);
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Warn \p Distribution process of Pod status state change
 */
VOID MES_UpdateTORPODStatus(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_POD_STATUS];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);
   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_POD_STATUS))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0u;
      ret = dil_ioctldevice(dilTORPodStatusHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_POD_STATUS);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_POD_STATUS);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }

   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_POD_STATUS);

   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_POD_STATUS,isTorSet);
      /* No Alarms check */

      /* Warn distribution process of Pod status state change */
      DIP_MesPodStatusChange(isTorSet);
   }
}


/*!
 *  @param isSecondBoundary [IN] BOOL <br>
 *                   is used to send Pdo on second boundary
 *                   as this proc is called with callback and
 *                   Record is expected to be sent every second.
 *
 *  @return void
 *
 *  @brief Update Tank refill with callback
 *
 *  @details
 *     -# if measure disabled, exit
 *     -# if DIL not bypassed, DIL Tor status read
 *     -# Warn \p Production process of tank refill state change
 */
VOID MES_UpdateTORTankRefill(BOOL isSecondBoundary)
{
   BOOL isTorSet;
   MeasData_t* CONST pMeasData = &g_MeasDataArray[TOR_TANK_REFILL];
   BOOL wasTorSet = (pMeasData->fMeasure==1.0F ? TRUE : FALSE);
   NOT_USED_PARAM(isSecondBoundary);
   /*  If Measure is not to be performed, exit */
   if (FALSE == pMeasData->isProcDepMeasEn)
   { return; }

   /* If TOR status to be updated */
   if (FALSE == g_doMesUpdateTorStatus)
   { return; }

   /* If DIL not bypassed, DIL is read */
   /*! it is assume that of DIL is bypassed , thus forced TOR value shall be set */
   if (FALSE == IS_DIL_READ_BYPASSED((UINT32)TOR_TANK_REFILL))
   {
      DIL_RC   ret;
      DIL_UINT8 bTorValue=0u;
      ret = dil_ioctldevice(dilTORTankRefillHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_GET_SIGNAL ,
                            (DIL_LPVOID)&bTorValue, sizeof(bTorValue), (DIL_LPUINT32)NULL);
      ASSERT(DIL_GOOD == ret,"Error in DIL  TOR get status init");
      /* Convert UINT8 to BOOL */
      isTorSet= (0u != bTorValue  ? TRUE : FALSE);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
      /* Update OD Tor status according mask and forced value */
      UpdateOdTorStatus(isTorSet,TOR_TANK_REFILL);
   }
   else
   {
      /* Retrieve OD TOR status */
      isTorSet = GetOdTorStatus(TOR_TANK_REFILL);
      /*  Save Raw data */
      pMeasData->fMeasure =( isTorSet==TRUE ? 1.0F : 0.0F );
   }
   /* Retrieve OD TOR status */
   isTorSet = GetOdTorStatus(TOR_TANK_REFILL);


   if(wasTorSet != isTorSet)
   {
      _LogTorState(TOR_TANK_REFILL,isTorSet);

      /* No Alarms check */

      /* Warn Process of tank refill state change */
      ROP_MesTankRefillChange(isTorSet);
   }
}


/*!
 * @fn MES_WarnTORChangedCallBack
 *
 *  @brief TOR change clall back
 *
 *  @details Called from ExtLine ISR
 *
 *  @return void
 */
static VOID MES_WarnTORChangedCallBack(VOID)
{
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   g_isTorsStatusModified=TRUE;
#endif
}



/*!
 *  @brief Initialize TOR drivers
 *
 *  @details
 */
VOID MES_InitTORDrivers(VOID)
{
#ifdef D_BSP_PROCESS_BOARD
   DIL_RC   ret;

#ifdef INCLUDE_ANALOG_LEAK_DETECTOR
   IsAnalogLeakDetectorResetAlarmAllowed=FALSE;

   ret = dil_opendevice(DIL_NAME_LEAKDETECTOR_ANALOGIC, &dilLeakDetectorHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  Analaog Leak detector init");
#else
   ret = dil_opendevice(DIL_NAME_TOR_INPUT_LEAK_DETECTOR, &dilLeakDetectorHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR Leak detector init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilLeakDetectorHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR leak detector call back  init");
#endif
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_SDS_LEAK_DETECTOR, &dilTORSdsLeakDetectorHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL TOR SDS Leak detector init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORSdsLeakDetectorHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR SDS leak detector  call back  init");
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_TANK_OVERFLOW, &dilTORTankOverflowHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR Tank overflow  init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORTankOverflowHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR tank overflow call back  init");
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_FAULT_REPORT_1, &dilTORFaultReport1Hdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR fault report1 init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORFaultReport1Hdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR fault report1 call back  init");
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_FAULT_REPORT_2, &dilTORFaultReport2Hdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR fault report2");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORFaultReport2Hdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR fault report2 call back  init");
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_BACKWASH_CARBON, &dilTORBackwashCarbonHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR backwash init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORBackwashCarbonHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR backwash call back  init");
#endif

   ret = dil_opendevice(DIL_NAME_TOR_INPUT_POD_STATUS, &dilTORPodStatusHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR pad status  init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORPodStatusHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR Pod status call back  init");
#endif
   ret = dil_opendevice(DIL_NAME_TOR_INPUT_TANK_REFILL, &dilTORTankRefillHdl, (DIL_PVOID)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR tank refill  init");
#ifdef MES_USE_CALLBACK_FOR_TOR_UPDATE
   ret = dil_ioctldevice(dilTORTankRefillHdl, (DIL_UINT32)IOCTL_CMD_TOR_IN_CALLBACK,
                         (DIL_LPVOID)&MES_WarnTORChangedCallBack, sizeof(DIL_LPVOID), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD == ret,"Error in DIL  TOR tank refill call back  init");
#endif
#endif
   g_MeasDataArray[LEAK_DETECTOR].bAverageTpdo        =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_SDS_LEAK_DETECTOR].bAverageTpdo=TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_TANK_OVERFLOW].bAverageTpdo    =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_FAULT_REPORT1].bAverageTpdo    =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_FAULT_REPORT2].bAverageTpdo    =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_BACKWASH_CARBON].bAverageTpdo  =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_POD_STATUS].bAverageTpdo       =TPDO_NOT_MAPPED;
   g_MeasDataArray[TOR_TANK_REFILL].bAverageTpdo      =TPDO_NOT_MAPPED;

   g_MeasDataArray[LEAK_DETECTOR].isProcDepMeasEn        =TRUE;
   g_MeasDataArray[TOR_SDS_LEAK_DETECTOR].isProcDepMeasEn=TRUE;
   g_MeasDataArray[TOR_TANK_OVERFLOW].isProcDepMeasEn    =TRUE;
   g_MeasDataArray[TOR_FAULT_REPORT1].isProcDepMeasEn    =TRUE;
   g_MeasDataArray[TOR_FAULT_REPORT2].isProcDepMeasEn    =TRUE;
   g_MeasDataArray[TOR_BACKWASH_CARBON].isProcDepMeasEn  =TRUE;
   g_MeasDataArray[TOR_POD_STATUS].isProcDepMeasEn       =TRUE;
   g_MeasDataArray[TOR_TANK_REFILL].isProcDepMeasEn      =TRUE;
}


/*!
 *  @}
 */


/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

