/*!
* @file rop_api.c
*
* @brief Production task API
*
* @details API definition to interface *Production module with others modules
*          such as *AlertAndAlarm or *Measure
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: rop_api.c 690 2014-12-24 09:31:40#$: Filename
* $Rev:: 690                              $: Revision of last commit
* $Author:: m175711                       $: Author of last commit
* $Date:: 2014-12-24 10:31:40 +0100 (mer.#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/production/src/rop_api.c $: URL of this file
* @endcond
*
*/

/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @ingroup Rop_API
 *  @{
 */


/* Includes ------------------------------------------------------------------*/
#include "millipore_types.h"
#include "msg_type.h"
#include "rop_data.h"
#include "sys_softerr.h"
#include "rop_private.h"

/* Defines -------------------------------------------------------------------*/

/* Declarations --------------------------------------------------------------*/
#ifndef NDEBUG
UINT32 g_ropWakeUpCnt = 0U;           /*<! allow test Wakeup Message reception */
#endif

/* Code ----------------------------------------------------------------------*/


/*!
 *  @brief  Start Production task
 *
 *  @details  Task is declared statically but not started because
 *            start order is set to 0. Once called , the task is started
 *
 */
VOID ROP_TaskStart(VOID)
{
   eOsRc_t rc=OS_TaskExecute("ROPTASK");
   ASSERT( OS_GOOD == rc ,"Unable to start Production task");
}


/*!
 *  @param order [IN] Order_t <br>
 *                   Order type
 *
 *  @return None
 *  @retval None
 *
 *  @brief  send comand (order) to Production task
 *
 *  @details   None
 *
 */
VOID ROP_SendComand(RopOrder_e order)
{

   sMSG_t Msg;
   eMSG_Rc_t rc;

   Msg.body.Rop.cmd = ROP_CMD_SEND_ORDER;
   Msg.body.Rop.extraField.order = order;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}


/*!
 *  @param csmType [IN] CSM_Type_t <br>
 *                   CSM type
 *
 *  @param csmStatus [IN] CSM_Status_t <br>
 *                   new CSM status
 *
 *  @brief  CSM status change API
 *
 *  @details   Allow \p Consumable task to inform \p Production task from a consumable change.
 *
 */
VOID ROP_CsmStatusChange(CSM_Type_t csmType, CSM_Status_t csmStatus)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_CSM_STATUS_CHANGE;
   Msg.body.Rop.extraField.csmStatusChange.csmType = csmType;
   Msg.body.Rop.extraField.csmStatusChange.csmStatus = csmStatus;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}


/*!
 *  @param isTankRefilNeeded [IN] BOOL <br>
 *                TRUE if Tank refill switch on
 *
 *  @brief  MES tank refill change API
 *
 *  @details   Allow \p Mesure task to inform \p Production task from Tank refill switch change
 *
 */
VOID ROP_MesTankRefillChange(BOOL isTankRefilNeeded)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_MES_TANK_REFILL_CHANGE;
   Msg.body.Rop.extraField.isTankRefilNeeded = isTankRefilNeeded;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}


/*!
 *  @param isBackwashCarbonOnGoing [IN] BOOL <br>
 *                TRUE if backwash carbon on going
 *
 *  @brief  MES Backwash carbon status change API
 *
 *  @details   Allow \p Mesure task to inform \p Production task from backwash carbon status change
 *
 */
VOID ROP_MesBackwashCarbonChange(BOOL isBackwashCarbonOnGoing)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_MES_TANK_REFILL_CHANGE;
   Msg.body.Rop.extraField.isBackwashCarbonOnGoing = isBackwashCarbonOnGoing;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}


/*!
 *  @param alarmType [IN] AlarmType_t <br>
 *                   Alarm type
 *
 *  @param isAlarmSet [IN] BOOL <br>
 *                   TRUE is Alarm set
 *
 *  @brief  New alarm raised API
 *
 *  @details   Allow \p Alert and Alarm  task to inform \p Production task from a new alarm.
 *
 */
VOID ROP_AlarmWakeup(AlarmType_t alarmType, BOOL isAlarmSet)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_ALARM_WAKEUP;
   Msg.body.Rop.extraField.alarmWakeup.alarmType = alarmType;
   Msg.body.Rop.extraField.alarmWakeup.isAlarmSet = isAlarmSet;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}


/*!
 *  @brief  A setpoint has been modified
 *
 *  @details   Allow \p CanOpen task to infomr \p Production task that setpoint has been modified
 *
 */
VOID ROP_UpdateSetpoints (VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_UPDATE_SETPOINT;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}

/*!
 *  @brief  A setpoint has been modified
 *
 *  @details   Allow \p CanOpen task to infomr \p Production task that setpoint has been modified
 *
 */
VOID ROP_UpdateFirstStartFlag (VOID)
{
   sMSG_t Msg;
   eMSG_Rc_t rc;
   Msg.body.Rop.cmd = ROP_CMD_UPDATE_FIRST_START_FLAG;
   rc=MSG_SendH(hRopMboxHld, &Msg, OS_MSG_PRIO_NORMAL );
   ASSERT( MSG_GOOD == rc,"Unable to post a message to Production mailbox");
}

/*!
 *  @brief  is the process in Test Mode
 *
 *  @details   Allow \p CanOpen task to know \p if Production task is in test mode
 *
 */
BOOL ROP_isProcessInTestMode (VOID)
{
   BOOL isTestMode = FALSE;
   
   if (ROP_MODE_TEST == ProductionProcessParameters.ProcessDescription.ProcessMode)
   {
      isTestMode = TRUE;
   }
   return(isTestMode);
}

/*!
 *  @}
 */


/*!
 *  @ingroup Production
 *  @{
 */


/*!
 *  @param  p_RopMsg :           [IN]    ROP_Msg_t           <br>
 *                      Production message
 *
 *  @return None
 *
 *  @brief  Production message parser
 */
void ROP_MsgParser(CONST  ROP_Msg_t* p_RopMsg)
{
   switch (p_RopMsg->cmd)
   {
      case ROP_CMD_ALARM_WAKEUP:
      {
#ifndef NDEBUG
         g_ropWakeUpCnt++;
#endif
         rop_productionStateMachine(ROP_LAST_ORDER);
         break;
      }

      case ROP_CMD_CSM_STATUS_CHANGE:
      case ROP_CMD_MES_TANK_REFILL_CHANGE:
      case ROP_CMD_MES_BACKWASH_CARBON_CHANGE:
      {
         rop_productionStateMachine(ROP_LAST_ORDER);
         break;
      }

      case ROP_CMD_SEND_ORDER:
      {
         rop_productionStateMachine((RopOrder_e)p_RopMsg->extraField.order);
         break;
      }

      case ROP_CMD_UPDATE_SETPOINT:
      {
         rop_readCanOpenSetpoints();
         break;
      }

      case ROP_CMD_UPDATE_FIRST_START_FLAG:
      {
         rop_readFirstStartFlag();
         break;
      }

      default:
      {
         ERROR("ROP_Parser : Msg type unknow: %4Xh ", p_RopMsg->cmd );
         break;
      }
   }
}


/*!
*  @}
*/

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

