/*!
* @file rop_state_machine.c
*
* @brief Production state machine
*
* @details Contain the state machine of the Production Process
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: rop_state_machine.c 683 2014-12-2#$: Filename
* $Rev:: 683                              $: Revision of last commit
* $Author:: m175711                       $: Author of last commit
* $Date:: 2014-12-22 11:36:49 +0100 (lun.#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/production/src/rop_state_machine.c $: URL of this file
* @endcond
*
*/

/* Groups Definition ---------------------------------------------------------*/
/*!
 *  @defgroup RopStateMachine State machine
 *  @brief Production state machine
 *  @ingroup Rop
 *  @{
 */

/* Defines -------------------------------------------------------------------*/

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "rop_api.h"
#include "rop_rec.h"
#include "rop_private.h"
#include "sys_softerr.h"
#include "rop_data.h"
#include "mes_api.h"
#include "aaa_api.h"
#include "csm_api.h"
#include "sys_api.h"
#include "sys_log.h"
#include "dip_api.h"

/* Declarations --------------------------------------------------------------*/
static RopSequence_e rop_getAssociatedSequence (RopMode_e RopMode);
static VOID rop_manageOrder(ROP_ProcessParam_t * p_ProcessParam, RopOrder_e NewOrder);
static VOID rop_manageAlarmStopAndPause(ROP_ProcessParam_t * p_ProcessParam);
static VOID rop_manageProgardsStatus(ROP_ProcessParam_t * p_ProcessParam);
static CSM_Status_t rop_getProgardsStatus (VOID);

ROP_ProcessParam_t ProductionProcessParameters;
BOOL cleaningSequenceStartedFromTheStart = FALSE;
static BOOL isAlarmUnpairedTrigged = FALSE;

/* Defines -------------------------------------------------------------------*/

/* Code ----------------------------------------------------------------------*/

/*!
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the state machine
 *
 *  @details
 *
 */
VOID rop_productionStateMachine(RopOrder_e Order)
{
   BOOL ReEnterMode=TRUE; // ReEnterMode is set to TRUE to forced one mode check at least 

   if (ROP_LAST_ORDER != Order)
   {
      // If the operator sends a command, process apply this command first
      rop_manageOrder(&ProductionProcessParameters, Order);
   }

   while (TRUE == ReEnterMode)
   {
      /* While the process is changing of mode:
       * The process checks the Alarm Stop and the Pause
       * Then the process checks the Progard status
       * And finally, the process performs the associated sequence */
      ProductionProcessParameters.ProcessDescription.isEnteringMode = FALSE;
      rop_manageAlarmStopAndPause(&ProductionProcessParameters);
      rop_manageProgardsStatus(&ProductionProcessParameters);
      rop_manageSequence(&ProductionProcessParameters);
      ReEnterMode = ProductionProcessParameters.ProcessDescription.isEnteringMode;
   }
   
   /* When mode and sequence are definives, the process checks the transitions and update the CanOpen Objets */
   rop_manageTransition(&(ProductionProcessParameters.TransitionDescription));
   ROP_CanUpdateProcessStatus(&ProductionProcessParameters);
}


/*!
 *  @param RopMode [IN] RopMode_e <br>
 *                    Process Mode
 *
 *  @return RopSequence_e : The associated sequence
 *
 *  @brief  Get the associated sequence of a process mode
 *
 *  @details   Allow the process to know which sequence to apply in a certain mode
 *
 */
static RopSequence_e rop_getAssociatedSequence (RopMode_e RopMode)
{
   RopSequence_e AssociatedSequence;
   switch(RopMode)
   {
      case ROP_MODE_INITIALIZATION:             {AssociatedSequence=ROP_SEQUENCE_INITIALIZATION; break;}
      case ROP_MODE_STANDBY_ALARM:              {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_STANDBY_OLD_PACK:           {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_STANDBY_NEW_PACK:           {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_STANDBY_BYPASS:             {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_STANDBY_TOOL_PACK:          {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_STANDBY_PACK_OVERRIDE:      {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_WAITING:                    {AssociatedSequence=ROP_SEQUENCE_WAITING; break;}
      case ROP_MODE_TANK_FILLING:               {AssociatedSequence=ROP_SEQUENCE_TANK_FILLING; break;}
      case ROP_MODE_DAILY_RO_DIAGNOSTIC:        {AssociatedSequence=ROP_SEQUENCE_RO_DIAGNOSTIC; break;}
      case ROP_MODE_ALARM_STOP:                 {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_PAUSE:                      {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_PROGARD_FLUSH:              {AssociatedSequence=ROP_SEQUENCE_PROGARD_FLUSH; break;}
      case ROP_MODE_PROGARD_FLUSH_ALARM_STOP:   {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_PROGARD_FLUSH_PAUSE:        {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_PROGARD_FLUSH: {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_PROGARD_FLUSH:      {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_FIRST_INSTALL:              {AssociatedSequence=ROP_SEQUENCE_FIRST_INSTALL; break;}
      case ROP_MODE_FIRST_INSTALL_ALARM_STOP:   {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_FIRST_INSTALL_PAUSE:        {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL: {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_FIRST_INSTALL:      {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RO_RINSE:                   {AssociatedSequence=ROP_SEQUENCE_RO_RINSE; break;}
      case ROP_MODE_RO_RINSE_ALARM_STOP:        {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_RO_RINSE_PAUSE:             {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_RO_RINSE:   {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_RO_RINSE:           {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_FORCED_RO_DIAGNOSTIC:       {AssociatedSequence=ROP_SEQUENCE_RO_DIAGNOSTIC; break;}
      case ROP_MODE_FORCED_RO_DIAGNOSTIC_ALARM_STOP:        {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_FORCED_RO_DIAGNOSTIC_PAUSE:             {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_PH_CLEANING:                {AssociatedSequence=ROP_SEQUENCE_PH_CLEANING; break;}
      case ROP_MODE_PH_CLEANING_ALARM_STOP:     {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_PH_CLEANING_PAUSE:          {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_START_AB_FLUSH:     {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_START_AB_FLUSH:             {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_CL2_CLEANING:               {AssociatedSequence=ROP_SEQUENCE_CL2_CLEANING; break;}
      case ROP_MODE_CL2_CLEANING_ALARM_STOP:    {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_CL2_CLEANING_PAUSE:         {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_CL2_CLEANING: {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_CL2_CLEANING:       {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_REAGENT_REMOVAL:            {AssociatedSequence=ROP_SEQUENCE_REAGENT_REMOVAL; break;}
      case ROP_MODE_REAGENT_REMOVAL_ALARM_STOP: {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_REAGENT_REMOVAL_PAUSE:      {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_REAGENT_REMOVAL_REQUESTED:  {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_START_REAGENT_REMOVAL:      {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_SAN1:                       {AssociatedSequence=ROP_SEQUENCE_SAN1; break;}
      case ROP_MODE_SAN1_ALARM_STOP:            {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_SAN1_PAUSE:                 {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_SAN1:       {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_SAN1:               {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_SAN2:                       {AssociatedSequence=ROP_SEQUENCE_SAN2; break;}
      case ROP_MODE_SAN2_ALARM_STOP:            {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_SAN2_PAUSE:                 {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_SAN2:       {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_SAN2:               {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_SAN3:                       {AssociatedSequence=ROP_SEQUENCE_SAN3; break;}
      case ROP_MODE_SAN3_ALARM_STOP:            {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_SAN3_PAUSE:                 {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      case ROP_MODE_CAN_NOT_RESTART_SAN3:       {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_RESTART_SAN3:               {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_CAN_NOT_TEST_H2O2:          {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_TEST_H2O2:                  {AssociatedSequence=ROP_SEQUENCE_STANDBY; break;}
      case ROP_MODE_TEST:                       {AssociatedSequence=ROP_SEQUENCE_TEST; break;}
      case ROP_MODE_TEST_RINSE:                 {AssociatedSequence=ROP_SEQUENCE_TEST_RINSE; break;}
      case ROP_MODE_TEST_FLUSH:                 {AssociatedSequence=ROP_SEQUENCE_TEST_FLUSH; break;}
      case ROP_MODE_TEST_ALARM_STOP:            {AssociatedSequence=ROP_SEQUENCE_ALARM_STOP; break;}
      case ROP_MODE_TEST_PAUSE:                 {AssociatedSequence=ROP_SEQUENCE_PAUSE; break;}
      default:
      {
         ERROR("Wrong RopMode %d",RopMode);
         AssociatedSequence=ROP_SEQUENCE_STANDBY;
         break;
      }
   }

   return(AssociatedSequence);
}

/*!
 *  @param RopMode [IN] RopMode_e <br>
 *                    Process Mode
 *
 *  @param pRopModeNominal [OUT] RopMode_e * <br>
 *                    Process Mode
 *
 *  @param pRopModeAlarmStop [OUT] RopMode_e * <br>
 *                    Process Mode
 *
 *  @param pRopModePause [OUT] RopMode_e * <br>
 *                    Process Mode
 *
 *  @return BOOL : If the requested mode have associated modes (because some modes do not have associated modes)
 *
 *  @brief  Get the associated mode (Nominal, AlarmStop and Pause) to a certain mode
 *
 *  @details   Allow the process to know which modes are associated (Nominal, AlarmStop and Pause)
 *
 */
BOOL rop_getAssociatedMode (RopMode_e RopMode,
                            RopMode_e * pRopModeNominal,
                            RopMode_e * pRopModeAlarmStop,
                            RopMode_e * pRopModePause)
{
   BOOL isModeFoundReturn = TRUE;

   * pRopModeNominal = LAST_ROP_MODE;
   * pRopModeAlarmStop = LAST_ROP_MODE;
   * pRopModePause = LAST_ROP_MODE;

   switch(RopMode)
   {
      case ROP_MODE_WAITING:
      case ROP_MODE_ALARM_STOP:
      case ROP_MODE_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_WAITING;
         * pRopModeAlarmStop = ROP_MODE_ALARM_STOP;
         * pRopModePause = ROP_MODE_PAUSE;
         break;
      }

      case ROP_MODE_TANK_FILLING:
      {
         * pRopModeNominal = ROP_MODE_TANK_FILLING;
         * pRopModeAlarmStop = ROP_MODE_ALARM_STOP;
         * pRopModePause = ROP_MODE_PAUSE;
         break;
      }

      case ROP_MODE_DAILY_RO_DIAGNOSTIC:
      {
         * pRopModeNominal = ROP_MODE_DAILY_RO_DIAGNOSTIC;
         * pRopModeAlarmStop = ROP_MODE_ALARM_STOP;
         * pRopModePause = ROP_MODE_PAUSE;
         break;
      }

      case ROP_MODE_PROGARD_FLUSH:
      case ROP_MODE_PROGARD_FLUSH_ALARM_STOP:
      case ROP_MODE_PROGARD_FLUSH_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_PROGARD_FLUSH;
         * pRopModeAlarmStop = ROP_MODE_PROGARD_FLUSH_ALARM_STOP;
         * pRopModePause = ROP_MODE_PROGARD_FLUSH_PAUSE;
         break;
      }

      case ROP_MODE_FIRST_INSTALL:
      case ROP_MODE_FIRST_INSTALL_ALARM_STOP:
      case ROP_MODE_FIRST_INSTALL_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_FIRST_INSTALL;
         * pRopModeAlarmStop = ROP_MODE_FIRST_INSTALL_ALARM_STOP;
         * pRopModePause = ROP_MODE_FIRST_INSTALL_PAUSE;
         break;
      }

      case ROP_MODE_RO_RINSE:
      case ROP_MODE_RO_RINSE_ALARM_STOP:
      case ROP_MODE_RO_RINSE_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_RO_RINSE;
         * pRopModeAlarmStop = ROP_MODE_RO_RINSE_ALARM_STOP;
         * pRopModePause = ROP_MODE_RO_RINSE_PAUSE;
         break;
      }

      case ROP_MODE_FORCED_RO_DIAGNOSTIC:
      case ROP_MODE_FORCED_RO_DIAGNOSTIC_ALARM_STOP:
      case ROP_MODE_FORCED_RO_DIAGNOSTIC_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_FORCED_RO_DIAGNOSTIC;
         * pRopModeAlarmStop = ROP_MODE_FORCED_RO_DIAGNOSTIC_ALARM_STOP;
         * pRopModePause = ROP_MODE_FORCED_RO_DIAGNOSTIC_PAUSE;
         break;
      }

      case ROP_MODE_PH_CLEANING:
      case ROP_MODE_PH_CLEANING_ALARM_STOP:
      case ROP_MODE_PH_CLEANING_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_PH_CLEANING;
         * pRopModeAlarmStop = ROP_MODE_PH_CLEANING_ALARM_STOP;
         * pRopModePause = ROP_MODE_PH_CLEANING_PAUSE;
         break;
      }

      case ROP_MODE_CL2_CLEANING:
      case ROP_MODE_CL2_CLEANING_ALARM_STOP:
      case ROP_MODE_CL2_CLEANING_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_CL2_CLEANING;
         * pRopModeAlarmStop = ROP_MODE_CL2_CLEANING_ALARM_STOP;
         * pRopModePause = ROP_MODE_CL2_CLEANING_PAUSE;
         break;
      }

      case ROP_MODE_REAGENT_REMOVAL:
      case ROP_MODE_REAGENT_REMOVAL_ALARM_STOP:
      case ROP_MODE_REAGENT_REMOVAL_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_REAGENT_REMOVAL;
         * pRopModeAlarmStop = ROP_MODE_REAGENT_REMOVAL_ALARM_STOP;
         * pRopModePause = ROP_MODE_REAGENT_REMOVAL_PAUSE;
         break;
      }

      case ROP_MODE_SAN1:
      case ROP_MODE_SAN1_ALARM_STOP:
      case ROP_MODE_SAN1_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_SAN1;
         * pRopModeAlarmStop = ROP_MODE_SAN1_ALARM_STOP;
         * pRopModePause = ROP_MODE_SAN1_PAUSE;
         break;
      }

      case ROP_MODE_SAN2:
      case ROP_MODE_SAN2_ALARM_STOP:
      case ROP_MODE_SAN2_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_SAN2;
         * pRopModeAlarmStop = ROP_MODE_SAN2_ALARM_STOP;
         * pRopModePause = ROP_MODE_SAN2_PAUSE;
         break;
      }

      case ROP_MODE_SAN3:
      case ROP_MODE_SAN3_ALARM_STOP:
      case ROP_MODE_SAN3_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_SAN3;
         * pRopModeAlarmStop = ROP_MODE_SAN3_ALARM_STOP;
         * pRopModePause = ROP_MODE_SAN3_PAUSE;
         break;
      }

      case ROP_MODE_TEST:
      case ROP_MODE_TEST_ALARM_STOP:
      case ROP_MODE_TEST_PAUSE:
      {
         * pRopModeNominal = ROP_MODE_TEST;
         * pRopModeAlarmStop = ROP_MODE_TEST_ALARM_STOP;
         * pRopModePause = ROP_MODE_TEST_PAUSE;
         break;
      }
      case ROP_MODE_TEST_RINSE:
      {
         * pRopModeNominal = ROP_MODE_TEST_RINSE;
         * pRopModeAlarmStop = ROP_MODE_TEST_ALARM_STOP;
         * pRopModePause = ROP_MODE_TEST_PAUSE;
         break;
      }

      case ROP_MODE_TEST_FLUSH:
      {
         * pRopModeNominal = ROP_MODE_TEST_FLUSH;
         * pRopModeAlarmStop = ROP_MODE_TEST_ALARM_STOP;
         * pRopModePause = ROP_MODE_TEST_PAUSE;
         break;
      }

      default:
      {
         isModeFoundReturn = FALSE;
         break;
      }
   }
   return (isModeFoundReturn);
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewMode [OUT] RopMode_e <br>
 *                    New Process Mode
 *
 *  @param isRecorded [OUT] BOOL <br>
 *                    Is the New Process Mode shall be recorded
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Set a new mode
 *
 *  @details: If the new mode is different from the old one, the parameters are reset
 *            and if the associated sequence is different, the process re-enter a new sequence
 *
 */
VOID rop_setNewMode(ROP_ProcessParam_t * p_ProcessParam, RopMode_e NewMode, BOOL isRecorded)
{
   if (NewMode != p_ProcessParam->ProcessDescription.ProcessMode)
   {
      RopSequence_e AssociatedSequence = rop_getAssociatedSequence(NewMode);
      p_ProcessParam->ProcessDescription.ProcessMode = NewMode;
      p_ProcessParam->ProcessDescription.isEnteringMode = TRUE;
      p_ProcessParam->ProcessDescription.SequenceTotalDuration = 0U;
      p_ProcessParam->ProcessDescription.SequenceRemainingTime = 0U;
      if (AssociatedSequence != p_ProcessParam->ProcessDescription.ProcessSequence)
      {
         p_ProcessParam->ProcessDescription.ProcessSequence = AssociatedSequence;
         p_ProcessParam->ProcessDescription.isEnteringSequence = TRUE;
      }
      if (TRUE == isRecorded)
      {
         p_ProcessParam->pProcessRecord->InterruptedProcessMode = NewMode;
         SYS_RefreshBackupRecord(SYS_REC_PROD);
      }
   }
}

/*!
 *  @param RopState [IN] RopState_e <br>
 *                    Process State
 *
 *  @return RopTransition_e : The associated transition
 *
 *  @brief  Get the associated transition to switch to a process state
 *
 *  @details   Allow the process to know which transition to apply to a certain state
 *
 */
RopTransition_e rop_getAssociatedTransition (RopState_e RopState)
{
   RopTransition_e AssociatedTransition;
   switch(RopState)
   {
      case ROP_STATE_WAIT:                   {AssociatedTransition=ROP_TRANSITION_TO_WAIT_SLOW;              break;}
      case ROP_STATE_FAST_WAIT:              {AssociatedTransition=ROP_TRANSITION_TO_WAIT_FAST;              break;}
      case ROP_STATE_UF_WAIT:                {AssociatedTransition=ROP_TRANSITION_TO_UF_WAIT;                break;}
      case ROP_STATE_RINSE:                  {AssociatedTransition=ROP_TRANSITION_TO_RINSE;                  break;}
      case ROP_STATE_RINSE_MESURE:           {AssociatedTransition=ROP_TRANSITION_TO_RINSE_MEASURE;          break;}
      case ROP_STATE_TANK_FILLING:           {AssociatedTransition=ROP_TRANSITION_TO_TANK_FILLING;           break;}
      case ROP_STATE_TANK_FILLING_MESURE:    {AssociatedTransition=ROP_TRANSITION_TO_TANK_FILLING_MEASURE;   break;}
      case ROP_STATE_FLUSH:                  {AssociatedTransition=ROP_TRANSITION_TO_FLUSH;                  break;}
      case ROP_STATE_LIMITED_FLUSH_DEAD_END: {AssociatedTransition=ROP_TRANSITION_TO_LIMITED_FLUSH_DEAD_END; break;}
      case ROP_STATE_SHORT_FLUSH:            {AssociatedTransition=ROP_TRANSITION_TO_SHORT_FLUSH;            break;}
      case ROP_STATE_SHORT_FLUSH_MEASURE:    {AssociatedTransition=ROP_TRANSITION_TO_SHORT_FLUSH_MEASURE;    break;}
      case ROP_STATE_SOAKING:                {AssociatedTransition=ROP_TRANSITION_TO_SOAKING;                break;}
      case ROP_STATE_TEST_TAP_PRESSURE:      {AssociatedTransition=ROP_TRANSITION_TO_TEST_TAP_PRESSURE;      break;}
      case ROP_STATE_TEST_RO_PUMP:           {AssociatedTransition=ROP_TRANSITION_TO_TEST_RO_PUMP;           break;}
      case ROP_STATE_SAN_RINSE:              {AssociatedTransition=ROP_TRANSITION_TO_SAN_RINSE;              break;}
      case ROP_STATE_SAN_TANK_FILLING:       {AssociatedTransition=ROP_TRANSITION_TO_SAN_TANK_FILLING;       break;}
      case ROP_STATE_WAIT_TEST:              {AssociatedTransition=ROP_TRANSITION_TO_WAIT_TEST;              break;}
      case ROP_STATE_SLOW_FLUSH:             {AssociatedTransition=ROP_TRANSITION_TO_SLOW_FLUSH;             break;}
      case ROP_STATE_SLOW_SHORT_FLUSH:       {AssociatedTransition=ROP_TRANSITION_TO_SLOW_SHORT_FLUSH;       break;}
      default:
      {
         ERROR("Wrong RopState %D ",RopState);
         AssociatedTransition=ROP_TRANSITION_TO_WAIT_SLOW; 
         break;
      }
   }
   return(AssociatedTransition);
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @param NewOrder [OUT] RopOrder_e <br>
 *                    Order
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage a order send to the process
 *
 *  @details: The order depends on the process mode
 *
 */
static VOID rop_manageOrder(ROP_ProcessParam_t * p_ProcessParam, RopOrder_e NewOrder)
{
   /* For each command, the result depends on the actual mode of the process */
   switch(NewOrder)
   {
      case ROP_ORDER_GOTO_STANDBY:
      {
         if((ROP_MODE_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode)        ||
            (ROP_MODE_WAITING == p_ProcessParam->ProcessDescription.ProcessMode)             ||
            (ROP_MODE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)          ||
            (ROP_MODE_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else if(ROP_MODE_DAILY_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
            rop_setAutoDiagnosticFlag (FALSE);
         }
         else if((ROP_MODE_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_PROGARD_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_PROGARD_FLUSH_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_PROGARD_FLUSH, TRUE);
         }
         else if((ROP_MODE_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_FIRST_INSTALL_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_FIRST_INSTALL_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_FIRST_INSTALL, TRUE);
         }
         else if((ROP_MODE_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_RO_RINSE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_RO_RINSE_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_RO_RINSE, TRUE);
         }
         else if((ROP_MODE_FORCED_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_FORCED_RO_DIAGNOSTIC_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_FORCED_RO_DIAGNOSTIC_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else if((ROP_MODE_PH_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_PH_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_PH_CLEANING_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_START_AB_FLUSH, TRUE);
         }
         else if((ROP_MODE_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_CL2_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_CL2_CLEANING_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_CL2_CLEANING, TRUE);
         }
         else if((ROP_MODE_REAGENT_REMOVAL == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_REAGENT_REMOVAL_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_REAGENT_REMOVAL_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_START_REAGENT_REMOVAL, TRUE);
         }
         else if((ROP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_SAN1_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_SAN1_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN1, TRUE);
         }
         else if((ROP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_SAN2_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_SAN2_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN2, TRUE);
         }
         else if((ROP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode)             ||
                 (ROP_MODE_SAN3_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
                 (ROP_MODE_SAN3_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN3, TRUE);
         }
         else if((ROP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)           ||
                 (ROP_MODE_TEST_RINSE == p_ProcessParam->ProcessDescription.ProcessMode)     ||
                 (ROP_MODE_TEST_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)     ||
                 (ROP_MODE_TEST_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)||
                 (ROP_MODE_TEST_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if ((p_ProcessParam->pProcessRecord->InterruptedProcessMode == ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL) ||
                (p_ProcessParam->pProcessRecord->InterruptedProcessMode == ROP_MODE_RESTART_FIRST_INSTALL))
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL, TRUE);
            }
            else
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
            }
         }
         else
         {
            /* Do Nothing */
         }
         break;
      }

      case ROP_ORDER_SHORTENING_RO_RINSE:
      {
         if((ROP_MODE_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_STATE_RINSE == p_ProcessParam->ProcessDescription.ProcessState)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_GOTO_READY:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_TEST_H2O2 == p_ProcessParam->ProcessDescription.ProcessMode)             ||
            (ROP_MODE_CAN_NOT_TEST_H2O2 == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_START_FIRST_INSTALL:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_STANDBY_NEW_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_RESTART_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               if (ROP_MODE_RESTART_FIRST_INSTALL != p_ProcessParam->ProcessDescription.ProcessMode)
               {
                  p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_WAIT;
                  p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               }
               rop_setNewMode(p_ProcessParam, ROP_MODE_FIRST_INSTALL, TRUE);
               // Write the install date on the Progard
               if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
               }
               else
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
                  CSM_WriteInstallDate(CSM_PROGARD2,FALSE);
               }
               // Disable the check new pack for a short delay
               p_ProcessParam->ProcessDescription.isCheckNewPackDisable = TRUE;
            }
         }
         break;
      }

      case ROP_ORDER_START_PROGARD_FLUSH:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_STANDBY_NEW_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_RESTART_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_PROGARD_FLUSH, TRUE);
               // Write the install date on the Progard
               if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
               }
               else
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
                  CSM_WriteInstallDate(CSM_PROGARD2,FALSE);
               }
               // Disable the check new pack for a short delay
               p_ProcessParam->ProcessDescription.isCheckNewPackDisable = TRUE;
            }
         }
         break;
      }

      case ROP_ORDER_START_RO_RINSE:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_RESTART_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               if(ROP_MODE_RESTART_RO_RINSE != p_ProcessParam->ProcessDescription.ProcessMode)
               {
                  p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_WAIT;
                  p_ProcessParam->pProcessRecord->RecordParameter = 0U;
               }
               rop_setNewMode(p_ProcessParam, ROP_MODE_RO_RINSE, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_START_PH_CLEANING:
      {
         if(ROP_MODE_STANDBY_TOOL_PACK == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               cleaningSequenceStartedFromTheStart = TRUE;
               rop_setNewMode(p_ProcessParam, ROP_MODE_PH_CLEANING, TRUE);
            }
         }
         else if(ROP_MODE_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               cleaningSequenceStartedFromTheStart = FALSE;
               rop_setNewMode(p_ProcessParam, ROP_MODE_PH_CLEANING, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_START_AB_FLUSH:
      {
         // Unused command
         break;
      }

      case ROP_ORDER_START_CL2_CLEANING:
      {
         if((ROP_MODE_STANDBY_TOOL_PACK == p_ProcessParam->ProcessDescription.ProcessMode)     ||
            (ROP_MODE_RESTART_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_CL2_CLEANING, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_START_REAGENT_REMOVAL:
      {
         if(ROP_MODE_START_REAGENT_REMOVAL == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL, TRUE);
               // Write the install date on the Progard
               if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
               }
               else
               {
                  CSM_WriteInstallDate(CSM_PROGARD1,FALSE);
                  CSM_WriteInstallDate(CSM_PROGARD2,FALSE);
               }
               // Disable the check new pack for a short delay
               p_ProcessParam->ProcessDescription.isCheckNewPackDisable = TRUE;
            }
         }
         break;
      }

      case ROP_ORDER_START_SAN1:
      case ROP_ORDER_START_SAN2:
      case ROP_ORDER_START_SAN3:
      {
         /* Unused */
         break;
      }

      case ROP_ORDER_START_FORCED_DIAGNOSTIC:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)    ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_FORCED_RO_DIAGNOSTIC, TRUE);
            }
         }
         break;
      }

      case ROP_ORDER_GOTO_TEST_MODE:
      {
         if((ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)       ||
            (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode)  ||
            (ROP_MODE_STANDBY_TOOL_PACK == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_STANDBY_ALARM == p_ProcessParam->ProcessDescription.ProcessMode)          ||
            (ROP_MODE_RESTART_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode)  ||
            (ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_TEST, FALSE);
            }
         }
         break;
      }

      case ROP_ORDER_START_FLUSH_TEST:
      {
         if(ROP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_TEST_FLUSH, FALSE);
            }
         }
         break;
      }

      case ROP_ORDER_START_RINSE_TEST:
      {
         if(ROP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            if(ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
            {
               rop_setNewMode(p_ProcessParam, ROP_MODE_TEST_RINSE, FALSE);
            }
         }
         break;
      }

      case ROP_ORDER_STOP_TEST:
      {
         if((ROP_MODE_TEST_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)    ||
            (ROP_MODE_TEST_RINSE == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_TEST, FALSE);
         }
         break;
      }

      case ROP_ORDER_SKIP_SEQUENCE:
      {
         if((ROP_MODE_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode) ||
            (ROP_MODE_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
            (ROP_MODE_REAGENT_REMOVAL == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            p_ProcessParam->pProcessRecord->ProcessState = ROP_STATE_WAIT;
            p_ProcessParam->pProcessRecord->RecordParameter = 0U;
            rop_setNewMode(p_ProcessParam, ROP_MODE_WAITING, TRUE);
            // Reset Flush Timer
            rop_resetFlushTimer(p_ProcessParam);
         }
         else if((ROP_MODE_PH_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (ROP_MODE_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
         }
         else if(ROP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN2, TRUE);
         }
         else if((ROP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode) ||
                 (ROP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode))
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_TEST_H2O2, TRUE);
         }
         else if(ROP_MODE_FORCED_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode)
         {
            rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
         }
         else
         {
           /* Do Nothing */
         }
         break;
      }

      default:
      {
         ERROR("Wrong Production order %d",NewOrder);
         break;
      }
   }
}

/*!
 *  @param includeLowFeedWaterPressure [IN] BOOL <br>
 *                    is the Low Feed Water Pressure alarm stop included in the check
 *
 *  @param includePackOut [IN] BOOL <br>
 *                    is the Pack Out alarm stop included in the check
 *  @return BOOL
 *  @retval TRUE if an alarm stop is present
 *  @retval FALSE if no alarm stop is present
 *
 *  @brief  Check if there is an alarm stop present
 *
 *  @details: The check can include Low Feed Water Pressure alarm or not and
 *            The check can include Progard Pack Out alarm or not
 *
 */
BOOL rop_isAlarmStopPresent(BOOL includeLowFeedWaterPressure, BOOL includePackOut)
{
   BOOL isAlarmStopPresent = FALSE;

   isAlarmStopPresent =((AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_FEED_PRESSURE_SENSOR_ERROR))||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_FEED_PRESSURE_MAX))         ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_RO_PRESSURE_SENSOR_ERROR))  ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_RO_PRESSURE_MAX))           ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_RO_PRESSURE_MIN))           ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_RO_RECOVERY_HIGH_SP))       ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_FCE_HIGH_SP))               ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_EDI_FLOWRATE_MAX))          ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_EDI_FLOWRATE_MIN))          ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TANKLEVEL_DISCONNECTED))    ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_TANK_OVERFLOW))         ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_LEAK_DETECTOR))         ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_TOR_SDS_LEAK_DETECTOR))     ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARDS_UNPAIRED))    ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD1_INCORRECT))   ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD1_INVALID))     ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD2_INCORRECT))   ||
                        (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD2_INVALID)));

   if (TRUE == includePackOut)
   {
      isAlarmStopPresent = (isAlarmStopPresent ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD1_OUT)) ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_PACK_PROGARD2_OUT)));
   }

   if (TRUE == includeLowFeedWaterPressure)
   {
      isAlarmStopPresent = (isAlarmStopPresent ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_DRAIN_AND_PERMEATE_FLOWRATE_LOW)) ||
                           (AAA_STATE_ACTIVE == AAA_GetAlarmState(ALARM_TYPE_FEED_PRESSURE_MIN)));
   }

   return (isAlarmStopPresent);
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Check if there is an alarm stop or a pause to switch the process mode
 *
 *  @details: The order depends on the process mode
 *
 */
static VOID rop_manageAlarmStopAndPause(ROP_ProcessParam_t * p_ProcessParam)
{
   BOOL isAlarmPresent = FALSE;
   BOOL isPausePresent = FALSE;
   FLOAT BackwashInProgress;
   RopMode_e RopModeNominal;
   RopMode_e RopModeAlarmStop;
   RopMode_e RopModePause;
   BOOL isModeFound = rop_getAssociatedMode (p_ProcessParam->ProcessDescription.ProcessMode,&RopModeNominal,&RopModeAlarmStop,&RopModePause);

   isAlarmPresent = rop_isAlarmStopPresent(TRUE, TRUE);

   if ((TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_BACKWASH_FILTER)) || // if External Backwash is in Progress
       (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_UF)))
   {
      BackwashInProgress = MES_GetMeasure(MES_TOR_BACKWASH_CARBON);
      if (0.0F != BackwashInProgress)
      {
         isPausePresent = TRUE;
      }
   }

   if (TRUE == isModeFound)
   {
      if(TRUE == isPausePresent)
      {
         if((p_ProcessParam->ProcessDescription.ProcessMode == RopModeNominal) ||
            (p_ProcessParam->ProcessDescription.ProcessMode == RopModeAlarmStop))
         {
           /* If the process is in nominal mode or alarm mode and a pause is present
            * the process shall switch to Pause mode */
            if (ROP_MODE_TEST_PAUSE == RopModePause)
            {
               rop_setNewMode(p_ProcessParam, RopModePause, FALSE);
            }
            else
            {
               rop_setNewMode(p_ProcessParam, RopModePause, TRUE);
            }
         }
      }
      else if(TRUE == isAlarmPresent)
      {
            /* If there is no Pause but an alarm stop */
         if((p_ProcessParam->ProcessDescription.ProcessMode == RopModeNominal) ||
            (p_ProcessParam->ProcessDescription.ProcessMode == RopModePause))
         {
            /* If the process is in nominal mode or pause mode and an alarm stop is present
             * the process shall switch to Alarm stop mode */
            if(p_ProcessParam->ProcessDescription.ProcessMode == RopModeNominal)
            {
               /* If the mode is nominal, we shall record the diagnostic measure */
               rop_recordDiagnosticMeasure ();
            }
            if (ROP_MODE_TEST_ALARM_STOP == RopModeAlarmStop)
            {
               rop_setNewMode(p_ProcessParam, RopModeAlarmStop, FALSE);
            }
            else
            {
               rop_setNewMode(p_ProcessParam, RopModeAlarmStop, TRUE);
            }
         }
      }
      else if (ROP_TRANSITION_DONE == p_ProcessParam->TransitionDescription.TransitionStatus)
      {
         if((p_ProcessParam->ProcessDescription.ProcessMode == RopModeAlarmStop) ||
            (p_ProcessParam->ProcessDescription.ProcessMode == RopModePause))
         {
            /* If the process is in alarm stop mode or pause mode and there is no Pause and no Alarm
             * the process shall switch to nominal mode */
            if (ROP_MODE_TEST == RopModeNominal)
            {
               rop_setNewMode(p_ProcessParam, RopModeNominal, FALSE);
            }
            else
            {
               rop_setNewMode(p_ProcessParam, RopModeNominal, TRUE);
            }
         }
      }
      else
      {
         /* Do Nothing */
      }
   }
}

/*!
 *  @param p_ProcessParam [IN] ROP_ProcessParam_t * <br>
 *                    Process Parameter Structure
 *
 *  @return None
 *  @retval None
 *
 *  @brief  Manage the Progard Pack Status impact on process mode
 *
 *  @details:
 *
 */
static VOID rop_manageProgardsStatus(ROP_ProcessParam_t * p_ProcessParam)
{
   CSM_Status_t ProgardStatus = rop_getProgardsStatus();
   
   /* Case of test for manufacturing bench */
   if (CSM_STATUS_TEST == ProgardStatus)
   {
      /* If one progard at Least is in Test mode, the process switches to Test mode */
      if((ROP_MODE_TEST != p_ProcessParam->ProcessDescription.ProcessMode)            &&
         (ROP_MODE_TEST_ALARM_STOP != p_ProcessParam->ProcessDescription.ProcessMode) &&
         (ROP_MODE_TEST_PAUSE != p_ProcessParam->ProcessDescription.ProcessMode)      &&
         (ROP_MODE_TEST_RINSE != p_ProcessParam->ProcessDescription.ProcessMode)      &&
         (ROP_MODE_TEST_FLUSH != p_ProcessParam->ProcessDescription.ProcessMode))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE); /* First we record the mode Standby Old Pack */
         rop_setNewMode(p_ProcessParam, ROP_MODE_TEST, FALSE);
      }
   }
   /* End of manufacturing bench case */
   
   /* Case of init */
   if (CSM_STATUS_INIT == ProgardStatus)
   {
      /* If the process is not in INIT mode there is a problem */
      if(ROP_MODE_INITIALIZATION != p_ProcessParam->ProcessDescription.ProcessMode)
      {
         ASSERT(FALSE, "Uncompatible production mode for INIT consumable Status");
      }
   }
   /* End of init case */
   
   if((ROP_MODE_TANK_FILLING == p_ProcessParam->ProcessDescription.ProcessMode)        ||
      (ROP_MODE_DAILY_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_WAITING == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)          ||
      (ROP_MODE_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_NEW == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_NEW_PACK, TRUE);
      }
      else if(CSM_STATUS_TOOL_PACK == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_TOOL_PACK, TRUE);
      }
      else if(CSM_STATUS_TOOL_BYPASS == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_BYPASS, TRUE);
      }
      else
      {
         /* Do Nothing */
      }
   }

   if((ROP_MODE_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_PROGARD_FLUSH_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (ROP_MODE_PROGARD_FLUSH_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_PROGARD_FLUSH, TRUE);
      }
      else if ((CSM_STATUS_TOOL_PACK == ProgardStatus) ||
               (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_PROGARD_FLUSH, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_PROGARD_FLUSH, TRUE);
      }
   }
   if((ROP_MODE_RESTART_PROGARD_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_NEW != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_PROGARD_FLUSH, TRUE);
      }
   }

   if((ROP_MODE_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_FIRST_INSTALL_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (ROP_MODE_FIRST_INSTALL_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_FIRST_INSTALL, TRUE);
      }
      else if ((CSM_STATUS_TOOL_PACK == ProgardStatus) ||
               (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL, TRUE);
      }
      else
      {
         /* Do Nothing */
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_FIRST_INSTALL, TRUE);
      }
   }
   if((ROP_MODE_RESTART_FIRST_INSTALL == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_NEW != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_FIRST_INSTALL, TRUE);
      }
   }

   if((ROP_MODE_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_RO_RINSE_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (ROP_MODE_RO_RINSE_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if ((CSM_STATUS_NEW == ProgardStatus)       ||
          (CSM_STATUS_TOOL_PACK == ProgardStatus) ||
          (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_RO_RINSE, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_RO_RINSE, TRUE);
      }
   }
   if((ROP_MODE_RESTART_RO_RINSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_RO_RINSE, TRUE);
      }
   }

   if((ROP_MODE_FORCED_RO_DIAGNOSTIC == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_FORCED_RO_DIAGNOSTIC_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (ROP_MODE_FORCED_RO_DIAGNOSTIC_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_NEW == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_NEW_PACK, TRUE);
      }
      else if(CSM_STATUS_TOOL_PACK == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_TOOL_PACK, TRUE);
      }
      else if(CSM_STATUS_TOOL_BYPASS == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_BYPASS, TRUE);
      }
      else
      {
         /* Do Nothing */
      }
   }

   if((ROP_MODE_PH_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_PH_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_PH_CLEANING_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_CAN_NOT_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus) ||
         (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_PACK == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_START_AB_FLUSH, TRUE);
      }
   }
   if((ROP_MODE_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_PACK != ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_START_AB_FLUSH, TRUE);
      }
   }

   if((ROP_MODE_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_CL2_CLEANING_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_CL2_CLEANING_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_CAN_NOT_RESTART_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_START_AB_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus) ||
         (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_PACK == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_CL2_CLEANING, TRUE);
      }
   }
   if((ROP_MODE_RESTART_CL2_CLEANING == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_PACK != ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_CL2_CLEANING, TRUE);
      }
   }

   if((ROP_MODE_REAGENT_REMOVAL == p_ProcessParam->ProcessDescription.ProcessMode)             ||
      (ROP_MODE_REAGENT_REMOVAL_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode)  ||
      (ROP_MODE_REAGENT_REMOVAL_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_START_REAGENT_REMOVAL, TRUE);
      }
      else if((CSM_STATUS_TOOL_PACK == ProgardStatus) ||
              (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
      }
      else
      {
         /* Do Nothing */
      }
   }
   if((ROP_MODE_REAGENT_REMOVAL_REQUESTED == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_START_REAGENT_REMOVAL, TRUE);
      }
   }
   if((ROP_MODE_START_REAGENT_REMOVAL == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_NEW != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_REAGENT_REMOVAL_REQUESTED, TRUE);
      }
   }

   if((ROP_MODE_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_SAN1_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_SAN1_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_CAN_NOT_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_TOOL_PACK == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN2, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_BYPASS == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN1, TRUE);
      }
   }
   if((ROP_MODE_RESTART_SAN1 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_BYPASS != ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN1, TRUE);
      }
   }

   if((ROP_MODE_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_SAN2_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_SAN2_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) &&
         (FALSE == p_ProcessParam->ProcessDescription.isCheckNewPackDisable))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN2, TRUE);
      }
      else if((CSM_STATUS_TOOL_PACK == ProgardStatus) ||
              (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN2, TRUE);
      }
      else
      {
         /* Do Nothing */
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN2, TRUE);
      }
   }
   if((ROP_MODE_RESTART_SAN2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_NEW != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN2, TRUE);
      }
   }

   if((ROP_MODE_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_SAN3_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_SAN3_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_NEW == ProgardStatus) ||
         (CSM_STATUS_TOOL_PACK == ProgardStatus) ||
         (CSM_STATUS_TOOL_BYPASS == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN3, TRUE);
      }
   }
   if((ROP_MODE_CAN_NOT_RESTART_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_RESTART_SAN3, TRUE);
      }
   }
   if((ROP_MODE_RESTART_SAN3 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_RESTART_SAN3, TRUE);
      }
   }

   if((ROP_MODE_CAN_NOT_TEST_H2O2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD == ProgardStatus) ||
         (CSM_STATUS_OVERRIDDEN == ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_TEST_H2O2, TRUE);
      }
   }
   if((ROP_MODE_TEST_H2O2 == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if((CSM_STATUS_OLD != ProgardStatus) &&
         (CSM_STATUS_OVERRIDDEN != ProgardStatus))
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_CAN_NOT_TEST_H2O2, TRUE);
      }
   }

   if((ROP_MODE_TEST == p_ProcessParam->ProcessDescription.ProcessMode)            ||
      (ROP_MODE_TEST_ALARM_STOP == p_ProcessParam->ProcessDescription.ProcessMode) ||
      (ROP_MODE_TEST_PAUSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_TEST_RINSE == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_TEST_FLUSH == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      /* Do Nothing */
   }

   if((ROP_MODE_STANDBY_ALARM == p_ProcessParam->ProcessDescription.ProcessMode)       ||
      (ROP_MODE_STANDBY_OLD_PACK == p_ProcessParam->ProcessDescription.ProcessMode)    ||
      (ROP_MODE_STANDBY_NEW_PACK == p_ProcessParam->ProcessDescription.ProcessMode)    ||
      (ROP_MODE_STANDBY_BYPASS == p_ProcessParam->ProcessDescription.ProcessMode)      ||
      (ROP_MODE_STANDBY_TOOL_PACK == p_ProcessParam->ProcessDescription.ProcessMode)   ||
      (ROP_MODE_STANDBY_PACK_OVERRIDE == p_ProcessParam->ProcessDescription.ProcessMode))
   {
      if(CSM_STATUS_TOOL_PACK == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_TOOL_PACK, TRUE);
      }
      else if(CSM_STATUS_TOOL_BYPASS == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_BYPASS, TRUE);
      }
      else if(CSM_STATUS_OLD == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_OLD_PACK, TRUE);
      }
      else if(CSM_STATUS_OVERRIDDEN == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_PACK_OVERRIDE, TRUE);
      }
      else if(CSM_STATUS_NEW == ProgardStatus)
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_NEW_PACK, TRUE);
      }
      else
      {
         rop_setNewMode(p_ProcessParam, ROP_MODE_STANDBY_ALARM, TRUE);
      }
   }
}

static CSM_Status_t rop_getProgardsStatus (VOID)
{
   CSM_Status_t ProgardStatus;
   CSM_Status_t Progard1Status = CSM_GetStatus(CSM_PROGARD1);
   CSM_Status_t Progard2Status;
   BOOL isAlarmUnpairedToTrig = FALSE;
   
   if (SYSTEM_ONE_PROGARD == SYS_GetNbProgards())
   {
      ProgardStatus = Progard1Status;
   }
   else
   {
      Progard2Status = CSM_GetStatus(CSM_PROGARD2);
      
      if ((CSM_STATUS_OUT == Progard2Status)       ||
          (CSM_STATUS_INCORRECT == Progard2Status) ||
          (CSM_STATUS_INVALID == Progard2Status)   ||
          (CSM_STATUS_TEST == Progard2Status)      ||
          (CSM_STATUS_MAX == Progard2Status)       ||
          (CSM_STATUS_INIT == Progard2Status))
      {
         ProgardStatus = Progard2Status;
      }
      else
      {
         switch(Progard1Status)
         {
            case CSM_STATUS_NEW:
            {
               if ((CSM_STATUS_NEW == Progard2Status) ||
                   (CSM_STATUS_OVERRIDDEN == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_NEW;
               }
               else if ((CSM_STATUS_TOOL_PACK == Progard2Status) ||
                        (CSM_STATUS_TOOL_CAP == Progard2Status) ||
                        (CSM_STATUS_TOOL_BYPASS == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else if (CSM_STATUS_OLD == Progard2Status)
               {
                  if(TRUE == ProductionProcessParameters.ProcessDescription.isCheckNewPackDisable)
                  {
                     ProgardStatus = CSM_STATUS_OLD;
                  }
                  else
                  {
                     ProgardStatus = CSM_STATUS_OUT;
                     isAlarmUnpairedToTrig = TRUE;
                  }
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }
                   
            case CSM_STATUS_OLD:
            {
               if ((CSM_STATUS_OLD == Progard2Status) ||
                   (CSM_STATUS_OVERRIDDEN == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OLD;
               }
               else if ((CSM_STATUS_TOOL_PACK == Progard2Status) ||
                        (CSM_STATUS_TOOL_CAP == Progard2Status) ||
                        (CSM_STATUS_TOOL_BYPASS == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else if (CSM_STATUS_NEW == Progard2Status)
               {
                  if(TRUE == ProductionProcessParameters.ProcessDescription.isCheckNewPackDisable)
                  {
                     ProgardStatus = CSM_STATUS_OLD;
                  }
                  else
                  {
                     ProgardStatus = CSM_STATUS_OUT;
                     isAlarmUnpairedToTrig = TRUE;
                  }
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }

            case CSM_STATUS_OVERRIDDEN:
            {
               if (CSM_STATUS_OVERRIDDEN == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_OVERRIDDEN;
               }
               else if (CSM_STATUS_OLD == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_OLD;
               }
               else if (CSM_STATUS_NEW == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_NEW;
               }
               else if ((CSM_STATUS_TOOL_PACK == Progard2Status) ||
                        (CSM_STATUS_TOOL_CAP == Progard2Status) ||
                        (CSM_STATUS_TOOL_BYPASS == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }

            case CSM_STATUS_TOOL_PACK:
            {
               if (CSM_STATUS_TOOL_CAP == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_TOOL_PACK;
               }
               else if ((CSM_STATUS_OVERRIDDEN == Progard2Status) ||
                        (CSM_STATUS_NEW == Progard2Status) ||
                        (CSM_STATUS_OLD == Progard2Status) ||
                        (CSM_STATUS_TOOL_PACK == Progard2Status) ||
                        (CSM_STATUS_TOOL_BYPASS == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }
            
            case CSM_STATUS_TOOL_CAP:
            {
               if (CSM_STATUS_TOOL_PACK == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_TOOL_PACK;
               }
               else if ((CSM_STATUS_OVERRIDDEN == Progard2Status) ||
                        (CSM_STATUS_NEW == Progard2Status) ||
                        (CSM_STATUS_OLD == Progard2Status) ||
                        (CSM_STATUS_TOOL_CAP == Progard2Status) ||
                        (CSM_STATUS_TOOL_BYPASS == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }
            
            case CSM_STATUS_TOOL_BYPASS:
            {
               if (CSM_STATUS_TOOL_BYPASS == Progard2Status)
               {
                  ProgardStatus = CSM_STATUS_TOOL_BYPASS;
               }
               else if ((CSM_STATUS_OVERRIDDEN == Progard2Status) ||
                        (CSM_STATUS_NEW == Progard2Status) ||
                        (CSM_STATUS_OLD == Progard2Status) ||
                        (CSM_STATUS_TOOL_CAP == Progard2Status) ||
                        (CSM_STATUS_TOOL_PACK == Progard2Status))
               {
                  ProgardStatus = CSM_STATUS_OUT;
                  isAlarmUnpairedToTrig = TRUE;
               }
               else
               {
                  /* Should not go here */
                  ProgardStatus = CSM_STATUS_OUT;
               }
               break;
            }
            
            case CSM_STATUS_OUT:
            case CSM_STATUS_INCORRECT:
            case CSM_STATUS_INVALID:     
            case CSM_STATUS_TEST:
            case CSM_STATUS_MAX:
            case CSM_STATUS_INIT:
            default:
            {
               ProgardStatus = Progard1Status;
            }
         }
      }
      if ((TRUE == isAlarmUnpairedToTrig) &&
          (FALSE == isAlarmUnpairedTrigged))
      {
         AAA_SetAlarm(ALARM_TYPE_PACK_PROGARDS_UNPAIRED);
         isAlarmUnpairedTrigged = TRUE;
      }
      else if ((FALSE == isAlarmUnpairedToTrig) &&
               (TRUE == isAlarmUnpairedTrigged))
      {
         AAA_ResetAlarm(ALARM_TYPE_PACK_PROGARDS_UNPAIRED);
         isAlarmUnpairedTrigged = FALSE;
      }
      else
      {
         /* Do Nothing */
      }
   }
   
   return (ProgardStatus);
}

/*!
 * @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

