/*!
* @file rop_transition.c
*
* @brief Production Task
*
* @details Contain all Production stuf
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: rop_transition.c 600 2014-10-14 0#$: Filename
* $Rev:: 600                              $: Revision of last commit
* $Author:: m175711                       $: Author of last commit
* $Date:: 2014-10-14 09:58:39 +0200 (mar.#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/production/src/rop_transition.c $: URL of this file
* @endcond
*
*/

/*!
 *  @ingroup RopStateMachine
 *  @{
 */


/* Defines -------------------------------------------------------------------*/


/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>

#include "millipore_types.h"
#include "os_adaptation.h"
#include "rop_api.h"
#include "sys_softerr.h"
#include "rop_data.h"
#include "mes_api.h"
#include "mvm_api.h"
#include "act_api.h"
#include "tmc_api.h"
#include "sys_api.h"
#include "sys_log.h"
#include "rop_private.h"
#include "system_parameters.h"
 
 
/* Defines -------------------------------------------------------------------*/
#define ROP_TRANS_WAIT_FOR_RO_PUMP_STOP (1000U)
#define ROP_SLOW_TRANS_PHASE_MAX (14U)

/* Types --------------------------------------------------------------*/
 
/* Timer used to check the duration of the transition */
typedef struct ROP_TransitionTimer_s
{
   BOOL           isTimerExpired;
   UINT32         TimerDelay;
   TimeCounter_t  TransitionTimeCounter;
} ROP_TransitionTimer_t;

/* Used for slow transition to flush */
typedef struct ROP_SlowTransition_s
{
   UINT32 PhaseDurationMilliSeconds;
   FLOAT DrainValveOpening;
} ROP_SlowTransition_t;

/* Declarations --------------------------------------------------------------*/
static ROP_TransitionTimer_t TransitionTimer;
static UINT8 ROP_SlowTransitionCounter;
static const ROP_SlowTransition_t SlowTransitionTab[ROP_SLOW_TRANS_PHASE_MAX] =
{
{15000U, 0.0F},
{15000U, 8.33F},
{15000U, 16.67F},
{120000U, 25.0F},
{120000U, 29.17F},
{15000U, 33.33F},
{15000U, 41.67F},
{15000U, 50.0F},
{15000U, 58.33F},
{15000U, 66.67F},
{15000U, 75.0F},
{15000U, 83.33F},
{15000U, 91.67F},
{1000U, 100.0F}
};

/* Code ----------------------------------------------------------------------*/

static VOID rop_startTransitionTimer(UINT32 Delay); // Delay is in milli-seconds
static BOOL rop_isTransitionTimerExpired (VOID);
static VOID rop_manageTransitionToWaitFast(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToWaitSlow(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToUfWait(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToRinse(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToTankFilling(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToFlush(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToLimitedFlushDeadEnd(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToShortFlush(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToSoaking(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToTestTapPressure(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToTestRoPump(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToSanRinse(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToSanTankFilling(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToSlowFlush(ROP_TransitionDescription_t * p_TransitionDescription);
static VOID rop_manageTransitionToSlowShortFlush(ROP_TransitionDescription_t * p_TransitionDescription);

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage the different type of transition
 *
 *  @details  Check which type of transition is start and call the associated transition
 *
 */
VOID rop_manageTransition (ROP_TransitionDescription_t * p_TransitionDescription)
{
   if (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus)
   {
      switch(p_TransitionDescription->TransitionType)
      {
         case ROP_TRANSITION_TO_WAIT_FAST:
         {
            rop_manageTransitionToWaitFast(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_WAIT_SLOW:
         case ROP_TRANSITION_TO_WAIT_TEST:
         {
            rop_manageTransitionToWaitSlow(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_UF_WAIT:
         {
            rop_manageTransitionToUfWait(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_RINSE:
         case ROP_TRANSITION_TO_RINSE_MEASURE:
         {
            rop_manageTransitionToRinse(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_TANK_FILLING:
         case ROP_TRANSITION_TO_TANK_FILLING_MEASURE:
         {
            rop_manageTransitionToTankFilling(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_FLUSH:
         {
            rop_manageTransitionToFlush(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_LIMITED_FLUSH_DEAD_END:
         {
            rop_manageTransitionToLimitedFlushDeadEnd(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SHORT_FLUSH:
         case ROP_TRANSITION_TO_SHORT_FLUSH_MEASURE:
         {
            rop_manageTransitionToShortFlush(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SOAKING:
         {
            rop_manageTransitionToSoaking(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_TEST_TAP_PRESSURE:
         {
            rop_manageTransitionToTestTapPressure(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_TEST_RO_PUMP:
         {
            rop_manageTransitionToTestRoPump(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SAN_RINSE:
         {
            rop_manageTransitionToSanRinse(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SAN_TANK_FILLING:
         {
            rop_manageTransitionToSanTankFilling(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SLOW_FLUSH:
         {
            rop_manageTransitionToSlowFlush(p_TransitionDescription);
            break;
         }

         case ROP_TRANSITION_TO_SLOW_SHORT_FLUSH:
         {
            rop_manageTransitionToSlowShortFlush(p_TransitionDescription);
            break;
         }

         default:
         {
            ERROR("Wrong production transition %d",p_TransitionDescription->TransitionType);
            break;
         }
      }
   }
}

/*!
 *  @param Delay [IN] UINT32 <br>
 *                  Delay in msec
 *
 *  @return none
 *
 *  @brief  Start the transition timer
 *
 *  @details
 *
 */
static VOID rop_startTransitionTimer(UINT32 Delay) // Timer is in milli-seconds
{
   TransitionTimer.isTimerExpired = TRUE;
   if (0U < Delay)
   {
      TransitionTimer.isTimerExpired = FALSE;
      TransitionTimer.TimerDelay = Delay;
      TransitionTimer.TransitionTimeCounter = TMC_StartTimeCounter();
   }
}

/*!
 *  @return BOOL: TRUE if the timer is expired, FALSE else
 *
 *  @brief  Check if the transition timer has expired
 *
 *  @details
 *
 */
static BOOL rop_isTransitionTimerExpired (VOID)
{
   UINT32 TimeCounterValue;

   if (FALSE == TransitionTimer.isTimerExpired)
   {
      TimeCounterValue = TMC_GetTimeCounter(TransitionTimer.TransitionTimeCounter);

      if (TimeCounterValue >= TransitionTimer.TimerDelay) // Check Timer compared to Delay
      {
         TransitionTimer.isTimerExpired = TRUE;
      }
   }

   return(TransitionTimer.isTimerExpired);
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToWaitFast(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Drain and the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      // Switch OFF EDI
      ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_OFF);
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_OFF);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      // Switch OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
      }

      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_RO_PUMP)) // RO Pump is ON
      {
          // Open the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
          // Open the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);
          // Stop the RO Pump
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);
          // Start Timer for 1 seconds = 1000 milliseconds
          rop_startTransitionTimer((UINT32)1000U);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
      else
      {
          // Confirme the RO Pump is stopped
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);

          // Set Timer to OK
          rop_startTransitionTimer((UINT32)0U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Inlet Valve
         ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_OFF);
         if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
         {
            // Close the external valve
            ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_OFF);
         }
          // Close the Permeate 1
         ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_OFF);
          // Close the Permeate 2
         ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_OFF);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION)))
      {
          // Motor Valve are open
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to WaitFast: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToWaitSlow(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Drain and the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      // Switch OFF EDI
      ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_OFF);
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_OFF);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      // Switch OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
      }

      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_RO_PUMP)) // RO Pump is ON
      {
          // Open the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
          // Open the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_RO_PRESSURE;
      }
      else
      {
         // Confirme the RO Pump is stopped
         ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);

          // Set Timer to OK
          rop_startTransitionTimer((UINT32)0U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_RO_PRESSURE == p_TransitionDescription->TransitionStatus)
   {
      if (((FLOAT)STOP_RO_PUMP_PRESSURE >= MES_GetMeasure(MES_RO_PRESSURE))       ||
          ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
           (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)))) // Wait for pressure or both Motor Valve opened
      {
         // Stop the RO Pump
         ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);
         // Start Timer for ROP_TRANS_WAIT_FOR_RO_PUMP_STOP
         rop_startTransitionTimer((UINT32)ROP_TRANS_WAIT_FOR_RO_PUMP_STOP);
         p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_INLET_SV_VALVE)) // Inlet Valve is open
          {
              // Close the Inlet Valve
              ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_OFF);
              // Start the Timer for 1 second = 1000 milliseconds
              rop_startTransitionTimer(1000U);
              p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          }
          else
          {
              // Set Timer to OK
              rop_startTransitionTimer(0U);
              p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          }
          
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Close the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_OFF);
          }
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_OFF);
          // Close the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_OFF);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
         (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
         (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
         (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are both open
      {
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_RO_PRESSURE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to WaitSlow: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToUfWait(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Drain and the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      // Switch OFF EDI
      ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_OFF);
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_OFF);

      // Switch OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
      }

      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_RO_PUMP)) // RO Pump is ON
      {
          // Open the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
          // Open the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_RO_PRESSURE;
      }
      else
      {
         // Confirme the RO Pump is stopped
         ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);

          // Set Timer to OK
          rop_startTransitionTimer((UINT32)0U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_RO_PRESSURE == p_TransitionDescription->TransitionStatus)
   {
      if (((FLOAT)STOP_RO_PUMP_PRESSURE >= MES_GetMeasure(MES_RO_PRESSURE))       ||
          ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
           (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)))) // Wait for pressure or both Motor Valve opened
      {
         // Stop the RO Pump
         ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);
         // Start Timer for ROP_TRANS_WAIT_FOR_RO_PUMP_STOP
         rop_startTransitionTimer((UINT32)ROP_TRANS_WAIT_FOR_RO_PUMP_STOP);
         p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_INLET_SV_VALVE)) // Inlet Valve is open
          {
              // Close the Inlet Valve
              ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_OFF);
              // Start the Timer for 1 second = 1000 milliseconds
              rop_startTransitionTimer(1000U);
              p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          }
          else
          {
              // Set Timer to OK
              rop_startTransitionTimer(0U);
              p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          }
          
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Close the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_OFF);
          }
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_OFF);
          // Close the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_OFF);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
         (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
         (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
         (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are both open
      {
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
          
          // Switch ON external UF
          ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_ON);
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_RO_PRESSURE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to WaitSlow: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToRinse(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Drain and the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      // Switch ON or OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         if(ROP_TRANSITION_TO_RINSE_MEASURE == p_TransitionDescription->TransitionType)
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_ON);
         }
         else
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
         }
      }

      // Open the Inlet Valve
      ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
      // Start Timer for 2 seconds = 2000 milliseconds
      rop_startTransitionTimer(2000U);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;

      rop_startMeasureRecordAndAlarm(CHECK_ALARM_PUMP, TRUE);// Start Check RO Pump Feed Pressure

      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
      {
         // Open the external valve
         ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Switch ON the RO Pump
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_ON);
          // Start Timer for 3 seconds = 3000 milliseconds
          rop_startTransitionTimer(3000U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          if(FALSE == ProductionProcessSetpoints.isManualRegulationActivated)
          {
             // Start the Regulation on both Motor Valve
             MVM_SetMode(MVM_MODE_REGULATED);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to Rinse: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToTankFilling(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch ON or OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         if(ROP_TRANSITION_TO_TANK_FILLING_MEASURE == p_TransitionDescription->TransitionType)
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_ON);
         }
         else
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
         }
      }

      // Switch ON the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_ON);
      // Open the Permeate 1
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
      // Start Timer for 1 second = 1000 milliseconds
      rop_startTransitionTimer(1000U);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE;
   }

   if(ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Switch ON the EDI
          ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_ON);
          // Close the Permeate 2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_OFF);

          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to TankFilling: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToFlush(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
      }

      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);

      // Open the Drain and close the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 0.0);

      // Switch OFF the EDI
      ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_OFF);
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_OFF);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_RO_PUMP)) // RO Pump is ON
      {
          // Open the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
          // Start Timer for 1 second = 1000 milliseconds
          rop_startTransitionTimer(1000U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE;
      }
      else
      {
          // Confirme the RO Pump is stopped
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);

          // Set Timer to OK
          rop_startTransitionTimer(0U);
          TransitionTimer.isTimerExpired = TRUE;
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Stop the RO Pump
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);
          // Start Timer for ROP_TRANS_WAIT_FOR_RO_PUMP_STOP
          rop_startTransitionTimer((UINT32)ROP_TRANS_WAIT_FOR_RO_PUMP_STOP);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Permeate1
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (1.0 >= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open the Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to Flush: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToShortFlush(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Switch ON or OFF Degaser (if option activated)
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_DEGASER))
      {
         if(ROP_TRANSITION_TO_SHORT_FLUSH_MEASURE == p_TransitionDescription->TransitionType)
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_ON);
         }
         else
         {
            ACT_SetPower(ACT_TYPE_VACUUM_PUMP, ACT_SWITCH_OFF);
         }
      }

      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);

      // Open the Drain and the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      // Switch OFF the EDI
      ACT_SetPower(ACT_TYPE_EDI, ACT_SWITCH_OFF);
      // Switch OFF the UV Lamp
      ACT_SetPower(ACT_TYPE_UV_PRODUCTION, ACT_SWITCH_OFF);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      if (ACT_SWITCH_ON == ACT_GetState(ACT_TYPE_RO_PUMP)) // RO Pump is ON
      {
          // Open the Permeate 1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_RO_PRESSURE;
      }
      else
      {
          // Confirme the RO Pump is stopped
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);

          // Set Timer to OK
          rop_startTransitionTimer(0U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_RO_PRESSURE == p_TransitionDescription->TransitionStatus)
   {
      if (((FLOAT)STOP_RO_PUMP_PRESSURE >= MES_GetMeasure(MES_RO_PRESSURE))       ||
          ((MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
           (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION)))) // Pressure is OK or Motor Valve are open
      {
          // Stop the RO Pump
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_OFF);
          // Start Timer for ROP_TRANS_WAIT_FOR_RO_PUMP_STOP
          rop_startTransitionTimer((UINT32)ROP_TRANS_WAIT_FOR_RO_PUMP_STOP);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_RO_PUMP;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_RO_PUMP == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Permeate1
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_OFF);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open the Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_RO_PRESSURE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_RO_PUMP != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to ShortFlush: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToLimitedFlushDeadEnd(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Close the Drain and close the Recirc to the good point
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 0.0);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, p_TransitionDescription->LimitedFlushDrainValvePosition);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open the Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to LimitedFlushDeadEnd: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToSoaking(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Close the Inlet Valve
      ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_OFF);
      if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
      {
         // Close the external valve
         ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_OFF);
      }
      // Open the Drain MV
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 100.0);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((99.0 <= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to Soaking: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToTestTapPressure(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Close the Drain MV
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 0.0);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to TestTapPressure: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToTestRoPump(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Close the Recirculation MV
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 0.0);

      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          // Start Timer for 2 seconds = 2000 milliseconds
          rop_startTransitionTimer(2000U);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
      }
   }

   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Start the RO Pump
          ACT_SetPower(ACT_TYPE_RO_PUMP, ACT_SWITCH_ON);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to TestRoPump: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToSanRinse(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Close the Drain MV and open the Recirculation MV
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 0.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to SanRinse: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToSanTankFilling(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Permeate 1
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_1, ACT_SWITCH_ON);
      // Start Timer for 1 second = 1000 milliseconds
          rop_startTransitionTimer(1000U);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE;
   }

   if(ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
          // Close the Permeate2
          ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_OFF);
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
          ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_PERMEATE_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to SanTankFilling: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToSlowFlush(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      // Close the Drain and close the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 0.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 0.0);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (1.0 >= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open the Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          ROP_SlowTransitionCounter = 0U;
          rop_startTransitionTimer(SlowTransitionTab[ROP_SlowTransitionCounter].PhaseDurationMilliSeconds);
      }
   }
   
   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
         ROP_SlowTransitionCounter++;
         if (ROP_SLOW_TRANS_PHASE_MAX > ROP_SlowTransitionCounter)
         {
            MVM_SetValvePosition(MVM_VALVE_DRAIN, SlowTransitionTab[ROP_SlowTransitionCounter].DrainValveOpening);
            rop_startTransitionTimer(SlowTransitionTab[ROP_SlowTransitionCounter].PhaseDurationMilliSeconds);
         }
         else
         {
            p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
            ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
         }
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to Slow Flush: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}

/*!
 *  @param p_TransitionDescription [IN] ROP_TransitionDescription_t <br>
 *                   Pointer to the structure of Transition Parameters
 *
 *  @return none
 *
 *  @brief  Manage a specific transition
 *
 *  @details
 *
 */
static VOID rop_manageTransitionToSlowShortFlush(ROP_TransitionDescription_t * p_TransitionDescription)
{
   // NOTE: Do not change the order of the if conditions !!!
   if(ROP_TRANSITION_START == p_TransitionDescription->TransitionStatus)
   {
      // Open the Permeate 2
      ACT_SetPower(ACT_TYPE_PERMEATE_SV_VALVE_2, ACT_SWITCH_ON);
      // Switch OFF external UF
      ACT_SetPower(ACT_TYPE_UF_TRIGGER, ACT_SWITCH_OFF);

      // Close the Drain and open the Recirc
      MVM_SetMode(MVM_MODE_FORCED);
      MVM_SetValvePosition(MVM_VALVE_DRAIN, 0.0);
      MVM_SetValvePosition(MVM_VALVE_RECIRCULATION, 100.0);

      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE;
   }

   if(ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if ((1.0 >= MVM_GetPosition(MVM_VALVE_DRAIN)) &&
          (99.0 <= MVM_GetPosition(MVM_VALVE_RECIRCULATION)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_DRAIN)) &&
          (MVM_STATUS_POSITION_REACHED == MVM_GetStatus(MVM_VALVE_RECIRCULATION))) // Motor Valve are OK
      {
          // Open the Inlet Valve
          ACT_SetPower(ACT_TYPE_INLET_SV_VALVE, ACT_SWITCH_ON);
          if (TRUE == SYS_GetDeviceOption(SYSTEM_OPT_EXT_INLET_VALVE)) // if External Inlet Valve is enable
          {
             // Open the external valve
             ACT_SetPower(ACT_TYPE_EXT_SV_VALVE, ACT_SWITCH_ON);
          }
          p_TransitionDescription->TransitionStatus = ROP_TRANSITION_WAIT_AFTER_INLET_VALVE;
          ROP_SlowTransitionCounter = 0U;
          rop_startTransitionTimer(SlowTransitionTab[ROP_SlowTransitionCounter].PhaseDurationMilliSeconds);
      }
   }
   
   if(ROP_TRANSITION_WAIT_AFTER_INLET_VALVE == p_TransitionDescription->TransitionStatus)
   {
      if (TRUE == rop_isTransitionTimerExpired()) // Timer is OK
      {
         ROP_SlowTransitionCounter++;
         if (ROP_SLOW_TRANS_PHASE_MAX > ROP_SlowTransitionCounter)
         {
            MVM_SetValvePosition(MVM_VALVE_DRAIN, SlowTransitionTab[ROP_SlowTransitionCounter].DrainValveOpening);
            rop_startTransitionTimer(SlowTransitionTab[ROP_SlowTransitionCounter].PhaseDurationMilliSeconds);
         }
         else
         {
            p_TransitionDescription->TransitionStatus = ROP_TRANSITION_DONE;
            ProductionProcessParameters.ProcessDescription.StateTimeCounter = TMC_StartTimeCounter(); // Restart State time Counter at the end of transition
         }
      }
   }

   // Check consistency
   if((ROP_TRANSITION_START != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_FOR_MOTOR_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_WAIT_AFTER_INLET_VALVE != p_TransitionDescription->TransitionStatus) &&
      (ROP_TRANSITION_DONE != p_TransitionDescription->TransitionStatus))
   {
      ERROR("Unknown Transition to Slow Flush: %d", p_TransitionDescription->TransitionStatus);
      p_TransitionDescription->TransitionStatus = ROP_TRANSITION_START;
   }
}
/*!
 *  @}
 */

/******* (C) COPYRIGHT 2011 MILLIPORE Corporation ***** End Of File ***********/

