/*!
* @file toc_state_machine.c
*
* @brief Total Organic Carbon measure
*
* @details Total Organic Carbon measure state machine
*
* @cond Copyright
* (c) Copyright 2013 MILLIPORE Corporation unpublished work.
* This computer program includes Confidential, Proprietary Information
* and is a Trade Secret of MILLIPORE corporation.
* All use, disclosure, and/or reproduction is prohibited
* unless authorized in writing.
* All rights reserved.
* @endcond
*
* @cond SVN
* $Id:: toc_state_machine.c 669 #$: FileName
* $Rev:: 669                     $: Revision of last commit
* $Author:: m175711              $: Author of last commit
* $Date:: 2014-12-09 18:12:04 +0#$: Date of last commit
* $URL: svn://frgu1w0001/Large2/Large2-process_board/branches/AFS/dev/sources/millipore/application/toc/src/toc_state_machine.c $: URL of this file
* @endcond
*
*/

/* ------------------------------------------------------------------------- */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "CanOpenIface.h"
#include "dil_api.h"
#include "dil_uart.h"
#include "toc_data.h"
#include "sys_softerr.h"
#include "messaging.h"
#include "toc_api.h"
#include "dip_api.h"
#include "sys_api.h"
#include "do_itf.h"
#include "do_def.h"

/* Groups Definition ---------------------------------------------------------*/
/*!
 *
 *  @ingroup Toc_data
 *  @{
 */


/* Definitions ---------------------------------------------------------------*/
#ifndef NDEBUG
/*! to define if mode log need */
#define MORE_LOG
#endif
#define DELAY_RX                 (10u)    /*!< 10 ms  , delay to receive one byte at 1200 bds. */
#define DELAY_TX                 (150u)   /*!< 150ms delay to response (!). */
#define DELAY_TX_CMD             (500u)   /*!< Require wait 500 ms after a cmd send  to trig an read. */
#define MAX_TRY_TO_RX            (3u)     /*!< Max retry to read one byte at 1200 bds. */
#define MAX_TOC_STR_LEN          (10u)    /*!< Largest string TOC string length. */
#define TOC_MAX                  (1000)   /*!< Largest TOC value. */
#define TOC_MIN                  (0)      /*!< Wrong smallest TOC value. */
#define M11CMD_LEN               (4u)     /*!< string Cmd length. */
#define TOC_IS_NB_RUN_COMPLETE   (1)      /*!< Nb cycle cycle reach value,  1 if reach , 0 otherwise. */
#define TOC_IS_UNEXPECT_ERROR    (-1)     /*!< Unexpected error during polling. */
#define TOC_NB_POLLING           (36u)    /*!< Nb polling to wait before retrieve a measure. Corespond to a delay of 6min/10sec   */
#define TOC_NB_POLLING_2ND_RND   (36u)    /*!< Nb polling to wait before retrieve a measure on 2 nd round. Correspond to a delay of 6min/10sec */
#define TOC_COMA10_ERROR_THRES   (5u)     /*!< Allowed coma10 error before cancelling and raising alarm. */
#define TOC_CLEAN_10SEC_COUNTER  (6u)     /*!< Clean period delay (step 10sec).  */



/*! TOC A10 commands list. */
typedef enum
{
   MZ_CMD_STANDBY_MODE,
   MR_CMD_RESTART_AUTO_TOC_MODE_MASTER_RESET,
   MD_CMD_SELECT_AUTO_TOC_MODE,
   RI_CMD_NB_OF_RUNS_COMPLETED,
   RO_CMD_LAST_TOC_IN_PPB,
   RR_CMD_CURRENT_RESISTIVITY,
   RE_CMD_READ_ERROR_CODE,
   ME_CMD_CLEAR_ERROR,
   FL_UV_LAMP_MINUTES_IN_SERVICE,
   M0_CMD_DISABLE_A100_COMPATIBILITY_MODE,
   M1_CMD_ENABLE_A100_COMPATIBILITY_MODE,
   FH_CMD_UPPER_TOC_RANGE_IN_PPB,
   MC_CMD_SELECT_CLEAN_MODE,
   FA_CMD_MANUAL_ANALYSIS_MODE,
     SV_CMD_OPEN_VALVE,
     CV_CMD_CLOSE_VALVE,
     S4_CMD_LAMP_ON,
     C4_CMD_LAMP_OFF,
   MAX_TOC_COMMANDS
}TocCmd_t;

/*! TOC COMA10 link status. */
typedef enum
{
   TOC_EXCH_OK,
   TOC_EXCH_READ_FAILURE
} Toc_EchgRes_t;


/* Declarations ------------------------------------------------------------- */
static DIL_HANDLE       _dilTocUartHdl=NULL;       /*! Uart port used to communicate with A10 module. */

static CONST PLAINCHAR * CONST _M11MCommandsTable[MAX_TOC_COMMANDS]=
{
   "MZ\r\n",  // MZ_CMD_STANDBY_MODE,
   "MR\r\n",  // MR_CMD_RESTART_AUTO_TOC_MODE_MASTER_RESET,
   "MD\r\n",  // MD_CMD_SELECT_AUTO_TOC_MODE,
   "RI\r\n",  // RI_CMD_NB_OF_RUNS_COMPLETED,
   "RO\r\n",  // RO_CMD_LAST_TOC_IN_PPB,
   "RR\r\n",  // RR_CMD_CURRENT_RESISTIVITY,
   "RE\r\n",  // RE_CMD_READ_ERROR_CODE,
   "ME\r\n",  // ME_CMD_CLEAR_ERROR,
   "FL\r\n",  // FL_UV_LAMP_MINUTES_IN_SERVICE,
   "M0\r\n",  // M0_CMD_DISABLE_A100_COMPATIBILITY_MODE,
   "M1\r\n",  // M1_CMD_ENABLE_A100_COMPATIBILITY_MODE,
   "FH\r\n",  // FH_CMD_UPPER_TOC_RANGE_IN_PPB,
   "MC\r\n",  // MC_CMD_SELECT_CLEAN_MODE,
   "FA\r\n",  // FA_CMD_MANUAL_ANALYSIS_MODE,
   "SV\r\n",  // SV_CMD_OPEN_VALVE,
   "CV\r\n",  // CV_CMD_CLOSE_VALVE,
   "S4\r\n",  // S4_CMD_LAMP_ON,
   "C4\r\n",  // C4_CMD_LAMP_OFF
};

#ifdef MORE_LOG
CONST PLAINCHAR *_szStateStringArray[TOC_STATE_LAST]=
{
   "STATE_IDLE    ",
   "STATE_AUTO    ",
   "STATE_CLEANING",
   "STATE_WAITING "
};

CONST PLAINCHAR *_szAutoSeqStepStringArray[TOC_MEAS_LAST_STEP]=
{
   "START_AUTO_MEAS_STEP",
   "WAIT_END_MEAS_STEP  ",
   "GET_TOC_VALUE_STEP  ",
   "GET_ERROR_CODE_STEP ",
   "CLEAR_ERROR_STEP    ",
   "MASTER_RESET_STEP   "
};

#define TOC_ERROR_LAST  (10)

CONST PLAINCHAR *_szA10TocErrorArray[TOC_ERROR_LAST]=
{
   "Eeprom failure",
   "Bad_A2D frequency from V/F converter too low or not present",
   "Bad thermistor reading out of bounds",
   "Conductivity and temperature readings error",
   "Temperature less than 4 degrees",
   "Temperature more than 41 degrees",
   "Conductivity more than 1.1 uS/cm",
   "Overheat (exceeds 55 deg C during oxidation)",
   "Incomplete oxydation after 6 min",
   "Low_oxidation_rate answer less than 0.1 ppb"
};
#endif

/*!
 *  @}
 */

/* Codes ------------------------------------------------------------------- */

/*! @ingroup Toc_func
 *  @{
 */

/*!
 *  @brief  Send command to A10 module and optionnaly receive anwer.
 *
 *  @details If we expect an answer to command sent user shall provide a buffer pointer \a pRxBuf \n
 *           different from NULL, and \a dwRxBufSize which represent the buffer size.\n
 *           The reception ends :
 *                   -# upon the '\\r' char reception .
 *                   -# upon a timeout whithout any reception AND a try counter over \a MAX_TRY_TO_RX.
 *                   -# Rx buffer too small.\n
 *            \n
 *           \p DELAY_TX is the delay betwwen Req send and Answer received. Ie for A10, this is 150 ms.\n
 *           \p DELAY_RX is the delay betwwen two consecutive received chars. At 1200 Bds, we shall wait 8.33 ms (round to 10ms)\n
 *            \n
 *  Communication with Toc use UART6 configure at 1200 bds-8-1s-No parity.\n
 *  This means one char can be transmit in 8.33 ms.
 *
 *  @param tocCmd [IN] TocCmd_t <br>
 *                Toc cmd tp send
 *
 *  @param pRxBuf [IN] PLAINCHAR * <br>
 *                Pointer to received bytes from A10 module if expected.
 *                If not needed, pRxBuf shall be set to NULL.
 *
 *  @param p_dwNbBytesRead [OUT] UINT32 * <br>
 *                Bytes received counter pointer. Store the \n
 *                 number of byte received
 *
 *  @param dwRxBufSize [IN] UINT32<br>
 *                Rx buffer size (bytes). Used to avoid write out of buffer.
 *
 *  @return Toc_EchgRes_t [OUT] exchange result code.
 *  @retval    TOC_EXCH_OK if everything is OK.
 *  @retval    TOC_EXCH_READ_FAILURE upon failure.
 *
 */
static Toc_EchgRes_t _SendA10Cmd(TocCmd_t tocCmd,PLAINCHAR *pRxBuf,UINT32 *p_dwNbBytesRead,UINT32 dwRxBufSize)
{
#ifndef  D_BSP_STM3240GEVAL
   DIL_RC dilRc=DIL_FAILURE;
   DIL_UINT32 dwByteWritten=0u;
   DIL_UINT32 dwTotalBytesRead=0u;
   Toc_EchgRes_t Toc_res=TOC_EXCH_OK;

   ASSERT(tocCmd<MAX_TOC_COMMANDS,"Wrong TocCmd to sent (%d)",tocCmd);

   /*! Send Cmd */
   dilRc = dil_writedevice(_dilTocUartHdl, (DIL_PVOID)_M11MCommandsTable[(UINT8)tocCmd],(DIL_UINT32) M11CMD_LEN, &dwByteWritten);
   ASSERT(DIL_GOOD==dilRc,"Error in sending TocCmd %d",tocCmd);

   if(NULL !=pRxBuf)
   {
      /*! Try read a char from Uart until timeout occurs
       *  Timeout is set to receive 3 consecutive char so that when A10Toc ends transmission,
       *  we shall trig a timeout and last receive char in buffer shall be '\\r'.
       */
      dilRc = dil_readdevice(_dilTocUartHdl,(DIL_PVOID)pRxBuf,dwRxBufSize,&dwTotalBytesRead);

      if( !dwTotalBytesRead )
      {
         Toc_res=TOC_EXCH_READ_FAILURE;
      }
      else if ( '\r' != pRxBuf[dwTotalBytesRead-1] )
      {
         Toc_res=TOC_EXCH_READ_FAILURE;
      }
      else
      {
         /**/
         Toc_res=TOC_EXCH_OK;
      }
   }
   else
   {
      /**/
      Toc_res=TOC_EXCH_OK;
   }
#endif //D_BSP_STM3240GEVAL

   /* update nbr bytes read */
   if(NULL != p_dwNbBytesRead)
   {
      *p_dwNbBytesRead=dwTotalBytesRead;
   }

   return Toc_res;
}


/*!
 *  @brief  reset \a _alarmType of Toc
 *
 *  @param tocAlarmType [IN] as AlarmType_t <br>
 *                Toc alarm type
 *
 *  @return None
 *
 *  @details Alarm is saved in Toc data context to avoid resetting it several times.
 */
static VOID _ResetA10Alarm(AlarmType_t tocAlarmType)
{
   if(tocAlarmType == g_TocData.alarmSet)
   {
      AAA_ResetAlarm(tocAlarmType);
      g_TocData.alarmSet=(AlarmType_t)0u;
   }
   else if((AlarmType_t)0u!=g_TocData.alarmSet)
   {
      ASSERT(tocAlarmType == g_TocData.alarmSet,"Wrong Toc alarm to reset %d",(UINT8)tocAlarmType);
   }
   else
   {
      /* Dear misra */
   }
   /* Reset CAN TOC Sensors diag bit */
   if(ALARM_TYPE_TOC_CHECK_A10_COM==tocAlarmType)
   {
      DO_AND(g_DoHardwareDiagnostic.SensorsDiagResult,~TOC_DIAG_CAN_BIT);
   }
}


/*!
 *  @brief  set \a _alarmType of Toc.\n
 *          Save alarm set to avoid set it again.\n
 *          Reset previous saved alarm set if exist.
 *
 *  @param tocAlarmType [IN] as AlarmType_t <br>
 *                Toc alarm type
 *
 *  @return None
 *
 *  @details Alarm is saved in Toc data context to avoid raising it several times.
 */
static VOID _SetA10Alarm(AlarmType_t tocAlarmType)
{
   if(tocAlarmType != g_TocData.alarmSet)
   {
      /* reset previous alarm set */
      if(0u!=g_TocData.alarmSet)
      {
         AAA_ResetAlarm(g_TocData.alarmSet);
      }
      /* Set new alarm */
      AAA_SetAlarm(tocAlarmType);
      g_TocData.alarmSet=tocAlarmType;
   }
   /* Set CAN TOC Sensors diag bit */
   if(ALARM_TYPE_TOC_CHECK_A10_COM==tocAlarmType)
   {
      DO_OR(g_DoHardwareDiagnostic.SensorsDiagResult,TOC_DIAG_CAN_BIT);
   }
}


/*!
 *  @brief  Check TOC value against SP stored in DO and update Can
 *
 *  @details if value is above SP, raised alarm.
 *
 *  @param wTocValue [IN] as UINT16 <br>
 *                Measured Toc value.
 *
 *  @return None
 *
 */
static VOID _UpdateAndCheckTocValue (UINT16 wTocValue)
{
   UINT16 wUpperTOCvalueSP;

   /* Check regarding SP */
   DO_READ(wUpperTOCvalueSP,g_DoAlertAndAlarmsSetpoints.TocSP);

   if(TOC_MAX > wTocValue)
   {
      if( wTocValue > wUpperTOCvalueSP )
      {
         _SetA10Alarm(ALARM_TYPE_TOC_ABOVE_SP);
      }
      else
      {
         _ResetA10Alarm(ALARM_TYPE_TOC_ABOVE_SP);
      }
   }
   else
   {
      wTocValue = 0xFFFF;
   }

   /* Update Toc value in DO */
   DO_WRITE(g_DoTotalOrganicCarbon.LastValue,wTocValue);

   /*! Trig PDO send */
   canSendPdo(TPDO_TOC_MEASURE);
}


/*! @fn _UpdateStatus(newstatus)
 *
 *  @brief  Store A10 error code in CAN Do.
 *
 *  @param bTocErrorCode [IN] UINT8  <br>
 *                A10 error code.
 *
 *  @return None
 *
 */
VOID _UpdateTocErrorCode(UINT8 bTocErrorCode)
{
   DO_WRITE(g_DoTotalOrganicCarbon.TocErrorStatus,bTocErrorCode);
   if(0u!=bTocErrorCode)
   {
      _SetA10Alarm(ALARM_TYPE_TOC_A10_ERROR);
   }
   else
   {
      /* If No A10 error to report , but alarm set , clear alarm */
      if(ALARM_TYPE_TOC_A10_ERROR==g_TocData.alarmSet)
      {
         _ResetA10Alarm(ALARM_TYPE_TOC_A10_ERROR);
      }
   }
}


/*! @fn VOID _UpdateStatus(TOC_Status_t newstatus)
 *
 *  @brief status change to \a newstatus
 *       and update DO toc Status
 *
 *  @param newstatus [IN] as TOC_Status_t <br>
 *                Toc status
 *
 *  @return None
 *
 */
static VOID _UpdateStatus(TOC_Status_t newstatus)
{
   DO_WRITE(g_DoTotalOrganicCarbon.State,(UINT8)newstatus);
   /* No PDO sent actually */
}


/*!
 * @brief Toc standby sequencer
 *
 * @return Toc_SeqRet_t sequencer staus
 * @retval TOC_SEQ_RUNNING if seq shall continue to be execute.
 * @retval TOC_SEQ_WAITING if seq shall wait 10sec period.
 * @retval TOC_SEQ_OVER if seq ends.
 *
 * @details No alarms raised
 *
 */
static Toc_SeqRet_t _TocStanbyStep(VOID)
{
   Toc_EchgRes_t tocRes=TOC_EXCH_OK;
   tocRes = _SendA10Cmd(MZ_CMD_STANDBY_MODE,NULL,NULL,0u);
   if( TOC_EXCH_READ_FAILURE==tocRes)
   {
      _SetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
   }

   return TOC_SEQ_OVER;
}


/*!
 * @brief Toc Automatic measure sequencer
 *
 * @return Toc_SeqRet_t sequencer staus
 * @retval TOC_SEQ_RUNNING if seq shall continue to be execute.
 * @retval TOC_SEQ_WAITING if seq shall wait 10sec period.
 * @retval TOC_SEQ_OVER if seq ends.
 *
 * @details Delay between polling : 10 sec. \n
 *          Overall polling period duration : 2x6 mn. \n
 *          According latest test, it has been observed that A10 Module require 9min 30 s to get a measure.
 *
 *  @dot digraph AutomaticMeasureSequencer
 *      {
 *         start;
 *
 *          subgraph cluster_0
 *          {
 *            label="Automatic measure sequencer";
 *
 *             node [shape=record, fontname=Helvetica, fontsize=10 ,style="blue" ];
 *
 *             start_step   [ label="Start measure"         URL="\ref TocStartAutoMeas"  ];
 *             wait_step    [ label="Wait end measure"      URL="\ref TocWaitEndMeas"];
 *             read_step    [ label="Read measure result"   URL="\ref TocReadMeasResult"];
 *             error_step   [ label="Get Error code"        URL="\ref TocGetErrorCode"];
 *             clear_step   [ label="Clear Error"           URL="\ref TocClearError"];
 *             reset_step   [ label="Reset A10 module"      URL="\ref TocReset"];
 *             standby_step [ label="Standby module"        URL="\ref TocStandby"];
 *
 *             start_step->wait_step;
 *             wait_step->wait_step;wait_step->read_step;wait_step->error_step;
 *             read_step->error_step;read_step->wait_step;read_step->standby_step;
 *             error_step->clear_step;
 *             clear_step->wait_step;clear_step->reset_step;
 *             reset_step->standby_step;
 *          };
 *
 *          start->start_step;
 *          standby_step->end;
 *      }
 *  @enddot
 */
static Toc_SeqRet_t _TocAutomaticSeq(VOID)
{
   Toc_SeqRet_t seqRc=TOC_SEQ_RUNNING;    /* By default , seq is running to switch next step  */
   UINT32 dwNbBytesRead;
   PLAINCHAR szTocReadStr[MAX_TOC_STR_LEN];
   INT16 swTocValue=0;
   INT8 sbIsTocNbRunComplete=0u;
   Toc_EchgRes_t tocRes=TOC_EXCH_OK;
   Toc_AutoMeasSeq_t oldStep=g_TocData.SeqData.Auto.seqStep;
   ASSERT(TOC_MEAS_LAST_STEP > g_TocData.SeqData.Auto.seqStep,"Wrong g_TocData.SeqData.Auto.seqStep %d in Toc automatic measure",g_TocData.SeqData.Auto.seqStep);

   switch(g_TocData.SeqData.Auto.seqStep)
   {
      /* Start Automatic Toc measure */
      /* =========================== */
      /*!\anchor TocStartAutoMeas    */
      case TOC_MEAS_START_AUTO_MEAS_STEP:
         {
         tocRes = _SendA10Cmd(MD_CMD_SELECT_AUTO_TOC_MODE,NULL,NULL,0u);
         g_TocData.SeqData.Auto.seqStep = TOC_MEAS_WAIT_END_MEAS_STEP;
         g_TocData.SeqData.Auto.bNbPolling=TOC_NB_POLLING;
         g_TocData.bNbA10ComError=0;
         g_TocData.SeqData.Auto.isSecondRound=FALSE;
         }
         break;

      /* Check Nbr run completed */
      /* ======================= */
      /*!\anchor  TocWaitEndMeas */
      case TOC_MEAS_WAIT_END_MEAS_STEP:
         {
         if(g_TocData.SeqData.Auto.bNbPolling)
         {
            g_TocData.SeqData.Auto.bNbPolling--;
            tocRes = _SendA10Cmd(RI_CMD_NB_OF_RUNS_COMPLETED,(PLAINCHAR*)szTocReadStr,&dwNbBytesRead,sizeof(szTocReadStr));
            if(TOC_EXCH_READ_FAILURE==tocRes)
            {
               /* Read operation error failure */
               g_TocData.bNbA10ComError++;
               if(TOC_COMA10_ERROR_THRES < g_TocData.bNbA10ComError)
               {
                  WARNING("T:Failed to get A10 anwser (%d bytes received)",dwNbBytesRead);
                  g_TocData.bNbA10ComError=0;
                  _SetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
                  seqRc=TOC_SEQ_OVER;
                  break;
               }
               /* wait for next try */
               seqRc=TOC_SEQ_WAITING;
               break;
            }
            else
            {
               /* Read success */
               if(ALARM_TYPE_TOC_CHECK_A10_COM==g_TocData.alarmSet)
               {
                  _ResetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
               }
               /* convert Run number */
               sbIsTocNbRunComplete=(INT8)strtol(szTocReadStr,NULL, 10u);
               /* Check compelete by receiving '1' */
               if(TOC_IS_NB_RUN_COMPLETE==sbIsTocNbRunComplete)
               {
                  /* If Nbr run complete, sw to next step  */
                  g_TocData.SeqData.Auto.bNbPolling=0;
                  g_TocData.SeqData.Auto.seqStep = TOC_MEAS_GET_TOC_VALUE_STEP;
                  OS_TaskDelay(DELAY_TX_CMD);
                  /* keep running seq */
               }
               if(TOC_IS_UNEXPECT_ERROR==sbIsTocNbRunComplete)
               {
                  /* Unexpected error */
                  /* sw to read error code step */
                  g_TocData.SeqData.Auto.seqStep = TOC_MEAS_GET_ERROR_CODE_STEP;
                  OS_TaskDelay(DELAY_TX_CMD);
                  /* keep running seq */
               }
               else
               {
                  /* if not complete, wait another run */
                  seqRc=TOC_SEQ_WAITING;
                  SYSLOG(INFO,"T:Wait %d sec",(g_TocData.SeqData.Auto.bNbPolling+1)*10);
               }
            }
         }
         else
         {
            /* No more polling :
             * Upon first round, we get measure evene if not finish (?)
             * On second round , we get error code.
             */
              if(FALSE==g_TocData.SeqData.Auto.isSecondRound)
            {
               /* sw to  the get ppb value step */
               g_TocData.SeqData.Auto.seqStep=TOC_MEAS_GET_TOC_VALUE_STEP;
               /* keep running seq */
            }
            else
            {
               /* sw to  the read error code step */
               g_TocData.SeqData.Auto.seqStep=TOC_MEAS_GET_ERROR_CODE_STEP;
               /* keep running seq */
               SYSLOG(INFO,"T:Second round over wo result");
            }
            OS_TaskDelay(DELAY_TX_CMD);
         }
         }
         break;

      /* Read Toc value in ppb */
      /* ===================== */
      /*!\anchor  TocReadMeasResult */
      case TOC_MEAS_GET_TOC_VALUE_STEP:
         {
         tocRes = _SendA10Cmd(RO_CMD_LAST_TOC_IN_PPB,(PLAINCHAR*)&szTocReadStr,&dwNbBytesRead,sizeof(szTocReadStr));
         if(TOC_EXCH_READ_FAILURE==tocRes)
         {
            /* Read operation error failure */
            _SetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
            seqRc=TOC_SEQ_OVER;
            break;
         }
         /* Convert and check Toc value against SP */
         swTocValue=strtol(szTocReadStr,NULL, 10u);
         SYSLOG(INFO,"T:Toc value = %d",swTocValue);
         /* On unexpected error , sw to Read error  step */
         if(   (TOC_IS_UNEXPECT_ERROR==swTocValue)
            || (TOC_MAX<swTocValue)
            || (TOC_MIN>=swTocValue)  )
         {
            if(FALSE==g_TocData.SeqData.Auto.isSecondRound)
            {
               SYSLOG(INFO,"T:First round over wo valid result");
               g_TocData.SeqData.Auto.isSecondRound=TRUE;
               /* Re-Init Nb Polling, delay 3' */
               g_TocData.SeqData.Auto.bNbPolling=TOC_NB_POLLING_2ND_RND;
               g_TocData.SeqData.Auto.seqStep = TOC_MEAS_WAIT_END_MEAS_STEP;
            }
            else
            {
               g_TocData.SeqData.Auto.seqStep = TOC_MEAS_GET_ERROR_CODE_STEP;
            }
            OS_TaskDelay(DELAY_TX_CMD);
            /* keep running seq */
         }
         else
         {
            /* Update Toc value in DO and go to Idle mode */
            _UpdateAndCheckTocValue((UINT16)swTocValue);
            g_TocData.SeqData.Auto.seqStep = TOC_MEAS_STANDBY_STEP;
            OS_TaskDelay(DELAY_TX_CMD);
         }
         }
         break;


      /* Read Error code */
      /* =============== */
      /*!\anchor  TocGetErrorCode */
      case TOC_MEAS_GET_ERROR_CODE_STEP:
         {
         tocRes = _SendA10Cmd(RE_CMD_READ_ERROR_CODE,(PLAINCHAR*)&szTocReadStr,&dwNbBytesRead,sizeof(szTocReadStr));
         if(TOC_EXCH_READ_FAILURE==tocRes)
         {
            /* Read operation error failure */
            _SetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
            g_TocData.SeqData.Auto.seqStep = TOC_MEAS_MASTER_RESET_STEP;
            OS_TaskDelay(DELAY_TX_CMD);
            /* keep running seq */
            break;
         }
         /* Convert and Record error code */
         swTocValue=strtol(szTocReadStr,NULL, 10u);
         /* On unexpected error , sw to send reset step */
         if(TOC_IS_UNEXPECT_ERROR==swTocValue)
         {
            g_TocData.SeqData.Auto.seqStep = TOC_MEAS_MASTER_RESET_STEP;
            OS_TaskDelay(DELAY_TX_CMD);
            /* keep running seq */
         }
         else
         {
            _UpdateTocErrorCode((UINT8)swTocValue);
#ifndef MORE_LOG
            SYSLOG(INFO,"T:ErrorCode = %d",swTocValue);
#else
            if(TOC_ERROR_LAST>swTocValue)
            {
               SYSLOG(INFO,"T:%s (%d)",_szA10TocErrorArray[swTocValue],swTocValue);
            }
            else
            {
               SYSLOG(INFO,"T:ErrorCode = %d",swTocValue);
            }
#endif
            g_TocData.SeqData.Auto.seqStep=TOC_MEAS_CLEAR_ERROR_STEP;
            /* keep running seq */
         }
         }
         break;

      /* Clear Error */
      /* =========== */
      /*!\anchor  TocClearError */
      case TOC_MEAS_CLEAR_ERROR_STEP:
         {
         tocRes = _SendA10Cmd(ME_CMD_CLEAR_ERROR,NULL,NULL,0u);
         if(TOC_EXCH_READ_FAILURE==tocRes)
         {
            /* Read operation error failure */
            _SetA10Alarm(ALARM_TYPE_TOC_CHECK_A10_COM);
            g_TocData.SeqData.Auto.seqStep = TOC_MEAS_MASTER_RESET_STEP;
            OS_TaskDelay(DELAY_TX_CMD);
            /* keep running seq */
            break;
         }
         /* Check remaining poll to handle */
         if(g_TocData.SeqData.Auto.bNbPolling)
         {
            /* return to polling step  */
            g_TocData.SeqData.Auto.seqStep=TOC_MEAS_WAIT_END_MEAS_STEP;
            seqRc=TOC_SEQ_WAITING;
         }
         else
         {
            /* normal end */
            seqRc=TOC_SEQ_OVER;
         }
         }
         break;

      /* Toc master Reset  */
      /* ================= */
      /*!\anchor  TocReset */
      case TOC_MEAS_MASTER_RESET_STEP:
         {
          /* send a master reset command */
         _SendA10Cmd(MR_CMD_RESTART_AUTO_TOC_MODE_MASTER_RESET,NULL,NULL,0u);
         g_TocData.SeqData.Auto.seqStep = TOC_MEAS_STANDBY_STEP;
         OS_TaskDelay(DELAY_TX_CMD);
         }
         break;

      /* Toc standby */
      /* =========== */
      /*!\anchor  TocStandby */
      case TOC_MEAS_STANDBY_STEP:
         {
          /* send a master reset command */
         _SendA10Cmd(MZ_CMD_STANDBY_MODE,NULL,NULL,0u);
         OS_TaskDelay(DELAY_TX_CMD);
         /* normal end */
         seqRc=TOC_SEQ_OVER;
         }
         break;

      default:
         {
         ERROR("Wrong step in Toc Auto %d",g_TocData.SeqData.Auto.seqStep);
         }
         break;
   }

   /* reset g_TocData.SeqData.Auto.seqStep if ending sequencer  */
   if(TOC_SEQ_OVER==seqRc)
   {
      g_TocData.SeqData.Auto.seqStep = TOC_MEAS_START_AUTO_MEAS_STEP;
   }

   if(TOC_SEQ_WAITING!=seqRc)
   {
#ifndef MORE_LOG
      SYSLOG(INFO,"T:AutoMeasSeq step=%d->%d",oldStep,g_TocData.SeqData.Auto.seqStep);
#else
      SYSLOG(INFO,"T:AutoMeasSeq step=%s -> %s",_szAutoSeqStepStringArray[oldStep],_szAutoSeqStepStringArray[g_TocData.SeqData.Auto.seqStep]);
#endif
   }

   return seqRc;
}


/*!
 * @brief Toc Cleanning sequencer.
 *
 * @return Toc_SeqRet_t sequencer staus
 * @retval TOC_SEQ_RUNNING if seq shall continue to be execute.
 * @retval TOC_SEQ_WAITING if seq shall wait 10sec period.
 * @retval TOC_SEQ_OVER if seq ends.
 *
 * @details No alarms raised.
 *          Operating delay managed with \p TOC_CLEAN_10SEC_COUNTER slice counter.
 *
 */
static Toc_SeqRet_t _TocCleaningSeq()
{
   if(0u==g_TocData.SeqData.Cleaner.wCleanSeqCnt)
   {
      _SendA10Cmd(MC_CMD_SELECT_CLEAN_MODE,NULL,NULL,0u);
   }
   if(TOC_CLEAN_10SEC_COUNTER <= g_TocData.SeqData.Cleaner.wCleanSeqCnt)
   {
      SYSLOG(INFO,"T:Cleaning : Over.");
      return TOC_SEQ_OVER;
   }
   else
   {
      SYSLOG(INFO,"T:Cleaning : Remaining %d sec.",(TOC_CLEAN_10SEC_COUNTER-g_TocData.SeqData.Cleaner.wCleanSeqCnt)*10);
      g_TocData.SeqData.Cleaner.wCleanSeqCnt++;
      return TOC_SEQ_WAITING;
   }

}


/*!
 *  @brief  Pack \a new state change to \a _newstate
 *
 *  @details
 *
 *  @dot digraph A10TocStateMachine
 *      {
 *         start;
 *
 *          subgraph cluster_0
 *          {
 *            label="A10 Toc State Machine";
 *
 *             node [shape=record, fontname=Helvetica, fontsize=10 ,style="blue" ];
 *
 *             idle_sta   [ label="Idle"              URL="\ref TocIdleState"];
 *             auto_sta   [ label="Automatic measure" URL="\ref TocMeasState"];
 *             clean_sta  [ label="Cleaning"          URL="\ref TocCleanState"];
 *             wait_sta   [ label="Waiting"           URL="\ref TocWaitState"];
 *
 *             idle_sta->auto_sta;idle_sta->clean_sta;
 *             auto_sta->idle_sta;
 *             clean_sta->idle_sta;
 *             wait_sta;
 *          };
 *
 *          start->idle_sta;
 *      }
 *  @enddot
 *
 *  @param newstate [IN] as Toc_State_t <br>
 *                Toc state request to switch to.
 *
 *  @return None
 */
static VOID _StateChange(Toc_State_t newstate)
{
   UINT8 oldstate=g_TocData.state;
   if(oldstate != newstate )
   {
      g_TocData.state  = newstate;
      switch(newstate)
      {
         /*! IDLE STATE
          * @anchor TocIdleState
          */
         case TOC_STATE_IDLE:
            {
            /*! Disarm potential wakeup */
            OS_AlarmCancel(g_szTocAlarmName,NULL);
            /*! select return to standby seq */
            g_TocData.seqFunc=_TocStanbyStep;
            g_TocData.status= TOC_STATUS_IDLE;
            /*! reset automatic measure seq step  */
            g_TocData.SeqData.Auto.seqStep=TOC_MEAS_START_AUTO_MEAS_STEP;
            }
            break;

         /*! AUTOMATIC MEASURE STATE
          *  @anchor TocMeasState
          */
         case TOC_STATE_AUTO_MEAS:
            {
            /*! Disarm potential wakeup */
            OS_AlarmCancel(g_szTocAlarmName,NULL);
            _UpdateTocErrorCode(0u);
            /*! select Automatic measure seq */
            g_TocData.seqFunc=_TocAutomaticSeq;
            g_TocData.status= TOC_STATUS_MEASURING;
            /*! reset automatic measure seq step  */
            g_TocData.SeqData.Auto.seqStep=TOC_MEAS_START_AUTO_MEAS_STEP;
            }
            break;

         /*! CLEANING STATE
          *  @anchor TocCleanState
          */
         case TOC_STATE_CLEANING:
            {
            /*! Disarm potential wakeup */
            OS_AlarmCancel(g_szTocAlarmName,NULL);
            /*! reset last Toc erroc code  */
            _UpdateTocErrorCode(0u);
            /*! select cleaning seq */
            g_TocData.seqFunc=_TocCleaningSeq;
            g_TocData.status= TOC_STATUS_CLEANING;
            /*! reset automatic measure seq step  */
            g_TocData.SeqData.Cleaner.wCleanSeqCnt=0u;
            }
            break;

         /*! WAITING STATE
          *  @anchor TocWaitState
          */
         case TOC_STATE_WAITING:
            /* Nothing to be done as we were waiting for long period */
            break;

         default:
            {
            ERROR("T:Wrong Toc state change req %d->%d",oldstate,newstate);
            }
            break;
      }
      /*! Update CAN Toc status */
      _UpdateStatus(g_TocData.status);
#ifndef MORE_LOG
      SYSLOG(INFO,"T:state chg %d->%d",oldstate,newstate);
#else
      SYSLOG(INFO,"T:state chg %s->%s",_szStateStringArray[oldstate],_szStateStringArray[newstate]);
#endif
   }
   else if(TOC_STATE_IDLE==newstate)
   {
      /*! When ending Idle sw back , need
       * to remove seq
       */
      g_TocData.seqFunc=NULL;
   }
   else
   {
      /* Dear MISRA ... */
   }
}


/*!
 *  @brief  TOC measure, and cleaning state machine.
 *
 *  @details Three sequencers are defined whcih manage three mains operations need.
 *           @li TOC measure.
 *           @li TOC cleaning
 *           @li TOC stop measuring or cleaning.
 *           \n
 *           Toc state machine manage the sequencer selection according request given by  \a stateReq.\n
 *           \n
 *           Sequencers may required to wait delay. Thus we exit sequence to stand to \p TOC_SEQ_WAITING state.\n
 *           Once delay ends, sequencer can continue.\n
 *           When sequencer end a step , it can switch directly to a new step with \p TOC_SEQ_RUNNING return.\n
 *           Or it can ends definitively by returning  \p TOC_SEQ_OVER.\n
 *           If ending a state change is called to switch to \p TOC_STATE_IDLE.
 *
 *  @param stateReq [IN] as Toc_State_t <br>
 *                Toc state request to switch to.
 *
 *  @return none
 */
VOID TOC_SmRun(Toc_State_t stateReq)
{
   /*! Manage state change */
   _StateChange(stateReq);

   /*! Call seq according state, */
   if(NULL != g_TocData.seqFunc)
   {
      while(1)
      {
         Toc_SeqRet_t seqRetCode=g_TocData.seqFunc();
         /*! Return to idle if seq ends */
         if(TOC_SEQ_OVER==seqRetCode)
         {
            _StateChange(TOC_STATE_IDLE);
            break;
         }
         /*! Wait request  */
         else if(TOC_SEQ_WAITING==seqRetCode)
         {
            /* leave loop to wait timer occurs */
            OS_AlarmArm(g_szTocAlarmName);
            break;
         }
         else
         {
            /*! Seq continue to run , is running */
         }
      }
   }
}


/*!
 * @brief  State machine initialization
 *
 * @return None
 *
 * @details if Uart2 (PG2) is used for LOG purpose, ...
 */
VOID TOC_SmInit(VOID)
{
   g_TocData.state=TOC_STATE_IDLE;
   g_TocData.status=TOC_STATUS_IDLE;
   g_TocData.alarmSet=(AlarmType_t)0u;
   g_TocData.bNbA10ComError=0;
   g_TocData.seqFunc=NULL;

   _UpdateStatus(TOC_STATUS_IDLE);

#ifndef  D_BSP_STM3240GEVAL
   DIL_RC   ret;
   /*! Attempt to open UART driver (Usart6) */
   ret = dil_opendevice(DIL_NAME_UART3, &_dilTocUartHdl, (DIL_PVOID)NULL);
   ret |= dil_ioctldevice(_dilTocUartHdl, IOCTL_CMD_UART_SET_BAUDRATE, (DIL_LPVOID)(UINT32)DIL_CLASS_UART_1200BPS, sizeof(UINT32), (DIL_LPUINT32)NULL);
   ret |= dil_ioctldevice(_dilTocUartHdl, IOCTL_CMD_UART_SET_PARITY_TYPE, (DIL_LPVOID)(UINT32)IOCTL_CMD_UART_PARITY_TYPE_NONE, sizeof(UINT32), (DIL_LPUINT32)NULL);
   ret |= dil_ioctldevice(_dilTocUartHdl, IOCTL_CMD_UART_SET_STOP_BIT, (DIL_LPVOID)(UINT32)IOCTL_CMD_UART_1STOP_BITS, sizeof(UINT32), (DIL_LPUINT32)NULL);
   ret |= dil_ioctldevice(_dilTocUartHdl, IOCTL_CMD_UART_SET_DATA_BITS, (DIL_LPVOID)(UINT32)IOCTL_CMD_UART_8BITS, sizeof(UINT32), (DIL_LPUINT32)NULL);
   ret |= dil_ioctldevice(_dilTocUartHdl, IOCTL_CMD_UART_SET_CHANNEL_MODE, (DIL_LPVOID)(UINT32)IOCTL_CMD_UART_Tx_RX, sizeof(UINT32), (DIL_LPUINT32)NULL);
   ASSERT(DIL_GOOD==ret,"T:Issue on initialising Toc UART");
#endif
   if(TRUE==SYS_GetDeviceOption(SYSTEM_OPT_TOC))
   {
      _TocStanbyStep();
   }
}

/*!
 *  @}
 */

/******* (C) COPYRIGHT 2014 MILLIPORE Corporation ***** End Of File ***********/


